// Generated by Rex version 1.0 alpha 5

package org.eclipse.photran.internal.core.f95modelparser;


/**
 * A compact representation of the LR machine.
 */
public class ParsingTable
{
    private AbstractParserAction ua;
    private AbstractProductionReductions pr;

    public ParsingTable(AbstractParserAction userActions)
    {
        ua = userActions;
        pr = userActions.getProductionReductions();
    }

    /**
     * Retrieve the object containing all of the user code
     */
    public AbstractParserAction getUserActions() { return ua; }
    protected static final int ACTION_MASK = 0xc000;
    protected static final int VALUE_MASK = 0x3fff;
    protected static final int SHIFT = 0x8000;
    protected static final int REDUCE = 0x4000;
    protected static final int ACCEPT = 0xc000;

    /**
     * Changes the state of the given parser based on its lookahead token
     */
    void processLookaheadFor(Parser parser) throws Exception
    {
        int actionRow = parser.getLookaheadToken().getTerminal().getActionRow();
        int entry = lookupAction(actionRow, parser.getCurrentState());
        int action = entry & ACTION_MASK;
        int value = entry & VALUE_MASK;
        switch (action)
        {
        case ACCEPT: parser.accept(); return;
        case REDUCE: reduceByProduction(value, parser); return;
        case SHIFT: parser.shiftAndGoToState(value); return;

        default:
            parser.syntaxError();
            return;
        }
    }

    /**
     * Runs the user code and reduces to the given production
     */
    void reduceByProduction(int production, Parser parser)
    {
        switch (production)
        {
        case 590: pr.reduceByProduction590(parser); return;
        case 65: pr.reduceByProduction65(parser); return;
        case 1199: pr.reduceByProduction1199(parser); return;
        case 1013: pr.reduceByProduction1013(parser); return;
        case 329: pr.reduceByProduction329(parser); return;
        case 116: pr.reduceByProduction116(parser); return;
        case 609: pr.reduceByProduction609(parser); return;
        case 1242: pr.reduceByProduction1242(parser); return;
        case 907: pr.reduceByProduction907(parser); return;
        case 994: pr.reduceByProduction994(parser); return;
        case 1032: pr.reduceByProduction1032(parser); return;
        case 1206: pr.reduceByProduction1206(parser); return;
        case 1088: pr.reduceByProduction1088(parser); return;
        case 363: pr.reduceByProduction363(parser); return;
        case 260: pr.reduceByProduction260(parser); return;
        case 972: pr.reduceByProduction972(parser); return;
        case 142: pr.reduceByProduction142(parser); return;
        case 1040: pr.reduceByProduction1040(parser); return;
        case 1310: pr.reduceByProduction1310(parser); return;
        case 730: pr.reduceByProduction730(parser); return;
        case 654: pr.reduceByProduction654(parser); return;
        case 463: pr.reduceByProduction463(parser); return;
        case 828: pr.reduceByProduction828(parser); return;
        case 270: pr.reduceByProduction270(parser); return;
        case 478: pr.reduceByProduction478(parser); return;
        case 18: pr.reduceByProduction18(parser); return;
        case 425: pr.reduceByProduction425(parser); return;
        case 306: pr.reduceByProduction306(parser); return;
        case 1224: pr.reduceByProduction1224(parser); return;
        case 689: pr.reduceByProduction689(parser); return;
        case 718: pr.reduceByProduction718(parser); return;
        case 444: pr.reduceByProduction444(parser); return;
        case 356: pr.reduceByProduction356(parser); return;
        case 846: pr.reduceByProduction846(parser); return;
        case 1114: pr.reduceByProduction1114(parser); return;
        case 198: pr.reduceByProduction198(parser); return;
        case 294: pr.reduceByProduction294(parser); return;
        case 295: pr.reduceByProduction295(parser); return;
        case 20: pr.reduceByProduction20(parser); return;
        case 1359: pr.reduceByProduction1359(parser); return;
        case 955: pr.reduceByProduction955(parser); return;
        case 381: pr.reduceByProduction381(parser); return;
        case 514: pr.reduceByProduction514(parser); return;
        case 240: pr.reduceByProduction240(parser); return;
        case 167: pr.reduceByProduction167(parser); return;
        case 1165: pr.reduceByProduction1165(parser); return;
        case 778: pr.reduceByProduction778(parser); return;
        case 752: pr.reduceByProduction752(parser); return;
        case 134: pr.reduceByProduction134(parser); return;
        case 507: pr.reduceByProduction507(parser); return;
        case 206: pr.reduceByProduction206(parser); return;
        case 199: pr.reduceByProduction199(parser); return;
        case 809: pr.reduceByProduction809(parser); return;
        case 1075: pr.reduceByProduction1075(parser); return;
        case 457: pr.reduceByProduction457(parser); return;
        case 937: pr.reduceByProduction937(parser); return;
        case 195: pr.reduceByProduction195(parser); return;
        case 1096: pr.reduceByProduction1096(parser); return;
        case 556: pr.reduceByProduction556(parser); return;
        case 1002: pr.reduceByProduction1002(parser); return;
        case 39: pr.reduceByProduction39(parser); return;
        case 181: pr.reduceByProduction181(parser); return;
        case 1111: pr.reduceByProduction1111(parser); return;
        case 503: pr.reduceByProduction503(parser); return;
        case 91: pr.reduceByProduction91(parser); return;
        case 884: pr.reduceByProduction884(parser); return;
        case 477: pr.reduceByProduction477(parser); return;
        case 542: pr.reduceByProduction542(parser); return;
        case 237: pr.reduceByProduction237(parser); return;
        case 249: pr.reduceByProduction249(parser); return;
        case 59: pr.reduceByProduction59(parser); return;
        case 427: pr.reduceByProduction427(parser); return;
        case 749: pr.reduceByProduction749(parser); return;
        case 1090: pr.reduceByProduction1090(parser); return;
        case 281: pr.reduceByProduction281(parser); return;
        case 1011: pr.reduceByProduction1011(parser); return;
        case 144: pr.reduceByProduction144(parser); return;
        case 1142: pr.reduceByProduction1142(parser); return;
        case 546: pr.reduceByProduction546(parser); return;
        case 814: pr.reduceByProduction814(parser); return;
        case 93: pr.reduceByProduction93(parser); return;
        case 102: pr.reduceByProduction102(parser); return;
        case 104: pr.reduceByProduction104(parser); return;
        case 1338: pr.reduceByProduction1338(parser); return;
        case 100: pr.reduceByProduction100(parser); return;
        case 592: pr.reduceByProduction592(parser); return;
        case 570: pr.reduceByProduction570(parser); return;
        case 1213: pr.reduceByProduction1213(parser); return;
        case 1358: pr.reduceByProduction1358(parser); return;
        case 1052: pr.reduceByProduction1052(parser); return;
        case 939: pr.reduceByProduction939(parser); return;
        case 1187: pr.reduceByProduction1187(parser); return;
        case 1361: pr.reduceByProduction1361(parser); return;
        case 1071: pr.reduceByProduction1071(parser); return;
        case 984: pr.reduceByProduction984(parser); return;
        case 323: pr.reduceByProduction323(parser); return;
        case 989: pr.reduceByProduction989(parser); return;
        case 480: pr.reduceByProduction480(parser); return;
        case 2: pr.reduceByProduction2(parser); return;
        case 988: pr.reduceByProduction988(parser); return;
        case 29: pr.reduceByProduction29(parser); return;
        case 523: pr.reduceByProduction523(parser); return;
        case 263: pr.reduceByProduction263(parser); return;
        case 635: pr.reduceByProduction635(parser); return;
        case 842: pr.reduceByProduction842(parser); return;
        case 676: pr.reduceByProduction676(parser); return;
        case 959: pr.reduceByProduction959(parser); return;
        case 1323: pr.reduceByProduction1323(parser); return;
        case 1243: pr.reduceByProduction1243(parser); return;
        case 303: pr.reduceByProduction303(parser); return;
        case 764: pr.reduceByProduction764(parser); return;
        case 203: pr.reduceByProduction203(parser); return;
        case 776: pr.reduceByProduction776(parser); return;
        case 37: pr.reduceByProduction37(parser); return;
        case 209: pr.reduceByProduction209(parser); return;
        case 1152: pr.reduceByProduction1152(parser); return;
        case 1036: pr.reduceByProduction1036(parser); return;
        case 1079: pr.reduceByProduction1079(parser); return;
        case 286: pr.reduceByProduction286(parser); return;
        case 1252: pr.reduceByProduction1252(parser); return;
        case 273: pr.reduceByProduction273(parser); return;
        case 355: pr.reduceByProduction355(parser); return;
        case 531: pr.reduceByProduction531(parser); return;
        case 638: pr.reduceByProduction638(parser); return;
        case 297: pr.reduceByProduction297(parser); return;
        case 318: pr.reduceByProduction318(parser); return;
        case 58: pr.reduceByProduction58(parser); return;
        case 160: pr.reduceByProduction160(parser); return;
        case 633: pr.reduceByProduction633(parser); return;
        case 506: pr.reduceByProduction506(parser); return;
        case 858: pr.reduceByProduction858(parser); return;
        case 275: pr.reduceByProduction275(parser); return;
        case 664: pr.reduceByProduction664(parser); return;
        case 1315: pr.reduceByProduction1315(parser); return;
        case 565: pr.reduceByProduction565(parser); return;
        case 1341: pr.reduceByProduction1341(parser); return;
        case 1277: pr.reduceByProduction1277(parser); return;
        case 311: pr.reduceByProduction311(parser); return;
        case 32: pr.reduceByProduction32(parser); return;
        case 387: pr.reduceByProduction387(parser); return;
        case 87: pr.reduceByProduction87(parser); return;
        case 603: pr.reduceByProduction603(parser); return;
        case 1171: pr.reduceByProduction1171(parser); return;
        case 55: pr.reduceByProduction55(parser); return;
        case 1048: pr.reduceByProduction1048(parser); return;
        case 1266: pr.reduceByProduction1266(parser); return;
        case 522: pr.reduceByProduction522(parser); return;
        case 715: pr.reduceByProduction715(parser); return;
        case 771: pr.reduceByProduction771(parser); return;
        case 1119: pr.reduceByProduction1119(parser); return;
        case 873: pr.reduceByProduction873(parser); return;
        case 426: pr.reduceByProduction426(parser); return;
        case 602: pr.reduceByProduction602(parser); return;
        case 230: pr.reduceByProduction230(parser); return;
        case 1023: pr.reduceByProduction1023(parser); return;
        case 309: pr.reduceByProduction309(parser); return;
        case 799: pr.reduceByProduction799(parser); return;
        case 291: pr.reduceByProduction291(parser); return;
        case 510: pr.reduceByProduction510(parser); return;
        case 1153: pr.reduceByProduction1153(parser); return;
        case 831: pr.reduceByProduction831(parser); return;
        case 703: pr.reduceByProduction703(parser); return;
        case 801: pr.reduceByProduction801(parser); return;
        case 1246: pr.reduceByProduction1246(parser); return;
        case 702: pr.reduceByProduction702(parser); return;
        case 357: pr.reduceByProduction357(parser); return;
        case 612: pr.reduceByProduction612(parser); return;
        case 99: pr.reduceByProduction99(parser); return;
        case 932: pr.reduceByProduction932(parser); return;
        case 933: pr.reduceByProduction933(parser); return;
        case 322: pr.reduceByProduction322(parser); return;
        case 824: pr.reduceByProduction824(parser); return;
        case 122: pr.reduceByProduction122(parser); return;
        case 845: pr.reduceByProduction845(parser); return;
        case 298: pr.reduceByProduction298(parser); return;
        case 42: pr.reduceByProduction42(parser); return;
        case 743: pr.reduceByProduction743(parser); return;
        case 1160: pr.reduceByProduction1160(parser); return;
        case 396: pr.reduceByProduction396(parser); return;
        case 763: pr.reduceByProduction763(parser); return;
        case 252: pr.reduceByProduction252(parser); return;
        case 317: pr.reduceByProduction317(parser); return;
        case 671: pr.reduceByProduction671(parser); return;
        case 833: pr.reduceByProduction833(parser); return;
        case 92: pr.reduceByProduction92(parser); return;
        case 1241: pr.reduceByProduction1241(parser); return;
        case 861: pr.reduceByProduction861(parser); return;
        case 147: pr.reduceByProduction147(parser); return;
        case 185: pr.reduceByProduction185(parser); return;
        case 1053: pr.reduceByProduction1053(parser); return;
        case 572: pr.reduceByProduction572(parser); return;
        case 813: pr.reduceByProduction813(parser); return;
        case 259: pr.reduceByProduction259(parser); return;
        case 1245: pr.reduceByProduction1245(parser); return;
        case 1034: pr.reduceByProduction1034(parser); return;
        case 1174: pr.reduceByProduction1174(parser); return;
        case 787: pr.reduceByProduction787(parser); return;
        case 348: pr.reduceByProduction348(parser); return;
        case 419: pr.reduceByProduction419(parser); return;
        case 631: pr.reduceByProduction631(parser); return;
        case 299: pr.reduceByProduction299(parser); return;
        case 566: pr.reduceByProduction566(parser); return;
        case 1101: pr.reduceByProduction1101(parser); return;
        case 1356: pr.reduceByProduction1356(parser); return;
        case 296: pr.reduceByProduction296(parser); return;
        case 430: pr.reduceByProduction430(parser); return;
        case 1126: pr.reduceByProduction1126(parser); return;
        case 74: pr.reduceByProduction74(parser); return;
        case 826: pr.reduceByProduction826(parser); return;
        case 1009: pr.reduceByProduction1009(parser); return;
        case 811: pr.reduceByProduction811(parser); return;
        case 40: pr.reduceByProduction40(parser); return;
        case 152: pr.reduceByProduction152(parser); return;
        case 1125: pr.reduceByProduction1125(parser); return;
        case 840: pr.reduceByProduction840(parser); return;
        case 1290: pr.reduceByProduction1290(parser); return;
        case 201: pr.reduceByProduction201(parser); return;
        case 454: pr.reduceByProduction454(parser); return;
        case 277: pr.reduceByProduction277(parser); return;
        case 110: pr.reduceByProduction110(parser); return;
        case 279: pr.reduceByProduction279(parser); return;
        case 885: pr.reduceByProduction885(parser); return;
        case 782: pr.reduceByProduction782(parser); return;
        case 52: pr.reduceByProduction52(parser); return;
        case 325: pr.reduceByProduction325(parser); return;
        case 6: pr.reduceByProduction6(parser); return;
        case 486: pr.reduceByProduction486(parser); return;
        case 461: pr.reduceByProduction461(parser); return;
        case 1022: pr.reduceByProduction1022(parser); return;
        case 1151: pr.reduceByProduction1151(parser); return;
        case 1063: pr.reduceByProduction1063(parser); return;
        case 950: pr.reduceByProduction950(parser); return;
        case 791: pr.reduceByProduction791(parser); return;
        case 887: pr.reduceByProduction887(parser); return;
        case 170: pr.reduceByProduction170(parser); return;
        case 445: pr.reduceByProduction445(parser); return;
        case 148: pr.reduceByProduction148(parser); return;
        case 253: pr.reduceByProduction253(parser); return;
        case 919: pr.reduceByProduction919(parser); return;
        case 578: pr.reduceByProduction578(parser); return;
        case 986: pr.reduceByProduction986(parser); return;
        case 1236: pr.reduceByProduction1236(parser); return;
        case 666: pr.reduceByProduction666(parser); return;
        case 681: pr.reduceByProduction681(parser); return;
        case 847: pr.reduceByProduction847(parser); return;
        case 319: pr.reduceByProduction319(parser); return;
        case 1284: pr.reduceByProduction1284(parser); return;
        case 103: pr.reduceByProduction103(parser); return;
        case 1332: pr.reduceByProduction1332(parser); return;
        case 759: pr.reduceByProduction759(parser); return;
        case 897: pr.reduceByProduction897(parser); return;
        case 879: pr.reduceByProduction879(parser); return;
        case 80: pr.reduceByProduction80(parser); return;
        case 1109: pr.reduceByProduction1109(parser); return;
        case 204: pr.reduceByProduction204(parser); return;
        case 190: pr.reduceByProduction190(parser); return;
        case 821: pr.reduceByProduction821(parser); return;
        case 1059: pr.reduceByProduction1059(parser); return;
        case 196: pr.reduceByProduction196(parser); return;
        case 1016: pr.reduceByProduction1016(parser); return;
        case 881: pr.reduceByProduction881(parser); return;
        case 1182: pr.reduceByProduction1182(parser); return;
        case 804: pr.reduceByProduction804(parser); return;
        case 75: pr.reduceByProduction75(parser); return;
        case 958: pr.reduceByProduction958(parser); return;
        case 775: pr.reduceByProduction775(parser); return;
        case 320: pr.reduceByProduction320(parser); return;
        case 393: pr.reduceByProduction393(parser); return;
        case 652: pr.reduceByProduction652(parser); return;
        case 1203: pr.reduceByProduction1203(parser); return;
        case 1184: pr.reduceByProduction1184(parser); return;
        case 1005: pr.reduceByProduction1005(parser); return;
        case 24: pr.reduceByProduction24(parser); return;
        case 856: pr.reduceByProduction856(parser); return;
        case 188: pr.reduceByProduction188(parser); return;
        case 431: pr.reduceByProduction431(parser); return;
        case 101: pr.reduceByProduction101(parser); return;
        case 35: pr.reduceByProduction35(parser); return;
        case 388: pr.reduceByProduction388(parser); return;
        case 1364: pr.reduceByProduction1364(parser); return;
        case 607: pr.reduceByProduction607(parser); return;
        case 292: pr.reduceByProduction292(parser); return;
        case 819: pr.reduceByProduction819(parser); return;
        case 1091: pr.reduceByProduction1091(parser); return;
        case 499: pr.reduceByProduction499(parser); return;
        case 1226: pr.reduceByProduction1226(parser); return;
        case 1194: pr.reduceByProduction1194(parser); return;
        case 1134: pr.reduceByProduction1134(parser); return;
        case 258: pr.reduceByProduction258(parser); return;
        case 772: pr.reduceByProduction772(parser); return;
        case 755: pr.reduceByProduction755(parser); return;
        case 975: pr.reduceByProduction975(parser); return;
        case 544: pr.reduceByProduction544(parser); return;
        case 866: pr.reduceByProduction866(parser); return;
        case 70: pr.reduceByProduction70(parser); return;
        case 634: pr.reduceByProduction634(parser); return;
        case 232: pr.reduceByProduction232(parser); return;
        case 200: pr.reduceByProduction200(parser); return;
        case 951: pr.reduceByProduction951(parser); return;
        case 874: pr.reduceByProduction874(parser); return;
        case 276: pr.reduceByProduction276(parser); return;
        case 1150: pr.reduceByProduction1150(parser); return;
        case 1147: pr.reduceByProduction1147(parser); return;
        case 974: pr.reduceByProduction974(parser); return;
        case 1275: pr.reduceByProduction1275(parser); return;
        case 1292: pr.reduceByProduction1292(parser); return;
        case 651: pr.reduceByProduction651(parser); return;
        case 49: pr.reduceByProduction49(parser); return;
        case 433: pr.reduceByProduction433(parser); return;
        case 541: pr.reduceByProduction541(parser); return;
        case 1214: pr.reduceByProduction1214(parser); return;
        case 186: pr.reduceByProduction186(parser); return;
        case 120: pr.reduceByProduction120(parser); return;
        case 417: pr.reduceByProduction417(parser); return;
        case 226: pr.reduceByProduction226(parser); return;
        case 694: pr.reduceByProduction694(parser); return;
        case 1128: pr.reduceByProduction1128(parser); return;
        case 22: pr.reduceByProduction22(parser); return;
        case 1324: pr.reduceByProduction1324(parser); return;
        case 372: pr.reduceByProduction372(parser); return;
        case 903: pr.reduceByProduction903(parser); return;
        case 354: pr.reduceByProduction354(parser); return;
        case 85: pr.reduceByProduction85(parser); return;
        case 308: pr.reduceByProduction308(parser); return;
        case 405: pr.reduceByProduction405(parser); return;
        case 502: pr.reduceByProduction502(parser); return;
        case 1095: pr.reduceByProduction1095(parser); return;
        case 250: pr.reduceByProduction250(parser); return;
        case 38: pr.reduceByProduction38(parser); return;
        case 1181: pr.reduceByProduction1181(parser); return;
        case 179: pr.reduceByProduction179(parser); return;
        case 416: pr.reduceByProduction416(parser); return;
        case 668: pr.reduceByProduction668(parser); return;
        case 1293: pr.reduceByProduction1293(parser); return;
        case 663: pr.reduceByProduction663(parser); return;
        case 1000: pr.reduceByProduction1000(parser); return;
        case 980: pr.reduceByProduction980(parser); return;
        case 497: pr.reduceByProduction497(parser); return;
        case 910: pr.reduceByProduction910(parser); return;
        case 952: pr.reduceByProduction952(parser); return;
        case 336: pr.reduceByProduction336(parser); return;
        case 121: pr.reduceByProduction121(parser); return;
        case 617: pr.reduceByProduction617(parser); return;
        case 1129: pr.reduceByProduction1129(parser); return;
        case 918: pr.reduceByProduction918(parser); return;
        case 1353: pr.reduceByProduction1353(parser); return;
        case 911: pr.reduceByProduction911(parser); return;
        case 739: pr.reduceByProduction739(parser); return;
        case 1259: pr.reduceByProduction1259(parser); return;
        case 470: pr.reduceByProduction470(parser); return;
        case 817: pr.reduceByProduction817(parser); return;
        case 1317: pr.reduceByProduction1317(parser); return;
        case 305: pr.reduceByProduction305(parser); return;
        case 714: pr.reduceByProduction714(parser); return;
        case 979: pr.reduceByProduction979(parser); return;
        case 848: pr.reduceByProduction848(parser); return;
        case 1250: pr.reduceByProduction1250(parser); return;
        case 1228: pr.reduceByProduction1228(parser); return;
        case 947: pr.reduceByProduction947(parser); return;
        case 280: pr.reduceByProduction280(parser); return;
        case 983: pr.reduceByProduction983(parser); return;
        case 615: pr.reduceByProduction615(parser); return;
        case 1105: pr.reduceByProduction1105(parser); return;
        case 1178: pr.reduceByProduction1178(parser); return;
        case 1321: pr.reduceByProduction1321(parser); return;
        case 422: pr.reduceByProduction422(parser); return;
        case 696: pr.reduceByProduction696(parser); return;
        case 841: pr.reduceByProduction841(parser); return;
        case 137: pr.reduceByProduction137(parser); return;
        case 1211: pr.reduceByProduction1211(parser); return;
        case 476: pr.reduceByProduction476(parser); return;
        case 1239: pr.reduceByProduction1239(parser); return;
        case 228: pr.reduceByProduction228(parser); return;
        case 711: pr.reduceByProduction711(parser); return;
        case 872: pr.reduceByProduction872(parser); return;
        case 883: pr.reduceByProduction883(parser); return;
        case 619: pr.reduceByProduction619(parser); return;
        case 1260: pr.reduceByProduction1260(parser); return;
        case 358: pr.reduceByProduction358(parser); return;
        case 661: pr.reduceByProduction661(parser); return;
        case 1163: pr.reduceByProduction1163(parser); return;
        case 521: pr.reduceByProduction521(parser); return;
        case 328: pr.reduceByProduction328(parser); return;
        case 109: pr.reduceByProduction109(parser); return;
        case 496: pr.reduceByProduction496(parser); return;
        case 888: pr.reduceByProduction888(parser); return;
        case 468: pr.reduceByProduction468(parser); return;
        case 481: pr.reduceByProduction481(parser); return;
        case 751: pr.reduceByProduction751(parser); return;
        case 362: pr.reduceByProduction362(parser); return;
        case 1136: pr.reduceByProduction1136(parser); return;
        case 146: pr.reduceByProduction146(parser); return;
        case 402: pr.reduceByProduction402(parser); return;
        case 505: pr.reduceByProduction505(parser); return;
        case 293: pr.reduceByProduction293(parser); return;
        case 1261: pr.reduceByProduction1261(parser); return;
        case 1078: pr.reduceByProduction1078(parser); return;
        case 710: pr.reduceByProduction710(parser); return;
        case 340: pr.reduceByProduction340(parser); return;
        case 82: pr.reduceByProduction82(parser); return;
        case 1094: pr.reduceByProduction1094(parser); return;
        case 518: pr.reduceByProduction518(parser); return;
        case 877: pr.reduceByProduction877(parser); return;
        case 437: pr.reduceByProduction437(parser); return;
        case 613: pr.reduceByProduction613(parser); return;
        case 537: pr.reduceByProduction537(parser); return;
        case 1225: pr.reduceByProduction1225(parser); return;
        case 571: pr.reduceByProduction571(parser); return;
        case 649: pr.reduceByProduction649(parser); return;
        case 773: pr.reduceByProduction773(parser); return;
        case 925: pr.reduceByProduction925(parser); return;
        case 484: pr.reduceByProduction484(parser); return;
        case 1267: pr.reduceByProduction1267(parser); return;
        case 424: pr.reduceByProduction424(parser); return;
        case 547: pr.reduceByProduction547(parser); return;
        case 56: pr.reduceByProduction56(parser); return;
        case 1303: pr.reduceByProduction1303(parser); return;
        case 832: pr.reduceByProduction832(parser); return;
        case 982: pr.reduceByProduction982(parser); return;
        case 28: pr.reduceByProduction28(parser); return;
        case 239: pr.reduceByProduction239(parser); return;
        case 17: pr.reduceByProduction17(parser); return;
        case 632: pr.reduceByProduction632(parser); return;
        case 287: pr.reduceByProduction287(parser); return;
        case 1313: pr.reduceByProduction1313(parser); return;
        case 768: pr.reduceByProduction768(parser); return;
        case 977: pr.reduceByProduction977(parser); return;
        case 493: pr.reduceByProduction493(parser); return;
        case 289: pr.reduceByProduction289(parser); return;
        case 159: pr.reduceByProduction159(parser); return;
        case 923: pr.reduceByProduction923(parser); return;
        case 338: pr.reduceByProduction338(parser); return;
        case 1180: pr.reduceByProduction1180(parser); return;
        case 345: pr.reduceByProduction345(parser); return;
        case 1030: pr.reduceByProduction1030(parser); return;
        case 14: pr.reduceByProduction14(parser); return;
        case 175: pr.reduceByProduction175(parser); return;
        case 1156: pr.reduceByProduction1156(parser); return;
        case 1069: pr.reduceByProduction1069(parser); return;
        case 944: pr.reduceByProduction944(parser); return;
        case 364: pr.reduceByProduction364(parser); return;
        case 78: pr.reduceByProduction78(parser); return;
        case 917: pr.reduceByProduction917(parser); return;
        case 84: pr.reduceByProduction84(parser); return;
        case 816: pr.reduceByProduction816(parser); return;
        case 48: pr.reduceByProduction48(parser); return;
        case 863: pr.reduceByProduction863(parser); return;
        case 862: pr.reduceByProduction862(parser); return;
        case 383: pr.reduceByProduction383(parser); return;
        case 321: pr.reduceByProduction321(parser); return;
        case 876: pr.reduceByProduction876(parser); return;
        case 407: pr.reduceByProduction407(parser); return;
        case 342: pr.reduceByProduction342(parser); return;
        case 224: pr.reduceByProduction224(parser); return;
        case 300: pr.reduceByProduction300(parser); return;
        case 131: pr.reduceByProduction131(parser); return;
        case 1107: pr.reduceByProduction1107(parser); return;
        case 1080: pr.reduceByProduction1080(parser); return;
        case 670: pr.reduceByProduction670(parser); return;
        case 956: pr.reduceByProduction956(parser); return;
        case 411: pr.reduceByProduction411(parser); return;
        case 453: pr.reduceByProduction453(parser); return;
        case 1041: pr.reduceByProduction1041(parser); return;
        case 1350: pr.reduceByProduction1350(parser); return;
        case 138: pr.reduceByProduction138(parser); return;
        case 1357: pr.reduceByProduction1357(parser); return;
        case 1336: pr.reduceByProduction1336(parser); return;
        case 1008: pr.reduceByProduction1008(parser); return;
        case 684: pr.reduceByProduction684(parser); return;
        case 1339: pr.reduceByProduction1339(parser); return;
        case 207: pr.reduceByProduction207(parser); return;
        case 351: pr.reduceByProduction351(parser); return;
        case 1185: pr.reduceByProduction1185(parser); return;
        case 1301: pr.reduceByProduction1301(parser); return;
        case 674: pr.reduceByProduction674(parser); return;
        case 682: pr.reduceByProduction682(parser); return;
        case 1360: pr.reduceByProduction1360(parser); return;
        case 482: pr.reduceByProduction482(parser); return;
        case 211: pr.reduceByProduction211(parser); return;
        case 1262: pr.reduceByProduction1262(parser); return;
        case 1097: pr.reduceByProduction1097(parser); return;
        case 598: pr.reduceByProduction598(parser); return;
        case 1133: pr.reduceByProduction1133(parser); return;
        case 380: pr.reduceByProduction380(parser); return;
        case 346: pr.reduceByProduction346(parser); return;
        case 1230: pr.reduceByProduction1230(parser); return;
        case 197: pr.reduceByProduction197(parser); return;
        case 1155: pr.reduceByProduction1155(parser); return;
        case 1291: pr.reduceByProduction1291(parser); return;
        case 576: pr.reduceByProduction576(parser); return;
        case 1014: pr.reduceByProduction1014(parser); return;
        case 1158: pr.reduceByProduction1158(parser); return;
        case 11: pr.reduceByProduction11(parser); return;
        case 1087: pr.reduceByProduction1087(parser); return;
        case 594: pr.reduceByProduction594(parser); return;
        case 636: pr.reduceByProduction636(parser); return;
        case 1051: pr.reduceByProduction1051(parser); return;
        case 1265: pr.reduceByProduction1265(parser); return;
        case 375: pr.reduceByProduction375(parser); return;
        case 929: pr.reduceByProduction929(parser); return;
        case 926: pr.reduceByProduction926(parser); return;
        case 515: pr.reduceByProduction515(parser); return;
        case 806: pr.reduceByProduction806(parser); return;
        case 169: pr.reduceByProduction169(parser); return;
        case 1347: pr.reduceByProduction1347(parser); return;
        case 894: pr.reduceByProduction894(parser); return;
        case 151: pr.reduceByProduction151(parser); return;
        case 784: pr.reduceByProduction784(parser); return;
        case 660: pr.reduceByProduction660(parser); return;
        case 1274: pr.reduceByProduction1274(parser); return;
        case 1047: pr.reduceByProduction1047(parser); return;
        case 1188: pr.reduceByProduction1188(parser); return;
        case 210: pr.reduceByProduction210(parser); return;
        case 246: pr.reduceByProduction246(parser); return;
        case 374: pr.reduceByProduction374(parser); return;
        case 285: pr.reduceByProduction285(parser); return;
        case 236: pr.reduceByProduction236(parser); return;
        case 969: pr.reduceByProduction969(parser); return;
        case 450: pr.reduceByProduction450(parser); return;
        case 1027: pr.reduceByProduction1027(parser); return;
        case 1055: pr.reduceByProduction1055(parser); return;
        case 687: pr.reduceByProduction687(parser); return;
        case 891: pr.reduceByProduction891(parser); return;
        case 805: pr.reduceByProduction805(parser); return;
        case 365: pr.reduceByProduction365(parser); return;
        case 183: pr.reduceByProduction183(parser); return;
        case 241: pr.reduceByProduction241(parser); return;
        case 304: pr.reduceByProduction304(parser); return;
        case 472: pr.reduceByProduction472(parser); return;
        case 534: pr.reduceByProduction534(parser); return;
        case 854: pr.reduceByProduction854(parser); return;
        case 559: pr.reduceByProduction559(parser); return;
        case 610: pr.reduceByProduction610(parser); return;
        case 244: pr.reduceByProduction244(parser); return;
        case 19: pr.reduceByProduction19(parser); return;
        case 677: pr.reduceByProduction677(parser); return;
        case 964: pr.reduceByProduction964(parser); return;
        case 935: pr.reduceByProduction935(parser); return;
        case 112: pr.reduceByProduction112(parser); return;
        case 213: pr.reduceByProduction213(parser); return;
        case 737: pr.reduceByProduction737(parser); return;
        case 796: pr.reduceByProduction796(parser); return;
        case 593: pr.reduceByProduction593(parser); return;
        case 938: pr.reduceByProduction938(parser); return;
        case 376: pr.reduceByProduction376(parser); return;
        case 283: pr.reduceByProduction283(parser); return;
        case 1103: pr.reduceByProduction1103(parser); return;
        case 1253: pr.reduceByProduction1253(parser); return;
        case 1342: pr.reduceByProduction1342(parser); return;
        case 1124: pr.reduceByProduction1124(parser); return;
        case 1001: pr.reduceByProduction1001(parser); return;
        case 420: pr.reduceByProduction420(parser); return;
        case 145: pr.reduceByProduction145(parser); return;
        case 5: pr.reduceByProduction5(parser); return;
        case 155: pr.reduceByProduction155(parser); return;
        case 1024: pr.reduceByProduction1024(parser); return;
        case 205: pr.reduceByProduction205(parser); return;
        case 692: pr.reduceByProduction692(parser); return;
        case 868: pr.reduceByProduction868(parser); return;
        case 530: pr.reduceByProduction530(parser); return;
        case 487: pr.reduceByProduction487(parser); return;
        case 644: pr.reduceByProduction644(parser); return;
        case 987: pr.reduceByProduction987(parser); return;
        case 238: pr.reduceByProduction238(parser); return;
        case 902: pr.reduceByProduction902(parser); return;
        case 218: pr.reduceByProduction218(parser); return;
        case 119: pr.reduceByProduction119(parser); return;
        case 532: pr.reduceByProduction532(parser); return;
        case 643: pr.reduceByProduction643(parser); return;
        case 1309: pr.reduceByProduction1309(parser); return;
        case 1244: pr.reduceByProduction1244(parser); return;
        case 875: pr.reduceByProduction875(parser); return;
        case 1281: pr.reduceByProduction1281(parser); return;
        case 878: pr.reduceByProduction878(parser); return;
        case 4: pr.reduceByProduction4(parser); return;
        case 413: pr.reduceByProduction413(parser); return;
        case 77: pr.reduceByProduction77(parser); return;
        case 526: pr.reduceByProduction526(parser); return;
        case 369: pr.reduceByProduction369(parser); return;
        case 88: pr.reduceByProduction88(parser); return;
        case 1012: pr.reduceByProduction1012(parser); return;
        case 802: pr.reduceByProduction802(parser); return;
        case 587: pr.reduceByProduction587(parser); return;
        case 45: pr.reduceByProduction45(parser); return;
        case 1238: pr.reduceByProduction1238(parser); return;
        case 591: pr.reduceByProduction591(parser); return;
        case 1247: pr.reduceByProduction1247(parser); return;
        case 1197: pr.reduceByProduction1197(parser); return;
        case 441: pr.reduceByProduction441(parser); return;
        case 709: pr.reduceByProduction709(parser); return;
        case 1068: pr.reduceByProduction1068(parser); return;
        case 569: pr.reduceByProduction569(parser); return;
        case 471: pr.reduceByProduction471(parser); return;
        case 359: pr.reduceByProduction359(parser); return;
        case 726: pr.reduceByProduction726(parser); return;
        case 1204: pr.reduceByProduction1204(parser); return;
        case 740: pr.reduceByProduction740(parser); return;
        case 1282: pr.reduceByProduction1282(parser); return;
        case 786: pr.reduceByProduction786(parser); return;
        case 1099: pr.reduceByProduction1099(parser); return;
        case 434: pr.reduceByProduction434(parser); return;
        case 1076: pr.reduceByProduction1076(parser); return;
        case 588: pr.reduceByProduction588(parser); return;
        case 604: pr.reduceByProduction604(parser); return;
        case 1025: pr.reduceByProduction1025(parser); return;
        case 640: pr.reduceByProduction640(parser); return;
        case 71: pr.reduceByProduction71(parser); return;
        case 44: pr.reduceByProduction44(parser); return;
        case 154: pr.reduceByProduction154(parser); return;
        case 552: pr.reduceByProduction552(parser); return;
        case 627: pr.reduceByProduction627(parser); return;
        case 189: pr.reduceByProduction189(parser); return;
        case 452: pr.reduceByProduction452(parser); return;
        case 844: pr.reduceByProduction844(parser); return;
        case 187: pr.reduceByProduction187(parser); return;
        case 282: pr.reduceByProduction282(parser); return;
        case 474: pr.reduceByProduction474(parser); return;
        case 473: pr.reduceByProduction473(parser); return;
        case 446: pr.reduceByProduction446(parser); return;
        case 691: pr.reduceByProduction691(parser); return;
        case 1235: pr.reduceByProduction1235(parser); return;
        case 1207: pr.reduceByProduction1207(parser); return;
        case 685: pr.reduceByProduction685(parser); return;
        case 837: pr.reduceByProduction837(parser); return;
        case 333: pr.reduceByProduction333(parser); return;
        case 912: pr.reduceByProduction912(parser); return;
        case 290: pr.reduceByProduction290(parser); return;
        case 324: pr.reduceByProduction324(parser); return;
        case 1212: pr.reduceByProduction1212(parser); return;
        case 1049: pr.reduceByProduction1049(parser); return;
        case 1272: pr.reduceByProduction1272(parser); return;
        case 630: pr.reduceByProduction630(parser); return;
        case 373: pr.reduceByProduction373(parser); return;
        case 25: pr.reduceByProduction25(parser); return;
        case 623: pr.reduceByProduction623(parser); return;
        case 823: pr.reduceByProduction823(parser); return;
        case 491: pr.reduceByProduction491(parser); return;
        case 908: pr.reduceByProduction908(parser); return;
        case 8: pr.reduceByProduction8(parser); return;
        case 601: pr.reduceByProduction601(parser); return;
        case 675: pr.reduceByProduction675(parser); return;
        case 551: pr.reduceByProduction551(parser); return;
        case 529: pr.reduceByProduction529(parser); return;
        case 723: pr.reduceByProduction723(parser); return;
        case 257: pr.reduceByProduction257(parser); return;
        case 621: pr.reduceByProduction621(parser); return;
        case 981: pr.reduceByProduction981(parser); return;
        case 83: pr.reduceByProduction83(parser); return;
        case 928: pr.reduceByProduction928(parser); return;
        case 629: pr.reduceByProduction629(parser); return;
        case 262: pr.reduceByProduction262(parser); return;
        case 792: pr.reduceByProduction792(parser); return;
        case 1043: pr.reduceByProduction1043(parser); return;
        case 961: pr.reduceByProduction961(parser); return;
        case 734: pr.reduceByProduction734(parser); return;
        case 850: pr.reduceByProduction850(parser); return;
        case 1044: pr.reduceByProduction1044(parser); return;
        case 645: pr.reduceByProduction645(parser); return;
        case 1029: pr.reduceByProduction1029(parser); return;
        case 808: pr.reduceByProduction808(parser); return;
        case 41: pr.reduceByProduction41(parser); return;
        case 464: pr.reduceByProduction464(parser); return;
        case 310: pr.reduceByProduction310(parser); return;
        case 683: pr.reduceByProduction683(parser); return;
        case 788: pr.reduceByProduction788(parser); return;
        case 64: pr.reduceByProduction64(parser); return;
        case 867: pr.reduceByProduction867(parser); return;
        case 865: pr.reduceByProduction865(parser); return;
        case 1123: pr.reduceByProduction1123(parser); return;
        case 1227: pr.reduceByProduction1227(parser); return;
        case 869: pr.reduceByProduction869(parser); return;
        case 519: pr.reduceByProduction519(parser); return;
        case 1269: pr.reduceByProduction1269(parser); return;
        case 494: pr.reduceByProduction494(parser); return;
        case 662: pr.reduceByProduction662(parser); return;
        case 998: pr.reduceByProduction998(parser); return;
        case 795: pr.reduceByProduction795(parser); return;
        case 855: pr.reduceByProduction855(parser); return;
        case 107: pr.reduceByProduction107(parser); return;
        case 968: pr.reduceByProduction968(parser); return;
        case 66: pr.reduceByProduction66(parser); return;
        case 90: pr.reduceByProduction90(parser); return;
        case 212: pr.reduceByProduction212(parser); return;
        case 488: pr.reduceByProduction488(parser); return;
        case 948: pr.reduceByProduction948(parser); return;
        case 86: pr.reduceByProduction86(parser); return;
        case 1296: pr.reduceByProduction1296(parser); return;
        case 779: pr.reduceByProduction779(parser); return;
        case 173: pr.reduceByProduction173(parser); return;
        case 81: pr.reduceByProduction81(parser); return;
        case 822: pr.reduceByProduction822(parser); return;
        case 820: pr.reduceByProduction820(parser); return;
        case 657: pr.reduceByProduction657(parser); return;
        case 254: pr.reduceByProduction254(parser); return;
        case 1089: pr.reduceByProduction1089(parser); return;
        case 665: pr.reduceByProduction665(parser); return;
        case 1319: pr.reduceByProduction1319(parser); return;
        case 95: pr.reduceByProduction95(parser); return;
        case 449: pr.reduceByProduction449(parser); return;
        case 618: pr.reduceByProduction618(parser); return;
        case 646: pr.reduceByProduction646(parser); return;
        case 31: pr.reduceByProduction31(parser); return;
        case 394: pr.reduceByProduction394(parser); return;
        case 516: pr.reduceByProduction516(parser); return;
        case 561: pr.reduceByProduction561(parser); return;
        case 941: pr.reduceByProduction941(parser); return;
        case 400: pr.reduceByProduction400(parser); return;
        case 934: pr.reduceByProduction934(parser); return;
        case 1169: pr.reduceByProduction1169(parser); return;
        case 1200: pr.reduceByProduction1200(parser); return;
        case 1115: pr.reduceByProduction1115(parser); return;
        case 655: pr.reduceByProduction655(parser); return;
        case 889: pr.reduceByProduction889(parser); return;
        case 757: pr.reduceByProduction757(parser); return;
        case 1064: pr.reduceByProduction1064(parser); return;
        case 1192: pr.reduceByProduction1192(parser); return;
        case 890: pr.reduceByProduction890(parser); return;
        case 1295: pr.reduceByProduction1295(parser); return;
        case 418: pr.reduceByProduction418(parser); return;
        case 456: pr.reduceByProduction456(parser); return;
        case 1217: pr.reduceByProduction1217(parser); return;
        case 859: pr.reduceByProduction859(parser); return;
        case 215: pr.reduceByProduction215(parser); return;
        case 1: pr.reduceByProduction1(parser); return;
        case 945: pr.reduceByProduction945(parser); return;
        case 330: pr.reduceByProduction330(parser); return;
        case 174: pr.reduceByProduction174(parser); return;
        case 504: pr.reduceByProduction504(parser); return;
        case 21: pr.reduceByProduction21(parser); return;
        case 852: pr.reduceByProduction852(parser); return;
        case 575: pr.reduceByProduction575(parser); return;
        case 163: pr.reduceByProduction163(parser); return;
        case 567: pr.reduceByProduction567(parser); return;
        case 1279: pr.reduceByProduction1279(parser); return;
        case 1132: pr.reduceByProduction1132(parser); return;
        case 1306: pr.reduceByProduction1306(parser); return;
        case 797: pr.reduceByProduction797(parser); return;
        case 316: pr.reduceByProduction316(parser); return;
        case 288: pr.reduceByProduction288(parser); return;
        case 540: pr.reduceByProduction540(parser); return;
        case 1362: pr.reduceByProduction1362(parser); return;
        case 265: pr.reduceByProduction265(parser); return;
        case 54: pr.reduceByProduction54(parser); return;
        case 513: pr.reduceByProduction513(parser); return;
        case 798: pr.reduceByProduction798(parser); return;
        case 403: pr.reduceByProduction403(parser); return;
        case 853: pr.reduceByProduction853(parser); return;
        case 43: pr.reduceByProduction43(parser); return;
        case 1219: pr.reduceByProduction1219(parser); return;
        case 827: pr.reduceByProduction827(parser); return;
        case 242: pr.reduceByProduction242(parser); return;
        case 611: pr.reduceByProduction611(parser); return;
        case 860: pr.reduceByProduction860(parser); return;
        case 754: pr.reduceByProduction754(parser); return;
        case 114: pr.reduceByProduction114(parser); return;
        case 1270: pr.reduceByProduction1270(parser); return;
        case 68: pr.reduceByProduction68(parser); return;
        case 1177: pr.reduceByProduction1177(parser); return;
        case 315: pr.reduceByProduction315(parser); return;
        case 731: pr.reduceByProduction731(parser); return;
        case 385: pr.reduceByProduction385(parser); return;
        case 149: pr.reduceByProduction149(parser); return;
        case 350: pr.reduceByProduction350(parser); return;
        case 931: pr.reduceByProduction931(parser); return;
        case 697: pr.reduceByProduction697(parser); return;
        case 892: pr.reduceByProduction892(parser); return;
        case 626: pr.reduceByProduction626(parser); return;
        case 214: pr.reduceByProduction214(parser); return;
        case 47: pr.reduceByProduction47(parser); return;
        case 744: pr.reduceByProduction744(parser); return;
        case 301: pr.reduceByProduction301(parser); return;
        case 716: pr.reduceByProduction716(parser); return;
        case 370: pr.reduceByProduction370(parser); return;
        case 596: pr.reduceByProduction596(parser); return;
        case 720: pr.reduceByProduction720(parser); return;
        case 1325: pr.reduceByProduction1325(parser); return;
        case 177: pr.reduceByProduction177(parser); return;
        case 954: pr.reduceByProduction954(parser); return;
        case 307: pr.reduceByProduction307(parser); return;
        case 16: pr.reduceByProduction16(parser); return;
        case 535: pr.reduceByProduction535(parser); return;
        case 143: pr.reduceByProduction143(parser); return;
        case 72: pr.reduceByProduction72(parser); return;
        case 766: pr.reduceByProduction766(parser); return;
        case 1120: pr.reduceByProduction1120(parser); return;
        case 231: pr.reduceByProduction231(parser); return;
        case 606: pr.reduceByProduction606(parser); return;
        case 284: pr.reduceByProduction284(parser); return;
        case 1015: pr.reduceByProduction1015(parser); return;
        case 443: pr.reduceByProduction443(parser); return;
        case 495: pr.reduceByProduction495(parser); return;
        case 886: pr.reduceByProduction886(parser); return;
        case 728: pr.reduceByProduction728(parser); return;
        case 136: pr.reduceByProduction136(parser); return;
        case 192: pr.reduceByProduction192(parser); return;
        case 1127: pr.reduceByProduction1127(parser); return;
        case 113: pr.reduceByProduction113(parser); return;
        case 545: pr.reduceByProduction545(parser); return;
        case 399: pr.reduceByProduction399(parser); return;
        case 957: pr.reduceByProduction957(parser); return;
        case 133: pr.reduceByProduction133(parser); return;
        case 389: pr.reduceByProduction389(parser); return;
        case 1299: pr.reduceByProduction1299(parser); return;
        case 139: pr.reduceByProduction139(parser); return;
        case 722: pr.reduceByProduction722(parser); return;
        case 936: pr.reduceByProduction936(parser); return;
        case 898: pr.reduceByProduction898(parser); return;
        case 880: pr.reduceByProduction880(parser); return;
        case 500: pr.reduceByProduction500(parser); return;
        case 973: pr.reduceByProduction973(parser); return;
        case 415: pr.reduceByProduction415(parser); return;
        case 608: pr.reduceByProduction608(parser); return;
        case 659: pr.reduceByProduction659(parser); return;
        case 1154: pr.reduceByProduction1154(parser); return;
        case 909: pr.reduceByProduction909(parser); return;
        case 347: pr.reduceByProduction347(parser); return;
        case 448: pr.reduceByProduction448(parser); return;
        case 1046: pr.reduceByProduction1046(parser); return;
        case 1164: pr.reduceByProduction1164(parser); return;
        case 1251: pr.reduceByProduction1251(parser); return;
        case 904: pr.reduceByProduction904(parser); return;
        case 753: pr.reduceByProduction753(parser); return;
        case 1264: pr.reduceByProduction1264(parser); return;
        case 1231: pr.reduceByProduction1231(parser); return;
        case 1337: pr.reduceByProduction1337(parser); return;
        case 1056: pr.reduceByProduction1056(parser); return;
        case 1139: pr.reduceByProduction1139(parser); return;
        case 1148: pr.reduceByProduction1148(parser); return;
        case 838: pr.reduceByProduction838(parser); return;
        case 536: pr.reduceByProduction536(parser); return;
        case 94: pr.reduceByProduction94(parser); return;
        case 708: pr.reduceByProduction708(parser); return;
        case 1257: pr.reduceByProduction1257(parser); return;
        case 1162: pr.reduceByProduction1162(parser); return;
        case 1137: pr.reduceByProduction1137(parser); return;
        case 1254: pr.reduceByProduction1254(parser); return;
        case 343: pr.reduceByProduction343(parser); return;
        case 498: pr.reduceByProduction498(parser); return;
        case 150: pr.reduceByProduction150(parser); return;
        case 171: pr.reduceByProduction171(parser); return;
        case 469: pr.reduceByProduction469(parser); return;
        case 117: pr.reduceByProduction117(parser); return;
        case 126: pr.reduceByProduction126(parser); return;
        case 1019: pr.reduceByProduction1019(parser); return;
        case 901: pr.reduceByProduction901(parser); return;
        case 1335: pr.reduceByProduction1335(parser); return;
        case 900: pr.reduceByProduction900(parser); return;
        case 1143: pr.reduceByProduction1143(parser); return;
        case 164: pr.reduceByProduction164(parser); return;
        case 920: pr.reduceByProduction920(parser); return;
        case 459: pr.reduceByProduction459(parser); return;
        case 1073: pr.reduceByProduction1073(parser); return;
        case 162: pr.reduceByProduction162(parser); return;
        case 406: pr.reduceByProduction406(parser); return;
        case 1218: pr.reduceByProduction1218(parser); return;
        case 1167: pr.reduceByProduction1167(parser); return;
        case 558: pr.reduceByProduction558(parser); return;
        case 1093: pr.reduceByProduction1093(parser); return;
        case 1327: pr.reduceByProduction1327(parser); return;
        case 349: pr.reduceByProduction349(parser); return;
        case 53: pr.reduceByProduction53(parser); return;
        case 1349: pr.reduceByProduction1349(parser); return;
        case 423: pr.reduceByProduction423(parser); return;
        case 108: pr.reduceByProduction108(parser); return;
        case 412: pr.reduceByProduction412(parser); return;
        case 733: pr.reduceByProduction733(parser); return;
        case 111: pr.reduceByProduction111(parser); return;
        case 548: pr.reduceByProduction548(parser); return;
        case 642: pr.reduceByProduction642(parser); return;
        case 1117: pr.reduceByProduction1117(parser); return;
        case 1346: pr.reduceByProduction1346(parser); return;
        case 803: pr.reduceByProduction803(parser); return;
        case 1329: pr.reduceByProduction1329(parser); return;
        case 835: pr.reduceByProduction835(parser); return;
        case 490: pr.reduceByProduction490(parser); return;
        case 895: pr.reduceByProduction895(parser); return;
        case 302: pr.reduceByProduction302(parser); return;
        case 978: pr.reduceByProduction978(parser); return;
        case 825: pr.reduceByProduction825(parser); return;
        case 721: pr.reduceByProduction721(parser); return;
        case 770: pr.reduceByProduction770(parser); return;
        case 368: pr.reduceByProduction368(parser); return;
        case 1098: pr.reduceByProduction1098(parser); return;
        case 946: pr.reduceByProduction946(parser); return;
        case 140: pr.reduceByProduction140(parser); return;
        case 391: pr.reduceByProduction391(parser); return;
        case 384: pr.reduceByProduction384(parser); return;
        case 1066: pr.reduceByProduction1066(parser); return;
        case 1166: pr.reduceByProduction1166(parser); return;
        case 1085: pr.reduceByProduction1085(parser); return;
        case 9: pr.reduceByProduction9(parser); return;
        case 1113: pr.reduceByProduction1113(parser); return;
        case 882: pr.reduceByProduction882(parser); return;
        case 1161: pr.reduceByProduction1161(parser); return;
        case 36: pr.reduceByProduction36(parser); return;
        case 1278: pr.reduceByProduction1278(parser); return;
        case 1221: pr.reduceByProduction1221(parser); return;
        case 1037: pr.reduceByProduction1037(parser); return;
        case 577: pr.reduceByProduction577(parser); return;
        case 1173: pr.reduceByProduction1173(parser); return;
        case 899: pr.reduceByProduction899(parser); return;
        case 555: pr.reduceByProduction555(parser); return;
        case 1205: pr.reduceByProduction1205(parser); return;
        case 727: pr.reduceByProduction727(parser); return;
        case 314: pr.reduceByProduction314(parser); return;
        case 1144: pr.reduceByProduction1144(parser); return;
        case 180: pr.reduceByProduction180(parser); return;
        case 268: pr.reduceByProduction268(parser); return;
        case 1084: pr.reduceByProduction1084(parser); return;
        case 940: pr.reduceByProduction940(parser); return;
        case 436: pr.reduceByProduction436(parser); return;
        case 533: pr.reduceByProduction533(parser); return;
        case 105: pr.reduceByProduction105(parser); return;
        case 930: pr.reduceByProduction930(parser); return;
        case 667: pr.reduceByProduction667(parser); return;
        case 1112: pr.reduceByProduction1112(parser); return;
        case 256: pr.reduceByProduction256(parser); return;
        case 428: pr.reduceByProduction428(parser); return;
        case 700: pr.reduceByProduction700(parser); return;
        case 1179: pr.reduceByProduction1179(parser); return;
        case 1108: pr.reduceByProduction1108(parser); return;
        case 379: pr.reduceByProduction379(parser); return;
        case 812: pr.reduceByProduction812(parser); return;
        case 1233: pr.reduceByProduction1233(parser); return;
        case 843: pr.reduceByProduction843(parser); return;
        case 672: pr.reduceByProduction672(parser); return;
        case 707: pr.reduceByProduction707(parser); return;
        case 404: pr.reduceByProduction404(parser); return;
        case 352: pr.reduceByProduction352(parser); return;
        case 1170: pr.reduceByProduction1170(parser); return;
        case 688: pr.reduceByProduction688(parser); return;
        case 783: pr.reduceByProduction783(parser); return;
        case 698: pr.reduceByProduction698(parser); return;
        case 489: pr.reduceByProduction489(parser); return;
        case 1256: pr.reduceByProduction1256(parser); return;
        case 864: pr.reduceByProduction864(parser); return;
        case 1328: pr.reduceByProduction1328(parser); return;
        case 312: pr.reduceByProduction312(parser); return;
        case 620: pr.reduceByProduction620(parser); return;
        case 767: pr.reduceByProduction767(parser); return;
        case 1004: pr.reduceByProduction1004(parser); return;
        case 106: pr.reduceByProduction106(parser); return;
        case 1273: pr.reduceByProduction1273(parser); return;
        case 439: pr.reduceByProduction439(parser); return;
        case 914: pr.reduceByProduction914(parser); return;
        case 223: pr.reduceByProduction223(parser); return;
        case 680: pr.reduceByProduction680(parser); return;
        case 738: pr.reduceByProduction738(parser); return;
        case 1058: pr.reduceByProduction1058(parser); return;
        case 622: pr.reduceByProduction622(parser); return;
        case 1159: pr.reduceByProduction1159(parser); return;
        case 586: pr.reduceByProduction586(parser); return;
        case 539: pr.reduceByProduction539(parser); return;
        case 1237: pr.reduceByProduction1237(parser); return;
        case 713: pr.reduceByProduction713(parser); return;
        case 61: pr.reduceByProduction61(parser); return;
        case 896: pr.reduceByProduction896(parser); return;
        case 1240: pr.reduceByProduction1240(parser); return;
        case 851: pr.reduceByProduction851(parser); return;
        case 1018: pr.reduceByProduction1018(parser); return;
        case 79: pr.reduceByProduction79(parser); return;
        case 1061: pr.reduceByProduction1061(parser); return;
        case 410: pr.reduceByProduction410(parser); return;
        case 1077: pr.reduceByProduction1077(parser); return;
        case 679: pr.reduceByProduction679(parser); return;
        case 264: pr.reduceByProduction264(parser); return;
        case 706: pr.reduceByProduction706(parser); return;
        case 774: pr.reduceByProduction774(parser); return;
        case 67: pr.reduceByProduction67(parser); return;
        case 741: pr.reduceByProduction741(parser); return;
        case 1344: pr.reduceByProduction1344(parser); return;
        case 915: pr.reduceByProduction915(parser); return;
        case 761: pr.reduceByProduction761(parser); return;
        case 756: pr.reduceByProduction756(parser); return;
        case 1072: pr.reduceByProduction1072(parser); return;
        case 1035: pr.reduceByProduction1035(parser); return;
        case 800: pr.reduceByProduction800(parser); return;
        case 543: pr.reduceByProduction543(parser); return;
        case 1316: pr.reduceByProduction1316(parser); return;
        case 335: pr.reduceByProduction335(parser); return;
        case 579: pr.reduceByProduction579(parser); return;
        case 960: pr.reduceByProduction960(parser); return;
        case 1354: pr.reduceByProduction1354(parser); return;
        case 1006: pr.reduceByProduction1006(parser); return;
        case 397: pr.reduceByProduction397(parser); return;
        case 390: pr.reduceByProduction390(parser); return;
        case 1135: pr.reduceByProduction1135(parser); return;
        case 1345: pr.reduceByProduction1345(parser); return;
        case 1330: pr.reduceByProduction1330(parser); return;
        case 1131: pr.reduceByProduction1131(parser); return;
        case 508: pr.reduceByProduction508(parser); return;
        case 701: pr.reduceByProduction701(parser); return;
        case 949: pr.reduceByProduction949(parser); return;
        case 1302: pr.reduceByProduction1302(parser); return;
        case 992: pr.reduceByProduction992(parser); return;
        case 1223: pr.reduceByProduction1223(parser); return;
        case 997: pr.reduceByProduction997(parser); return;
        case 233: pr.reduceByProduction233(parser); return;
        case 1060: pr.reduceByProduction1060(parser); return;
        case 327: pr.reduceByProduction327(parser); return;
        case 1263: pr.reduceByProduction1263(parser); return;
        case 60: pr.reduceByProduction60(parser); return;
        case 466: pr.reduceByProduction466(parser); return;
        case 1121: pr.reduceByProduction1121(parser); return;
        case 1145: pr.reduceByProduction1145(parser); return;
        case 485: pr.reduceByProduction485(parser); return;
        case 924: pr.reduceByProduction924(parser); return;
        case 1355: pr.reduceByProduction1355(parser); return;
        case 501: pr.reduceByProduction501(parser); return;
        case 1157: pr.reduceByProduction1157(parser); return;
        case 1062: pr.reduceByProduction1062(parser); return;
        case 421: pr.reduceByProduction421(parser); return;
        case 1031: pr.reduceByProduction1031(parser); return;
        case 178: pr.reduceByProduction178(parser); return;
        case 1308: pr.reduceByProduction1308(parser); return;
        case 243: pr.reduceByProduction243(parser); return;
        case 1305: pr.reduceByProduction1305(parser); return;
        case 1298: pr.reduceByProduction1298(parser); return;
        case 1106: pr.reduceByProduction1106(parser); return;
        case 30: pr.reduceByProduction30(parser); return;
        case 647: pr.reduceByProduction647(parser); return;
        case 1249: pr.reduceByProduction1249(parser); return;
        case 1042: pr.reduceByProduction1042(parser); return;
        case 990: pr.reduceByProduction990(parser); return;
        case 208: pr.reduceByProduction208(parser); return;
        case 1294: pr.reduceByProduction1294(parser); return;
        case 1050: pr.reduceByProduction1050(parser); return;
        case 1198: pr.reduceByProduction1198(parser); return;
        case 717: pr.reduceByProduction717(parser); return;
        case 462: pr.reduceByProduction462(parser); return;
        case 729: pr.reduceByProduction729(parser); return;
        case 669: pr.reduceByProduction669(parser); return;
        case 648: pr.reduceByProduction648(parser); return;
        case 625: pr.reduceByProduction625(parser); return;
        case 129: pr.reduceByProduction129(parser); return;
        case 966: pr.reduceByProduction966(parser); return;
        case 1067: pr.reduceByProduction1067(parser); return;
        case 132: pr.reduceByProduction132(parser); return;
        case 1074: pr.reduceByProduction1074(parser); return;
        case 1039: pr.reduceByProduction1039(parser); return;
        case 334: pr.reduceByProduction334(parser); return;
        case 182: pr.reduceByProduction182(parser); return;
        case 794: pr.reduceByProduction794(parser); return;
        case 1195: pr.reduceByProduction1195(parser); return;
        case 361: pr.reduceByProduction361(parser); return;
        case 401: pr.reduceByProduction401(parser); return;
        case 1307: pr.reduceByProduction1307(parser); return;
        case 996: pr.reduceByProduction996(parser); return;
        case 1138: pr.reduceByProduction1138(parser); return;
        case 597: pr.reduceByProduction597(parser); return;
        case 467: pr.reduceByProduction467(parser); return;
        case 1186: pr.reduceByProduction1186(parser); return;
        case 746: pr.reduceByProduction746(parser); return;
        case 628: pr.reduceByProduction628(parser); return;
        case 367: pr.reduceByProduction367(parser); return;
        case 1340: pr.reduceByProduction1340(parser); return;
        case 216: pr.reduceByProduction216(parser); return;
        case 157: pr.reduceByProduction157(parser); return;
        case 34: pr.reduceByProduction34(parser); return;
        case 742: pr.reduceByProduction742(parser); return;
        case 63: pr.reduceByProduction63(parser); return;
        case 560: pr.reduceByProduction560(parser); return;
        case 130: pr.reduceByProduction130(parser); return;
        case 366: pr.reduceByProduction366(parser); return;
        case 248: pr.reduceByProduction248(parser); return;
        case 678: pr.reduceByProduction678(parser); return;
        case 1007: pr.reduceByProduction1007(parser); return;
        case 520: pr.reduceByProduction520(parser); return;
        case 69: pr.reduceByProduction69(parser); return;
        case 871: pr.reduceByProduction871(parser); return;
        case 221: pr.reduceByProduction221(parser); return;
        case 690: pr.reduceByProduction690(parser); return;
        case 614: pr.reduceByProduction614(parser); return;
        case 360: pr.reduceByProduction360(parser); return;
        case 153: pr.reduceByProduction153(parser); return;
        case 458: pr.reduceByProduction458(parser); return;
        case 517: pr.reduceByProduction517(parser); return;
        case 563: pr.reduceByProduction563(parser); return;
        case 447: pr.reduceByProduction447(parser); return;
        case 719: pr.reduceByProduction719(parser); return;
        case 1210: pr.reduceByProduction1210(parser); return;
        case 217: pr.reduceByProduction217(parser); return;
        case 810: pr.reduceByProduction810(parser); return;
        case 1352: pr.reduceByProduction1352(parser); return;
        case 1322: pr.reduceByProduction1322(parser); return;
        case 963: pr.reduceByProduction963(parser); return;
        case 465: pr.reduceByProduction465(parser); return;
        case 1248: pr.reduceByProduction1248(parser); return;
        case 46: pr.reduceByProduction46(parser); return;
        case 176: pr.reduceByProduction176(parser); return;
        case 344: pr.reduceByProduction344(parser); return;
        case 1191: pr.reduceByProduction1191(parser); return;
        case 235: pr.reduceByProduction235(parser); return;
        case 386: pr.reduceByProduction386(parser); return;
        case 125: pr.reduceByProduction125(parser); return;
        case 222: pr.reduceByProduction222(parser); return;
        case 589: pr.reduceByProduction589(parser); return;
        case 271: pr.reduceByProduction271(parser); return;
        case 326: pr.reduceByProduction326(parser); return;
        case 1193: pr.reduceByProduction1193(parser); return;
        case 378: pr.reduceByProduction378(parser); return;
        case 564: pr.reduceByProduction564(parser); return;
        case 985: pr.reduceByProduction985(parser); return;
        case 255: pr.reduceByProduction255(parser); return;
        case 247: pr.reduceByProduction247(parser); return;
        case 650: pr.reduceByProduction650(parser); return;
        case 266: pr.reduceByProduction266(parser); return;
        case 98: pr.reduceByProduction98(parser); return;
        case 760: pr.reduceByProduction760(parser); return;
        case 1348: pr.reduceByProduction1348(parser); return;
        case 584: pr.reduceByProduction584(parser); return;
        case 1028: pr.reduceByProduction1028(parser); return;
        case 234: pr.reduceByProduction234(parser); return;
        case 549: pr.reduceByProduction549(parser); return;
        case 870: pr.reduceByProduction870(parser); return;
        case 408: pr.reduceByProduction408(parser); return;
        case 789: pr.reduceByProduction789(parser); return;
        case 339: pr.reduceByProduction339(parser); return;
        case 118: pr.reduceByProduction118(parser); return;
        case 1092: pr.reduceByProduction1092(parser); return;
        case 1222: pr.reduceByProduction1222(parser); return;
        case 600: pr.reduceByProduction600(parser); return;
        case 893: pr.reduceByProduction893(parser); return;
        case 1300: pr.reduceByProduction1300(parser); return;
        case 524: pr.reduceByProduction524(parser); return;
        case 1149: pr.reduceByProduction1149(parser); return;
        case 1255: pr.reduceByProduction1255(parser); return;
        case 829: pr.reduceByProduction829(parser); return;
        case 392: pr.reduceByProduction392(parser); return;
        case 574: pr.reduceByProduction574(parser); return;
        case 341: pr.reduceByProduction341(parser); return;
        case 1304: pr.reduceByProduction1304(parser); return;
        case 637: pr.reduceByProduction637(parser); return;
        case 73: pr.reduceByProduction73(parser); return;
        case 15: pr.reduceByProduction15(parser); return;
        case 395: pr.reduceByProduction395(parser); return;
        case 777: pr.reduceByProduction777(parser); return;
        case 1146: pr.reduceByProduction1146(parser); return;
        case 993: pr.reduceByProduction993(parser); return;
        case 780: pr.reduceByProduction780(parser); return;
        case 1021: pr.reduceByProduction1021(parser); return;
        case 1130: pr.reduceByProduction1130(parser); return;
        case 398: pr.reduceByProduction398(parser); return;
        case 562: pr.reduceByProduction562(parser); return;
        case 724: pr.reduceByProduction724(parser); return;
        case 353: pr.reduceByProduction353(parser); return;
        case 1216: pr.reduceByProduction1216(parser); return;
        case 156: pr.reduceByProduction156(parser); return;
        case 595: pr.reduceByProduction595(parser); return;
        case 1287: pr.reduceByProduction1287(parser); return;
        case 1054: pr.reduceByProduction1054(parser); return;
        case 7: pr.reduceByProduction7(parser); return;
        case 382: pr.reduceByProduction382(parser); return;
        case 834: pr.reduceByProduction834(parser); return;
        case 736: pr.reduceByProduction736(parser); return;
        case 451: pr.reduceByProduction451(parser); return;
        case 568: pr.reduceByProduction568(parser); return;
        case 1286: pr.reduceByProduction1286(parser); return;
        case 557: pr.reduceByProduction557(parser); return;
        case 693: pr.reduceByProduction693(parser); return;
        case 953: pr.reduceByProduction953(parser); return;
        case 1116: pr.reduceByProduction1116(parser); return;
        case 166: pr.reduceByProduction166(parser); return;
        case 1289: pr.reduceByProduction1289(parser); return;
        case 553: pr.reduceByProduction553(parser); return;
        case 765: pr.reduceByProduction765(parser); return;
        case 1190: pr.reduceByProduction1190(parser); return;
        case 1343: pr.reduceByProduction1343(parser); return;
        case 1140: pr.reduceByProduction1140(parser); return;
        case 818: pr.reduceByProduction818(parser); return;
        case 13: pr.reduceByProduction13(parser); return;
        case 371: pr.reduceByProduction371(parser); return;
        case 33: pr.reduceByProduction33(parser); return;
        case 172: pr.reduceByProduction172(parser); return;
        case 1234: pr.reduceByProduction1234(parser); return;
        case 1003: pr.reduceByProduction1003(parser); return;
        case 1033: pr.reduceByProduction1033(parser); return;
        case 57: pr.reduceByProduction57(parser); return;
        case 942: pr.reduceByProduction942(parser); return;
        case 1010: pr.reduceByProduction1010(parser); return;
        case 12: pr.reduceByProduction12(parser); return;
        case 916: pr.reduceByProduction916(parser); return;
        case 1202: pr.reduceByProduction1202(parser); return;
        case 857: pr.reduceByProduction857(parser); return;
        case 705: pr.reduceByProduction705(parser); return;
        case 1318: pr.reduceByProduction1318(parser); return;
        case 432: pr.reduceByProduction432(parser); return;
        case 970: pr.reduceByProduction970(parser); return;
        case 313: pr.reduceByProduction313(parser); return;
        case 695: pr.reduceByProduction695(parser); return;
        case 245: pr.reduceByProduction245(parser); return;
        case 1196: pr.reduceByProduction1196(parser); return;
        case 790: pr.reduceByProduction790(parser); return;
        case 1026: pr.reduceByProduction1026(parser); return;
        case 1229: pr.reduceByProduction1229(parser); return;
        case 967: pr.reduceByProduction967(parser); return;
        case 927: pr.reduceByProduction927(parser); return;
        case 748: pr.reduceByProduction748(parser); return;
        case 1320: pr.reduceByProduction1320(parser); return;
        case 337: pr.reduceByProduction337(parser); return;
        case 758: pr.reduceByProduction758(parser); return;
        case 442: pr.reduceByProduction442(parser); return;
        case 135: pr.reduceByProduction135(parser); return;
        case 202: pr.reduceByProduction202(parser); return;
        case 1208: pr.reduceByProduction1208(parser); return;
        case 251: pr.reduceByProduction251(parser); return;
        case 1215: pr.reduceByProduction1215(parser); return;
        case 492: pr.reduceByProduction492(parser); return;
        case 905: pr.reduceByProduction905(parser); return;
        case 830: pr.reduceByProduction830(parser); return;
        case 475: pr.reduceByProduction475(parser); return;
        case 839: pr.reduceByProduction839(parser); return;
        case 673: pr.reduceByProduction673(parser); return;
        case 62: pr.reduceByProduction62(parser); return;
        case 995: pr.reduceByProduction995(parser); return;
        case 267: pr.reduceByProduction267(parser); return;
        case 1258: pr.reduceByProduction1258(parser); return;
        case 455: pr.reduceByProduction455(parser); return;
        case 1082: pr.reduceByProduction1082(parser); return;
        case 1070: pr.reduceByProduction1070(parser); return;
        case 815: pr.reduceByProduction815(parser); return;
        case 414: pr.reduceByProduction414(parser); return;
        case 554: pr.reduceByProduction554(parser); return;
        case 605: pr.reduceByProduction605(parser); return;
        case 1363: pr.reduceByProduction1363(parser); return;
        case 769: pr.reduceByProduction769(parser); return;
        case 582: pr.reduceByProduction582(parser); return;
        case 220: pr.reduceByProduction220(parser); return;
        case 1065: pr.reduceByProduction1065(parser); return;
        case 573: pr.reduceByProduction573(parser); return;
        case 1288: pr.reduceByProduction1288(parser); return;
        case 1283: pr.reduceByProduction1283(parser); return;
        case 512: pr.reduceByProduction512(parser); return;
        case 1280: pr.reduceByProduction1280(parser); return;
        case 639: pr.reduceByProduction639(parser); return;
        case 735: pr.reduceByProduction735(parser); return;
        case 89: pr.reduceByProduction89(parser); return;
        case 1100: pr.reduceByProduction1100(parser); return;
        case 1045: pr.reduceByProduction1045(parser); return;
        case 1122: pr.reduceByProduction1122(parser); return;
        case 97: pr.reduceByProduction97(parser); return;
        case 332: pr.reduceByProduction332(parser); return;
        case 331: pr.reduceByProduction331(parser); return;
        case 1175: pr.reduceByProduction1175(parser); return;
        case 581: pr.reduceByProduction581(parser); return;
        case 686: pr.reduceByProduction686(parser); return;
        case 943: pr.reduceByProduction943(parser); return;
        case 115: pr.reduceByProduction115(parser); return;
        case 585: pr.reduceByProduction585(parser); return;
        case 971: pr.reduceByProduction971(parser); return;
        case 538: pr.reduceByProduction538(parser); return;
        case 712: pr.reduceByProduction712(parser); return;
        case 962: pr.reduceByProduction962(parser); return;
        case 1038: pr.reduceByProduction1038(parser); return;
        case 1331: pr.reduceByProduction1331(parser); return;
        case 1118: pr.reduceByProduction1118(parser); return;
        case 51: pr.reduceByProduction51(parser); return;
        case 141: pr.reduceByProduction141(parser); return;
        case 438: pr.reduceByProduction438(parser); return;
        case 1086: pr.reduceByProduction1086(parser); return;
        case 10: pr.reduceByProduction10(parser); return;
        case 1271: pr.reduceByProduction1271(parser); return;
        case 1311: pr.reduceByProduction1311(parser); return;
        case 921: pr.reduceByProduction921(parser); return;
        case 750: pr.reduceByProduction750(parser); return;
        case 616: pr.reduceByProduction616(parser); return;
        case 191: pr.reduceByProduction191(parser); return;
        case 128: pr.reduceByProduction128(parser); return;
        case 1351: pr.reduceByProduction1351(parser); return;
        case 1312: pr.reduceByProduction1312(parser); return;
        case 1183: pr.reduceByProduction1183(parser); return;
        case 807: pr.reduceByProduction807(parser); return;
        case 1168: pr.reduceByProduction1168(parser); return;
        case 269: pr.reduceByProduction269(parser); return;
        case 76: pr.reduceByProduction76(parser); return;
        case 479: pr.reduceByProduction479(parser); return;
        case 762: pr.reduceByProduction762(parser); return;
        case 849: pr.reduceByProduction849(parser); return;
        case 227: pr.reduceByProduction227(parser); return;
        case 725: pr.reduceByProduction725(parser); return;
        case 225: pr.reduceByProduction225(parser); return;
        case 704: pr.reduceByProduction704(parser); return;
        case 194: pr.reduceByProduction194(parser); return;
        case 1268: pr.reduceByProduction1268(parser); return;
        case 124: pr.reduceByProduction124(parser); return;
        case 272: pr.reduceByProduction272(parser); return;
        case 219: pr.reduceByProduction219(parser); return;
        case 656: pr.reduceByProduction656(parser); return;
        case 732: pr.reduceByProduction732(parser); return;
        case 1102: pr.reduceByProduction1102(parser); return;
        case 3: pr.reduceByProduction3(parser); return;
        case 440: pr.reduceByProduction440(parser); return;
        case 1176: pr.reduceByProduction1176(parser); return;
        case 1220: pr.reduceByProduction1220(parser); return;
        case 999: pr.reduceByProduction999(parser); return;
        case 278: pr.reduceByProduction278(parser); return;
        case 1172: pr.reduceByProduction1172(parser); return;
        case 1209: pr.reduceByProduction1209(parser); return;
        case 184: pr.reduceByProduction184(parser); return;
        case 747: pr.reduceByProduction747(parser); return;
        case 1276: pr.reduceByProduction1276(parser); return;
        case 1057: pr.reduceByProduction1057(parser); return;
        case 229: pr.reduceByProduction229(parser); return;
        case 793: pr.reduceByProduction793(parser); return;
        case 127: pr.reduceByProduction127(parser); return;
        case 528: pr.reduceByProduction528(parser); return;
        case 1232: pr.reduceByProduction1232(parser); return;
        case 1314: pr.reduceByProduction1314(parser); return;
        case 409: pr.reduceByProduction409(parser); return;
        case 23: pr.reduceByProduction23(parser); return;
        case 781: pr.reduceByProduction781(parser); return;
        case 96: pr.reduceByProduction96(parser); return;
        case 913: pr.reduceByProduction913(parser); return;
        case 525: pr.reduceByProduction525(parser); return;
        case 429: pr.reduceByProduction429(parser); return;
        case 1110: pr.reduceByProduction1110(parser); return;
        case 274: pr.reduceByProduction274(parser); return;
        case 658: pr.reduceByProduction658(parser); return;
        case 50: pr.reduceByProduction50(parser); return;
        case 583: pr.reduceByProduction583(parser); return;
        case 527: pr.reduceByProduction527(parser); return;
        case 550: pr.reduceByProduction550(parser); return;
        case 976: pr.reduceByProduction976(parser); return;
        case 699: pr.reduceByProduction699(parser); return;
        case 1081: pr.reduceByProduction1081(parser); return;
        case 26: pr.reduceByProduction26(parser); return;
        case 1297: pr.reduceByProduction1297(parser); return;
        case 785: pr.reduceByProduction785(parser); return;
        case 158: pr.reduceByProduction158(parser); return;
        case 509: pr.reduceByProduction509(parser); return;
        case 1326: pr.reduceByProduction1326(parser); return;
        case 965: pr.reduceByProduction965(parser); return;
        case 165: pr.reduceByProduction165(parser); return;
        case 483: pr.reduceByProduction483(parser); return;
        case 1017: pr.reduceByProduction1017(parser); return;
        case 836: pr.reduceByProduction836(parser); return;
        case 123: pr.reduceByProduction123(parser); return;
        case 1201: pr.reduceByProduction1201(parser); return;
        case 641: pr.reduceByProduction641(parser); return;
        case 991: pr.reduceByProduction991(parser); return;
        case 1333: pr.reduceByProduction1333(parser); return;
        case 1189: pr.reduceByProduction1189(parser); return;
        case 377: pr.reduceByProduction377(parser); return;
        case 745: pr.reduceByProduction745(parser); return;
        case 906: pr.reduceByProduction906(parser); return;
        case 161: pr.reduceByProduction161(parser); return;
        case 599: pr.reduceByProduction599(parser); return;
        case 27: pr.reduceByProduction27(parser); return;
        case 460: pr.reduceByProduction460(parser); return;
        case 1334: pr.reduceByProduction1334(parser); return;
        case 1083: pr.reduceByProduction1083(parser); return;
        case 435: pr.reduceByProduction435(parser); return;
        case 653: pr.reduceByProduction653(parser); return;
        case 511: pr.reduceByProduction511(parser); return;
        case 168: pr.reduceByProduction168(parser); return;
        case 261: pr.reduceByProduction261(parser); return;
        case 1104: pr.reduceByProduction1104(parser); return;
        case 1020: pr.reduceByProduction1020(parser); return;
        case 922: pr.reduceByProduction922(parser); return;
        case 1141: pr.reduceByProduction1141(parser); return;
        case 1285: pr.reduceByProduction1285(parser); return;
        case 624: pr.reduceByProduction624(parser); return;
        case 193: pr.reduceByProduction193(parser); return;
        case 580: pr.reduceByProduction580(parser); return;
        }
    }

    /**
     * Returns the parser's next state when the given nonterminal appears on top of the state stack
     */
    int getGoToFor(Nonterminal nonterminal, Parser parser)
    {
        return lookupGoTo(nonterminal.getGoToRow(), parser.getCurrentState());
    }

    /**
     * Looks up a row in the action table
     */
    private static short lookupAction(int row, int col) { return ActionTable.get(row, col); }

    protected static class ActionTable
    {
        protected static final int[] rowmap = { 0, 0, 1, 0, 0, 0, 2, 3, 0, 1, 4, 5, 0, 0, 6, 7, 8, 1, 0, 9, 10, 11, 12, 13, 0, 2, 0, 14, 15, 1, 1, 16, 17, 18, 2, 19, 20, 3, 21, 22, 4, 3, 1, 23, 24, 5, 2, 25, 5, 6, 1, 26, 27, 28, 29, 0, 30, 6, 31, 7, 32, 0, 33, 27, 8, 3, 7, 34, 35, 36, 8, 37, 9, 38, 39, 10, 10, 40, 41, 11, 42, 11, 12, 0, 1, 0, 43, 44, 45, 2, 46, 5, 47, 48, 3, 13, 0, 14, 2, 49, 12, 13, 0, 0, 15, 50, 3, 51, 15, 16, 17, 2, 0, 18, 16, 11, 17, 52, 53, 19, 7, 54, 55, 56, 57, 58, 59, 8, 18, 20, 19, 21, 0, 0, 10, 22, 60, 23, 24, 61, 20, 1, 62, 63, 25, 26, 64, 5, 21, 65, 66, 28, 67, 22, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 23, 78, 24, 7, 79, 80, 7, 81, 82, 83, 84, 85, 86, 25, 29, 0, 26, 30, 87, 88, 89, 11, 27, 90, 8, 31, 0, 0 };
        protected static final int[] columnmap = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 15, 34, 35, 36, 37, 15, 38, 39, 40, 41, 42, 43, 44, 10, 45, 46, 47, 48, 49, 36, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 34, 77, 78, 79, 80, 11, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 61, 10, 0, 95, 11, 32, 96, 97, 97, 32, 15, 34, 0, 66, 76, 98, 55, 99, 100, 101, 102, 61, 103, 66, 104, 0, 105, 106, 99, 107, 108, 51, 76, 109, 110, 43, 10, 10, 111, 112, 113, 55, 114, 115, 116, 92, 117, 118, 119, 98, 120, 92, 121, 101, 102, 122, 95, 1, 123, 124, 125, 103, 105, 126, 106, 127, 128, 129, 111, 130, 131, 132, 133, 134, 100, 135, 101, 102, 103, 136, 2, 15, 105, 137, 106, 108, 112, 113, 107, 115, 138, 119, 3, 139, 98, 109, 108, 140, 141, 142, 143, 116, 144, 145, 61, 146, 4, 5, 6, 121, 1, 32, 2, 126, 147, 148, 149, 150, 32, 112, 10, 151, 121, 124, 125, 126, 127, 129, 152, 114, 1, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166, 167, 11, 168, 109, 169, 170, 171, 32, 172, 173, 174, 131, 175, 176, 177, 100, 178, 110, 7, 179, 113, 129, 88, 115, 131, 180, 181, 182, 183, 117, 184, 185, 186, 187, 188, 189, 190, 10, 133, 191, 136, 139, 140, 117, 8, 9, 141, 192, 146, 118, 151, 11, 167, 120, 12, 13, 193, 14, 15, 194, 116, 119, 3, 16, 124, 125, 195, 196, 136, 197, 198, 199, 200, 201, 133, 2, 202, 139, 98, 203, 110, 140, 204, 141, 17, 205, 206, 207, 146, 208, 209, 210, 151, 211, 167, 212, 213, 122, 214, 168, 123, 11, 55, 215, 216, 217, 218, 172, 219, 220, 221, 178, 222, 181, 179, 18, 223, 224, 225, 182, 183, 226, 184, 227, 228, 229, 230, 231, 164, 232, 233, 234, 185, 235, 236, 237, 238, 239, 240, 241, 242, 243, 10, 244, 245, 19, 246, 20, 21, 4, 22, 247, 186, 43, 248, 187, 66, 23, 24, 76, 92, 95, 25, 249, 250, 102, 251, 198, 172, 252, 200, 197, 202, 204, 32, 253, 209, 254, 255, 218, 256, 228, 103, 211, 229, 242, 217, 246, 257, 258, 259, 260, 261, 262, 263, 264, 265, 266, 267, 268, 269, 270, 271, 272, 273, 274, 275, 276, 277, 26, 27, 278, 279, 280, 258, 28, 29, 261, 168, 30, 264, 31, 188, 5, 33, 35, 6, 36, 37, 38, 178, 265, 281, 282, 189, 283, 39, 15, 8, 40, 268, 41, 42, 44, 45, 284, 285, 269, 101, 270, 106, 193, 107, 286, 287, 288, 46, 190, 47, 273, 48, 201, 49, 9, 205, 50, 51, 289, 52, 53, 290, 54, 291, 292, 3, 56, 57, 58, 112, 275, 293, 294, 295, 296, 59, 60, 12, 62, 206, 63, 297, 64, 13, 213, 65, 140, 127, 215, 141, 298, 181, 123, 182, 183, 67, 299, 68, 300, 301, 69, 70, 36, 190, 302, 303, 71, 304, 72, 73, 305, 306, 74, 307, 75, 308, 309, 310, 311, 312, 77, 313, 10, 114, 78, 79, 314, 80, 81, 82, 83, 315, 316, 317, 14, 84, 85, 318, 319, 66, 320, 86, 87, 216, 16, 226, 88, 89, 321, 232, 34, 55, 297, 61, 118, 96, 322, 323, 191, 324, 325, 196, 233, 326, 236, 238, 327, 328, 90, 32, 329, 191, 91, 93, 330, 331, 332, 34, 333, 94, 334, 201, 205, 335, 336, 337, 120, 128, 338, 339, 173, 133, 184, 340, 341, 342, 343, 130, 132, 344, 345, 346, 134, 135, 173, 186, 347, 10, 199, 348, 43, 17, 18, 26, 27, 28, 200, 37, 38, 39, 40, 41, 207, 42, 349, 350, 351, 352, 353, 354, 15, 210, 11, 355, 356, 357, 358, 359, 137, 138, 360, 193, 206, 207, 208, 179, 210, 212, 213, 214, 144, 216, 361, 199, 362, 15, 4, 363, 364, 365, 366, 367, 368, 369, 370, 371, 372, 373, 76, 374, 209, 142, 143, 375, 376, 145, 211, 377, 378, 185, 188, 147, 5, 212, 214, 218, 379, 219, 221, 225, 98, 102, 125, 0, 131, 239, 139, 146, 380, 2, 3, 151, 167, 381, 55, 382, 383, 215, 189, 384, 385, 386, 387, 148, 149, 388, 109, 389, 390, 224, 128, 7, 8, 150, 391, 152, 392, 393, 394, 11, 395, 241, 396, 15, 153, 154, 397, 398, 243, 155, 156, 15, 19, 286, 399, 221, 219, 400, 96, 401, 157, 158, 224, 225, 159, 402, 403, 160, 226, 61, 404, 161, 405, 162, 44, 20, 406, 407, 408, 34, 409, 410, 228, 163, 303, 130, 309, 164, 411, 412, 245, 413, 165, 414, 229, 166, 415, 134, 191, 416, 169, 170, 417, 418, 171, 419, 227, 420, 174, 175, 21, 176, 177, 244, 421, 180, 34, 22, 192, 194, 422, 423, 220, 424, 195, 245, 425, 222, 208, 223, 426, 227, 427, 428, 230, 231, 241, 429, 430, 431, 432, 234, 23, 235, 433, 237, 434, 240, 435, 436, 239, 437, 247, 438, 248, 439, 440, 249, 441, 250, 442, 443, 444, 445, 251, 252, 446, 447, 253, 448, 449, 92, 450, 254, 255, 451, 279, 281, 248, 15, 452, 453, 454, 455, 456, 457, 458, 285, 43, 459, 460, 461, 462, 463, 464, 465, 256, 257, 466, 467, 259, 260, 468, 262, 469, 470, 471, 9, 10, 11, 472, 144, 473, 474, 475, 476, 291, 10, 295, 477, 478, 479, 480, 481, 482, 15, 34, 483, 298, 316, 320, 484, 485, 486, 487, 488, 299, 489, 490, 324, 491, 492, 493, 494, 495, 346, 304, 349, 363, 496, 497, 498, 499, 500, 501, 502, 503, 504, 505, 506, 507, 508, 509, 510, 511, 512, 513, 514, 515, 516, 517, 518, 409, 519, 413, 307, 315, 321, 520, 105, 10, 108, 115, 521, 522, 523, 418, 524, 525, 119, 526, 527, 55, 528, 17, 293, 296, 29, 249, 45, 76, 46, 95, 263, 529, 530, 531, 532, 533, 534, 535, 286, 293, 47, 50, 328, 331, 536, 537, 538, 120, 1, 539, 51, 540, 296, 25, 541, 4, 542, 543, 242, 266, 267, 544, 545, 97, 52, 272, 546, 547, 548, 15, 549, 32, 61, 301, 271, 30, 550, 551, 552, 553, 554, 555, 53, 54, 58, 556, 557, 558, 175, 196, 559, 560, 211, 561, 562, 274, 563, 147, 564, 565, 566, 567, 43, 136, 276, 277, 568, 278, 569, 280, 282, 570, 571, 572, 573, 31, 574, 575, 283, 576, 577, 578, 579, 43, 394, 580, 581, 582, 403, 404, 583, 65, 69, 70, 124, 214, 584, 585, 586, 587, 71, 72, 73, 588, 589, 590, 591, 592, 593, 594, 595, 596, 597, 598, 215, 599, 600, 601, 602, 603, 604, 605, 606, 607, 608, 609, 610, 611, 612, 284, 613, 614, 615, 616, 287, 617, 74, 618, 97, 5, 619, 75, 620, 168, 6, 170, 621, 171, 622, 178, 184, 623, 188, 624, 189, 625, 288, 190, 626, 84, 333, 289, 290, 292, 294, 627, 628, 629, 630, 631, 632, 85, 90, 91, 185, 93, 633, 634, 300, 66, 635, 636, 94, 132, 137, 637, 638, 639, 640, 641, 642, 279, 643, 644, 645, 646, 647, 648, 12, 13, 14, 16, 302, 649, 650, 651, 652, 653, 246, 138, 142, 306, 55, 654, 655, 656, 657, 658, 659, 660, 143, 144, 145, 258, 334, 147, 661, 662, 305, 663, 664, 308, 665, 666, 667, 43, 668, 669, 310, 311, 670, 671, 314, 317, 672, 673, 674, 318, 675, 319, 325, 676, 677, 327, 678, 679, 326, 680, 329, 330, 681, 682, 332, 335, 337, 395, 282, 410, 261, 419, 148, 18, 19, 33, 207, 149, 150, 152, 153, 154, 20, 157, 158, 683, 684, 114, 338, 95, 685, 686, 339, 687, 341, 688, 159, 210, 161, 689, 342, 344, 690, 691, 303, 309, 312, 347, 692, 32, 345, 348, 693, 694, 695, 350, 351, 352, 696, 353, 697, 698, 354, 264, 699, 700, 701, 358, 360, 702, 703, 704, 99, 705, 706, 265, 707, 361, 708, 709, 364, 710, 711, 99, 712, 713, 714, 61, 715, 716, 717, 718, 719, 720, 721, 212, 722, 288, 723, 310, 724, 214, 725, 726, 100, 727, 728, 365, 729, 366, 367, 730, 731, 163, 169, 281, 732, 733, 734, 176, 177, 180, 735, 192, 368, 194, 61, 736, 369, 737, 738, 739, 370, 371, 740, 372, 741, 11, 321, 742, 313, 743, 203, 221, 268, 223, 744, 745, 148, 375, 377, 746, 747, 378, 34, 35, 322, 323, 380, 748, 21, 24, 26, 27, 28, 30, 31, 32, 33, 35, 36, 37, 749, 195, 199, 224, 750, 389, 751, 36, 752, 392, 381, 7, 269, 273, 275, 753, 301, 315, 227, 231, 288, 754, 755, 310, 233, 382, 756, 383, 757, 385, 386, 758, 388, 759, 236, 760, 761, 762, 101, 763, 764, 765, 766, 237, 328, 238, 390, 767, 768, 769, 331, 333, 244, 327, 43, 770, 771, 772, 391, 393, 773, 774, 106, 334, 397, 775, 776, 777, 778, 245, 779, 780, 781, 399, 782, 401, 783, 402, 784, 785, 786, 787, 406, 788, 247, 789, 790, 248, 791, 405, 792, 793, 794, 249, 795, 38, 796, 250, 797, 39, 407, 253, 408, 798, 799, 800, 801, 411, 802, 256, 41, 412, 803, 804, 805, 806, 807, 414, 808, 316, 257, 259, 66, 809, 810, 811, 415, 812, 813, 416, 814, 815, 417, 420, 816, 817, 42, 818, 819, 421, 118, 422, 285, 260, 266, 267, 336, 320, 276, 105, 107, 820, 425, 277, 278, 340, 309, 280, 284, 287, 821, 433, 428, 424, 426, 427, 822, 431, 430, 434, 443, 92, 44, 450, 305, 438, 452, 464, 470, 471, 474, 823, 477, 824, 312, 308, 483, 484, 491, 530, 825, 532, 826, 108, 827, 828, 343, 829, 830, 831, 435, 832, 833, 437, 834, 439, 48, 440, 441, 835, 442, 311, 355, 836, 49, 837, 109, 313, 46, 444, 838, 839, 840, 325, 841, 842, 843, 453, 844, 454, 845, 846, 847, 314, 446, 848, 120, 849, 317, 318, 319, 329, 330, 339, 341, 342, 344, 356, 447, 76, 462, 466, 850, 851, 852, 853, 345, 854, 448, 449, 347, 455, 855, 472, 479, 856, 520, 348, 522, 523, 55, 61, 456, 524, 353, 354, 357, 457, 458, 47, 32, 222, 461, 358, 359, 360, 361, 362, 364, 56, 365, 366, 367, 368, 369, 370, 371, 373, 374, 375, 376, 377, 382, 383, 384, 386, 387, 389, 396, 857, 398, 110, 463, 858, 467, 468, 859, 860, 400, 407, 414, 861, 862, 415, 863, 469, 864, 865, 391, 473, 475, 476, 866, 133, 867, 868, 869, 478, 870, 480, 186, 481, 420, 423, 429, 432, 434, 445, 393, 399, 402, 459, 460, 465, 525, 871, 57, 872, 482, 59, 485, 873, 874, 534, 416, 467, 535, 486, 487, 488, 326, 489, 490, 492, 493, 495, 356, 541, 496, 357, 497, 498, 499, 544, 327, 875, 500, 876, 545, 877, 546, 878, 879, 417, 549, 548, 422, 880, 325, 426, 486, 881, 430, 487, 882, 488, 883, 884, 489, 885, 886, 490, 492, 887, 888, 493, 889, 890, 891, 494, 892, 893, 894, 501, 551, 60, 895, 896, 552, 897, 898, 899, 502, 62, 63, 900, 495, 901, 496, 902, 903, 904, 905, 906, 907, 556, 908, 497, 909, 910, 911, 498, 499, 912, 503, 336, 913, 914, 500, 915, 916, 917, 112, 918, 113, 5, 919, 114, 920, 115, 504, 921, 922, 64, 923, 525, 505, 435, 924, 560, 437, 513, 557, 925, 518, 561, 444, 456, 110, 926, 927, 562, 461, 928, 118, 929, 930, 565, 931, 559, 535, 572, 506, 932, 933, 934, 507, 508, 463, 509, 536, 935, 936, 510, 468, 346, 469, 937, 511, 938, 939, 567, 349, 569, 481, 49, 65, 494, 512, 36, 50, 940, 514, 513, 941, 942, 943, 515, 516, 944, 343, 504, 945, 517, 519, 946, 526, 516, 518, 947, 359, 948, 521, 949, 950, 362, 951, 527, 952, 953, 528, 954, 955, 122, 956, 957, 363, 373, 526, 527, 958, 959, 960, 961, 962, 528, 529, 963, 531, 116, 533, 538, 539, 540, 541, 964, 542, 965, 543, 52, 544, 966, 967, 968, 969, 970, 971, 972, 529, 973, 547, 974, 545, 550, 53, 546, 68, 975, 548, 976, 549, 553, 977, 978, 554, 979, 555, 980, 556, 981, 982, 531, 557, 983, 376, 379, 384, 533, 538, 558, 984, 985, 77, 986, 987, 988, 539, 989, 559, 573, 575, 990, 991, 576, 992, 993, 994, 578, 579, 995, 996, 997, 998, 580, 442, 326, 999, 585, 1000, 588, 589, 1001, 590, 582, 505, 583, 506, 1002, 1003, 117, 1004, 587, 540, 543, 591, 553, 554, 1005, 584, 558, 1006, 581, 1007, 563, 1008, 1009, 564, 1010, 1011, 1012, 1013, 1014, 354, 592, 54, 78, 79, 593, 1015, 387, 507, 1016, 1017, 560, 374, 392, 6, 394, 56, 566, 568, 570, 571, 561, 1018, 1019, 92, 594, 562, 1020, 574, 595, 596, 1021, 597, 577, 586, 1022, 1023, 1024, 1025, 1026, 598, 1027, 1028, 1029, 563, 1030, 396, 1031, 1032, 1033, 1034, 1035, 599, 600, 1036, 601, 602, 603, 1037, 1038, 604, 1039, 1040, 605, 606, 607, 608, 1041, 609, 610, 611, 1042, 57, 612, 613, 614, 644, 1043, 650, 10, 1044, 1045, 1046, 1047, 508, 1048, 1049, 509, 80, 622, 615, 616, 123, 510, 398, 1050, 564, 1051, 617, 618, 1052, 565, 1053, 568, 1054, 1055, 1056, 1057, 1058, 619, 81, 58, 620, 82, 626, 627, 83, 59, 95, 628, 629, 62, 630, 1059, 511, 515, 1060, 1061, 631, 632, 1062, 570, 1063, 1064, 355, 633, 1065, 121, 8, 9, 1066, 122, 571, 12, 1067, 572, 587, 1068, 1069, 400, 652, 66, 1070, 634, 654, 1071, 591, 401, 1072, 635, 1073, 598, 405, 637, 1074, 1075, 1076, 1077, 1078, 1079, 599, 636, 403, 1080, 85, 638, 1081, 1082, 639, 640, 1083, 86, 359, 1084, 577, 1085, 1086, 63, 641, 642, 579, 1087, 1088, 1089, 643, 1090, 1091, 126, 1092, 128, 580, 1093, 1094, 87, 1095, 600, 582, 127, 1096, 1097, 128, 1098, 583, 584, 586, 601, 592, 602, 1099, 1100, 603, 1101, 362, 1102, 1103, 1104, 1105, 1106, 1107, 1108, 368, 1109, 645, 605, 606, 1110, 607, 1111, 1112, 1113, 1114, 1115, 1116, 1117, 1118, 1119, 1120, 1121, 1122, 1123, 1124, 1125, 646, 1126, 608, 1127, 1128, 1129, 409, 404, 1130, 1131, 613, 614, 1132, 373, 647, 648, 130, 649, 656, 657, 1133, 651, 653, 1134, 660, 683, 655, 695, 659, 1135, 1136, 661, 663, 715, 720, 1137, 131, 666, 139, 140, 743, 92, 95, 96, 88, 668, 130, 1138, 615, 1139, 1140, 374, 413, 411, 418, 13, 420, 1141, 1142, 1143, 1144, 1145, 684, 1146, 1147, 744, 665, 750, 752, 667, 669, 670, 616, 672, 674, 1148, 717, 675, 676, 617, 723, 421, 671, 89, 141, 1149, 1150, 677, 1151, 1152, 678, 689, 700, 673, 679, 685, 686, 680, 692, 694, 698, 704, 707, 708, 711, 1153, 681, 1154, 728, 376, 682, 1155, 735, 1156, 687, 688, 1157, 14, 1158, 1159, 618, 1160, 620, 384, 621, 1161, 1162, 146, 147, 736, 623, 1163, 148, 1164, 1165, 1166, 738, 624, 1167, 151, 1168, 1169, 1170, 690, 691, 429, 693, 696, 1171, 1172, 387, 1173, 96, 1174, 133, 622, 16, 1175, 625, 628, 1176, 1177, 1178, 629, 423, 1179, 432, 1180, 1181, 1182, 1183, 626, 1184, 1185, 392, 134, 627, 396, 1186, 1187, 1188, 630, 1189, 631, 135, 1190, 697, 1191, 436, 446, 1192, 1193, 632, 1194, 1195, 763, 1196, 745, 747, 153, 769, 154, 770, 786, 791, 1197, 1198, 1199, 1200, 1201, 634, 1202, 186, 398, 699, 701, 1203, 1204, 1205, 193, 142, 712, 773, 1206, 1207, 1208, 1209, 1210, 635, 1211, 702, 1212, 703, 1213, 713, 783, 705, 710, 714, 143, 1214, 636, 1215, 637, 1216, 1217, 400, 1218, 1219, 1220, 640, 642, 706, 1221, 1222, 645, 648, 716, 790, 401, 719, 649, 651, 403, 652, 1223, 1224, 654, 1225, 156, 1226, 1227, 664, 157, 1228, 1229, 1230, 653, 404, 721, 1231, 1232, 656, 1233, 452, 454, 1234, 800, 1235, 1236, 809, 820, 827, 829, 831, 1237, 810, 811, 812, 817, 833, 1238, 1239, 1240, 405, 196, 144, 1241, 1242, 835, 844, 722, 724, 758, 1243, 828, 843, 1244, 657, 1245, 1246, 1247, 812, 1248, 1249, 460, 817, 1250, 1251, 462, 725, 411, 419, 1252, 1253, 1254, 826, 660, 1255, 1256, 833, 661, 1257, 420, 1258, 845, 848, 726, 1259, 1260, 850, 662, 1261, 1262, 1263, 851, 1264, 1265, 159, 160, 1266, 163, 164, 837, 1267, 1268, 838, 1269, 1270, 663, 1271, 1272, 856, 858, 860, 1273, 861, 1274, 741, 1275, 1276, 1277, 863, 865, 874, 879, 1278, 742, 1279, 760, 1280, 727, 201, 1281, 729, 1282, 730, 205, 206, 208, 762, 213, 216, 813, 822, 731, 733, 734 };

        public static short get(int row, int col)
        {
            if (isErrorEntry(row, col))
                return (short)0;
            else
                return lookupValue(rowmap[row], columnmap[col]);
        }

        protected static boolean isErrorEntry(int row, int col)
        {
            final int INT_BITS = 32;
            int sigmapRow = row;

            int sigmapCol = col / INT_BITS;
            int bitNumberFromLeft = col % INT_BITS;
            int sigmapMask = 0x1 << (INT_BITS - bitNumberFromLeft - 1);

            return (lookupSigmap(sigmapRow, sigmapCol) & sigmapMask) == 0;
        }

        protected static int[][] sigmap = null;

        protected static void sigmapInit()
        {
            sigmap = new int[][]
            {
                { 0x0, 0x0, 0x0, 0x4, 0x1000, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x208000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0xf25fd267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x70483, 0xc000005e, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104025, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x40200, 0x1000008, 0x1200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200003, 0x400, 0x800000, 0x8, 0x0, 0x1200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x1080000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20001, 0xa0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x920, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x110, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x1efc0, 0x0, 0x0, 0x54, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x30100, 0x0, 0x0, 0x0, 0x0, 0xf0a01000, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x60000, 0x0, 0x5200000, 0x0, 0x0, 0x80000, 0x0, 0x3240, 0x0, 0x0, 0x60000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x580, 0x60, 0x1, 0x80000000, 0x2600000, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x180, 0x4000, 0x200000, 0x80, 0x800000, 0x0, 0x0, 0x2, 0x10000000, 0xc0000000, 0x0, 0x84, 0x280000, 0x0, 0x19822000, 0x48d8018, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x31000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5a02d98, 0x56100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x85407b34, 0x3fffffa0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x40100c0, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4d20, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e4, 0x4fffc24, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xfd0100, 0x107fc088, 0x46, 0x2c120101, 0x200480, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x45030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x40021, 0xd9862280, 0xc8d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0xc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc24, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200480, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x40021, 0xd9862280, 0xc8d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0xffffffff, 0xdafaffef, 0xffffbdff, 0xd8040204, 0x182008c, 0x12bf1ff, 0x98f80061, 0xfe24c020, 0xf801000b, 0xcfb040c0, 0x20, 0xe54, 0x80000000, 0x0, 0x8060800b, 0x100500, 0x48010000, 0x1040c, 0x88000, 0x120650e, 0x9c630000, 0x1001ff, 0xb871c080, 0x66, 0x2c120101, 0xc1a00400, 0x20091020, 0x42141000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x181044, 0xc3023004, 0x1270003a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x403e082, 0x82460400, 0x49860, 0x5208010, 0xd0008200, 0x62848e9, 0xb4c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8008, 0x10e0080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a551, 0x1800000, 0x1c00580, 0x2980460, 0x401e4041, 0xad410949, 0x2640010, 0x800204a5, 0xa8200021, 0x1845301, 0x21000000, 0xaf8001, 0x12e80980, 0xfc4036, 0x19060003, 0x5eb175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200b44, 0xc1ec1a50, 0x1401, 0x21000084, 0x10280ad0, 0x60020, 0x19922280, 0x48d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x81000084, 0x10009100, 0x84000006, 0x40000004, 0x0, 0x88081, 0x40c00001, 0x3a000000, 0x88000000, 0x4000080, 0x0, 0x800, 0x0, 0x0, 0x8, 0x0, 0x8010000, 0x0, 0x0, 0x4522, 0x98620000, 0xa5, 0x28000000, 0x46, 0x20000000, 0x0, 0x20, 0x41000, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc0000000, 0x100000, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x2922a6, 0x93000000, 0x6000, 0x0, 0x49800, 0x0, 0x0, 0x0, 0x30000820, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0xa66ee00, 0x100, 0x0, 0x1c00000, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2f8000, 0x200000, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x19, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5a02d98, 0x56100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x4000, 0x320, 0x400000, 0x8000000, 0x4000002, 0x8000, 0x85407b34, 0x3fffffa0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xaab00080, 0x10090000, 0x0, 0x1400, 0x0, 0x0, 0x8088000, 0x8200, 0x5021240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xffdeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa81ff, 0x80d80060, 0x424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x450e, 0x98630000, 0x100100, 0x1071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4030082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120109, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200012, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x42900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x26, 0x28000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x410000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0xc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80060, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630000, 0x10015a, 0x9271c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038482, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0xf25fd267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x70483, 0xc000005e, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104025, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0xffffffff, 0xdafaffef, 0xffffbdff, 0xd8040204, 0x182008c, 0x12bf1ff, 0x98f80061, 0xfe24c020, 0xf801000b, 0xcfb040c0, 0x20, 0xe54, 0x80000000, 0x0, 0x8060800b, 0x100500, 0x48010000, 0x1040c, 0x88000, 0x120650e, 0x9c630000, 0x1001ff, 0xb871c080, 0x66, 0x2c120101, 0xc1a00400, 0x20091020, 0x42141000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x181044, 0xc3023004, 0x1270003a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x403e082, 0x82460400, 0x49860, 0x5208010, 0xd0008200, 0x62848e9, 0xb4c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8008, 0x10e0080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a551, 0x1800000, 0x1c00580, 0x2980460, 0x401e4041, 0xad410949, 0x2640010, 0x800204a5, 0xa8200021, 0x1845301, 0x21000000, 0xaf8001, 0x12e80980, 0xfc4036, 0x19060003, 0x5eb175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200b44, 0xc1ec1a50, 0x1401, 0x21000084, 0x10280ad0, 0x60020, 0x19922280, 0x48d811c, 0x0 },
                { 0xffffffff, 0xdafaffef, 0xffffbdff, 0xd8040204, 0x182008c, 0x12bf1ff, 0x98f80061, 0xfe24c020, 0xf801000b, 0xcfb040c0, 0x20, 0xe54, 0x80000000, 0x0, 0x8060800b, 0x100500, 0x48010000, 0x1040c, 0x88000, 0x120650e, 0x9c630000, 0x1001ff, 0xb871c080, 0x66, 0x2c120101, 0xd1a00400, 0x20091020, 0x42141000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x181044, 0xc3023004, 0x1270003a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x403e082, 0x82460400, 0x49860, 0x5208090, 0xd0008200, 0x62848e9, 0xb4c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8008, 0x10e0080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a551, 0x1800000, 0x1c00580, 0x2980460, 0x401e4041, 0xad410949, 0x2640010, 0x800204a5, 0xa8200021, 0x1845301, 0x21000000, 0xaf8001, 0x12e80980, 0xfc4036, 0x19060003, 0x5eb175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200b44, 0xc1ec1a50, 0x1401, 0x21000084, 0x10280ad0, 0x60020, 0x19922280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0xf25fd267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x70483, 0xc000005e, 0xf1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4025, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5be00000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x60800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xd4001800 },
                { 0x81000084, 0x9100, 0x84000006, 0x40000004, 0x200000, 0x88081, 0xc00000, 0x0, 0x88000000, 0x4400080, 0x8, 0x800, 0x0, 0x0, 0x0, 0x0, 0x8010000, 0x0, 0x0, 0x4502, 0x98620000, 0x0, 0x0, 0x46, 0x20000000, 0x400000, 0x0, 0x41000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x2922a6, 0x93000000, 0x0, 0x0, 0x49800, 0x0, 0x0, 0x0, 0x820, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0xa66ee00, 0x0, 0x0, 0x1c00000, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2f8000, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x10900, 0x400000, 0x0, 0x19, 0x0, 0x0, 0x4000001, 0x12000000, 0x4002f, 0x93515ff7, 0x3a100000, 0x40000800, 0x1008007, 0xbd402280, 0x4004001, 0x0, 0x480, 0x1000, 0x4, 0xf78e80, 0x4000, 0x40320, 0x2400000, 0x8020814, 0x25001002, 0x8005, 0x854343b6, 0xffffe354, 0x9020010, 0x940000, 0x4800, 0x20002b0, 0xa0000420, 0x3000000, 0x8000, 0x240100c0, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x3a300080, 0x10090000, 0x0, 0x1400, 0x8, 0x0, 0x8088000, 0x410c224, 0xff3f1240, 0x400c004, 0x1ac, 0x40000000, 0x0, 0x10, 0x4524, 0x37, 0x280004, 0x20215c55, 0x64010000, 0x1000000a, 0x2c200000, 0x10000000, 0x5e68, 0x7e100000, 0x40000000, 0x78030000, 0xcd20, 0x14000000, 0x2b, 0x28000c00, 0x808800, 0x200, 0x906, 0x64814018, 0x58000000, 0x404000, 0x70e0000, 0x24000000, 0x83e0062, 0x14400042, 0x33000, 0xff010604, 0x1100c, 0x20003840, 0xea500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xffceffff, 0x4efafbff, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc24, 0xb801efd0, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c138901, 0x200480, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3084, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x25200010, 0x10008200, 0x62848e1, 0x80cc7ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x9800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19260003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80060, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630000, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5a02d98, 0x56100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x85407b34, 0x3fffffa0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x40100c0, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x904504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4d20, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffda4, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fd888, 0x46, 0x2c120101, 0x200480, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x16706, 0x92535267, 0x28000000, 0x0, 0x7, 0xac000000, 0x60000001, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x50270082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x230, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x904124, 0xfa3d0000, 0x0, 0x10001ac, 0x40000000, 0x8000001, 0x3f16010, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x19100, 0x5a000000, 0x0, 0x68000000, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x64800000, 0x58000000, 0x0, 0x0, 0x0, 0x10380002, 0x0, 0x0, 0x3e000600, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x400000, 0x2200, 0x0, 0x101000, 0x20, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x1, 0x0, 0x1800, 0x0, 0x0, 0x1000100, 0x2042, 0x20000, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x500, 0x1000840, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x110000, 0x0, 0x100, 0x5000000, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x1000000, 0x100000, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x10, 0x20, 0x88, 0x0, 0x82, 0x10, 0x0, 0x13000000, 0x0, 0xa0000000, 0x10000001, 0x100000, 0x2000, 0x8000, 0xa0000000, 0x40, 0x42000200, 0x1200, 0x0, 0x2008000, 0xf000800, 0x4800, 0x8001, 0x88100000, 0xa00000c, 0x1400000, 0x0, 0x2004400c, 0x0, 0x4400000, 0x3600000, 0x0 },
                { 0x5a469570, 0x6386b4d, 0x218731d8, 0x10000004, 0x0, 0x0, 0x60, 0xfffc20, 0x10000, 0x0, 0x0, 0x54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x40000000, 0x10404, 0x8000, 0x0, 0x0, 0x100100, 0x7fc088, 0x0, 0xc020100, 0x400, 0x90000, 0x40100000, 0x88102000, 0xf0a01000, 0x800008, 0x0, 0x180000, 0x1000, 0x600000, 0x402000, 0x0, 0xc528000, 0x1100100, 0x8, 0x9000000, 0x0, 0x4030002, 0x8a660400, 0x0, 0x5200000, 0x0, 0x60008e0, 0x403002, 0x103aa188, 0x32003240, 0x0, 0x0, 0x60000, 0x14000, 0x0, 0x4002200, 0x0, 0x0, 0x40000000, 0x1820059, 0x1800000, 0x580, 0x60, 0x40000001, 0x85010901, 0x2640010, 0x80000404, 0x200020, 0x1045301, 0x21000000, 0x800000, 0x10080980, 0xbc4000, 0x60001, 0x589125b7, 0x10800000, 0x44, 0x80000000, 0x100042, 0x10200004, 0xc1e03810, 0x0, 0x20002d84, 0x280040, 0x40021, 0xd9862280, 0xc8d811c, 0x0 },
                { 0xfffeffff, 0x7efefaff, 0xffffbdfd, 0xf8b80000, 0x60011901, 0x9d1fb, 0x80e801e1, 0xfcffec39, 0xd809ef8b, 0x8ea04090, 0x2e000101, 0x32200fdf, 0xd0040029, 0x5a02d98, 0xd6508008, 0xc0180900, 0x4b018000, 0x414126a4, 0x408c000, 0x650e, 0x98630580, 0xf011f7, 0x307bc08c, 0x40042c6, 0x3d134101, 0x81a00722, 0x22491020, 0x6a161814, 0xad50b013, 0xf0e1940d, 0x85c07b3c, 0x3fffffa1, 0xeda1054, 0xc0963084, 0x1270fa2a, 0xa6238c8, 0x68000c20, 0xf528854, 0x81102100, 0x640540c8, 0xd2922a6, 0x93080002, 0x513e082, 0x8e76a6e7, 0xaab499e0, 0x152b8010, 0xd0008201, 0xf285ce9, 0xb4cd7bab, 0x543aa1a8, 0xfa09b360, 0x8200, 0x5021248, 0x406e194, 0x34200, 0x0, 0x44c2208, 0x20000400, 0x904506, 0x4a66ee37, 0x15eaa55d, 0x29a01045, 0x21c10580, 0x1298046a, 0x6c3e4041, 0xfd414949, 0x3644e79, 0xa41004a4, 0x48200021, 0x11875721, 0x21004d60, 0x14af8001, 0x32e80989, 0x28fc4c36, 0x19a68803, 0x7eb177f7, 0x108c0906, 0x1707c, 0x80100400, 0x554856, 0x172e4b44, 0xe5ee7a50, 0x680604e1, 0x35402fc6, 0x116b3ad2, 0xc1059825, 0xd99f338c, 0xec8db15c, 0x2a500000 },
                { 0xffffffff, 0xdafaffef, 0xffffbdff, 0xd8040204, 0x182008c, 0x12bf1ff, 0x98f80061, 0xfe24c020, 0xf801000b, 0xcfb040c0, 0x20, 0xe54, 0x80000000, 0x0, 0x8060800b, 0x100500, 0x48010000, 0x1040c, 0x88000, 0x120650e, 0x9c630000, 0x1001ff, 0xb871c080, 0x66, 0x2c120101, 0xc1a00400, 0x20091020, 0x42141000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x181044, 0xc3023004, 0x1270003a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x403e082, 0x82460400, 0x49860, 0x5208010, 0xd0008200, 0x62848e9, 0xb4c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8008, 0x10e0080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a551, 0x1800000, 0x1c00580, 0x2980460, 0x401e4041, 0xad410949, 0x2640010, 0x800204a5, 0xa8200021, 0x1845301, 0x21000000, 0xaf8001, 0x12e80980, 0xfc4036, 0x19060003, 0x5eb175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200b44, 0xc1ec1a50, 0x1401, 0x21000084, 0x10280ad0, 0x60020, 0x19922280, 0x48d811c, 0x0 },
                { 0x81000084, 0x9100, 0x84000006, 0x40000004, 0x0, 0x88481, 0xc00000, 0x0, 0x88000000, 0x4000080, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x8010000, 0x0, 0x0, 0x14502, 0x98620000, 0x0, 0x0, 0x46, 0x20000000, 0x0, 0x0, 0x41000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x2922a6, 0x93000000, 0x0, 0x0, 0x49800, 0x0, 0x0, 0x0, 0x820, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0xa66ee00, 0x0, 0x0, 0x1c00000, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2f8000, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x980000, 0x40010901, 0x2000000, 0x0, 0x18, 0x0, 0x0, 0x26000101, 0x12204000, 0x48069, 0x5a02d98, 0x56100000, 0x40000800, 0x1008000, 0x414022a2, 0x84004010, 0x18000000, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x2400000, 0x8020814, 0x25481080, 0x8015, 0x87407b34, 0x3fffffa0, 0x8020010, 0x940000, 0x1c880, 0x20018c8, 0x20000420, 0x23000a54, 0x80e8001f, 0xaf0100f4, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x8, 0x40, 0x8008000, 0x8200, 0x5020240, 0x400c014, 0x0, 0x0, 0xc410b, 0x80000040, 0x5504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100008, 0x40000000, 0x10030022, 0x40005f60, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x1000208, 0x906, 0x14018, 0x40, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x844, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xffcefffd, 0x4abafbef, 0xbfff3dff, 0xd8000004, 0x0, 0xa8081, 0xc00060, 0x424c020, 0x88010000, 0xfa04080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0x100100, 0x1071c080, 0x46, 0x2c020101, 0x200400, 0x20090000, 0x40141000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4030082, 0x82460400, 0x49800, 0x5200010, 0x10000200, 0x60848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900060, 0x40044041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x1060001, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200844, 0xc1e01a50, 0x1, 0x21000084, 0x2800d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x4, 0x8000000, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x10, 0x400000, 0x80400000, 0x0, 0x0, 0x100000, 0x0, 0x1010, 0x100000, 0x6, 0x92505267, 0x28000000, 0x2f000, 0x80007, 0xac000020, 0x20000f, 0xe0000000, 0x0, 0x0, 0x0, 0xb78c00, 0x0, 0x40000, 0x0, 0x10004000, 0x410000, 0x0, 0x30082, 0xc0000050, 0x1000000, 0x13, 0x40000001, 0xc0000230, 0x0, 0x5dfd, 0x80001600, 0x400, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x1400, 0x7001ac, 0x40000000, 0xf9000, 0x17, 0xdf6f2020, 0x0, 0x0, 0x14c00, 0x44000000, 0x0, 0x0, 0x0, 0x1000, 0x5a000040, 0x40000c, 0x680000f8, 0xa090, 0x0, 0x22, 0x0, 0x0, 0x0, 0x0, 0x64800002, 0x5800201f, 0x9e000000, 0x0, 0x0, 0x380006, 0x200000, 0x0, 0x3e000600, 0x0, 0x820, 0xc0008000 },
                { 0x0, 0x100, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0xb, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc1800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x100000, 0x0, 0x0 },
                { 0x0, 0x0, 0x4000, 0x184845, 0x52205402, 0xc0400c00, 0x6007e202, 0x280, 0x6f60020, 0x304d8404, 0x6244588, 0x28580000, 0x7697f86, 0xfa5fd267, 0x29865370, 0x25eae2bb, 0xb47a775f, 0xbeb85943, 0xbbc30ee9, 0x58db1a71, 0x639c7800, 0xee00, 0x44002b63, 0x84000019, 0xc02816f2, 0x2648a00d, 0x849646c9, 0xb4e9e6ab, 0x528e43e0, 0x30e6800, 0x50070483, 0xc000005e, 0x1000080, 0x1000610, 0x8000040, 0x35808230, 0x0, 0x10800002, 0x1e000800, 0x80410, 0x252dd59, 0x6cf50000, 0xba0c1b5d, 0x61865800, 0x36000, 0x8ad0016c, 0x20000461, 0x568002, 0x308054, 0x20400a11, 0x4064c14, 0x807de5, 0xfa3d0095, 0xa0103424, 0xb035ac, 0xc6200000, 0xf00473, 0xd4000290, 0x1000, 0x95991188, 0x8a115a20, 0x2180000, 0xa364250, 0x8d61a8e5, 0x8000afaa, 0x10b4f2a2, 0x4139100, 0x5a000041, 0x410446, 0xe8208000, 0x80e4a200, 0x8504000, 0x110a780, 0x84021208, 0xa20860fc, 0x80068800, 0xa0720078, 0x6080a002, 0x5fef8138, 0x83aa, 0x3041a013, 0xd0108520, 0x90382006, 0xa06d0b5, 0xaeb88420, 0x3e0007c2, 0x3ba2ec01, 0x49fc818, 0xc1280700 },
                { 0x0, 0x100, 0x0, 0x4, 0x0, 0x0, 0x1, 0x38000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x220000, 0xa5, 0x28000000, 0x2, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x93000000, 0x6000, 0x0, 0x49800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe00, 0x100, 0x0, 0x1c00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x200000, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x10900, 0x400000, 0x0, 0x19, 0x0, 0x0, 0x4000001, 0x12000000, 0x4002f, 0x93515ff7, 0x3a100000, 0x40000800, 0x1008007, 0xbd402280, 0x4004001, 0x0, 0x480, 0x1000, 0x4, 0xf78e80, 0x4000, 0x40320, 0x2400000, 0x8020814, 0x25001002, 0x8005, 0x854343b6, 0xffffe354, 0x9020010, 0x940000, 0x4800, 0x20002b0, 0xa0000420, 0x3000000, 0x8000, 0x240100c0, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x3a300080, 0x10090000, 0x0, 0x1400, 0x8, 0x0, 0x8088000, 0x410c224, 0xff3f1240, 0x400c004, 0x1ac, 0x40000000, 0x0, 0x10, 0x4524, 0x37, 0x280004, 0x20215c55, 0x64010000, 0x1000000a, 0x2c200000, 0x10000000, 0x5e68, 0x7e100000, 0x40000000, 0x78030000, 0xcd20, 0x14000000, 0x2b, 0x28000c00, 0x808800, 0x200, 0x906, 0x64814018, 0x58000000, 0x404000, 0x70e0000, 0x24000000, 0x83e0062, 0x14400042, 0x33000, 0xff010604, 0x1100c, 0x20003840, 0xea500000 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x5a469570, 0x22386b4d, 0x218731d8, 0x11000004, 0x0, 0x0, 0x1f0, 0x3dcc20, 0x10000, 0x0, 0x40000000, 0x54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x40000000, 0x10404, 0x8000, 0x0, 0x0, 0xf80100, 0x71c080, 0x40000000, 0xc020100, 0x400, 0x90000, 0x40100000, 0x88102000, 0xf0a01000, 0x800008, 0x0, 0x180000, 0x1000, 0x600000, 0x402000, 0x0, 0xc528000, 0x1100100, 0x8, 0x9000000, 0x0, 0x5030002, 0x82460400, 0x0, 0x5200000, 0x10, 0x60008e0, 0x403002, 0x103aa188, 0x32003240, 0x0, 0x0, 0x60000, 0x14000, 0x0, 0x4002200, 0x0, 0x0, 0x40000000, 0x5c20051, 0x1800000, 0x580, 0x60, 0x40000001, 0x85010901, 0x2640011, 0x80000404, 0x200020, 0x1045301, 0x21000000, 0x800000, 0x30080980, 0xbc4000, 0x60001, 0x789125b7, 0x10840000, 0x4, 0x80000000, 0x100012, 0x10200004, 0xc1e21810, 0x0, 0x20000284, 0x1280040, 0x40020, 0x198a2280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x10100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091004, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x80000000, 0x0, 0x40304, 0x3005008, 0x1640000, 0x1f200, 0x0, 0x700004, 0xd8440, 0x10000, 0x20000000, 0x1697f86, 0xf25fd267, 0x28205343, 0x2e43a, 0x683757, 0xbea00009, 0x3bc00eef, 0xf9240000, 0x781e, 0xfa000000, 0x0, 0x4ffbe00, 0xc40072, 0x8087004, 0x84800600, 0x1081428a, 0x60380, 0x8e0, 0x50070483, 0xc000005e, 0xb121ea80, 0x3f480030, 0x80e0040, 0x40008237, 0x83800000, 0x215dfd, 0x7e07c8e0, 0x81c13, 0xb4800000, 0xf58fd8, 0x80e0001c, 0x1001, 0x1040069d, 0x40103404, 0x20007001, 0x3085a016, 0x8008400, 0x29044615, 0x8c00013, 0xe8fff527, 0xfafd8805, 0x99481463, 0xffb0bdbc, 0x7e1fbfff, 0xfa0b90f0, 0x53ffe29f, 0xdf6f3a79, 0x2000018c, 0x800c0200, 0x5ef1c, 0x54082859, 0x88059081, 0xe10020, 0x2900084, 0x18099586, 0x5be3fb53, 0xa44c0784, 0xee0800fc, 0x9ee7a297, 0x43004b1a, 0xcc100026, 0x31011300, 0xc2000000, 0x80000, 0x800376f9, 0xe4840002, 0x7f6ea13f, 0xffaab000, 0x4081b000, 0x20, 0x16b81306, 0x2be0008, 0x68000101, 0x3efa0798, 0x200072, 0x10000ea3, 0xd5289f00 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xffffffff, 0xdafaffef, 0xffffbdff, 0xd8040204, 0x182008c, 0x12bf1ff, 0x98f80061, 0xfe24c020, 0xf801000b, 0xcfb040c0, 0x20, 0xe54, 0x80000000, 0x0, 0x8060800b, 0x100500, 0x48010000, 0x1040c, 0x88000, 0x120650e, 0x9c630000, 0x1001ff, 0xb871c080, 0x66, 0x2c120101, 0xc1a00400, 0x20091020, 0x42141000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x181044, 0xc3023004, 0x1270003a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x403e082, 0x82460400, 0x49860, 0x5208010, 0xd0008200, 0x62848e9, 0xb4c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8008, 0x10e0080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a551, 0x1800000, 0x1c00580, 0x2980460, 0x401e4041, 0xad410949, 0x2640010, 0x800204a5, 0xa8200021, 0x1845301, 0x21000000, 0xaf8001, 0x12e80980, 0xfc4036, 0x19060003, 0x5eb175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200b44, 0xc1ec1a50, 0x1401, 0x21000084, 0x10280ad0, 0x60020, 0x19922280, 0x48d811c, 0x0 },
                { 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x400000, 0x2200, 0x0, 0x1000, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x1800, 0x0, 0x0, 0x1000000, 0x2042, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x110000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x800000, 0x0, 0x10, 0x0, 0x88, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x1, 0x100000, 0x0, 0x8000, 0x0, 0x0, 0x42000200, 0x0, 0x0, 0x0, 0x800, 0x80000, 0x8000, 0x0, 0xa000000, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xa54, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x4000, 0x800, 0x2004000, 0x0, 0x0, 0x0, 0x0, 0x20004, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x110000, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80060, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630000, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80060, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630000, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x4a84a09, 0x488201a0, 0x18680c20, 0x88000004, 0x0, 0x17000, 0x0, 0x4000000, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x80000, 0x2000, 0x10000, 0x0, 0x10000000, 0x0, 0x1, 0x200000, 0x20000000, 0x2000000, 0x8000, 0x400, 0x0, 0x0, 0x44, 0x4, 0x1200002a, 0x220000, 0x8000000, 0x8000, 0x2000, 0x44008, 0x1000000, 0x0, 0x80, 0x0, 0x0, 0x10, 0x10000200, 0x84001, 0x80054381, 0x44002020, 0xe2000020, 0x0, 0x8, 0x80, 0x200, 0x0, 0x2000, 0x20000400, 0x0, 0x40000000, 0x1000a400, 0x0, 0x0, 0x2900000, 0x44040, 0x20400849, 0x0, 0xa0, 0x1, 0x800100, 0x0, 0x1, 0x2c00000, 0x16, 0x1000000, 0x6005010, 0x0, 0x0, 0x100400, 0x800, 0x840, 0x240, 0x1, 0x1000000, 0x90, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x844, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xfffeffff, 0x6efafbff, 0xbfff3dff, 0xf8000004, 0x0, 0xbf1ff, 0x80f801f1, 0xfcfffc20, 0xf801efcb, 0x8fa04080, 0x0, 0xddf, 0xf0000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x9c630000, 0xf801ff, 0xb87fc088, 0x66, 0x2c130101, 0xc1a00400, 0x20091000, 0x42141000, 0x8810a011, 0xf0f11400, 0x800008, 0x0, 0x181044, 0x3084, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x503e082, 0x8a660400, 0x49860, 0x5208010, 0xd0008200, 0x62848e9, 0x80cd7ba3, 0x543aa1a8, 0xf2013360, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a559, 0x9800000, 0x1c00580, 0x2980460, 0x401e4041, 0xed410949, 0x3640011, 0x800004a4, 0x260021, 0x1845701, 0x21000000, 0xaf8001, 0x32e80980, 0xfc4036, 0x19260003, 0x7e9175f7, 0x108c0000, 0x44, 0x80100400, 0x110856, 0x10200b44, 0xc1ee3a50, 0x60000001, 0x21002f84, 0x116808d2, 0x41021, 0xd99e2280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5a02d98, 0x56100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x85407b34, 0x3fffffa0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x40100c0, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4d20, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x85994a8d, 0xd88295a0, 0x9e688c26, 0xc8040204, 0x102008c, 0x12881ff, 0x98e40001, 0x3e000000, 0xc8000000, 0xfb040c0, 0x0, 0xa00, 0x0, 0x0, 0x20800b, 0x400, 0x8010000, 0x8, 0x80000, 0x120450e, 0x9f630000, 0xa5, 0x38000000, 0x66, 0x20000001, 0x200000, 0x20001020, 0x41000, 0x8010, 0x400, 0x0, 0x0, 0x1044, 0xc3000004, 0x210002a, 0x8220000, 0x8000000, 0x8000, 0x2000, 0x44008, 0x12922a7, 0x93000000, 0x6080, 0x0, 0x49800, 0x10, 0x10000200, 0x84009, 0xb0044ba1, 0x44002020, 0xe2000020, 0x0, 0x8008, 0x1080080, 0x20200, 0x0, 0x2000, 0x20000400, 0x0, 0x4a66ee00, 0x1000a500, 0x0, 0x1c00000, 0x2900000, 0x44040, 0x28400849, 0x0, 0x200a1, 0xa0000001, 0x800100, 0x0, 0x2f8001, 0x2e00000, 0x400016, 0x1000000, 0x6005010, 0x0, 0x0, 0x100400, 0x10800, 0x840, 0x240, 0x1001, 0x1000000, 0x90, 0x20000, 0x0, 0x0, 0x0 },
                { 0x5eceff7b, 0x4aba6bef, 0x39ff3df8, 0x98000004, 0x0, 0x0, 0xf80060, 0x8424c020, 0x88010000, 0xaa04000, 0x0, 0x454, 0x80000000, 0x0, 0x80400000, 0x100000, 0x40000000, 0x10404, 0x88000, 0x0, 0xc630000, 0x100158, 0x1071c080, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40100000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x1004, 0x260002a, 0x622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x9000000, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10000200, 0x62848e1, 0x80447383, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x14200, 0x0, 0x4002200, 0x20000400, 0x0, 0x40000e00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900060, 0x401c4041, 0xa5410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0x808001, 0x12c80980, 0xfc4016, 0x1060001, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x100802, 0x10200844, 0xc1e01a50, 0x1, 0x21000084, 0x2800d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x920, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x800200, 0x10901, 0x400000, 0x2200, 0x0, 0x0, 0x0, 0x20000001, 0x12200000, 0x40006, 0x92505267, 0x28000000, 0x40000000, 0x1008007, 0xad402080, 0x4004001, 0x0, 0x880, 0x0, 0x0, 0xb78c00, 0x40042, 0x40000, 0x2000000, 0x20814, 0x21001000, 0x4, 0x30082, 0xc0000050, 0x1000010, 0x840000, 0x8800, 0x2000230, 0x20000400, 0x0, 0x0, 0x20010000, 0x4800000, 0x2, 0x0, 0x0, 0x10000002, 0x0, 0x0, 0x0, 0x8, 0x0, 0x8000000, 0x104024, 0xfa3d0000, 0x10, 0x1ac, 0x40000000, 0x8, 0x12, 0x410024, 0x3c, 0x0, 0x14c00, 0x44008000, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x68000000, 0x8000, 0x14000000, 0x22, 0x200, 0x0, 0x0, 0x0, 0x64800000, 0x58000000, 0x4000, 0x0, 0x0, 0x380002, 0x0, 0x0, 0x3e000600, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0xc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x92505227, 0x28000000, 0x0, 0x5, 0x8c000000, 0x1, 0x0, 0x0, 0x0, 0x0, 0xb28c00, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30080, 0x40000000, 0x1000000, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0x7a3d0000, 0x0, 0x20, 0x40000000, 0x0, 0x10, 0x20, 0x0, 0x0, 0x10400, 0x4000000, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x68000000, 0x8000, 0x0, 0x22, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x58000000, 0x0, 0x0, 0x0, 0x380002, 0x0, 0x0, 0x3e000600, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x40000, 0x100, 0x4000000, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x40000, 0x1000, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120109, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200012, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x42900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0xc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x85994a8d, 0xd88295a0, 0x9e688c26, 0xc8040204, 0x102008c, 0x12881ff, 0x98e00001, 0x3e000000, 0xc8000000, 0xfb040c0, 0x0, 0xa00, 0x0, 0x0, 0x20800b, 0x400, 0x8010000, 0x8, 0x80000, 0x120450e, 0x9c630000, 0xa5, 0x38000000, 0x66, 0x20000001, 0x200000, 0x20001020, 0x41000, 0x8010, 0x400, 0x0, 0x0, 0x1044, 0xc3000004, 0x210002a, 0x8220000, 0x8000000, 0x8000, 0x2000, 0x44008, 0x12922a6, 0x93000000, 0x6080, 0x0, 0x49800, 0x10, 0x10000200, 0x84009, 0xb0044ba1, 0x44002020, 0xe2000020, 0x0, 0x8008, 0x1080080, 0x20200, 0x0, 0x2000, 0x20000400, 0x0, 0x4a66ee00, 0x1000a500, 0x0, 0x1c00000, 0x2900000, 0x44040, 0x28400849, 0x0, 0x200a1, 0xa0000001, 0x800100, 0x0, 0x2f8001, 0x2e00000, 0x400016, 0x1000000, 0x6005010, 0x0, 0x0, 0x100400, 0x10800, 0x840, 0x240, 0x1001, 0x1000000, 0x90, 0x20000, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x1000000, 0x1402080, 0x4004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x20014, 0x21001000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x8, 0x0, 0x0, 0x18, 0x3, 0x400000, 0xc, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x1000000, 0x1402080, 0x4004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x20014, 0x21001000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x8, 0x0, 0x0, 0x18, 0x3, 0x400000, 0xc, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] sigmap1 = null;

        protected static void sigmap1Init()
        {
            sigmap1 = new int[][]
            {
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x1000000, 0x1402080, 0x4004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x20014, 0x21001000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x8, 0x0, 0x0, 0x18, 0x3, 0x400000, 0xc, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x21, 0x910, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x40300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404334, 0x38000300, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0xb257d267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x70083, 0xc000005e, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80060, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630000, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xfdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb801000b, 0xc4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x10100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0xc1a00400, 0x20091004, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x23004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5208010, 0xd0008200, 0x62848e9, 0x84c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2980460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x8200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4036, 0x19060003, 0x5eb175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200b44, 0xc1ec1a50, 0x401, 0x21000084, 0x10280ad0, 0x40020, 0x19922280, 0x48d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x200000, 0xa8081, 0xdc01e0, 0x4fffd24, 0xbc010000, 0x4004080, 0x8, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x99630000, 0xfc0100, 0x107fd088, 0x56, 0x2c120101, 0x200480, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x45030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x40021, 0xd9862280, 0xc8d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x40021, 0xd9862280, 0xc8d811c, 0x0 },
                { 0xffceffff, 0x4efafbff, 0xbdff3dff, 0xf8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc24, 0xb801efc0, 0x4004080, 0x0, 0xddf, 0xf8000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c138101, 0x200480, 0x20090000, 0x40141000, 0x8810a009, 0xf4f11400, 0x800008, 0x0, 0x180044, 0x3084, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x25200010, 0x10008200, 0x62848e1, 0x80cc7ba3, 0x543aa1a8, 0xf2213360, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x9800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x3640010, 0x800004a4, 0x260021, 0x1845701, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19260003, 0x5e9175f7, 0x10880000, 0x44, 0x80100400, 0x110846, 0x10200944, 0xc1ec3a50, 0x60000001, 0x21002d84, 0x6808d2, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xa54, 0x80000000, 0x40, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc4000, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x920, 0x14000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x60000, 0x300, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5a02d98, 0x56100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x85407b34, 0x3fffffa0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x40100c0, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x904504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4d20, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x100, 0x0, 0x20000004, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1df, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0xf0a01000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3240, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x2600000, 0x0, 0x0, 0x1400, 0x0, 0x0, 0x100, 0x0, 0x0, 0xc0, 0x880000, 0x0, 0x0, 0x6, 0x10000000, 0x80000000, 0x60000000, 0x84, 0x680002, 0x0, 0x11022000, 0x4890018, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0xf257d267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x70483, 0xc000005e, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x4000, 0x800, 0x2004000, 0x0, 0x0, 0x0, 0x0, 0x20004, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x110000, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x100, 0x0, 0x20000004, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1df, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0xf0f11000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x13340, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x3600000, 0x0, 0x0, 0x1400, 0x0, 0x0, 0x100, 0x0, 0x0, 0xc0, 0x880000, 0x0, 0x0, 0x6, 0x10000000, 0x80000000, 0x60000000, 0x84, 0x680002, 0x0, 0x11022000, 0x4890018, 0x0 },
                { 0x85994a8d, 0xd88295a0, 0x9e688c26, 0xc8040204, 0x102008c, 0x12881ff, 0x98e00001, 0x3e000000, 0xc8000000, 0xfb040c0, 0x0, 0xa00, 0x0, 0x0, 0x20800b, 0x400, 0x8010000, 0x8, 0x80000, 0x120450e, 0x9c630000, 0xa5, 0x38000000, 0x66, 0x20000001, 0x200000, 0x20001020, 0x41000, 0x8010, 0x400, 0x0, 0x0, 0x1044, 0xc3000004, 0x210002a, 0x8220000, 0x8000000, 0x8000, 0x2000, 0x44008, 0x12922a6, 0x93000000, 0x6080, 0x0, 0x49800, 0x10, 0x10000200, 0x84009, 0xb0044ba1, 0x44002020, 0xe2000020, 0x0, 0x8008, 0x1080080, 0x20200, 0x0, 0x2000, 0x20000400, 0x0, 0x4a66ee00, 0x1000a500, 0x0, 0x1c00000, 0x2900000, 0x44040, 0x28400849, 0x0, 0x200a1, 0xa0000001, 0x800100, 0x0, 0x2f8001, 0x2e00000, 0x400016, 0x1000000, 0x6005010, 0x0, 0x0, 0x100400, 0x10800, 0x840, 0x240, 0x1001, 0x1000000, 0x90, 0x20000, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x4a84a09, 0x488201a0, 0x18680c20, 0x88000004, 0x0, 0x17000, 0x0, 0x4000000, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x80000, 0x2000, 0x10000, 0x0, 0x10000000, 0x0, 0x1, 0x200000, 0x20000000, 0x2000000, 0x8000, 0x400, 0x0, 0x0, 0x44, 0x4, 0x1200002a, 0x220000, 0x8000000, 0x8000, 0x2000, 0x44008, 0x1000000, 0x0, 0x80, 0x0, 0x0, 0x10, 0x10000200, 0x84001, 0x80054381, 0x44002020, 0xe2000020, 0x0, 0x8, 0x80, 0x200, 0x0, 0x2000, 0x20000400, 0x0, 0x40000000, 0x1000a400, 0x0, 0x0, 0x2900000, 0x44040, 0x20400849, 0x0, 0xa0, 0x1, 0x800100, 0x0, 0x1, 0x2c00000, 0x16, 0x1000000, 0x6005010, 0x0, 0x0, 0x100400, 0x800, 0x840, 0x240, 0x1, 0x1000000, 0x90, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0xc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x81000000, 0x4000, 0x4043300, 0x830c4008, 0x89640000, 0x1400, 0x1, 0x1004, 0x865, 0x190206, 0x110020, 0x100006, 0xf25fd267, 0x28200483, 0x8000404, 0x820, 0x28, 0x40101, 0x1240000, 0xbe, 0xfa001000, 0x0, 0xf78e00, 0x6004, 0x8001820, 0x102, 0x100, 0x400002, 0x8000000, 0x40744a3, 0xc000005e, 0x1000010, 0x3010000, 0x4000000, 0x40080, 0x0, 0x5dfd, 0x80000000, 0x10080, 0x0, 0x180a, 0x400000, 0x25, 0x3a400000, 0x10013401, 0x100, 0x0, 0x2000000, 0x0, 0x80000, 0x18c035, 0xfa3d9200, 0x5280000, 0x0, 0x1000000, 0xf9000, 0x1010, 0x500, 0x23, 0x200000, 0x16c10, 0x44000004, 0x30040000, 0x4, 0x80010, 0x5680, 0x7e020001, 0xa6000800, 0x100000f8, 0x8c000, 0x1064, 0xc0000022, 0x101, 0x0, 0x0, 0x1000, 0x60800000, 0x2000, 0x0, 0x1021000, 0x0, 0x381200, 0x0, 0x0, 0x3f020000, 0x0, 0x800, 0xc0000000 },
                { 0x85994a8d, 0xd88295a0, 0x9e688c26, 0xc8040204, 0x102008c, 0x12881ff, 0x98e00001, 0x3e000000, 0xcc000000, 0xfb040c0, 0x0, 0xa00, 0x0, 0x0, 0x20800b, 0x400, 0x8010000, 0x8, 0x80000, 0x120450e, 0x9c630000, 0xa5, 0x78000000, 0x76, 0x20000001, 0x200000, 0x20001020, 0x41000, 0x8010, 0x400, 0x0, 0x0, 0x1044, 0xc3000004, 0x210002a, 0x8220000, 0x8000000, 0x8000, 0x2000, 0x44008, 0x12922a6, 0x93000000, 0x7080, 0x0, 0x49800, 0x10, 0x10000200, 0x84009, 0xb0044ba1, 0x44002020, 0xe2000020, 0x0, 0x8008, 0x1080080, 0x20200, 0x0, 0x2000, 0x20000400, 0x0, 0x4a66ee00, 0x1000a500, 0x0, 0x1c00000, 0x2900000, 0x44040, 0x28400849, 0x0, 0x200a1, 0xa0000001, 0x800100, 0x0, 0x2f8001, 0x2e00000, 0x400016, 0x1000000, 0x6005010, 0x0, 0x0, 0x100400, 0x10800, 0x840, 0x240, 0x1001, 0x1000000, 0x90, 0x20000, 0x0, 0x0, 0x0 },
                { 0xfffeffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d801e0, 0xc4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630000, 0xf8015a, 0x907fc088, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5038082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x40021, 0xd9862280, 0xc8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x400000, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x2f, 0xffdfffff, 0x7a100000, 0x40000800, 0x0, 0x200, 0x1, 0x0, 0x480, 0x1000, 0x7, 0x4280, 0x0, 0x40300, 0x500000, 0x8000000, 0x4000000, 0x8000, 0x85577fb7, 0xfffffffe, 0x89260010, 0xd40000, 0x20004000, 0x80, 0x20, 0x3080000, 0x0, 0x4010080, 0x4000000, 0x40802, 0x500000, 0x418a2e7, 0xa8f80080, 0x90000, 0x40, 0x1600, 0x20000, 0x0, 0x8008000, 0x218e235, 0xffbf4240, 0x480c004, 0x80000000, 0x8, 0x3000000, 0x8010, 0x4534, 0x37, 0x2c0006, 0xb4303045, 0x38030040, 0x1000000a, 0x2c201000, 0x10000000, 0x5eec, 0x7f540200, 0x44000000, 0x10030000, 0xc404, 0x14000220, 0x404420d, 0xbd020c00, 0xd0c810, 0x200, 0x39d6, 0x64834018, 0x8000, 0x20484000, 0x67ce0021, 0x34000000, 0xebe0870, 0x14500063, 0x13b001, 0xffc1405c, 0x2201986e, 0x30123ec3, 0xead76000 },
                { 0xffffffff, 0xdafaffef, 0xffffbdff, 0xd8040204, 0x182008c, 0x12bf1ff, 0x98f80061, 0xfe24c020, 0xf801000b, 0xcfb040c0, 0x20, 0xe54, 0x80000000, 0x0, 0x8060800b, 0x100500, 0x48010000, 0x1040c, 0x88000, 0x120650e, 0x9c630000, 0x1001ff, 0xb871c080, 0x66, 0x2c120101, 0xc1a00400, 0x20091020, 0x42141000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x181044, 0xc3023004, 0x1270003a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x403e082, 0x82460400, 0x49860, 0x5208010, 0xd0008200, 0x62848e9, 0xb4c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8008, 0x10e0080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a551, 0x1800000, 0x1c00580, 0x2980460, 0x401e4041, 0xad410949, 0x2640010, 0x800204a5, 0xa8200021, 0x1845301, 0x21000000, 0xaf8001, 0x12e80980, 0xfc4036, 0x19060003, 0x5eb175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200b44, 0xc1ec1a50, 0x1401, 0x21000084, 0x10280ad0, 0x60020, 0x19922280, 0x48d811c, 0x0 },
                { 0x85994a8d, 0xd88295a0, 0x9e688c26, 0xc8040204, 0x102008c, 0x12881ff, 0x98e00001, 0x3e000000, 0xc8000000, 0xfb040c0, 0x0, 0xa00, 0x0, 0x0, 0x20800b, 0x400, 0x8010000, 0x8, 0x80000, 0x120450e, 0x9c630000, 0xa5, 0x38000000, 0x66, 0x20000001, 0x200000, 0x20001020, 0x41000, 0x8010, 0x400, 0x0, 0x0, 0x1044, 0xc3000004, 0x210002a, 0x8220000, 0x8000000, 0x8000, 0x2000, 0x44008, 0x12922a6, 0x93000000, 0x6080, 0x0, 0x49800, 0x10, 0x10000200, 0x84009, 0xb0044ba1, 0x44002020, 0xe2000020, 0x0, 0x8008, 0x1080080, 0x20200, 0x0, 0x2000, 0x20000400, 0x0, 0x4a66ee00, 0x1000a500, 0x0, 0x1c00000, 0x2900000, 0x44040, 0x28400849, 0x0, 0x200a1, 0xa0000001, 0x800100, 0x0, 0x2f8001, 0x2e00000, 0x400016, 0x1000000, 0x6005010, 0x0, 0x0, 0x100400, 0x10800, 0x840, 0x240, 0x1001, 0x1000000, 0x90, 0x20000, 0x0, 0x0, 0x0 },
                { 0x5a469570, 0x6386b4d, 0x218731d8, 0x10000004, 0x0, 0x0, 0x60, 0xfdfc20, 0x10000, 0x0, 0x0, 0x54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x40000000, 0x10404, 0x8000, 0x0, 0x0, 0x100100, 0x73c080, 0x0, 0xc020100, 0x400, 0x90000, 0x40100000, 0x88102000, 0xf0a01000, 0x800008, 0x0, 0x180000, 0x1000, 0x600000, 0x402000, 0x0, 0xc528000, 0x1100100, 0x8, 0x9000000, 0x0, 0x4030002, 0x82460400, 0x0, 0x5200000, 0x0, 0x60008e0, 0x403002, 0x103aa188, 0x32003240, 0x0, 0x0, 0x60000, 0x14000, 0x0, 0x4002200, 0x0, 0x0, 0x40000000, 0x1820051, 0x1800000, 0x580, 0x60, 0x40000001, 0x85010901, 0x2640010, 0x80000404, 0x200020, 0x1045301, 0x21000000, 0x800000, 0x10080980, 0xbc4000, 0x60001, 0x589125b7, 0x10800000, 0x44, 0x80000000, 0x100042, 0x10200004, 0xc1e03810, 0x0, 0x20002d84, 0x280040, 0x40021, 0xd9862280, 0xc8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x4, 0x0, 0x400000, 0x0, 0x1, 0x500000, 0x0, 0x0, 0x0, 0x97786, 0xf25fd267, 0x38000000, 0x4000000, 0x7, 0xbe000020, 0x9, 0x40000000, 0x0, 0x0, 0x0, 0xffbe00, 0xc40000, 0x4020, 0x0, 0x0, 0x400000, 0xe0, 0x55371da3, 0xd9ffed7e, 0x125ea00, 0x3c684020, 0x200e0000, 0x400002b3, 0x83800000, 0x2d7dfd, 0x6003c0e0, 0xf01400, 0x4800000, 0xeff0, 0xe00000, 0x80001, 0x3048061d, 0x40000000, 0xfff7000, 0x30812214, 0x8020400, 0x9055406, 0x8c00083, 0xeeffc12f, 0xfabd4802, 0x1ac00043, 0xff0089bc, 0x781fffff, 0xf90b9000, 0x3ffe918, 0x79, 0x20000044, 0x600c0002, 0x9445ef1c, 0x5408280b, 0x1000, 0xc11000, 0x2020004, 0x18899584, 0x5a05fb12, 0x5800380, 0x6e0800fc, 0x1e03a00b, 0x3002f9a, 0xc044026, 0x31012040, 0x40510000, 0x0, 0x37681, 0xe48e0000, 0x78003018, 0x61aa3000, 0x408000a0, 0x10080, 0x16b80a1a, 0x890008, 0x40000001, 0x3e386608, 0x12, 0x10000e83, 0xc0876000 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x5a469570, 0x6386b4d, 0x218731d8, 0x10000004, 0x0, 0x0, 0x60, 0xfdfc20, 0x10000, 0x0, 0x0, 0x54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x40000000, 0x10404, 0x8000, 0x0, 0x0, 0x100100, 0x73c080, 0x0, 0xc020100, 0x400, 0x90000, 0x40100000, 0x88102000, 0xf0a01000, 0x800008, 0x0, 0x180000, 0x1000, 0x600000, 0x402000, 0x0, 0xc528000, 0x1100100, 0x8, 0x9000000, 0x0, 0x4030002, 0x82460400, 0x0, 0x5200000, 0x0, 0x60008e0, 0x403002, 0x103aa188, 0x32003240, 0x0, 0x0, 0x60000, 0x14000, 0x0, 0x4002200, 0x0, 0x0, 0x40000000, 0x1820051, 0x1800000, 0x580, 0x60, 0x40000001, 0x85010901, 0x2640010, 0x80000404, 0x200020, 0x1045301, 0x21000000, 0x800000, 0x10080980, 0xbc4000, 0x60001, 0x589125b7, 0x10800000, 0x44, 0x80000000, 0x100042, 0x10200004, 0xc1e03810, 0x0, 0x20002d84, 0x280040, 0x40021, 0xd9862280, 0xc8d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0xc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80860, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630040, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x5eceff79, 0x4afa6bef, 0x39ff3df8, 0x98000004, 0x0, 0x0, 0xc00060, 0x4424c020, 0xf8010000, 0xaa04000, 0x0, 0x454, 0x80000000, 0x0, 0x80400000, 0x100000, 0x40000000, 0x10404, 0x88000, 0x0, 0x8630000, 0x100142, 0x9071c080, 0x66, 0x2c020101, 0x200400, 0x20090000, 0x40100000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x1004, 0x260002a, 0x622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x9000000, 0x93000000, 0x4038082, 0x82460400, 0x49800, 0x5200010, 0x10008200, 0x60848e1, 0x80447383, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x14200, 0x0, 0x4002200, 0x20000400, 0x0, 0x40000e00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x40064041, 0xa5410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0x808001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x100802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x920, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x5a469570, 0x22386b4d, 0x218731d8, 0x10000004, 0x0, 0x0, 0x1f0, 0x3dcc20, 0x10000, 0x0, 0x0, 0x54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x40000000, 0x10404, 0x8000, 0x0, 0x0, 0xf80100, 0x71c080, 0x0, 0xc020100, 0x400, 0x90000, 0x40100000, 0x88102000, 0xf0a01000, 0x800008, 0x0, 0x180000, 0x1000, 0x600000, 0x402000, 0x0, 0xc528000, 0x1100100, 0x8, 0x9000000, 0x0, 0x5030002, 0x82460400, 0x0, 0x5200000, 0x0, 0x60008e0, 0x403002, 0x103aa188, 0x32003240, 0x0, 0x0, 0x60000, 0x14000, 0x0, 0x4002200, 0x0, 0x0, 0x40000000, 0x5c20051, 0x1800000, 0x580, 0x60, 0x40000001, 0x85010901, 0x2640011, 0x80000404, 0x200020, 0x1045301, 0x21000000, 0x800000, 0x30080980, 0xbc4000, 0x60001, 0x789125b7, 0x10840000, 0x4, 0x80000000, 0x100012, 0x10200004, 0xc1e21810, 0x0, 0x20000284, 0x1280040, 0x40020, 0x198a2280, 0x48d811c, 0x0 },
                { 0xfffeffff, 0x4afafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d80060, 0xc424c020, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630000, 0x10015a, 0x9071c080, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x4038082, 0x82460400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a451, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x12c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200944, 0xc1ec1a50, 0x1, 0x21000084, 0x2808d0, 0x40020, 0x19822280, 0x48d811c, 0x0 },
                { 0xfffeffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf3ff, 0x80d801e0, 0xc4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0xe58e, 0x98630000, 0xf8015a, 0x907fc088, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92d22a6, 0x93000000, 0x5038082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x40021, 0xd9862280, 0xc8d811c, 0x0 },
                { 0xffffffff, 0xdafaffef, 0xffffbdff, 0xd8040204, 0x182008c, 0x12bf1ff, 0x98f80061, 0xfe24c020, 0xf801000b, 0xcfb040c0, 0x20, 0xe54, 0x80000000, 0x0, 0x8060800b, 0x100500, 0x48010000, 0x1040c, 0x88000, 0x120650e, 0x9c630000, 0x1001ff, 0xb871c080, 0x66, 0x2c120101, 0xc1a00400, 0x20091020, 0x42141000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x181044, 0xc3023004, 0x1270003a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x403e082, 0x82460400, 0x49860, 0x5208010, 0xd0008200, 0x62848e9, 0xb4c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8008, 0x10e0080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a551, 0x1800000, 0x1c00580, 0x2980460, 0x401e4041, 0xad410949, 0x2640010, 0x800204a5, 0xa8200021, 0x1845301, 0x21000000, 0xaf8001, 0x12e80980, 0xfc4036, 0x19060003, 0x5eb175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200b44, 0xc1ec1a50, 0x1401, 0x21000084, 0x10280ad0, 0x60020, 0x19922280, 0x48d811c, 0x0 },
                { 0xffceffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xa8081, 0xd801e0, 0x4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80400000, 0x100000, 0x48010000, 0x10404, 0x88000, 0x4502, 0x98630000, 0xf80100, 0x107fc088, 0x46, 0x2c120101, 0x200400, 0x20090000, 0x40141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x180044, 0x3004, 0x270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5030082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e1, 0x80c47ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60180, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xed410949, 0x2640010, 0x800004a4, 0x260021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x41021, 0xd9862280, 0x8c8d811c, 0x0 },
                { 0x4984a09, 0x488201a0, 0x18680c20, 0x88000004, 0x0, 0x17e, 0x80c00000, 0x4000000, 0x88000000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x80000, 0xc, 0x8630000, 0x0, 0x10000000, 0x46, 0x20000001, 0x200000, 0x20001000, 0x0, 0x8000, 0x400, 0x0, 0x0, 0x1044, 0x4, 0x200002a, 0x220000, 0x8000000, 0x8000, 0x2000, 0x44008, 0x1000000, 0x93000000, 0x80, 0x0, 0x49800, 0x10, 0x10000200, 0x84009, 0x80044381, 0x44002020, 0xe2000020, 0x0, 0x8, 0x80, 0x200, 0x0, 0x2000, 0x20000400, 0x0, 0x40000e00, 0x1000a400, 0x0, 0x1c00000, 0x2900000, 0x44040, 0x20400849, 0x0, 0xa0, 0x1, 0x800100, 0x0, 0x8001, 0x2c00000, 0x400016, 0x1000000, 0x6005010, 0x0, 0x0, 0x100400, 0x800, 0x840, 0x240, 0x1, 0x1000000, 0x90, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x101000, 0x20, 0x0, 0x20, 0x4004, 0x10000000, 0x0, 0x0, 0x1, 0x0, 0x9, 0x0, 0x0, 0x0, 0x0, 0x1820000, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x20000000, 0x0, 0xc000, 0xc000800, 0x0, 0x400, 0x0, 0x0, 0x20000000, 0x200, 0x60000000, 0x0, 0x0, 0x10, 0x5000000, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x1000000, 0x30904024, 0x800, 0x8010000, 0x20, 0x0, 0x0, 0x10, 0xa20, 0x0, 0x0, 0x10400, 0x1000, 0x0, 0x3000000, 0x0, 0xe0011100, 0x2012000, 0x100000, 0x48002004, 0xa000, 0x20000000, 0x72, 0x42008000, 0x0, 0x0, 0x3000000, 0x4740800, 0x4000, 0x1800000, 0x0, 0xa00000c, 0x10380002, 0x0, 0xc, 0x380000, 0x4400000, 0x800, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x100, 0x40000000, 0x4, 0x0, 0x0, 0x0, 0x0, 0xb, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc1800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0xc0000000, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x400, 0x0, 0x10000200, 0x0, 0x100000, 0x0, 0x0 },
                { 0xffceffff, 0xcefffbff, 0xbdff7fff, 0xfc953bd5, 0xb31ddd49, 0xcbbe8881, 0x23da01e2, 0x4fffebb, 0xba41efe4, 0x40240df, 0x249dd687, 0xba397ddf, 0xf6848069, 0x5a02d98, 0xd6792007, 0x421d1c40, 0x4b07c080, 0x4161a6be, 0x8438d010, 0x1b264753, 0x98f38480, 0x1f8f900, 0x14ffe8cc, 0x40042cf, 0x2c175181, 0x28314760, 0x6a49a810, 0x69163c56, 0xad79b487, 0xf3f99717, 0x8fc8fb7c, 0x3fffffa1, 0xeda007f, 0x394b18e, 0x83f1cdaf, 0xa7b79c8, 0x3c7ff7ff, 0xef528a54, 0x81f8331f, 0xbf07c1fc, 0xdbb33a6, 0xd3000006, 0x351308e2, 0xce76b6ef, 0xeab698e2, 0x17294318, 0x10008a81, 0x6295de1, 0xc1cc7bab, 0xd6baa1e8, 0xfa1db360, 0x1008200, 0x502b768, 0x450eeb94, 0xf4240, 0x0, 0x44c6b0b, 0xa0001460, 0x5584, 0x4b66ee37, 0x15fae4dd, 0x29a21065, 0xa1c18580, 0x1392456b, 0x6c3e4051, 0xfd414d59, 0x3746e78, 0xa41a04ad, 0xf0260031, 0x11f75f23, 0x61104f60, 0x14af8001, 0x32c88989, 0x28fc4c96, 0x1da68d03, 0x5fd17fff, 0x5cc80906, 0x1001607d, 0x801006c0, 0x514b46, 0x172e0d44, 0xe5ec3a53, 0x6807d0e1, 0xf540adc6, 0x6b38d2, 0xc1071825, 0xd987328c, 0xec8db15c, 0x2a500000 },
                { 0xffffffff, 0xdafaffef, 0xffffbdff, 0xd8040204, 0x182008c, 0x12bf1ff, 0x98f80061, 0xfe24c020, 0xf801000b, 0xcfb040c0, 0x20, 0xe54, 0x80000000, 0x0, 0x8060800b, 0x100500, 0x48010000, 0x1040c, 0x88000, 0x120650e, 0x9c630000, 0x1001ff, 0xb871c080, 0x66, 0x2c120101, 0xc1a00400, 0x20091020, 0x42141000, 0x8810a010, 0xf0a01400, 0x800008, 0x0, 0x181044, 0xc3023004, 0x1270003a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x403e082, 0x82460400, 0x49860, 0x5208010, 0xd0008200, 0x62848e9, 0xb4c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8008, 0x10e0080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x1182a551, 0x1800000, 0x1c00580, 0x2980460, 0x401e4041, 0xad410949, 0x2640010, 0x800204a5, 0xa8200021, 0x1845301, 0x21000000, 0xaf8001, 0x12e80980, 0xfc4036, 0x19060003, 0x5eb175b7, 0x10800000, 0x4, 0x80100400, 0x110802, 0x10200b44, 0xc1ec1a50, 0x1401, 0x21000084, 0x10280ad0, 0x60020, 0x19922280, 0x48d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x920, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x92535267, 0x28000000, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x30082, 0xc0000054, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104024, 0xfa3d0000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x5a000000, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x64800000, 0x0, 0x0, 0x0, 0x0, 0x380000, 0x0, 0x0, 0x3e000000, 0x0, 0x800, 0xc0000000 },
                { 0xfffeffff, 0x4efafbef, 0xbdff3dff, 0xd8000004, 0x0, 0xbf1ff, 0x80d801e0, 0xc4fffc20, 0xb8010000, 0x4004080, 0x0, 0xc54, 0x80000000, 0x0, 0x80408000, 0x100100, 0x48010000, 0x10404, 0x88000, 0x650e, 0x98630000, 0xf8015a, 0x907fc088, 0x46, 0x2c120101, 0x200400, 0x20091000, 0x42141000, 0x8810a000, 0xf0a01400, 0x800008, 0x0, 0x181044, 0x3004, 0x1270002a, 0x8622000, 0x8000000, 0xc528000, 0x1102100, 0x44008, 0x92922a6, 0x93000000, 0x5038082, 0x8a660400, 0x49860, 0x5200010, 0x10008200, 0x62848e9, 0x80c57ba3, 0x543aa1a8, 0xf2003260, 0x0, 0x8, 0x60080, 0x34200, 0x0, 0x4002200, 0x20000400, 0x0, 0x4a66ee00, 0x15c2a459, 0x1800000, 0x1c00580, 0x2900460, 0x401e4041, 0xad410949, 0x2640010, 0x800004a4, 0x200021, 0x1845301, 0x21000000, 0xaf8001, 0x32c80980, 0xfc4016, 0x19060003, 0x5e9175b7, 0x10800000, 0x44, 0x80100400, 0x110842, 0x10200944, 0xc1ec3a50, 0x1, 0x21002d84, 0x2808d0, 0x40021, 0xd9862280, 0xc8d811c, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0xc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x1040000, 0x0, 0x2d68728, 0x24012b39, 0x37600000, 0x4002208, 0x59, 0x1361000, 0x3330, 0xb5432841, 0x57a6c020, 0x406c06d, 0x15a02d99, 0x56100800, 0xc0004812, 0x3008211, 0x414222a2, 0x84004289, 0x1ed80000, 0x581, 0x4021000, 0x414, 0x3b824280, 0x2006042, 0x335, 0x13400000, 0x8020814, 0x254810e2, 0xa008015, 0x87407b34, 0x3fffffa0, 0x802c010, 0xd940800, 0xc880, 0x20018c8, 0x20000420, 0x23000a54, 0xa0e8001f, 0xaf0120f4, 0x64000000, 0xb0002, 0x100000, 0x1411a2e7, 0xafb00080, 0x100b0800, 0xe, 0xc0001400, 0x8, 0x40, 0x9088000, 0x3090c224, 0x5021a40, 0xc01c014, 0x20, 0x0, 0xc410b, 0x80000010, 0x2091dfa4, 0x37, 0x280004, 0x20211445, 0x20011020, 0x1000021a, 0x2f200000, 0x10000000, 0xe0015e68, 0x26112001, 0x4010d000, 0x58032024, 0x4000ef20, 0x34000000, 0x6b, 0x6a008c00, 0x808800, 0x200, 0x3000906, 0x4014e98, 0x4000, 0x404000, 0x70e4000, 0x2e00000c, 0x83e0062, 0x14400042, 0x3300c, 0xc1390004, 0x441100c, 0x20003840, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x1000084, 0x9000, 0x84000004, 0x40000000, 0x0, 0x88081, 0xc00000, 0x0, 0x88000000, 0x4000080, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x8010000, 0x0, 0x0, 0x4502, 0x98620000, 0x0, 0x0, 0x46, 0x20000000, 0x0, 0x0, 0x41000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x2922a6, 0x93000000, 0x0, 0x0, 0x49800, 0x0, 0x0, 0x0, 0x820, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0xa66ee00, 0x0, 0x0, 0x1c00000, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2f8000, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 }
            };
        }

        protected static int lookupSigmap(int row, int col)
        {
            if (row <= 111)
                return sigmap[row][col];
            else if (row >= 112)
                return sigmap1[row-112][col];
            else
                throw new Error("Unexpected location requested in sigmap1 lookup");
        }

        protected static int[][] value = null;

        protected static void valueInit()
        {
            value = new int[][]
            {
                { 0x81f582c8, 0x82788459, 0x855a8910, 0x82104006, 0x89f689f7, 0x82108210, 0x89fc8374, 0x8a708210, 0x8a880000, 0x0, 0x0, 0x0, 0x40080000, 0x0, 0x4001, 0x0, 0x0, 0x81320000, 0x0, 0x0, 0x0, 0x820b, 0x400c, 0x0, 0x40030000, 0x0, 0x0, 0x8374, 0x0, 0x0, 0x40058210, 0x0, 0x4007, 0x84e90000, 0x0, 0x0, 0x0, 0x0, 0x83740000, 0x0, 0x0, 0x0, 0x0, 0xc000, 0x0, 0x850d4004, 0x84e90000, 0x8374, 0x81bb40f9, 0x813240fe, 0x82108239, 0x83e3, 0x40a10000, 0x0, 0x412d, 0x40fb406e, 0x82b20000, 0x84e9830c, 0x412e822e, 0x83bb0000, 0x844f0000, 0x40fa8370, 0x833044d3, 0x0, 0x40fc0000, 0x40fd44dd, 0x4009844f, 0x4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bb81bb, 0x423781bb, 0x40130000, 0x4011, 0x4348, 0x8388, 0x0, 0x83740000, 0x84e00000, 0x8374, 0x0, 0x400b8132, 0x4002, 0x4249, 0x42128132, 0x420a0000, 0x424581bb, 0x420c0000, 0x8132, 0x40a5, 0x82b24243, 0x0, 0x0, 0x4247, 0x4217822e, 0x0, 0x84890000, 0x0, 0x83e30000, 0x420b4241, 0x400a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x423d0000, 0x85ee, 0x422e83bb, 0x0, 0x81bb81bb, 0x81bb81bb, 0x81bb81bb, 0x81bb81bb, 0x420881bb, 0x81bb422c, 0x81bb4525, 0x423240b0, 0x81bb4081, 0x40a440a2, 0x42090000, 0x408240af, 0x409040a3, 0x409f81bb, 0x40ff, 0x81bb4122, 0x410381bb, 0x0, 0x81bb, 0x41010000, 0x44060000, 0x844f0000, 0x44040000, 0x41d3, 0x4242424a, 0x8489822e, 0x822e, 0x42440000, 0x81bb4246, 0x84b885eb, 0x41000000, 0x0, 0x844f423e, 0x42130000, 0x84e0, 0x45104248, 0x0, 0x420d4102, 0x41040000, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b30000, 0x423881bb, 0x81bb4233, 0x0, 0x0, 0x0, 0x0, 0x81bb0000, 0x0, 0x422d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bb822e, 0x81bb81bb, 0x81bb4014, 0x4012, 0x449a0000, 0x449c87f6, 0x0, 0x0, 0x81bb, 0x0, 0x0, 0x0, 0x8132, 0x44fc0000, 0x81bb, 0x81bb422f, 0x81bb8132, 0x4239, 0x83bb81bb, 0x0, 0x41ce0000, 0x0, 0x0, 0x81bb0000, 0x4408, 0x844f0000, 0x44050000, 0x446f, 0x8489, 0x822e0000, 0x83e30000, 0x81bb0000, 0x84b84214, 0x0, 0x0, 0x84e0, 0x420e0000, 0x451e81bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40aa0000, 0x81bb, 0x0, 0x40b240a9, 0x0, 0x407b0000, 0x81bb40e5, 0x81bb0000, 0x40830000, 0x81bb40a8, 0x40a681bb, 0x4520407e, 0x407d4527, 0x409240b1, 0x81bb81bb, 0x40a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874521, 0x407f4080, 0x45264091, 0x81bb407c, 0x81bb, 0x0, 0x0, 0x87f60000, 0x81bb87f6, 0x40a781bb, 0x42160000, 0x41d40000, 0x4128412a, 0x81bb81bb, 0x81bb0000, 0x0, 0x0, 0x0, 0x420c, 0x0, 0x0, 0x4407, 0x0, 0x81bb0000, 0x0, 0x0, 0x0, 0x0, 0x440a0000, 0x0, 0x0, 0x0, 0x44700000, 0x0, 0x81bb, 0x0, 0x4215, 0x86e7, 0x41d2420f, 0x45070000, 0x87f60000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bb0000, 0x0, 0x0, 0x0, 0x0, 0x81bb, 0x81bb0000, 0x0, 0x84b8, 0x4400, 0x4401, 0x84b80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41d50000, 0x81bb0000, 0x0, 0x81bb, 0x0, 0x81bb, 0x87470000, 0x0, 0x0, 0x81bb0000, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x81bb0000, 0x0, 0x81bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bb0000, 0x0, 0x81bb0000, 0x81bb, 0x81bb822e, 0x81bb, 0x81bb81bb, 0x81bb4509, 0x81bb81bb, 0x81bb81bb, 0x450881bb, 0x81bb81bb, 0x81bb81bb, 0x81bb449b, 0x449e, 0x44a0449d, 0x81bb0000, 0x0, 0x81bb0000, 0x0, 0x81bb, 0x81bb40ae, 0x0, 0x0, 0x0, 0x0, 0x822e, 0x0, 0x0, 0x0, 0x81bb0000, 0x81bb0000, 0x0, 0x0, 0x0, 0x45110000, 0x0, 0x0, 0x0, 0x44090000, 0x0, 0x440c0000, 0x0, 0x450a0000, 0x0, 0x882c4210, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bb, 0x4512, 0x81bb0000, 0x81bb, 0x0, 0x0, 0x0, 0x451381bb, 0x81bb81bb, 0x81bb81bb, 0x4121450c, 0x81bb450b, 0x81bb4211, 0x81bb0000, 0x887c, 0x81bb4127, 0x41050000, 0x81bb, 0x81bb0000, 0x0, 0x888f0000, 0x844f81bb, 0x0, 0x440b0000, 0x0, 0x450d0000, 0x84b8, 0x84b80000, 0x88be, 0x0, 0x844f0000, 0x0, 0x0, 0x0, 0x450e81bb, 0x81bb, 0x81bb450f, 0x81bb0000, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881bb, 0x81bb81bb, 0x81bb81bb, 0x0, 0x0, 0x81bb, 0x81bb, 0x0, 0x0, 0x0, 0x0, 0x81bb, 0x81bb, 0x0, 0x81bb, 0x81bb, 0x81bb0000, 0x81bb81bb, 0x81bb, 0x44a2449f, 0x44a181bb, 0x0, 0x0, 0x81bb0000, 0x0, 0x0, 0x0, 0x0, 0x81bb, 0x81bb822e, 0x81bb, 0x81bb81bb, 0x81bb0000, 0x0, 0x0, 0x8967, 0x0, 0x896d0000, 0x81bb440d, 0x0, 0x0, 0x0, 0x897c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bb0000, 0x81bb81bb, 0x81bb81bb, 0x81bb0000, 0x81bb0000, 0x81bb81bb, 0x81bb0000, 0x41200000, 0x0, 0x0, 0x41294125, 0x81bb4126, 0x0, 0x0, 0x0, 0x81bb, 0x0, 0x8132, 0x0, 0x81bb81bb, 0x0, 0x0, 0x0, 0x84b80000, 0x0, 0x0, 0x81bb, 0x874781bb, 0x8747, 0x8747, 0x81bb81bb, 0x4437, 0x443e443b, 0x443d4419, 0x44ba0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bb81bb, 0x81bb81bb, 0x44a3, 0x0, 0x0, 0x81bb, 0x81bb0000, 0x0, 0x0, 0x0, 0x81bb, 0x0, 0x81bb, 0x81bb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81320000, 0x81bb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bb, 0x81bb81bb, 0x81bb0000, 0x81bb81bb, 0x0, 0x81bb, 0x81bb0000, 0x0, 0x0, 0x0, 0x81bb, 0x443f, 0x81bb0000, 0x0, 0x0, 0x0, 0x81bb81bb, 0x81bb0000, 0x81bb81bb, 0x81bb0000, 0x0, 0x0, 0x0, 0x81bb81bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bb81bb, 0x81bb0000, 0x0, 0x41240000, 0x81bb81bb, 0x8132, 0x0, 0x81bb81bb, 0x81bb, 0x0, 0x0, 0x0, 0x81bb0000, 0x81bb0000, 0x0, 0x81bb81bb, 0x81bb81bb, 0x0, 0x0, 0x81bb, 0x81bb81bb, 0x4123, 0x81bb0000, 0x0, 0x81bb0000, 0x0, 0x0, 0x0, 0x0, 0x81bb0000, 0x81bb0000, 0x81bb, 0x81bb0000, 0x81bb, 0x0, 0x0, 0x0, 0x81bb, 0x81bb0000, 0x81bb0000, 0x0, 0x81bb81bb, 0x81bb81bb, 0x81bb0000 },
                { 0x80654055, 0x80654068, 0x40524049, 0x40604006, 0x40484067, 0x80658065, 0x403e406b, 0x406580fb, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80654043, 0x829d4042, 0x8133402d, 0x84a64062, 0x40408065, 0x4057405c, 0x4058829d, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x4025858b, 0x404f4064, 0x4010406c, 0x400585cf, 0x40614029, 0x40264007, 0x84ea44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x869944e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668065, 0x813f8065, 0x402a4004, 0x84ea406a, 0x4028407a, 0x81ba86b0, 0x813387aa, 0x40244022, 0x813f8065, 0x40a1401c, 0x403c813f, 0x403a412d, 0x891f406e, 0x4039401e, 0x84ea403b, 0x412e80fb, 0x89248065, 0x84508295, 0x895089e9, 0x80658065, 0x44d98a22, 0x8a2541cf, 0x88548a4d, 0x40098450, 0x813f4021, 0x83364019, 0x401b8a4e, 0x44d58a65, 0x40174015, 0x813f8065, 0x452a452b, 0x8a7942f3, 0x42f58a7e, 0x44408aa0, 0x8aa142cd, 0x8acf8ad3, 0x42cf452e, 0x452d42cc, 0x42d14531, 0x45300000, 0x0, 0x44c344c5, 0x81ba81ba, 0x8295, 0x4013813f, 0x80654011, 0x813f, 0x42ad0000, 0x0, 0x80fb, 0x82950000, 0x0, 0x442c4023, 0x400b8133, 0x415b4002, 0x0, 0x8133, 0x401d, 0x81ba, 0x401f, 0x445a8133, 0x80650000, 0x0, 0x80650000, 0x813f0000, 0x0, 0x0, 0x8578, 0x848a0000, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42898336, 0x80fb85ac, 0x442d, 0x0, 0x440e, 0x81ba81ba, 0x81ba81ba, 0x81ba81ba, 0x81ba81ba, 0x81ba, 0x81ba0000, 0x81ba4525, 0x83ff, 0x81ba4081, 0x84058406, 0x44bf, 0x40828408, 0x40908409, 0x409f81ba, 0x44e50000, 0x81ba0000, 0x81ba, 0x43398336, 0x44ce81ba, 0x87b9434d, 0x44060000, 0x84500000, 0x44040000, 0x41d3, 0x0, 0x848a442a, 0x431e445b, 0x432d, 0x81ba0000, 0x84ba0000, 0x87bf434b, 0x0, 0x84508804, 0x85784161, 0x43350000, 0x0, 0x0, 0x8336, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x88544016, 0x44b10000, 0x44b340f5, 0x81ba, 0x81ba0000, 0x0, 0x42f4, 0x0, 0x0, 0x81ba0000, 0x43064441, 0x8954, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d287bf, 0x42e9, 0x87b90000, 0x81ba0000, 0x81ba81ba, 0x81ba4014, 0x88044012, 0x449a80fb, 0x449c433d, 0x44ed44a4, 0x42ae, 0x8295, 0x80fb42c4, 0x44d540b3, 0x45298a05, 0x41578133, 0x44fc8a0c, 0x44c681ba, 0x81ba8954, 0x8133, 0x415c8336, 0x81ba, 0x44c044e6, 0x44f7, 0x8336, 0x433b8a8e, 0x81ba0000, 0x434e4408, 0x84500000, 0x44050000, 0x8a920000, 0x848a, 0x8a05431f, 0x432f, 0x81ba0000, 0x84ba0000, 0x434c0000, 0x8a0c4162, 0x43370000, 0x0, 0x8a8e0000, 0x8a92428e, 0x428c428a, 0x428f0000, 0x452c4290, 0x452f4291, 0x0, 0x81ba, 0x42a54150, 0x0, 0x0, 0x0, 0x81ba0000, 0x81ba0000, 0x4083438c, 0x81ba0000, 0x81ba, 0x4520407e, 0x407d4527, 0x40920000, 0x81ba81ba, 0x44c140a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874521, 0x407f4080, 0x45264091, 0x81ba407c, 0x81ba, 0x0, 0x0, 0x44e744e9, 0x81ba416c, 0x81ba, 0x4463, 0x41d488d3, 0x412a, 0x81ba81ba, 0x81ba88d9, 0x4402433a, 0x407b0000, 0x41c0, 0x88e5, 0x417344f7, 0x44f6434f, 0x43514407, 0x0, 0x81ba0000, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a85ac, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x81ba, 0x0, 0x0, 0x441a0000, 0x41d20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81ba0000, 0x0, 0x0, 0x0, 0x431a, 0x431c81ba, 0x81ba43b1, 0x432e43b5, 0x43eb84ba, 0x84ba4400, 0x84ba4401, 0x84ba4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x85ac, 0x0, 0x0, 0x41d50000, 0x81ba0000, 0x4184, 0x81ba, 0x41b8, 0x81ba, 0x81ba0000, 0x4192, 0x4471, 0x81ba44bb, 0x4432, 0x44d5442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b288d3, 0x44b544b7, 0x44b40000, 0x81ba0000, 0x88d94308, 0x81ba, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x81ba42ea, 0x42ed0000, 0x81ba4251, 0x427d81ba, 0x81ba0000, 0x432681ba, 0x81ba81ba, 0x81ba0000, 0x81ba81ba, 0x81ba81ba, 0x81ba, 0x81ba0000, 0x81ba81ba, 0x81ba449b, 0x449e, 0x44a0449d, 0x81ba44a5, 0x44a88295, 0x42c6, 0x42c5, 0x42c081ba, 0x81ba0000, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x81ba438d, 0x81ba0000, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x85ac0000, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x447281ba, 0x44bd0000, 0x0, 0x81ba, 0x42a74292, 0x42a90000, 0x42a64151, 0x81ba, 0x81ba81ba, 0x81ba81ba, 0x0, 0x0, 0x81ba0000, 0x81ba0000, 0x416d0000, 0x81ba0000, 0x42ca, 0x42c881ba, 0x81ba0000, 0x0, 0x4174, 0x845081ba, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x84ba, 0x84ba84ba, 0x43e70000, 0x43700000, 0x84500000, 0x0, 0x41640000, 0x41850000, 0x81ba, 0x81ba, 0x81ba0000, 0x81ba44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881ba, 0x81ba81ba, 0x81ba81ba, 0x40f30000, 0x430c4309, 0x81ba, 0x81ba, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x81ba, 0x42ee81ba, 0x4322, 0x432481ba, 0x81ba, 0x81ba0000, 0x81ba81ba, 0x81ba, 0x44a2449f, 0x44a181ba, 0x42c7, 0x42c242c1, 0x81ba40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e81ba, 0x81ba0000, 0x432781ba, 0x81ba81ba, 0x81ba44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x81ba440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x81ba0000, 0x81ba81ba, 0x81ba81ba, 0x81ba438a, 0x81ba44eb, 0x81ba81ba, 0x81ba0000, 0x41fa, 0x418d4465, 0x41ef, 0x41290000, 0x81ba0000, 0x42f1, 0x0, 0x43880000, 0x446b81ba, 0x43bf441c, 0x8133, 0x43c7, 0x81ba81ba, 0x43b34286, 0x0, 0x43b743e8, 0x84ba0000, 0x42060000, 0x43744355, 0x81ba, 0x81ba81ba, 0x81ba, 0x81ba, 0x81ba81ba, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x81ba81ba, 0x81ba81ba, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c381ba, 0x81ba0000, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f481ba, 0x4323, 0x432581ba, 0x81ba0000, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x813343c8, 0x81ba43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x81ba, 0x81ba81ba, 0x81ba0000, 0x81ba81ba, 0x0, 0x81ba, 0x81ba4386, 0x43bb, 0x441e4149, 0x0, 0x81ba, 0x4474443f, 0x81ba4453, 0x0, 0x0, 0x425a4253, 0x81ba81ba, 0x81ba44c8, 0x81ba81ba, 0x81ba40e1, 0x4487, 0x40f24321, 0x425b4280, 0x81ba81ba, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x81ba81ba, 0x81ba0000, 0x0, 0x0, 0x81ba81ba, 0x441f8133, 0x43c9, 0x81ba81ba, 0x414a81ba, 0x0, 0x0, 0x4255, 0x81ba4257, 0x81ba0000, 0x425c4254, 0x81ba81ba, 0x81ba81ba, 0x448b4421, 0x43ca414c, 0x81ba, 0x81ba81ba, 0x41ed0000, 0x81ba0000, 0x4204, 0x81ba0000, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x81ba4258, 0x81ba41ee, 0x420581ba, 0x81ba0000, 0x81ba, 0x425e, 0x4262425f, 0x42824263, 0x428481ba, 0x81ba4349, 0x81ba4260, 0x4264434a, 0x81ba81ba, 0x81ba81ba, 0x81ba0000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x87340000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e1, 0x0, 0x811a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x84eb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x84eb0000, 0x0, 0x81bd0000, 0x0, 0x0, 0x83e4, 0x40a10000, 0x0, 0x0, 0x406e, 0x0, 0x84eb0000, 0x0, 0x0, 0x84510000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8451, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bd81bd, 0x42370000, 0x0, 0x0, 0x0, 0x42ad811a, 0x42bf42bc, 0x42be42bd, 0x42bb42ba, 0x42af8313, 0x0, 0x0, 0x0, 0x4249, 0x42120000, 0x420a0000, 0x424581bd, 0x420c0000, 0x0, 0x40a5, 0x4243, 0x0, 0x0, 0x4247, 0x42170000, 0x0, 0x848b0000, 0x0, 0x83e40000, 0x420b4241, 0x0, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x0, 0x422e0000, 0x0, 0x81bd81bd, 0x81bd81bd, 0x81bd81bd, 0x81bd81bd, 0x420881bd, 0x81bd422c, 0x81bd4525, 0x423240b0, 0x81bd4081, 0x40a440a2, 0x42090000, 0x408240af, 0x409040a3, 0x409f81bd, 0x0, 0x81bd0000, 0x81bd, 0x0, 0x81bd, 0x0, 0x0, 0x84510000, 0x0, 0x41d3, 0x4242424a, 0x848b0000, 0x0, 0x42440000, 0x81bd4246, 0x84bb0000, 0x0, 0x0, 0x8451423e, 0x42130000, 0x0, 0x45104248, 0x0, 0x420d0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x420c0000, 0x0, 0x0, 0x423881bd, 0x81bd4233, 0x0, 0x0, 0x0, 0x81bd81bd, 0x81bd0000, 0x0, 0x422d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bd0000, 0x81bd81bd, 0x81bd0000, 0x0, 0x0, 0x0, 0x0, 0x42ae, 0x42b00000, 0x42c4, 0x0, 0x0, 0x0, 0x0, 0x81bd, 0x81bd422f, 0x0, 0x4239, 0x81bd, 0x0, 0x41ce0000, 0x0, 0x0, 0x81bd0000, 0x0, 0x84510000, 0x0, 0x446f, 0x848b, 0x0, 0x83e40000, 0x81bd0000, 0x84bb4214, 0x0, 0x0, 0x0, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x81bd, 0x42a50000, 0x40b240a9, 0x0, 0x407b0000, 0x81bd40e5, 0x81bd0000, 0x40830000, 0x81bd40a8, 0x40a681bd, 0x4520407e, 0x407d4527, 0x409240b1, 0x81bd81bd, 0x40a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874521, 0x407f4080, 0x45264091, 0x81bd407c, 0x81bd, 0x0, 0x0, 0x0, 0x81bd0000, 0x40a781bd, 0x42160000, 0x41d40000, 0x0, 0x81bd81bd, 0x81bd0000, 0x0, 0x42b1, 0x0, 0x420c, 0x0, 0x0, 0x0, 0x0, 0x81bd0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44700000, 0x0, 0x81bd, 0x0, 0x4215, 0x0, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bd0000, 0x0, 0x0, 0x0, 0x0, 0x81bd, 0x81bd0000, 0x0, 0x84bb, 0x4400, 0x4401, 0x84bb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41d50000, 0x81bd0000, 0x0, 0x81bd, 0x0, 0x81bd, 0x81bd0000, 0x0, 0x0, 0x81bd0000, 0x0, 0x406e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bd0000, 0x0, 0x81bd, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bd0000, 0x0, 0x81bd4251, 0x427d81bd, 0x81bd0000, 0x81bd, 0x81bd81bd, 0x81bd4509, 0x81bd81bd, 0x81bd81bd, 0x450881bd, 0x81bd0000, 0x81bd81bd, 0x81bd0000, 0x0, 0x0, 0x81bd0000, 0x0, 0x42c6, 0x42c5, 0x42c081bd, 0x81bd40ae, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bd0000, 0x81bd0000, 0x0, 0x0, 0x0, 0x45110000, 0x42b2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x450a0000, 0x0, 0x4210, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bd, 0x4512, 0x0, 0x81bd, 0x42a74292, 0x42a90000, 0x42a60000, 0x451381bd, 0x81bd81bd, 0x81bd81bd, 0x450c, 0x450b, 0x81bd4211, 0x81bd0000, 0x0, 0x81bd0000, 0x42ca, 0x42c881bd, 0x81bd0000, 0x0, 0x0, 0x845181bd, 0x0, 0x0, 0x0, 0x450d0000, 0x84bb, 0x84bb0000, 0x0, 0x0, 0x84510000, 0x0, 0x0, 0x0, 0x450e81bd, 0x81bd, 0x81bd450f, 0x81bd0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bd, 0x81bd81bd, 0x81bd81bd, 0x0, 0x0, 0x81bd, 0x81bd, 0x0, 0x0, 0x0, 0x0, 0x81bd, 0x81bd, 0x42b30000, 0x81bd, 0x81bd, 0x81bd0000, 0x81bd81bd, 0x81bd, 0x0, 0x81bd, 0x42c7, 0x42c242c1, 0x81bd0000, 0x0, 0x0, 0x0, 0x4252, 0x427e81bd, 0x81bd0000, 0x81bd, 0x81bd81bd, 0x81bd0000, 0x0, 0x0, 0x0, 0x42cb42c9, 0x0, 0x81bd0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ab0000, 0x42a842aa, 0x0, 0x81bd0000, 0x81bd81bd, 0x81bd81bd, 0x81bd0000, 0x81bd0000, 0x81bd81bd, 0x81bd0000, 0x0, 0x0, 0x0, 0x0, 0x81bd0000, 0x0, 0x0, 0x0, 0x81bd, 0x0, 0x0, 0x0, 0x81bd81bd, 0x4286, 0x0, 0x0, 0x84bb0000, 0x0, 0x0, 0x81bd, 0x81bd81bd, 0x81bd, 0x81bd, 0x81bd81bd, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4259427f, 0x81bd81bd, 0x81bd81bd, 0x0, 0x0, 0x0, 0x42c381bd, 0x81bd0000, 0x0, 0x0, 0x0, 0x81bd, 0x42b40000, 0x81bd, 0x81bd0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bd0000, 0x42870000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ac0000, 0x81bd, 0x81bd81bd, 0x81bd0000, 0x81bd81bd, 0x0, 0x81bd, 0x81bd0000, 0x0, 0x0, 0x0, 0x81bd, 0x0, 0x81bd0000, 0x0, 0x0, 0x425a4253, 0x81bd81bd, 0x81bd0000, 0x81bd81bd, 0x81bd0000, 0x0, 0x0, 0x425b4280, 0x81bd81bd, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81bd81bd, 0x81bd0000, 0x0, 0x0, 0x81bd81bd, 0x0, 0x0, 0x81bd81bd, 0x81bd, 0x0, 0x0, 0x4255, 0x81bd4257, 0x81bd0000, 0x425c4254, 0x81bd81bd, 0x81bd81bd, 0x0, 0x0, 0x81bd, 0x81bd81bd, 0x0, 0x81bd0000, 0x0, 0x81bd0000, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x81bd4258, 0x81bd0000, 0x81bd, 0x81bd0000, 0x81bd, 0x425e, 0x4262425f, 0x42824263, 0x428481bd, 0x81bd0000, 0x81bd4260, 0x42640000, 0x81bd81bd, 0x81bd81bd, 0x81bd0000 },
                { 0x80664055, 0x80664068, 0x40524049, 0x40604006, 0x40484067, 0x86860000, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80664043, 0x4042, 0x402d, 0x80664062, 0x40408066, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668066, 0x81408066, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x424b, 0x40a140e6, 0x0, 0x40e70000, 0x406e, 0x0, 0x40ea, 0x0, 0x0, 0x84520000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40098452, 0x81404021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81408066, 0x42f6, 0x806642f3, 0x42f50000, 0x44408066, 0x42d742cd, 0x40334030, 0x42cf8066, 0x403242cc, 0x42d18066, 0x8140402f, 0x40310000, 0x44c5, 0x0, 0x42370000, 0x40138140, 0x80664011, 0x0, 0x42ad0000, 0x0, 0x0, 0x84db0000, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x4249, 0x42120000, 0x420a0000, 0x42450000, 0x420c0000, 0x0, 0x40a5, 0x4243, 0x0, 0x0, 0x4247, 0x42170000, 0x0, 0x848c0000, 0x0, 0x0, 0x420b4241, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x0, 0x422e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x840e0000, 0x842b, 0x43390000, 0x0, 0x434d, 0x44060000, 0x84520000, 0x44040000, 0x41d3, 0x4242424a, 0x848c0000, 0x431e0000, 0x4244432d, 0x4246, 0x0, 0x434b, 0x0, 0x8452423e, 0x42134161, 0x433584db, 0x45104248, 0x0, 0x420d0000, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x42380000, 0x4233, 0x42f842f7, 0x42f942f4, 0x81400000, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d58066, 0x814042d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x422f, 0x0, 0x415c4239, 0x0, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x84520000, 0x44050000, 0x446f, 0x848c, 0x431f, 0x424c432f, 0x0, 0x4214, 0x434c0000, 0x4162, 0x433784db, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x40e9, 0x40ec0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x40e840eb, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x874c0000, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ed40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x40ef438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x40ee, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x40f00000, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80664174, 0x84520000, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x84520000, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80664176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x89af0000, 0x42f1, 0x44e18140, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x874c0000, 0x874c, 0x874c, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x8066, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8066, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80674055, 0x80674068, 0x40524049, 0x40604006, 0x40484067, 0x82b48543, 0x403e406b, 0x406582b4, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80674043, 0x82e14042, 0x85db402d, 0x80674062, 0x40408067, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668067, 0x81418067, 0x402a4004, 0x406a, 0x40288543, 0x81bc0000, 0x83f40000, 0x0, 0x42790000, 0x40a183f4, 0x0, 0x0, 0x406e, 0x0, 0x81bc0000, 0x0, 0x81bc0000, 0x81bc0000, 0x83f4, 0x83f4427b, 0x423a, 0x0, 0x4270, 0x40090000, 0x81414021, 0x4019, 0x401b4273, 0x42244230, 0x40174015, 0x81418067, 0x427742f6, 0x806742f3, 0x42f54275, 0x44408067, 0x42d742cd, 0x40334030, 0x42cf8067, 0x403242cc, 0x42d18067, 0x8141402f, 0x40314278, 0x44c5, 0x81bc81bc, 0x423781bc, 0x40138141, 0x80674011, 0x0, 0x42ad4234, 0x42224223, 0x83f4, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a0000, 0x81bc, 0x420c0000, 0x0, 0x40a5, 0x0, 0x42710000, 0x4225, 0x81bc, 0x42170000, 0x81bc0000, 0x81bc0000, 0x427a, 0x83f40000, 0x420b0000, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x44dc, 0x42890000, 0x83f40000, 0x0, 0x422e81bc, 0x0, 0x81bc81bc, 0x81bc0000, 0x81bc0000, 0x0, 0x420881bc, 0x81bc422c, 0x81bc4525, 0x423240b0, 0x81bc0000, 0x40a440a2, 0x420944bf, 0x40af, 0x409040a3, 0x81bc, 0x44e50000, 0x81bc0000, 0x81bc, 0x43390000, 0x81bc, 0x434d, 0x44060000, 0x81bc0000, 0x440481bc, 0x41d3, 0x81bc, 0x81bc0000, 0x431e0000, 0x81bc432d, 0x81bc81bc, 0x0, 0x434b, 0x0, 0x81bc83f4, 0x42134161, 0x433581bc, 0x45100000, 0x0, 0x420d0000, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x423881bc, 0x81bc4233, 0x42f842f7, 0x42f942f4, 0x81410000, 0x0, 0x81bc0000, 0x43064441, 0x422d, 0x42d342d4, 0x42d58067, 0x814142d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x81bc0000, 0x81bc, 0x81bc4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x422681bc, 0x42c4, 0x40b3, 0x0, 0x41574231, 0x423b0000, 0x44c681bc, 0x81bc422f, 0x81bc0000, 0x415c4239, 0x81bc81bc, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x81bc0000, 0x434e4408, 0x81bc0000, 0x440581bc, 0x446f, 0x81bc81bc, 0x431f, 0x81bc432f, 0x81bc81bc, 0x4214, 0x434c0000, 0x4162, 0x433781bc, 0x420e0000, 0x451e81bc, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x81bc, 0x42a54150, 0x40b240a9, 0x423681bc, 0x407b81bc, 0x81bc40e5, 0x81bc4235, 0x4083438c, 0x81bc40a8, 0x40a681bc, 0x0, 0x4527, 0x409240b1, 0x81bc81bc, 0x44c140a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874272, 0x0, 0x45264091, 0x42740000, 0x422781bc, 0x451f423c, 0x422b0000, 0x44e744e9, 0x81bc416c, 0x40a781bc, 0x42164463, 0x41d40000, 0x0, 0x81bc81bc, 0x81bc0000, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x81bc81bc, 0x41d841d6, 0x83f441aa, 0x41ac4276, 0x41b10000, 0x440a81bc, 0x43314469, 0x427c0000, 0x417b0000, 0x447043bd, 0x0, 0x81bc, 0x0, 0x81bc4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x4229, 0x42284517, 0x81bc, 0x0, 0x0, 0x81bc0000, 0x0, 0x0, 0x0, 0x431a, 0x431c81bc, 0x81bc43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x81bc, 0x41d50000, 0x81bc0000, 0x4184, 0x81bc, 0x41d941b8, 0x41d781bc, 0x87480000, 0x4192, 0x4471, 0x81bc44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x81bc0000, 0x4308, 0x430281bc, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x81bc42ea, 0x42ed42e7, 0x81bc4251, 0x427d81bc, 0x81bc0000, 0x432681bc, 0x81bc81bc, 0x81bc4509, 0x81bc0000, 0x81bc0000, 0x450881bc, 0x81bc81bc, 0x81bc, 0x81bc449b, 0x449e, 0x44a0449d, 0x81bc44a5, 0x44a80000, 0x81bc42c6, 0x42c5, 0x42c081bc, 0x81bc40ae, 0x422a0000, 0x40da0000, 0x4158448c, 0x451940f6, 0x45180000, 0x433e44ee, 0x44a64159, 0x4152, 0x81bc438d, 0x81bc0000, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c451b, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c81bc, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x447281bc, 0x44bd4512, 0x81bc0000, 0x81bc, 0x42a74292, 0x42a90000, 0x42a64151, 0x451381bc, 0x81bc81bc, 0x81bc81bc, 0x450c, 0x81bc450b, 0x81bc4211, 0x81bc0000, 0x416d451d, 0x81bc0000, 0x42ca, 0x42c881bc, 0x81bc0000, 0x451c0000, 0x4174, 0x81bc, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x450e81bc, 0x81bc, 0x81bc450f, 0x81bc44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881bc, 0x81bc81bc, 0x81bc81bc, 0x40f34303, 0x430c4309, 0x430481bc, 0x81bc, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e881bc, 0x42ee81bc, 0x4322, 0x432481bc, 0x81bc, 0x81bc0000, 0x81bc81bc, 0x81bc, 0x44a2449f, 0x44a181bc, 0x42c7, 0x42c242c1, 0x81bc40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e81bc, 0x81bc0000, 0x432781bc, 0x81bc81bc, 0x81bc44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x81bc440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x81bc0000, 0x81bc81bc, 0x81bc81bc, 0x81bc438a, 0x81bc44eb, 0x81bc81bc, 0x81bc0000, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x81bc0000, 0x42f1, 0x0, 0x43880000, 0x446b81bc, 0x43bf441c, 0x0, 0x43c7, 0x81bc81bc, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x81bc, 0x874881bc, 0x8748, 0x8748, 0x81bc81bc, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x81bc81bc, 0x81bc81bc, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c381bc, 0x81bc0000, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f481bc, 0x4323, 0x432581bc, 0x81bc0000, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x81bc43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x81bc, 0x81bc81bc, 0x81bc0000, 0x81bc81bc, 0x42d9, 0x81bc, 0x81bc4386, 0x43bb, 0x441e4149, 0x0, 0x81bc, 0x4474443f, 0x81bc4453, 0x0, 0x42dd0000, 0x425a4253, 0x81bc81bc, 0x81bc44c8, 0x81bc81bc, 0x81bc40e1, 0x4487, 0x40f24321, 0x425b4280, 0x81bc81bc, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x81bc81bc, 0x81bc0000, 0x0, 0x0, 0x81bc81bc, 0x441f0000, 0x43c9, 0x81bc81bc, 0x414a81bc, 0x42de, 0x42e142df, 0x42db4255, 0x81bc4257, 0x81bc0000, 0x425c4254, 0x81bc81bc, 0x81bc81bc, 0x448b4421, 0x43ca414c, 0x81bc, 0x81bc81bc, 0x41ed0000, 0x81bc0000, 0x4204, 0x81bc42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x81bc4258, 0x81bc41ee, 0x420581bc, 0x81bc0000, 0x81bc, 0x42e4425e, 0x4262425f, 0x42824263, 0x428481bc, 0x81bc4349, 0x81bc4260, 0x4264434a, 0x81bc81bc, 0x81bc81bc, 0x81bc0000 },
                { 0x80684055, 0x80684068, 0x40524049, 0x40604006, 0x40484067, 0x83c70000, 0x403e406b, 0x406583c7, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80684043, 0x4042, 0x402d, 0x80684062, 0x40408068, 0x4057405c, 0x40584221, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x83c744e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x423f44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668068, 0x81428068, 0x402a4004, 0x4218406a, 0x4028421a, 0x81c20000, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x406e, 0x0, 0x83c70000, 0x0, 0x0, 0x84560000, 0x0, 0x0, 0x83c7, 0x0, 0x0, 0x40098456, 0x81424021, 0x4019, 0x401b0000, 0x42244230, 0x40174015, 0x81428068, 0x42f6, 0x806842f3, 0x42f50000, 0x44408068, 0x42d742cd, 0x40334030, 0x42cf8068, 0x403242cc, 0x42d18068, 0x8142402f, 0x40310000, 0x44c5, 0x81c281c2, 0x42370000, 0x40138142, 0x80684011, 0x0, 0x42ad4234, 0x42224223, 0x0, 0x84dc421b, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a0000, 0x81c2, 0x420c0000, 0x0, 0x40a5, 0x0, 0x0, 0x4225, 0x0, 0x42170000, 0x0, 0x848d0000, 0x0, 0x83c70000, 0x420b83c7, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x0, 0x422e0000, 0x0, 0x81c281c2, 0x81c281c2, 0x81c281c2, 0x81c281c2, 0x420881c2, 0x81c2422c, 0x81c24525, 0x423240b0, 0x81c24081, 0x40a440a2, 0x420944bf, 0x408240af, 0x409040a3, 0x409f81c2, 0x44e50000, 0x81c20000, 0x81c2, 0x433983c7, 0x81c2, 0x434d, 0x44064240, 0x84560000, 0x4404421c, 0x41d3, 0x83c70000, 0x848d0000, 0x431e0000, 0x432d, 0x81c20000, 0x0, 0x434b, 0x0, 0x8456423e, 0x42134161, 0x433584dc, 0x45100000, 0x0, 0x420d0000, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x423881c2, 0x81c24233, 0x42f842f7, 0x42f942f4, 0x81420000, 0x83c70000, 0x81c20000, 0x43064441, 0x422d, 0x42d342d4, 0x42d58068, 0x814242d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x81c20000, 0x81c281c2, 0x81c24014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x42260000, 0x83c742c4, 0x40b3, 0x0, 0x41574231, 0x83c70000, 0x44c681c2, 0x81c2422f, 0x0, 0x415c4239, 0x81c2, 0x44c044e6, 0x41ce0000, 0x83c70000, 0x433b83c7, 0x81c2421d, 0x434e4408, 0x845683c7, 0x44054219, 0x83c7446f, 0x83c7848d, 0x431f, 0x432f, 0x81c20000, 0x4214, 0x434c83c7, 0x4162, 0x433784dc, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f83c7, 0x4290, 0x4291, 0x40aa83c7, 0x81c2, 0x42a54150, 0x40b240a9, 0x42360000, 0x407b0000, 0x81c240e5, 0x81c24235, 0x4083438c, 0x81c240a8, 0x40a681c2, 0x4520407e, 0x407d4527, 0x409240b1, 0x81c281c2, 0x44c140a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874521, 0x407f4080, 0x45264091, 0x81c2407c, 0x422781c2, 0x451f83c7, 0x422b83c7, 0x44e744e9, 0x81c2416c, 0x40a781c2, 0x42164463, 0x41d40000, 0x0, 0x81c281c2, 0x81c20000, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x81c20000, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x421f, 0x417b421e, 0x447043bd, 0x45140000, 0x81c2, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x4229, 0x42284517, 0x0, 0x0, 0x0, 0x81c20000, 0x0, 0x0, 0x0, 0x431a, 0x431c81c2, 0x81c243b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x81c24220, 0x4184, 0x81c2, 0x41b8, 0x81c2, 0x81c20000, 0x4192, 0x4471, 0x81c244bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x81c20000, 0x4308, 0x430281c2, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x81c242ea, 0x42ed42e7, 0x81c24251, 0x427d81c2, 0x81c20000, 0x432681c2, 0x81c281c2, 0x81c24509, 0x81c281c2, 0x81c281c2, 0x450881c2, 0x81c20000, 0x81c281c2, 0x81c2449b, 0x449e, 0x44a0449d, 0x81c244a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c081c2, 0x81c240ae, 0x422a0000, 0x40da0000, 0x4158448c, 0x451940f6, 0x45180000, 0x433e44ee, 0x44a64159, 0x4152, 0x81c2438d, 0x81c24516, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c451b, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x447281c2, 0x44bd4512, 0x0, 0x81c2, 0x42a74292, 0x42a90000, 0x42a64151, 0x451381c2, 0x81c281c2, 0x81c281c2, 0x450c, 0x450b, 0x81c24211, 0x81c20000, 0x416d451d, 0x81c20000, 0x42ca, 0x42c881c2, 0x81c20000, 0x451c0000, 0x80684174, 0x845681c2, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x84560000, 0x43850000, 0x41640000, 0x41850000, 0x450e81c2, 0x81c2, 0x81c2450f, 0x81c244bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881c2, 0x81c281c2, 0x81c281c2, 0x40f34303, 0x430c4309, 0x430481c2, 0x81c2, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e881c2, 0x42ee81c2, 0x4322, 0x432481c2, 0x81c2, 0x81c20000, 0x81c281c2, 0x81c2, 0x44a2449f, 0x44a181c2, 0x42c7, 0x42c242c1, 0x81c240dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e81c2, 0x81c20000, 0x432781c2, 0x81c281c2, 0x81c244a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80684176, 0x81c2440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x81c20000, 0x81c281c2, 0x81c281c2, 0x81c2438a, 0x81c244eb, 0x81c281c2, 0x81c20000, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x81c20000, 0x42f1, 0x44e18142, 0x43880000, 0x446b81c2, 0x43bf441c, 0x0, 0x43c7, 0x81c281c2, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x81c2, 0x81c281c2, 0x81c2, 0x81c2, 0x81c281c2, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x81c281c2, 0x81c281c2, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c381c2, 0x81c20000, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f481c2, 0x4323, 0x432581c2, 0x81c20000, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x81c243b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x81c2, 0x81c281c2, 0x81c20000, 0x81c281c2, 0x42d9, 0x81c2, 0x81c24386, 0x43bb, 0x441e4149, 0x0, 0x81c2, 0x4474443f, 0x81c24453, 0x0, 0x42dd0000, 0x425a4253, 0x81c281c2, 0x81c244c8, 0x81c281c2, 0x81c240e1, 0x4487, 0x40f24321, 0x425b4280, 0x81c281c2, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x81c281c2, 0x81c20000, 0x0, 0x0, 0x81c281c2, 0x441f0000, 0x43c9, 0x81c281c2, 0x414a81c2, 0x42de, 0x42e142df, 0x42db4255, 0x81c24257, 0x81c20000, 0x425c4254, 0x81c281c2, 0x81c281c2, 0x448b4421, 0x43ca414c, 0x81c2, 0x81c281c2, 0x41ed0000, 0x81c28068, 0x4204, 0x81c242e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x81c24258, 0x81c241ee, 0x420581c2, 0x81c28068, 0x81c2, 0x42e4425e, 0x4262425f, 0x42824263, 0x428481c2, 0x81c24349, 0x81c24260, 0x4264434a, 0x81c281c2, 0x81c281c2, 0x81c20000 },
                { 0x80694055, 0x80694068, 0x40524049, 0x40604006, 0x40484067, 0x89dd0000, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80694043, 0x82e24042, 0x85dc402d, 0x80694062, 0x40408069, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668069, 0x81438069, 0x402a4004, 0x406a, 0x40288197, 0x81bf81c8, 0x8200, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x834e406e, 0x0, 0x82250000, 0x0, 0x82250000, 0x82250000, 0x836a8197, 0x0, 0x8265, 0x0, 0x0, 0x40090000, 0x81434021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81438069, 0x42f6, 0x806942f3, 0x42f50000, 0x44408069, 0x42d742cd, 0x40334030, 0x42cf8069, 0x403242cc, 0x42d18069, 0x8143402f, 0x40310000, 0x44c5, 0x81bf81bf, 0x42370000, 0x40138143, 0x80694011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a0000, 0x819781bf, 0x420c0000, 0x0, 0x40a5, 0x0, 0x0, 0x0, 0x8225, 0x42170000, 0x82250000, 0x82258197, 0x0, 0x83e78265, 0x420b83e7, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x0, 0x422e8225, 0x81978197, 0x81bf81bf, 0x81bf81bf, 0x81bf81bf, 0x81bf81bf, 0x420881bf, 0x81bf422c, 0x81bf4525, 0x423240b0, 0x81bf4081, 0x40a440a2, 0x420944bf, 0x408240af, 0x409040a3, 0x409f81bf, 0x44e50000, 0x81bf0000, 0x81bf, 0x43390000, 0x81bf, 0x434d, 0x44060000, 0x82250000, 0x44048225, 0x41d3, 0x82258225, 0x82250000, 0x431e0000, 0x8225432d, 0x81bf8225, 0x84b70000, 0x434b, 0x82654378, 0x8225423e, 0x42134161, 0x43358225, 0x45108265, 0x41948265, 0x420d0000, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x423881bf, 0x81bf4233, 0x42f842f7, 0x42f942f4, 0x81430000, 0x0, 0x81bf0000, 0x43064441, 0x422d, 0x42d342d4, 0x42d58069, 0x814342d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x81bf0000, 0x81bf81bf, 0x81bf4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c681bf, 0x81bf422f, 0x0, 0x415c4239, 0x822581bf, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x81bf0000, 0x434e4408, 0x82250000, 0x44058225, 0x446f, 0x82258225, 0x431f, 0x8225432f, 0x81bf8225, 0x84b74214, 0x434c8265, 0x4162, 0x43378225, 0x420e8265, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x81bf, 0x42a54150, 0x40b240a9, 0x81978225, 0x407b8225, 0x81bf40e5, 0x81bf4523, 0x4083438c, 0x81bf40a8, 0x40a681bf, 0x4520407e, 0x407d4527, 0x409240b1, 0x81bf81bf, 0x44c140a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874521, 0x407f4080, 0x45264091, 0x81bf407c, 0x81bf, 0x0, 0x0, 0x44e744e9, 0x81bf416c, 0x40a781bf, 0x42164463, 0x41d40000, 0x0, 0x81bf81bf, 0x81bf0000, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x82658197, 0x81bf8225, 0x0, 0x819741aa, 0x41ac0000, 0x41b10000, 0x440a8225, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x82258225, 0x81bf, 0x82258225, 0x82254215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x8225, 0x0, 0x0, 0x81bf0000, 0x0, 0x0, 0x0, 0x8197431a, 0x431c81bf, 0x81bf43b1, 0x432e43b5, 0x43eb84b7, 0x84b74400, 0x84b74401, 0x84b74372, 0x82658197, 0x81974376, 0x82654163, 0x0, 0x43368197, 0x81978197, 0x81978197, 0x81978225, 0x41d58197, 0x81bf8197, 0x81974184, 0x81bf, 0x41b8, 0x81bf, 0x874a8265, 0x41954192, 0x82654471, 0x81bf44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x81bf0000, 0x4308, 0x430281bf, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x81bf42ea, 0x42ed42e7, 0x81bf4251, 0x427d81bf, 0x81bf0000, 0x432681bf, 0x81bf81bf, 0x81bf4509, 0x81bf81bf, 0x81bf81bf, 0x450881bf, 0x81bf0000, 0x81bf81bf, 0x81bf449b, 0x449e, 0x44a0449d, 0x81bf44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c081bf, 0x81bf40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x81bf438d, 0x81bf0000, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c8225, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x82654165, 0x43384186, 0x41b94193, 0x447281bf, 0x44bd4512, 0x0, 0x81bf, 0x42a74292, 0x42a90000, 0x42a64151, 0x451381bf, 0x81bf81bf, 0x81bf81bf, 0x450c, 0x450b, 0x81bf4211, 0x81bf0000, 0x416d0000, 0x81bf0000, 0x42ca, 0x42c881bf, 0x81bf0000, 0x0, 0x4174, 0x819781bf, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x84b7, 0x84b784b7, 0x43e70000, 0x43708265, 0x81974379, 0x437a437b, 0x41640000, 0x41858265, 0x450e81bf, 0x419681bf, 0x81bf450f, 0x81bf44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881bf, 0x81bf81bf, 0x81bf81bf, 0x40f34303, 0x430c4309, 0x430481bf, 0x81bf, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e881bf, 0x42ee81bf, 0x4322, 0x432481bf, 0x81bf, 0x81bf0000, 0x81bf81bf, 0x81bf, 0x44a2449f, 0x44a181bf, 0x42c7, 0x42c242c1, 0x81bf40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e81bf, 0x81bf0000, 0x432781bf, 0x81bf81bf, 0x81bf44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x81bf440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x81bf0000, 0x81bf81bf, 0x81bf81bf, 0x81bf438a, 0x81bf44eb, 0x81bf81bf, 0x81bf0000, 0x41fa, 0x418d4465, 0x826541ef, 0x0, 0x81bf0000, 0x42f1, 0x0, 0x43880000, 0x446b81bf, 0x43bf441c, 0x0, 0x43c7, 0x81bf81bf, 0x43b34286, 0x0, 0x43b743e8, 0x84b78265, 0x42068265, 0x43744355, 0x81bf, 0x874a81bf, 0x4197874a, 0x89e4874a, 0x81bf81bf, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x81bf81bf, 0x81bf81bf, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c381bf, 0x81bf0000, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f481bf, 0x4323, 0x432581bf, 0x81bf0000, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x826541f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x81bf43b4, 0x42870000, 0x43b843ea, 0x82654207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x81bf, 0x81bf81bf, 0x81bf0000, 0x81bf81bf, 0x42d9, 0x81bf, 0x81bf4386, 0x43bb, 0x441e4149, 0x0, 0x826581bf, 0x4474443f, 0x81bf4453, 0x0, 0x42dd0000, 0x425a4253, 0x81bf81bf, 0x81bf44c8, 0x81bf81bf, 0x81bf40e1, 0x4487, 0x40f24321, 0x425b4280, 0x81bf81bf, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x81bf81bf, 0x81bf0000, 0x0, 0x0, 0x81bf81bf, 0x441f0000, 0x43c9, 0x81bf81bf, 0x414a81bf, 0x42de, 0x42e142df, 0x42db4255, 0x81bf4257, 0x81bf0000, 0x425c4254, 0x81bf81bf, 0x81bf81bf, 0x448b4421, 0x43ca414c, 0x81bf, 0x81bf81bf, 0x41ed0000, 0x81bf0000, 0x4204, 0x81bf42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x81bf4258, 0x81bf41ee, 0x420581bf, 0x81bf0000, 0x81bf, 0x42e4425e, 0x4262425f, 0x42824263, 0x428481bf, 0x81bf4349, 0x81bf4260, 0x4264434a, 0x81bf81bf, 0x81bf81bf, 0x81bf0000 },
                { 0x806a4055, 0x806a4068, 0x40524049, 0x40604006, 0x40484067, 0x820d820d, 0x403e406b, 0x406586b6, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x806a4043, 0x44cd4042, 0x419a402d, 0x806a4062, 0x4040806a, 0x4057405c, 0x405886cd, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x4025419b, 0x404f4064, 0x4010406c, 0x400584f9, 0x40614029, 0x40264007, 0x872344e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x889b44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066806a, 0x8144806a, 0x402a4004, 0x89bf406a, 0x40280000, 0x41980000, 0x0, 0x41ca, 0x427981f3, 0x40a10000, 0x81f341c2, 0x0, 0x406e, 0x845d0000, 0x0, 0x0, 0x41cd0000, 0x84570000, 0x0, 0x824b427b, 0x83d3, 0x41cf, 0x83d3, 0x40098457, 0x81444021, 0x44d84019, 0x401b4273, 0x42244230, 0x40174015, 0x8144806a, 0x427742f6, 0x806a42f3, 0x42f54275, 0x4440806a, 0x42d742cd, 0x40334030, 0x42cf806a, 0x403242cc, 0x42d1806a, 0x8144402f, 0x40314278, 0x43ed44c5, 0x84b943f5, 0x83d30000, 0x40138144, 0x806a4011, 0x43fc0000, 0x42ad4234, 0x42224223, 0x43f70000, 0x84dd0000, 0x43fb43ff, 0x43fd0000, 0x400b41c9, 0x415b4002, 0x0, 0x42120000, 0x420a0000, 0x0, 0x420c0000, 0x0, 0x40a5, 0x81f3, 0x83d30000, 0x4225, 0x0, 0x42170000, 0x81f30000, 0x848e0000, 0x824b427a, 0x83d30000, 0x420b0000, 0x400a400e, 0x41cb0000, 0x428d4288, 0x428b, 0x44dc, 0x4289845d, 0x0, 0x0, 0x422e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x41c340af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x842d, 0x43390000, 0x84400000, 0x434d, 0x44060000, 0x84570000, 0x44040000, 0x41d3, 0x0, 0x848e0000, 0x431e0000, 0x432d, 0x41cc0000, 0x84b90000, 0x434b, 0x0, 0x8457824b, 0x42134161, 0x433584dd, 0x45100000, 0x0, 0x420d0000, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x83d30000, 0x4233, 0x42f842f7, 0x42f942f4, 0x81440000, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d5806a, 0x814442d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x42260000, 0x42c4, 0x40b3, 0x43ef0000, 0x41574231, 0x83d30000, 0x44c643f6, 0x422f, 0x0, 0x415c83d3, 0x43f984b9, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x84570000, 0x44050000, 0x446f, 0x848e, 0x431f, 0x432f, 0x0, 0x84b94214, 0x434c0000, 0x824b4162, 0x433784dd, 0x420e0000, 0x451e41c5, 0x41c4428e, 0x428c428a, 0x428f0000, 0x4290, 0x41b64291, 0x40aa0000, 0x41b741a3, 0x42a54150, 0x40b240a9, 0x42360000, 0x407b0000, 0x40e5, 0x4235, 0x438c, 0x40a8, 0x40a60000, 0x40704077, 0x41a888d6, 0x407840b1, 0x41a54076, 0x44c140ab, 0x407340ad, 0x407940ac, 0x41990000, 0x0, 0x0, 0x0, 0x40714072, 0x0, 0x83d3, 0x0, 0x0, 0x42740000, 0x42270000, 0x451f83d3, 0x422b0000, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d4406f, 0x0, 0x0, 0x40a2, 0x4402433a, 0x407b0000, 0x41c0, 0x81f3420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x41d841d6, 0x41aa, 0x41ac4276, 0x41b186cc, 0x440a0000, 0x43314469, 0x427c0000, 0x417b0000, 0x447043bd, 0x0, 0x4172, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x41c6, 0x41b34229, 0x42284517, 0x43f1, 0x43f243fa, 0x43fe0000, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb43ee, 0x4400, 0x43f84401, 0x84b94372, 0x0, 0x4376, 0x4163, 0x824b44fd, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x45020000, 0x41d941b8, 0x41d70000, 0x874e0000, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x4431, 0x443443f4, 0x44384075, 0x44114414, 0x44164413, 0x44b2406f, 0x44b544b7, 0x44b40000, 0x41b441a7, 0x40a24308, 0x43024503, 0x430a4307, 0x444c, 0x41a40000, 0x42d642eb, 0x407442e6, 0x450442ea, 0x42ed42e7, 0x41a64251, 0x427d41a9, 0x0, 0x43260000, 0x41b540e2, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c0419c, 0x40ae, 0x422a0000, 0x40da0000, 0x4158448c, 0x451940f6, 0x45180000, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x419d0000, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c451b, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d451d, 0x0, 0x42ca, 0x42c80000, 0x0, 0x451c0000, 0x806a4174, 0x84570000, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x43f0, 0x84b90000, 0x43e70000, 0x43700000, 0x84570000, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x806a4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18144, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x43f30000, 0x42060000, 0x43744355, 0x0, 0x874e0000, 0x874e, 0x874e, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x806a, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x806a, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x806b4055, 0x806b4068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x806b4043, 0x4042, 0x402d, 0x806b4062, 0x4040806b, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066806b, 0x8145806b, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x406e, 0x0, 0x0, 0x0, 0x0, 0x84580000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40098458, 0x81454021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x8145806b, 0x42f6, 0x806b42f3, 0x42f50000, 0x4440806b, 0x42d742cd, 0x40334030, 0x42cf806b, 0x403242cc, 0x42d1806b, 0x8145402f, 0x40310000, 0x44c5, 0x0, 0x42370000, 0x40138145, 0x806b4011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a0000, 0x0, 0x420c0000, 0x0, 0x40a5, 0x0, 0x0, 0x0, 0x0, 0x42170000, 0x0, 0x0, 0x0, 0x83e80000, 0x420b83e8, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x0, 0x422e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x84580000, 0x44040000, 0x41d3, 0x0, 0x848f0000, 0x431e0000, 0x432d, 0x0, 0x84bc0000, 0x434b, 0x0, 0x8458423e, 0x42134161, 0x43350000, 0x45100000, 0x0, 0x420d0000, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x42380000, 0x4233, 0x42f842f7, 0x42f942f4, 0x81450000, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d5806b, 0x814542d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x422f, 0x0, 0x415c4239, 0x0, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x84580000, 0x44050000, 0x446f, 0x8628, 0x431f, 0x432f, 0x0, 0x84bc4214, 0x434c0000, 0x4162, 0x43370000, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb84bc, 0x84bc4400, 0x84bc4401, 0x84bc4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x874d0000, 0x4192, 0x4471, 0x875f44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x4472875f, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x806b4174, 0x84580000, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x84bc, 0x84bc84bc, 0x43e70000, 0x43700000, 0x84580000, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x875f44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x806b4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18145, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x84bc0000, 0x42060000, 0x43744355, 0x0, 0x874d0000, 0x874d, 0x874d, 0x875f89ee, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x806b, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x806b, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x806c4055, 0x806c4068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x806c4043, 0x4042, 0x402d, 0x806c4062, 0x4040806c, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066806c, 0x8146806c, 0x402a4004, 0x406a, 0x40280000, 0x81c10000, 0x83f50000, 0x0, 0x42790000, 0x40a183f5, 0x0, 0x0, 0x406e, 0x0, 0x81c10000, 0x0, 0x81c10000, 0x81c10000, 0x83f5, 0x83f5427b, 0x423a, 0x0, 0x4270, 0x40090000, 0x81464021, 0x4019, 0x401b4273, 0x42244230, 0x40174015, 0x8146806c, 0x427742f6, 0x806c42f3, 0x42f54275, 0x4440806c, 0x42d742cd, 0x40334030, 0x42cf806c, 0x403242cc, 0x42d1806c, 0x8146402f, 0x40314278, 0x44c5, 0x81c181c1, 0x423781c1, 0x40138146, 0x806c4011, 0x0, 0x42ad4234, 0x42224223, 0x83f5, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a0000, 0x81c1, 0x420c0000, 0x0, 0x40a5, 0x0, 0x42710000, 0x4225, 0x81c1, 0x42170000, 0x81c10000, 0x81c10000, 0x427a, 0x83f50000, 0x420b0000, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x44dc, 0x42890000, 0x83f50000, 0x0, 0x422e81c1, 0x0, 0x81c181c1, 0x81c10000, 0x81c10000, 0x0, 0x420881c1, 0x81c1422c, 0x81c14525, 0x423240b0, 0x81c10000, 0x40a440a2, 0x420944bf, 0x40af, 0x409040a3, 0x81c1, 0x44e50000, 0x81c10000, 0x81c1, 0x43390000, 0x81c1, 0x434d, 0x44060000, 0x81c10000, 0x440481c1, 0x41d3, 0x81c1, 0x81c10000, 0x431e0000, 0x81c1432d, 0x81c181c1, 0x0, 0x434b, 0x0, 0x81c183f5, 0x42134161, 0x433581c1, 0x45100000, 0x0, 0x420d0000, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x423881c1, 0x81c14233, 0x42f842f7, 0x42f942f4, 0x81460000, 0x0, 0x81c10000, 0x43064441, 0x422d, 0x42d342d4, 0x42d5806c, 0x814642d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x81c10000, 0x81c1, 0x81c14014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x422681c1, 0x42c4, 0x40b3, 0x0, 0x41574231, 0x423b0000, 0x44c681c1, 0x81c1422f, 0x81c10000, 0x415c4239, 0x81c181c1, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x81c10000, 0x434e4408, 0x81c10000, 0x440581c1, 0x446f, 0x81c181c1, 0x431f, 0x81c1432f, 0x81c181c1, 0x4214, 0x434c0000, 0x4162, 0x433781c1, 0x420e0000, 0x451e81c1, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x81c1, 0x42a54150, 0x40b240a9, 0x423681c1, 0x407b81c1, 0x81c140e5, 0x81c14235, 0x4083438c, 0x81c140a8, 0x40a681c1, 0x0, 0x4527, 0x409240b1, 0x81c181c1, 0x44c140a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874272, 0x0, 0x45264091, 0x42740000, 0x422781c1, 0x451f423c, 0x422b0000, 0x44e744e9, 0x81c1416c, 0x40a781c1, 0x42164463, 0x41d40000, 0x0, 0x81c181c1, 0x81c10000, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x81c181c1, 0x41d841d6, 0x83f541aa, 0x41ac4276, 0x41b10000, 0x440a81c1, 0x43314469, 0x427c0000, 0x417b0000, 0x447043bd, 0x0, 0x81c1, 0x0, 0x81c14215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x4229, 0x42284517, 0x81c1, 0x0, 0x0, 0x81c10000, 0x0, 0x0, 0x0, 0x431a, 0x431c81c1, 0x81c143b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x81c1, 0x41d50000, 0x81c10000, 0x4184, 0x81c1, 0x41d941b8, 0x41d781c1, 0x874b0000, 0x4192, 0x4471, 0x81c144bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x81c10000, 0x4308, 0x430281c1, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x81c142ea, 0x42ed42e7, 0x81c14251, 0x427d81c1, 0x81c10000, 0x432681c1, 0x81c181c1, 0x81c14509, 0x81c10000, 0x81c10000, 0x450881c1, 0x81c181c1, 0x81c1, 0x81c1449b, 0x449e, 0x44a0449d, 0x81c144a5, 0x44a80000, 0x81c142c6, 0x42c5, 0x42c081c1, 0x81c140ae, 0x422a0000, 0x40da0000, 0x4158448c, 0x451940f6, 0x45180000, 0x433e44ee, 0x44a64159, 0x4152, 0x81c1438d, 0x81c10000, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c451b, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c81c1, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x447281c1, 0x44bd4512, 0x81c10000, 0x81c1, 0x42a74292, 0x42a90000, 0x42a64151, 0x451381c1, 0x81c181c1, 0x81c181c1, 0x450c, 0x81c1450b, 0x81c14211, 0x81c10000, 0x416d451d, 0x81c10000, 0x42ca, 0x42c881c1, 0x81c10000, 0x451c0000, 0x806c4174, 0x81c1, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x450e81c1, 0x81c1, 0x81c1450f, 0x81c144bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881c1, 0x81c181c1, 0x81c181c1, 0x40f34303, 0x430c4309, 0x430481c1, 0x81c1, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e881c1, 0x42ee81c1, 0x4322, 0x432481c1, 0x81c1, 0x81c10000, 0x81c181c1, 0x81c1, 0x44a2449f, 0x44a181c1, 0x42c7, 0x42c242c1, 0x81c140dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e81c1, 0x81c10000, 0x432781c1, 0x81c181c1, 0x81c144a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x806c4176, 0x81c1440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x81c10000, 0x81c181c1, 0x81c181c1, 0x81c1438a, 0x81c144eb, 0x81c181c1, 0x81c10000, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x81c10000, 0x42f1, 0x44e18146, 0x43880000, 0x446b81c1, 0x43bf441c, 0x0, 0x43c7, 0x81c181c1, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x81c1, 0x874b81c1, 0x874b, 0x874b, 0x81c181c1, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x81c181c1, 0x81c181c1, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c381c1, 0x81c10000, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f481c1, 0x4323, 0x432581c1, 0x81c10000, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x81c143b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x81c1, 0x81c181c1, 0x81c10000, 0x81c181c1, 0x42d9, 0x81c1, 0x81c14386, 0x43bb, 0x441e4149, 0x0, 0x81c1, 0x4474443f, 0x81c14453, 0x0, 0x42dd0000, 0x425a4253, 0x81c181c1, 0x81c144c8, 0x81c181c1, 0x81c140e1, 0x4487, 0x40f24321, 0x425b4280, 0x81c181c1, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x81c181c1, 0x81c10000, 0x0, 0x0, 0x81c181c1, 0x441f0000, 0x43c9, 0x81c181c1, 0x414a81c1, 0x42de, 0x42e142df, 0x42db4255, 0x81c14257, 0x81c10000, 0x425c4254, 0x81c181c1, 0x81c181c1, 0x448b4421, 0x43ca414c, 0x81c1, 0x81c181c1, 0x41ed0000, 0x81c1806c, 0x4204, 0x81c142e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x81c14258, 0x81c141ee, 0x420581c1, 0x81c1806c, 0x81c1, 0x42e4425e, 0x4262425f, 0x42824263, 0x428481c1, 0x81c14349, 0x81c14260, 0x4264434a, 0x81c181c1, 0x81c181c1, 0x81c10000 },
                { 0x806d4055, 0x806d4068, 0x40524049, 0x40604006, 0x40484067, 0x806d806d, 0x403e406b, 0x4065806d, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x806d4043, 0x44964042, 0x806d402d, 0x44944062, 0x4040806d, 0x4057405c, 0x4058806d, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40254497, 0x404f4064, 0x4010406c, 0x4005806d, 0x40614029, 0x40264007, 0x806d44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x806d44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066806d, 0x806d806d, 0x402a4004, 0x4498406a, 0x40284499, 0x81c440f9, 0x449640fe, 0x40244022, 0x806d806d, 0x40a1401c, 0x403c806d, 0x403a4495, 0x40fb406e, 0x4039401e, 0x806d403b, 0x806d806d, 0x806d806d, 0x806d4444, 0x40fa806d, 0x806d806d, 0x44454442, 0x40fc806d, 0x40fd806d, 0x400940b6, 0x806d4021, 0x40b74019, 0x401b806d, 0x40b44036, 0x40174015, 0x806d806d, 0x4038806d, 0x42f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x806d0000, 0x4035, 0x44c344c5, 0x81c481c4, 0x4237806d, 0x4013806d, 0x806d4011, 0x444a806d, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b4455, 0x415b4002, 0x44960000, 0x4212806d, 0x420a401d, 0x81c4, 0x420c401f, 0x445a4459, 0x806d40a5, 0x44570000, 0x806d4454, 0x806d806d, 0x44430000, 0x4217806d, 0x40b5, 0x84900000, 0x40be40c0, 0x83e944c4, 0x420b83e9, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x444b442d, 0x422e0000, 0x4034440e, 0x81c481c4, 0x81c481c4, 0x81c481c4, 0x81c481c4, 0x420881c4, 0x81c4422c, 0x81c44525, 0x423240b0, 0x81c44081, 0x40a440a2, 0x420944bf, 0x408240af, 0x409040a3, 0x409f81c4, 0x44e540ff, 0x81c44122, 0x410381c4, 0x43390000, 0x444881c4, 0x4101434d, 0x44060000, 0x4446, 0x44040000, 0x40b841d3, 0x0, 0x8490442a, 0x431e445b, 0x432d, 0x81c40000, 0x4458, 0x4100434b, 0x445640c6, 0x423e, 0x42134161, 0x43350000, 0x45100000, 0x40bf40c1, 0x420d4102, 0x410440b9, 0x440f4020, 0x44304449, 0x442e4447, 0x442b401a, 0x445c4410, 0x44124018, 0x420c4016, 0x44b140c7, 0x44b340f5, 0x423881c4, 0x81c44233, 0x0, 0x42f4, 0x40ba445d, 0x81c481c4, 0x81c440c4, 0x43064441, 0x422d, 0x42d342d4, 0x42d50000, 0x40bb0000, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x81c40000, 0x81c481c4, 0x81c44014, 0x4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40bc40b3, 0x40c2, 0x41570000, 0x40bd0000, 0x44c681c4, 0x81c4422f, 0x40c3, 0x415c4239, 0x81c4, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x81c40000, 0x434e4408, 0x0, 0x44050000, 0x446f, 0x8490, 0x431f, 0x432f, 0x81c40000, 0x4214, 0x434c0000, 0x4162, 0x43370000, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x81c4, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x81c440e5, 0x81c40000, 0x4083438c, 0x81c440a8, 0x40a681c4, 0x4520407e, 0x407d4527, 0x409240b1, 0x81c481c4, 0x44c140a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874521, 0x407f4080, 0x45264091, 0x81c4407c, 0x81c4, 0x0, 0x0, 0x44e744e9, 0x81c4416c, 0x40a781c4, 0x42164463, 0x41d40000, 0x4128412a, 0x81c481c4, 0x81c40000, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x81c40000, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x81c4, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81c40000, 0x0, 0x0, 0x0, 0x431a, 0x431c81c4, 0x81c443b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x81c40000, 0x4184, 0x81c4, 0x41b8, 0x81c4, 0x81c40000, 0x4192, 0x4471, 0x81c444bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x81c40000, 0x4308, 0x81c4, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x81c442ea, 0x42ed0000, 0x81c44251, 0x427d81c4, 0x81c40000, 0x432681c4, 0x81c481c4, 0x81c44509, 0x81c481c4, 0x81c481c4, 0x450881c4, 0x81c40000, 0x81c481c4, 0x81c4449b, 0x449e, 0x44a0449d, 0x81c444a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c081c4, 0x81c440ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x81c4438d, 0x81c40000, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x447281c4, 0x44bd4512, 0x0, 0x81c4, 0x42a74292, 0x42a90000, 0x42a64151, 0x451381c4, 0x81c481c4, 0x81c481c4, 0x4121450c, 0x450b, 0x81c44211, 0x81c40000, 0x416d0000, 0x81c44127, 0x410542ca, 0x42c881c4, 0x81c40000, 0x0, 0x4174, 0x81c4, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x450e81c4, 0x81c4, 0x81c4450f, 0x81c444bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881c4, 0x81c481c4, 0x81c481c4, 0x40f30000, 0x430c4309, 0x81c4, 0x81c4, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x81c4, 0x42ee81c4, 0x4322, 0x432481c4, 0x81c4, 0x81c40000, 0x81c481c4, 0x81c4, 0x44a2449f, 0x44a181c4, 0x42c7, 0x42c242c1, 0x81c440dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e81c4, 0x81c40000, 0x432781c4, 0x81c481c4, 0x81c444a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x81c4440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x81c40000, 0x81c481c4, 0x81c481c4, 0x81c4438a, 0x81c444eb, 0x81c481c4, 0x81c40000, 0x412041fa, 0x418d4465, 0x41ef, 0x41294125, 0x81c44126, 0x42f1, 0x0, 0x43880000, 0x446b81c4, 0x43bf441c, 0x0, 0x43c7, 0x81c481c4, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x81c4, 0x81c481c4, 0x81c4, 0x81c4, 0x81c481c4, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x81c481c4, 0x81c481c4, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c381c4, 0x81c40000, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f481c4, 0x4323, 0x432581c4, 0x81c40000, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x81c443b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x81c4, 0x81c481c4, 0x81c40000, 0x81c481c4, 0x0, 0x81c4, 0x81c44386, 0x43bb, 0x441e4149, 0x0, 0x81c4, 0x4474443f, 0x81c44453, 0x0, 0x0, 0x425a4253, 0x81c481c4, 0x81c444c8, 0x81c481c4, 0x81c440e1, 0x4487, 0x40f24321, 0x425b4280, 0x81c481c4, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x81c481c4, 0x81c40000, 0x0, 0x41240000, 0x81c481c4, 0x441f0000, 0x43c9, 0x81c481c4, 0x414a81c4, 0x0, 0x0, 0x4255, 0x81c44257, 0x81c40000, 0x425c4254, 0x81c481c4, 0x81c481c4, 0x448b4421, 0x43ca414c, 0x81c4, 0x81c481c4, 0x41ed4123, 0x81c40000, 0x4204, 0x81c40000, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x81c44258, 0x81c441ee, 0x420581c4, 0x81c40000, 0x81c4, 0x425e, 0x4262425f, 0x42824263, 0x428481c4, 0x81c44349, 0x81c44260, 0x4264434a, 0x81c481c4, 0x81c481c4, 0x81c40000 },
                { 0x806e4055, 0x806e4068, 0x40524049, 0x40604006, 0x40484067, 0x852e87da, 0x403e406b, 0x406580fc, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x806e4043, 0x88ab4042, 0x402d, 0x806e4062, 0x4040806e, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066806e, 0x8147806e, 0x402a4004, 0x406a, 0x40280000, 0x81c30000, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x406e, 0x0, 0x0, 0x80fc, 0x0, 0x8296, 0x0, 0x0, 0x44d90000, 0x0, 0x0, 0x40090000, 0x81474021, 0x83374019, 0x401b0000, 0x0, 0x40174015, 0x8147806e, 0x42f6, 0x806e42f3, 0x42f50000, 0x4440806e, 0x42d742cd, 0x40334030, 0x42cf806e, 0x403242cc, 0x42d1806e, 0x8147402f, 0x40310000, 0x44c5, 0x81c381c3, 0x8296, 0x40138147, 0x806e4011, 0x0, 0x42ad0000, 0x0, 0x0, 0x82960000, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x81c3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8579, 0x84910000, 0x0, 0x81c30000, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42898337, 0x80fc85ad, 0x0, 0x0, 0x0, 0x81c30000, 0x81c30000, 0x0, 0x0, 0x81c3, 0x0, 0x81c30000, 0x0, 0x81c30000, 0x0, 0x44bf, 0x0, 0x0, 0x81c3, 0x44e50000, 0x81c30000, 0x81c3, 0x43390000, 0x44ce81c3, 0x434d, 0x44060000, 0x0, 0x44040000, 0x41d3, 0x0, 0x84910000, 0x431e0000, 0x432d, 0x81c30000, 0x0, 0x434b, 0x0, 0x8805, 0x4161, 0x43350000, 0x0, 0x0, 0x8337, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x81c3, 0x81c30000, 0x42f842f7, 0x42f942f4, 0x81470000, 0x0, 0x81c30000, 0x43064441, 0x0, 0x42d342d4, 0x42d5806e, 0x814742d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x81c30000, 0x81c3, 0x81c34014, 0x4012, 0x449a80fc, 0x449c433d, 0x44ed44a4, 0x42ae, 0x8296, 0x42c4, 0x40b3, 0x8a06, 0x41570000, 0x8a0d, 0x44c681c3, 0x81c30000, 0x0, 0x415c8337, 0x81c3, 0x44c044e6, 0x44f7, 0x0, 0x433b8a8f, 0x81c30000, 0x434e4408, 0x0, 0x44050000, 0x8a930000, 0x8491, 0x431f, 0x432f, 0x81c30000, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x81c3, 0x42a54150, 0x0, 0x0, 0x0, 0x81c30000, 0x81c30000, 0x438c, 0x81c30000, 0x81c3, 0x0, 0x4527, 0x40920000, 0x81c381c3, 0x44c140a0, 0x45284094, 0x40930000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x45264091, 0x0, 0x81c3, 0x0, 0x0, 0x44e744e9, 0x81c3416c, 0x81c3, 0x4463, 0x41d40000, 0x412a, 0x81c381c3, 0x81c30000, 0x4402433a, 0x0, 0x41c0, 0x0, 0x417344f7, 0x44f6434f, 0x43514407, 0x0, 0x81c30000, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a85ad, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x81c3, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81c30000, 0x0, 0x0, 0x0, 0x431a, 0x431c81c3, 0x81c343b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x85ad, 0x0, 0x0, 0x41d50000, 0x81c30000, 0x4184, 0x81c3, 0x41b8, 0x81c3, 0x87500000, 0x4192, 0x4471, 0x81c344bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x81c30000, 0x4308, 0x430281c3, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x81c342ea, 0x42ed42e7, 0x81c34251, 0x427d81c3, 0x81c30000, 0x432681c3, 0x81c381c3, 0x81c30000, 0x81c38a61, 0x81c30000, 0x81c3, 0x81c30000, 0x81c3, 0x81c3449b, 0x449e, 0x44a0449d, 0x81c344a5, 0x44a88296, 0x42c6, 0x42c5, 0x42c081c3, 0x81c30000, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x81c3438d, 0x81c30000, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x85ad0000, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x447281c3, 0x44bd0000, 0x0, 0x81c3, 0x42a74292, 0x42a90000, 0x42a64151, 0x81c3, 0x81c381c3, 0x81c381c3, 0x0, 0x0, 0x81c30000, 0x81c30000, 0x416d0000, 0x81c30000, 0x42ca, 0x42c881c3, 0x81c30000, 0x0, 0x806e4174, 0x81c3, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x81c3, 0x81c3, 0x81c30000, 0x81c344bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881c3, 0x81c381c3, 0x81c381c3, 0x40f34303, 0x430c4309, 0x430481c3, 0x81c3, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e881c3, 0x42ee81c3, 0x4322, 0x432481c3, 0x81c3, 0x81c30000, 0x81c381c3, 0x81c3, 0x44a2449f, 0x44a181c3, 0x42c7, 0x42c242c1, 0x81c340dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e81c3, 0x81c30000, 0x432781c3, 0x81c381c3, 0x81c344a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x806e4176, 0x81c3440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x81c30000, 0x81c381c3, 0x81c381c3, 0x81c3438a, 0x81c344eb, 0x81c381c3, 0x81c30000, 0x41fa, 0x418d4465, 0x41ef, 0x41290000, 0x81c30000, 0x42f1, 0x44e18147, 0x43880000, 0x446b81c3, 0x43bf441c, 0x0, 0x43c7, 0x81c381c3, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x81c3, 0x875081c3, 0x8750, 0x8750, 0x81c381c3, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x81c381c3, 0x81c381c3, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c381c3, 0x81c30000, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f481c3, 0x4323, 0x432581c3, 0x81c30000, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x81c343b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x81c3, 0x81c381c3, 0x81c30000, 0x81c381c3, 0x42d9, 0x81c3, 0x81c34386, 0x43bb, 0x441e4149, 0x0, 0x81c3, 0x4474443f, 0x81c34453, 0x0, 0x42dd0000, 0x425a4253, 0x81c381c3, 0x81c344c8, 0x81c381c3, 0x81c340e1, 0x4487, 0x40f24321, 0x425b4280, 0x81c381c3, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x81c381c3, 0x81c30000, 0x0, 0x0, 0x81c381c3, 0x441f0000, 0x43c9, 0x81c381c3, 0x414a81c3, 0x42de, 0x42e142df, 0x42db4255, 0x81c34257, 0x81c30000, 0x425c4254, 0x81c381c3, 0x81c381c3, 0x448b4421, 0x43ca414c, 0x81c3, 0x81c381c3, 0x41ed0000, 0x81c3806e, 0x4204, 0x81c342e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x81c34258, 0x81c341ee, 0x420581c3, 0x81c3806e, 0x81c3, 0x42e4425e, 0x4262425f, 0x42824263, 0x428481c3, 0x81c34349, 0x81c34260, 0x4264434a, 0x81c381c3, 0x81c381c3, 0x81c30000 },
                { 0x806f0000, 0x827f, 0x0, 0x4006, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40080000, 0x0, 0x4001, 0x0, 0x0, 0x80e70000, 0x0, 0x0, 0x0, 0x0, 0x400c, 0x0, 0x40030000, 0x44e1, 0x0, 0x0, 0x0, 0x0, 0x40050000, 0x0, 0x4007, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x806f, 0x81480000, 0x4004, 0x0, 0x85170000, 0x0, 0x0, 0x0, 0x42790000, 0x40a10000, 0x0, 0x0, 0x406e, 0x0, 0x82c70000, 0x0, 0x0, 0x4444, 0x0, 0x427b, 0x44454442, 0x80e7, 0x0, 0x40090000, 0x4021, 0x4019, 0x401b0000, 0x422483fa, 0x40174015, 0x0, 0x42770000, 0x0, 0x83fa, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4278, 0x0, 0x0, 0x0, 0x40130000, 0x4011, 0x444a0000, 0x0, 0x42224223, 0x0, 0x84de0000, 0x0, 0x0, 0x400b4455, 0x4002, 0x0, 0x421282c7, 0x420a0000, 0x0, 0x420c0000, 0x4459, 0x40a5, 0x44570000, 0x4454, 0x4225, 0x44430000, 0x42170000, 0x0, 0x84920000, 0x427a, 0x83fa0000, 0x420b0000, 0x400a0000, 0x0, 0x0, 0x0, 0x44dc, 0x0, 0x0, 0x444b0000, 0x83fa0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x40b0, 0x0, 0x40a440a2, 0x42090000, 0x40af, 0x40a3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44480000, 0x0, 0x44060000, 0x4446, 0x44040000, 0x41d3, 0x0, 0x84920000, 0x0, 0x0, 0x0, 0x4458, 0x0, 0x44560000, 0x0, 0x42130000, 0x84de, 0x45100000, 0x0, 0x420d0000, 0x0, 0x4020, 0x44304449, 0x442e4447, 0x401a, 0x445c4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b30000, 0x0, 0x0, 0x0, 0x0, 0x445d, 0x0, 0x0, 0x0, 0x422d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c0000, 0x0, 0x0, 0x42260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41ce0000, 0x0, 0x0, 0x0, 0x4408, 0x0, 0x44050000, 0x446f, 0x8492, 0x0, 0x0, 0x0, 0x4214, 0x0, 0x0, 0x84de, 0x420e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40aa0000, 0x0, 0x0, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x0, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42270000, 0x0, 0x422b0000, 0x0, 0x0, 0x40a70000, 0x42160000, 0x41d40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x420c, 0x0, 0x0, 0x4407, 0x0, 0x0, 0x41d841d6, 0x0, 0x0, 0x0, 0x440a0000, 0x0, 0x427c0000, 0x0, 0x44700000, 0x0, 0x0, 0x0, 0x4215, 0x0, 0x41d2420f, 0x45070000, 0x0, 0x4229, 0x42284517, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x0, 0x0, 0x41d90000, 0x41d70000, 0x0, 0x0, 0x0, 0x0, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40ae, 0x422a0000, 0x0, 0x0, 0x45190000, 0x45180000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x45110000, 0x451b, 0x0, 0x0, 0x44090000, 0x0, 0x440c0000, 0x0, 0x450a0000, 0x0, 0x4210, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4512, 0x0, 0x0, 0x0, 0x0, 0x0, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x451d, 0x0, 0x0, 0x0, 0x0, 0x451c0000, 0x0, 0x0, 0x0, 0x440b0000, 0x0, 0x450d0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x450e0000, 0x0, 0x450f, 0x0, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x440d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4437, 0x443e443b, 0x443d4419, 0x44ba0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x443f, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80704055, 0x80704068, 0x40524049, 0x40604006, 0x40484067, 0x80708070, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80704043, 0x829e4042, 0x402d, 0x4062, 0x40408070, 0x4057405c, 0x4058829e, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668070, 0x81498070, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x81498070, 0x40a1401c, 0x403c8149, 0x403a0000, 0x406e, 0x4039401e, 0x403b, 0x0, 0x8070, 0x0, 0x0, 0x80708070, 0x0, 0x0, 0x0, 0x40090000, 0x81494021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81498070, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x42370000, 0x40138149, 0x80704011, 0x8149, 0x42ad0000, 0x0, 0x0, 0x84df0000, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a401d, 0x0, 0x420c401f, 0x445a0000, 0x807040a5, 0x0, 0x80700000, 0x81490000, 0x0, 0x42170000, 0x0, 0x84930000, 0x0, 0x83ea44c4, 0x420b83ea, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x442d, 0x422e0000, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x41d3, 0x0, 0x8493442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x423e, 0x42134161, 0x433584df, 0x45100000, 0x0, 0x420d0000, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x42380000, 0x4233, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x422f, 0x0, 0x415c4239, 0x0, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x446f, 0x8493, 0x431f, 0x432f, 0x0, 0x4214, 0x434c0000, 0x4162, 0x433784df, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80714055, 0x80714068, 0x40524049, 0x40604006, 0x40484067, 0x8071, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80714043, 0x4042, 0x402d, 0x4062, 0x40408071, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668071, 0x814a8071, 0x402a4004, 0x406a, 0x40288198, 0x0, 0x81e0, 0x0, 0x42790000, 0x40a1401c, 0x403c814a, 0x403a0000, 0x406e, 0x4039401e, 0x8226403b, 0x0, 0x82268071, 0x82268297, 0x8244, 0x427b, 0x44d983d4, 0x0, 0x83d4, 0x40090000, 0x814a4021, 0x4019, 0x401b4273, 0x42244230, 0x40174015, 0x814a8071, 0x42770000, 0x42f3, 0x42f54275, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x4278, 0x44c344c5, 0x0, 0x83d40000, 0x4013814a, 0x80714011, 0x0, 0x42ad4234, 0x42224223, 0x0, 0x852c0000, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a401d, 0x81980000, 0x420c401f, 0x0, 0x40a5, 0x0, 0x83d40000, 0x4225, 0x8226, 0x42170000, 0x82260000, 0x82268244, 0x427a, 0x83d444c4, 0x420b0000, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x44dc, 0x42890000, 0x85ae, 0x0, 0x422e8226, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x842c, 0x43390000, 0x44ce0000, 0x434d, 0x44060000, 0x82260000, 0x44048226, 0x41d3, 0x8226, 0x82260000, 0x431e0000, 0x8226432d, 0x8226, 0x0, 0x434b, 0x0, 0x84d10000, 0x42134161, 0x43358226, 0x45100000, 0x0, 0x420d0000, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x83d40000, 0x853a4233, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x859f42ae, 0x42260000, 0x42c4, 0x40b3, 0x0, 0x41574231, 0x83d40000, 0x44c60000, 0x422f, 0x0, 0x415c83d4, 0x82260000, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x82260000, 0x44058226, 0x446f, 0x82268226, 0x431f, 0x8226432f, 0x8226, 0x4214, 0x434c0000, 0x4162, 0x43378226, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x42368226, 0x407b0000, 0x40e5, 0x4235, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x83d4, 0x0, 0x0, 0x42740000, 0x42270000, 0x451f83d4, 0x422b0000, 0x44e744e9, 0x416c, 0x40a7853a, 0x42164463, 0x41d488d4, 0x0, 0x842c0000, 0x842c88da, 0x4402433a, 0x407b0000, 0x41c0, 0x420c, 0x41730000, 0x44f6434f, 0x43514407, 0x8198, 0x8226, 0x41d841d6, 0x41aa, 0x41ac4276, 0x41b10000, 0x440a8226, 0x43314469, 0x427c0000, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x82264215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x4229, 0x42284517, 0x8226, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x8a11, 0x0, 0x8226, 0x41d50000, 0x0, 0x4184, 0x853a, 0x41d941b8, 0x41d70000, 0x0, 0x4192, 0x4471, 0x875e44bb, 0x859f4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x853a0000, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a8859f, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x422a0000, 0x40da0000, 0x4158448c, 0x451940f6, 0x45180000, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c451b, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c8226, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x4472875e, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d451d, 0x0, 0x42ca, 0x42c80000, 0x0, 0x451c0000, 0x4174, 0x81980000, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x81980000, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x875e44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b8853a, 0x88f588f8, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x859f42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x875e89ed, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x8a6f4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] value1 = null;

        protected static void value1Init()
        {
            value1 = new int[][]
            {
                { 0x80724055, 0x80724068, 0x40524049, 0x40604006, 0x40484067, 0x80728072, 0x403e406b, 0x406585b4, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80724043, 0x829f4042, 0x85b4402d, 0x85b44062, 0x40408072, 0x4057405c, 0x4058829f, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40258481, 0x404f4064, 0x4010406c, 0x40058481, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668072, 0x814b8072, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x814b8072, 0x40a1401c, 0x403c814b, 0x403a0000, 0x406e, 0x4039401e, 0x403b, 0x0, 0x83bc8072, 0x84530000, 0x0, 0x80728072, 0x0, 0x0, 0x0, 0x40098453, 0x814b4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x814b8072, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x42370000, 0x4013814b, 0x80724011, 0x814b, 0x42ad0000, 0x0, 0x0, 0x84e10000, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a401d, 0x0, 0x420c401f, 0x445a0000, 0x807240a5, 0x0, 0x80720000, 0x814b0000, 0x0, 0x42170000, 0x0, 0x84940000, 0x0, 0x83eb44c4, 0x420b83eb, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x442d, 0x422e83bc, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x84530000, 0x44040000, 0x41d3, 0x8481, 0x8494442a, 0x431e445b, 0x8481432d, 0x8481, 0x0, 0x434b, 0x0, 0x8453423e, 0x42134161, 0x433584e1, 0x45100000, 0x0, 0x420d0000, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x42380000, 0x4233, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x422f, 0x0, 0x415c4239, 0x83bc0000, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x84530000, 0x44050000, 0x446f, 0x84818494, 0x431f, 0x8481432f, 0x8481, 0x4214, 0x434c0000, 0x4162, 0x433784e1, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a8481, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c8481, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x84530000, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x84530000, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80734055, 0x80734068, 0x40524049, 0x40604006, 0x40484067, 0x80738073, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80734043, 0x82a04042, 0x402d, 0x4062, 0x40408073, 0x4057405c, 0x405882a0, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40258482, 0x404f4064, 0x4010406c, 0x40058482, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668073, 0x814c8073, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x814c8073, 0x40a1401c, 0x403c814c, 0x403a0000, 0x406e, 0x4039401e, 0x403b, 0x0, 0x83bd8073, 0x84540000, 0x0, 0x80738073, 0x0, 0x0, 0x0, 0x40098454, 0x814c4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x814c8073, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x42370000, 0x4013814c, 0x80734011, 0x814c, 0x42ad0000, 0x0, 0x0, 0x84e20000, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a401d, 0x0, 0x420c401f, 0x445a0000, 0x807340a5, 0x0, 0x80730000, 0x814c0000, 0x0, 0x42170000, 0x0, 0x84950000, 0x0, 0x83ec44c4, 0x420b83ec, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x442d, 0x422e83bd, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x84540000, 0x44040000, 0x41d3, 0x8482, 0x8495442a, 0x431e445b, 0x8482432d, 0x8482, 0x0, 0x434b, 0x0, 0x8454423e, 0x42134161, 0x433584e2, 0x45100000, 0x0, 0x420d0000, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x42380000, 0x4233, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x422f, 0x0, 0x415c4239, 0x83bd0000, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x84540000, 0x44050000, 0x446f, 0x84828495, 0x431f, 0x8482432f, 0x8482, 0x4214, 0x434c0000, 0x4162, 0x433784e2, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a8482, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c8482, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x84540000, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x84540000, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80744055, 0x80744068, 0x40524049, 0x40604006, 0x40484067, 0x80748074, 0x403e406b, 0x40658559, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80744043, 0x87914042, 0x402d, 0x4062, 0x40408074, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668074, 0x814d8074, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x814d8074, 0x40a1401c, 0x403c814d, 0x403a0000, 0x406e, 0x4039401e, 0x403b, 0x0, 0x8074, 0x0, 0x0, 0x80748074, 0x0, 0x0, 0x0, 0x40090000, 0x814d4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x814d8074, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x42370000, 0x4013814d, 0x80744011, 0x814d, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a401d, 0x42450000, 0x420c401f, 0x445a0000, 0x807440a5, 0x4243, 0x80740000, 0x814d0000, 0x83f7, 0x42170000, 0x0, 0x84960000, 0x0, 0x83f744c4, 0x420b4241, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x442d, 0x422e0000, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x41d3, 0x42420000, 0x8496442a, 0x431e445b, 0x4244432d, 0x4246, 0x0, 0x434b, 0x0, 0x423e, 0x42134161, 0x43350000, 0x451083f7, 0x0, 0x420d0000, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x42380000, 0x4233, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x422f, 0x0, 0x415c4239, 0x0, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x446f, 0x8496, 0x431f, 0x432f, 0x0, 0x4214, 0x434c0000, 0x4162, 0x43370000, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80754055, 0x80754068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80754043, 0x4042, 0x402d, 0x80754062, 0x40408075, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40258483, 0x404f4064, 0x4010406c, 0x40058483, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668075, 0x814e8075, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x406e, 0x0, 0x0, 0x0, 0x83be0000, 0x84550000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40098455, 0x814e4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x814e8075, 0x42f6, 0x807542f3, 0x42f50000, 0x44408075, 0x42d742cd, 0x40334030, 0x42cf8075, 0x403242cc, 0x42d18075, 0x814e402f, 0x40310000, 0x44c5, 0x0, 0x42370000, 0x4013814e, 0x80754011, 0x0, 0x42ad0000, 0x0, 0x0, 0x84e30000, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a0000, 0x0, 0x420c0000, 0x0, 0x40a5, 0x0, 0x0, 0x0, 0x0, 0x42170000, 0x0, 0x84970000, 0x0, 0x83ed0000, 0x420b83ed, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x0, 0x422e83be, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x84550000, 0x44040000, 0x41d3, 0x8483, 0x84970000, 0x431e0000, 0x8483432d, 0x8483, 0x0, 0x434b, 0x0, 0x8455423e, 0x42134161, 0x433584e3, 0x45100000, 0x0, 0x420d0000, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x42380000, 0x4233, 0x42f842f7, 0x42f942f4, 0x814e0000, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d58075, 0x814e42d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x422f, 0x0, 0x415c4239, 0x83be0000, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x84550000, 0x44050000, 0x446f, 0x84838497, 0x431f, 0x8483432f, 0x8483, 0x4214, 0x434c0000, 0x4162, 0x433784e3, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a8483, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c8483, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80754174, 0x84550000, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x84550000, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80754176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e1814e, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x8075, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8075, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80764055, 0x80764068, 0x40524049, 0x40604006, 0x40484067, 0x80768076, 0x403e406b, 0x40658076, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80764043, 0x44964042, 0x8076402d, 0x44944062, 0x40408076, 0x4057405c, 0x40588076, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40254497, 0x404f4064, 0x4010406c, 0x40058076, 0x40614029, 0x40264007, 0x807644e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x807644e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668076, 0x80768076, 0x402a4004, 0x4498406a, 0x40284499, 0x40f9, 0x449640fe, 0x40244022, 0x80768076, 0x40a1401c, 0x403c8076, 0x403a4495, 0x40fb406e, 0x4039401e, 0x8076403b, 0x80768076, 0x80768076, 0x80764444, 0x40fa8076, 0x80768076, 0x44454442, 0x40fc8076, 0x40fd8076, 0x400940b6, 0x80764021, 0x40b74019, 0x401b8076, 0x40b44036, 0x40174015, 0x80768076, 0x40388076, 0x42f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x80760000, 0x4035, 0x44c344c5, 0x0, 0x42378076, 0x40138076, 0x80764011, 0x444a8076, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b4455, 0x415b4002, 0x44960000, 0x42128076, 0x420a401d, 0x0, 0x420c401f, 0x445a4459, 0x807640a5, 0x44570000, 0x80764454, 0x80768076, 0x44430000, 0x42178076, 0x40b5, 0x84980000, 0x40be40c0, 0x83ee44c4, 0x420b83ee, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x444b442d, 0x422e0000, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e540ff, 0x4122, 0x41030000, 0x43390000, 0x44480000, 0x4101434d, 0x44060000, 0x4446, 0x44040000, 0x40b841d3, 0x0, 0x8498442a, 0x431e445b, 0x432d, 0x0, 0x4458, 0x4100434b, 0x445640c6, 0x423e, 0x42134161, 0x43350000, 0x45100000, 0x40bf40c1, 0x420d4102, 0x410440b9, 0x440f4020, 0x44304449, 0x442e4447, 0x442b401a, 0x445c4410, 0x44124018, 0x420c4016, 0x44b140c7, 0x44b340f5, 0x42380000, 0x4233, 0x0, 0x42f4, 0x40ba445d, 0x0, 0x40c4, 0x43064441, 0x422d, 0x42d342d4, 0x42d50000, 0x40bb0000, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40bc40b3, 0x40c2, 0x41570000, 0x40bd0000, 0x44c60000, 0x422f, 0x40c3, 0x415c4239, 0x0, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x446f, 0x8498, 0x431f, 0x432f, 0x0, 0x4214, 0x434c0000, 0x4162, 0x43370000, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x4128412a, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x4121450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x4127, 0x410542ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x412041fa, 0x418d4465, 0x41ef, 0x41294125, 0x4126, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed4123, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80774055, 0x80774068, 0x40524049, 0x40604006, 0x40484067, 0x80778077, 0x403e406b, 0x406580c5, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80774043, 0x44964042, 0x80c5402d, 0x44944062, 0x40408077, 0x4057405c, 0x405880c5, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40254497, 0x404f4064, 0x4010406c, 0x400580c5, 0x40614029, 0x40264007, 0x80c544e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x80c544e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668077, 0x814f8077, 0x402a4004, 0x4498406a, 0x40284499, 0x85bc40f9, 0x449640fe, 0x40244022, 0x814f8077, 0x40a1401c, 0x403c814f, 0x403a4495, 0x40fb406e, 0x4039401e, 0x80c5403b, 0x80c580c5, 0x80c58077, 0x80c54444, 0x40fa80c5, 0x80778077, 0x44454442, 0x40fc80c5, 0x40fd80c5, 0x400940b6, 0x814f4021, 0x40b74019, 0x401b80c5, 0x40b44036, 0x40174015, 0x814f8077, 0x403880c5, 0x87e742f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x80c50000, 0x4035, 0x44c344c5, 0x0, 0x423780c5, 0x4013814f, 0x80774011, 0x444a814f, 0x42ad0000, 0x0, 0x0, 0x84e40000, 0x0, 0x442c4023, 0x400b4455, 0x415b4002, 0x449683d7, 0x421280c5, 0x420a401d, 0x42450000, 0x420c401f, 0x445a4459, 0x807740a5, 0x44574243, 0x80774454, 0x814f80c5, 0x44434247, 0x421780c5, 0x40b5, 0x84990000, 0x40be40c0, 0x83d744c4, 0x420b4241, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x444b442d, 0x422e0000, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e540ff, 0x4122, 0x41030000, 0x43390000, 0x44480000, 0x4101434d, 0x44060000, 0x4446, 0x44040000, 0x40b841d3, 0x424283d7, 0x8499442a, 0x431e445b, 0x4244432d, 0x4246, 0x4458, 0x4100434b, 0x445640c6, 0x423e, 0x42134161, 0x433584e4, 0x45104248, 0x40bf40c1, 0x420d4102, 0x410440b9, 0x440f4020, 0x44304449, 0x442e4447, 0x442b401a, 0x445c4410, 0x44124018, 0x420c4016, 0x44b140c7, 0x44b340f5, 0x42380000, 0x4233, 0x0, 0x42f4, 0x40ba445d, 0x0, 0x40c4, 0x43064441, 0x422d, 0x42d342d4, 0x42d50000, 0x40bb0000, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40bc40b3, 0x40c2, 0x41570000, 0x40bd0000, 0x44c60000, 0x422f, 0x40c3, 0x415c4239, 0x0, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x446f, 0x8499, 0x431f, 0x432f, 0x0, 0x4214, 0x434c0000, 0x4162, 0x433784e4, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x4128412a, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x4121450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x4127, 0x410542ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x412041fa, 0x418d4465, 0x41ef, 0x41294125, 0x4126, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed4123, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x8078829a, 0x82a7832f, 0x83b7841c, 0x841f4006, 0x84308448, 0x820e820e, 0x846c8471, 0x84d680fd, 0x84f0853d, 0x40f785b6, 0x85d88601, 0x86078620, 0x400886aa, 0x4107410d, 0x41064001, 0x853b853b, 0x80d5877d, 0x813487ee, 0x87f7410a, 0x4108410c, 0x410e410f, 0x4110881d, 0x85d7400c, 0x888388aa, 0x40038935, 0x893844e1, 0x89698974, 0x89b18372, 0x853b89d8, 0x89f189f2, 0x4005818b, 0x89f38a15, 0x4007, 0x81d50000, 0x0, 0x0, 0x0, 0x0, 0x81d80000, 0x0, 0x0, 0x0, 0x0, 0x8078, 0x81500000, 0x4004, 0x88550000, 0x413e, 0x40f9, 0x857f40fe, 0x81e40000, 0x4279424b, 0x40a10000, 0x81fd0000, 0x412d, 0x40fb406e, 0x8214821a, 0x0, 0x412e8310, 0x0, 0x0, 0x40fa0000, 0x824d427b, 0x82570000, 0x40fc0000, 0x40fd4270, 0x400980d5, 0x4021, 0x83384019, 0x401b4273, 0x0, 0x40174015, 0x0, 0x42770000, 0x0, 0x4275, 0x0, 0x0, 0x0, 0x40f88372, 0x0, 0x0, 0x0, 0x4278, 0x43ed0000, 0x84bd43f5, 0x423782f6, 0x40130000, 0x4011, 0x43fc0000, 0x0, 0x0, 0x43f7839c, 0x84e50000, 0x43fb43ff, 0x43fd831d, 0x400b0000, 0x4002, 0x83344249, 0x42128134, 0x420a818b, 0x424540c8, 0x420c81d5, 0x8357413e, 0x835e40a5, 0x878c4243, 0x427181fd, 0x87c28377, 0x821a4247, 0x42170000, 0x87e2, 0x849a40ca, 0x824d427a, 0x82570000, 0x420b4241, 0x400a0000, 0x0, 0x0, 0x0, 0x44dc, 0x85a2, 0x423d0000, 0x0, 0x422e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x42090000, 0x40af, 0x40a3, 0x0, 0x40ff, 0x4122, 0x41030000, 0x864a, 0x0, 0x41010000, 0x44060000, 0x0, 0x44040000, 0x41d3, 0x4242424a, 0x849a0000, 0x0, 0x42440000, 0x4246, 0x84bd8855, 0x41000000, 0x0, 0x423e, 0x42130000, 0x84e5, 0x45104248, 0x0, 0x420d4102, 0x41048334, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b30000, 0x42380000, 0x853b4233, 0x0, 0x0, 0x0, 0x0, 0x854b0000, 0x0, 0x422d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x858a, 0x4014, 0x4012, 0x449a0000, 0x449c0000, 0x0, 0x0, 0x82f6, 0x0, 0x0, 0x43ef85bf, 0x0, 0x0, 0x43f6, 0x422f, 0x82f60000, 0x4239, 0x43f984bd, 0x0, 0x41ce0000, 0x0, 0x0, 0x0, 0x4408, 0x0, 0x44050000, 0x446f, 0x849a, 0x0, 0x424c0000, 0x0, 0x84bd4214, 0x0, 0x0, 0x84e5, 0x420e0000, 0x451e82f6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40aa0000, 0x0, 0x0, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x86700000, 0x0, 0x40a8, 0x40a68670, 0x0, 0x0, 0x40b1, 0x86700000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x893f0000, 0x0, 0x0, 0x4109, 0x4272, 0x410b, 0x0, 0x42740000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a7853b, 0x42160000, 0x41d40000, 0x4128412a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4407, 0x891c, 0x0, 0x41d841d6, 0x89220000, 0x4276, 0x0, 0x440a0000, 0x0, 0x427c0000, 0x0, 0x44700000, 0x8997, 0x8670, 0x0, 0x4215, 0x0, 0x41d2420f, 0x45070000, 0x41e00000, 0x0, 0x0, 0x43f1, 0x43f243fa, 0x43fe0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43ee, 0x4400, 0x43f84401, 0x84bd0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40c90000, 0x0, 0x41d50000, 0x0, 0x0, 0x853b, 0x41d90000, 0x41d78670, 0x8a44, 0x0, 0x0, 0x0, 0x4432, 0x442f, 0x4431, 0x443443f4, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x853b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40cb0000, 0x8aaf, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x86704509, 0x86700000, 0x87b30000, 0x45088670, 0x867087bd, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x82f60000, 0x0, 0x0, 0x853b40ae, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x45110000, 0x0, 0x0, 0x0, 0x44090000, 0x0, 0x440c0000, 0x0, 0x450a0000, 0x0, 0x4210, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4512, 0x82f60000, 0x0, 0x0, 0x0, 0x0, 0x45138859, 0x0, 0x0, 0x4121450c, 0x82f6450b, 0x86704211, 0x0, 0x0, 0x86704127, 0x41050000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x440b0000, 0x0, 0x450d0000, 0x43f0, 0x84bd0000, 0x0, 0x0, 0x0, 0x0, 0x88cc, 0x0, 0x450e0000, 0x8670, 0x8670450f, 0x0, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b8853b, 0x0, 0x88fc853b, 0x0, 0x0, 0x854b, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x0, 0x0, 0x853b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x440d, 0x0, 0x0, 0x0, 0x0, 0x88cc, 0x0, 0x0, 0x0, 0x0, 0x86700000, 0x8994, 0x86700000, 0x0, 0x0, 0x86708670, 0x0, 0x41200000, 0x0, 0x0, 0x41294125, 0x4126, 0x0, 0x0, 0x0, 0x8670, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43f30000, 0x0, 0x0, 0x8670, 0x0, 0x0, 0x0, 0x0, 0x4437, 0x443e443b, 0x443d4419, 0x44ba0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x443f, 0x0, 0x0, 0x0, 0x0, 0x0, 0x86700000, 0x0, 0x86700000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8670, 0x4123, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80794055, 0x80794068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80794043, 0x83dd4042, 0x83dd402d, 0x80794062, 0x40408079, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x402583dd, 0x404f4064, 0x4010406c, 0x400583dd, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668079, 0x81518079, 0x402a4004, 0x83dd406a, 0x402883dd, 0x81c583dd, 0x83dd, 0x83dd83dd, 0x424b, 0x40a183dd, 0x83dd83dd, 0x83dd83dd, 0x83dd406e, 0x83dd83dd, 0x83dd83dd, 0x83dd83dd, 0x83dd, 0x83dd83dd, 0x83dd0000, 0x0, 0x83dd0000, 0x83dd0000, 0x83dd0000, 0x400983dd, 0x81514021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81518079, 0x42f6, 0x807942f3, 0x42f50000, 0x44408079, 0x42d742cd, 0x40334030, 0x42cf8079, 0x403242cc, 0x42d18079, 0x8151402f, 0x40310000, 0x44c5, 0x81c581c5, 0x42370000, 0x40138151, 0x80794011, 0x0, 0x42ad0000, 0x0, 0x83dd, 0x83dd0000, 0x83dd0000, 0x83dd, 0x400b83dd, 0x415b4002, 0x83dd4249, 0x42120000, 0x420a83dd, 0x424581c5, 0x420c83dd, 0x83dd83dd, 0x83dd40a5, 0x83dd4243, 0x83dd, 0x83dd0000, 0x83dd4247, 0x42170000, 0x0, 0x849b0000, 0x0, 0x85ce0000, 0x420b4241, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x0, 0x422e0000, 0x0, 0x81c581c5, 0x81c581c5, 0x81c581c5, 0x81c5, 0x420881c5, 0x81c5422c, 0x81c54525, 0x423240b0, 0x81c54081, 0x40a440a2, 0x420944bf, 0x408240af, 0x409040a3, 0x81c5, 0x44e50000, 0x81c50000, 0x81c5, 0x43390000, 0x81c5, 0x434d, 0x44060000, 0x0, 0x44040000, 0x41d3, 0x4242424a, 0x849b0000, 0x431e0000, 0x4244432d, 0x81c54246, 0x83dd, 0x434b, 0x83dd0000, 0x423e, 0x42134161, 0x433583dd, 0x45104248, 0x0, 0x420d0000, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x423881c5, 0x81c54233, 0x42f842f7, 0x42f942f4, 0x81510000, 0x0, 0x81c50000, 0x43064441, 0x422d, 0x42d342d4, 0x42d58079, 0x815142d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x83dd42e5, 0x81c50000, 0x81c581c5, 0x81c54014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x83dd42ae, 0x0, 0x42c4, 0x83dd40b3, 0x0, 0x41570000, 0x0, 0x44c681c5, 0x81c5422f, 0x83dd0000, 0x415c4239, 0x81c5, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x81c50000, 0x434e4408, 0x83dd0000, 0x44050000, 0x446f, 0x849b, 0x431f, 0x424c432f, 0x81c50000, 0x4214, 0x434c0000, 0x4162, 0x43370000, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x83dd4290, 0x4291, 0x40aa0000, 0x81c5, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x81c540e5, 0x81c50000, 0x4083438c, 0x81c540a8, 0x40a681c5, 0x4520407e, 0x407d4527, 0x409240b1, 0x81c581c5, 0x44c140a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874521, 0x407f4080, 0x45264091, 0x81c5407c, 0x81c5, 0x0, 0x83dd, 0x44e744e9, 0x81c5416c, 0x40a781c5, 0x42164463, 0x41d483dd, 0x83dd0000, 0x81c581c5, 0x81c50000, 0x4402433a, 0x0, 0x41c0, 0x83dd420c, 0x41730000, 0x434f, 0x43514407, 0x83dd83dd, 0x81c583dd, 0x0, 0x41aa, 0x41ac0000, 0x41b183dd, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x83dd, 0x83dd81c5, 0x83dd83dd, 0x83dd4215, 0x441a83dd, 0x41d2420f, 0x450783dd, 0x83dd0000, 0x0, 0x0, 0x83dd83dd, 0x83dd83dd, 0x0, 0x81c50000, 0x0, 0x0, 0x0, 0x431a, 0x431c81c5, 0x81c543b1, 0x432e43b5, 0x43eb0000, 0x83dd83dd, 0x83dd0000, 0x83dd4372, 0x83dd83dd, 0x4376, 0x83dd4163, 0x83dd83dd, 0x433683dd, 0x0, 0x83dd0000, 0x0, 0x41d583dd, 0x81c50000, 0x4184, 0x81c5, 0x41b8, 0x83dd81c5, 0x83dd83dd, 0x4192, 0x83dd4471, 0x81c544bb, 0x83dd4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b483dd, 0x81c50000, 0x4308, 0x430281c5, 0x430a4307, 0x83dd444c, 0x83dd, 0x42d642eb, 0x42e6, 0x81c542ea, 0x42ed42e7, 0x81c54251, 0x427d81c5, 0x81c50000, 0x432681c5, 0x81c581c5, 0x81c54509, 0x81c581c5, 0x81c581c5, 0x450881c5, 0x81c583dd, 0x81c581c5, 0x81c5449b, 0x83dd449e, 0x44a0449d, 0x81c544a5, 0x44a80000, 0x83dd42c6, 0x83dd42c5, 0x42c081c5, 0x81c540ae, 0x83dd, 0x40da83dd, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x83dd4152, 0x81c5438d, 0x81c50000, 0x44c244e8, 0x44ea416e, 0x446483dd, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c83dd, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x83dd4165, 0x43384186, 0x41b94193, 0x447281c5, 0x44bd4512, 0x83dd83dd, 0x83dd81c5, 0x42a74292, 0x42a983dd, 0x42a64151, 0x451381c5, 0x81c581c5, 0x81c581c5, 0x83dd450c, 0x83dd450b, 0x81c54211, 0x81c583dd, 0x416d0000, 0x81c583dd, 0x83dd42ca, 0x42c881c5, 0x81c583dd, 0x83dd, 0x80794174, 0x83dd81c5, 0x83dd83dd, 0x440b43b9, 0x4332417c, 0x450d4318, 0x83dd0000, 0x83dd83dd, 0x43e783dd, 0x437083dd, 0x0, 0x0, 0x41640000, 0x41850000, 0x450e81c5, 0x81c5, 0x81c5450f, 0x81c544bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881c5, 0x81c581c5, 0x81c581c5, 0x40f34303, 0x430c4309, 0x430481c5, 0x81c5, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e881c5, 0x42ee81c5, 0x4322, 0x432481c5, 0x81c5, 0x81c50000, 0x81c581c5, 0x81c5, 0x44a2449f, 0x44a181c5, 0x42c7, 0x42c242c1, 0x81c540dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e81c5, 0x81c50000, 0x432781c5, 0x81c581c5, 0x81c544a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80794176, 0x81c5440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x81c50000, 0x81c581c5, 0x81c581c5, 0x81c5438a, 0x81c544eb, 0x81c581c5, 0x81c50000, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x81c50000, 0x42f1, 0x44e18151, 0x43880000, 0x446b81c5, 0x43bf441c, 0x0, 0x43c7, 0x81c581c5, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x81c5, 0x81c5, 0x0, 0x0, 0x81c581c5, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x81c581c5, 0x81c581c5, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c381c5, 0x81c50000, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f481c5, 0x4323, 0x432581c5, 0x81c50000, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x81c543b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x81c5, 0x81c581c5, 0x81c50000, 0x81c581c5, 0x42d9, 0x81c5, 0x81c54386, 0x43bb, 0x441e4149, 0x0, 0x81c5, 0x4474443f, 0x81c54453, 0x0, 0x42dd0000, 0x425a4253, 0x81c581c5, 0x81c544c8, 0x81c581c5, 0x81c540e1, 0x4487, 0x40f24321, 0x425b4280, 0x81c581c5, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x81c581c5, 0x81c50000, 0x0, 0x0, 0x81c581c5, 0x441f0000, 0x43c9, 0x81c581c5, 0x414a81c5, 0x42de, 0x42e142df, 0x42db4255, 0x81c54257, 0x81c50000, 0x425c4254, 0x81c581c5, 0x81c581c5, 0x448b4421, 0x43ca414c, 0x81c5, 0x81c581c5, 0x41ed0000, 0x81c58079, 0x4204, 0x81c542e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x81c54258, 0x81c541ee, 0x420581c5, 0x81c58079, 0x81c5, 0x42e4425e, 0x4262425f, 0x42824263, 0x428481c5, 0x81c54349, 0x81c54260, 0x4264434a, 0x81c581c5, 0x81c581c5, 0x81c50000 },
                { 0x807a4055, 0x807a4068, 0x40524049, 0x40604006, 0x40484067, 0x831882e3, 0x403e406b, 0x406585a6, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x807a4043, 0x82e34042, 0x85dd402d, 0x807a4062, 0x4040807a, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x4025807a, 0x404f4064, 0x4010406c, 0x400585dd, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066807a, 0x8152807a, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x406e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81524021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x8152807a, 0x42f6, 0x807a42f3, 0x42f50000, 0x4440807a, 0x42d742cd, 0x40334030, 0x42cf807a, 0x403242cc, 0x42d1807a, 0x8152402f, 0x40310000, 0x44c5, 0x0, 0x42370000, 0x40138152, 0x807a4011, 0x0, 0x42ad807a, 0x42bf42bc, 0x42be42bd, 0x42bb42ba, 0x42af8152, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a0000, 0x0, 0x420c0000, 0x0, 0x40a5, 0x0, 0x0, 0x0, 0x0, 0x42170000, 0x0, 0x849c0000, 0x0, 0x83ef0000, 0x420b83ef, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x0, 0x422e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x41d3, 0x0, 0x849c0000, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x423e, 0x42134161, 0x43350000, 0x45100000, 0x0, 0x420d0000, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x42380000, 0x4233, 0x42f842f7, 0x42f942f4, 0x81520000, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d5807a, 0x815242d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x42b00000, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x422f, 0x0, 0x415c4239, 0x0, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x446f, 0x849c, 0x431f, 0x432f, 0x0, 0x4214, 0x434c0000, 0x4162, 0x43370000, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x0, 0x0, 0x0, 0x4402433a, 0x42b1, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c42b2, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x88904174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x42b34322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x88904176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e189b5, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x42b44323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x8890, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8890, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x807b4055, 0x807b4068, 0x40524049, 0x40604006, 0x40484067, 0x807b807b, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x807b4043, 0x4042, 0x402d, 0x4062, 0x4040807b, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066807b, 0x8153807b, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x8153807b, 0x40a1401c, 0x403c8153, 0x403a0000, 0x406e, 0x4039401e, 0x403b, 0x0, 0x807b, 0x0, 0x0, 0x807b807b, 0x0, 0x0, 0x0, 0x40090000, 0x81534021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x8153807b, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x42370000, 0x40138153, 0x807b4011, 0x8153, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x42120000, 0x420a401d, 0x0, 0x420c401f, 0x445a0000, 0x807b40a5, 0x0, 0x807b0000, 0x81530000, 0x0, 0x42170000, 0x0, 0x849d0000, 0x0, 0x83f044c4, 0x420b83f0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x442d, 0x422e0000, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x41d3, 0x0, 0x849d442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x423e, 0x42134161, 0x43350000, 0x45100000, 0x0, 0x420d0000, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x420c4016, 0x44b10000, 0x44b340f5, 0x42380000, 0x4233, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x422d, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x422f, 0x0, 0x415c4239, 0x0, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x446f, 0x849d, 0x431f, 0x432f, 0x0, 0x4214, 0x434c0000, 0x4162, 0x43370000, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a54150, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x40a70000, 0x42164463, 0x41d40000, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x807c4055, 0x807c4068, 0x40524049, 0x40604006, 0x40484067, 0x866782d9, 0x403e406b, 0x406580fe, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x807c4043, 0x82e44042, 0x85de402d, 0x807c4062, 0x4040807c, 0x4057405c, 0x405886d9, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x4025873a, 0x404f4064, 0x4010406c, 0x400587ae, 0x40614029, 0x40264007, 0x450644e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x856144e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066807c, 0x8154807c, 0x402a4004, 0x8a4f406a, 0x40288a50, 0x8a510000, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x406e, 0x0, 0x0, 0x80fe, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41cf, 0x0, 0x40090000, 0x81544021, 0x83394019, 0x401b0000, 0x0, 0x40174015, 0x8154807c, 0x42f6, 0x807c42f3, 0x42f50000, 0x4440807c, 0x42d742cd, 0x40334030, 0x42cf807c, 0x403242cc, 0x42d1807c, 0x8154402f, 0x40310000, 0x82db44c5, 0x0, 0x0, 0x40138154, 0x807c4011, 0x0, 0x42ad0000, 0x0, 0x839d, 0x83c4, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x83d00000, 0x0, 0x0, 0x83d90000, 0x0, 0x0, 0x0, 0x0, 0x8477, 0x0, 0x0, 0x84f7, 0x849e0000, 0x857c, 0x0, 0x0, 0x400a400e, 0x84f70000, 0x428d4288, 0x428b, 0x8581, 0x42898339, 0x80fe0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x86950000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x4339864b, 0x0, 0x87ba434d, 0x44060000, 0x8692, 0x44040000, 0x869541d3, 0x0, 0x849e0000, 0x431e0000, 0x432d, 0x0, 0x0, 0x87c0434b, 0x0, 0x84f7, 0x84f74161, 0x43350000, 0x86950000, 0x0, 0x8339, 0x0, 0x41f84020, 0x44308695, 0x442e8695, 0x8695401a, 0x86954410, 0x44124018, 0x83d94016, 0x44b14203, 0x44b340f5, 0x87c00000, 0x0, 0x42f842f7, 0x42f942f4, 0x815441f9, 0x0, 0x0, 0x43064441, 0x8955, 0x42d342d4, 0x42d5807c, 0x815442d8, 0x42ce42d0, 0x42d28957, 0x856342e9, 0x898442e5, 0x8985, 0x0, 0x4014, 0x84f74012, 0x449a80fe, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x839d42c4, 0x41cf40b3, 0x450584f7, 0x41578957, 0x8a3f84f7, 0x44c60000, 0x8a3e, 0x0, 0x415c8339, 0x0, 0x44c044e6, 0x0, 0x864b, 0x433b84f7, 0x84f7, 0x434e4408, 0x0, 0x44050000, 0x84f7446f, 0x849e, 0x84f7431f, 0x432f, 0x0, 0x84f7, 0x434c0000, 0x84f74162, 0x43370000, 0x84f70000, 0x84f70000, 0x84f7428e, 0x428c428a, 0x428f0000, 0x45054290, 0x45054291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x84f70000, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x41d40000, 0x0, 0x0, 0x0, 0x4402433a, 0x41da, 0x41db41c0, 0x83d9, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x41d841d6, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x84f7, 0x417b0000, 0x447043bd, 0x45150000, 0x0, 0x0, 0x0, 0x441a4202, 0x0, 0x89a341dd, 0x0, 0x84f7, 0x451a, 0x0, 0x0, 0x8695, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x4184, 0x45050000, 0x41d941b8, 0x41d70000, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x406e442f, 0x8a514431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x8a510000, 0x43268a51, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x4505, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45054403, 0x433c4505, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x4505417d, 0x43be441b, 0x84f7, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x42000000, 0x42a74292, 0x42a94201, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41dc, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x88914174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x45054318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x88914176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e189b6, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x8891, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8891, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x4055, 0x4068, 0x40520000, 0x40600000, 0x4067, 0x0, 0x406b, 0x40650000, 0x404c0000, 0x405d, 0x405f, 0x405a, 0x4069, 0x404b4317, 0x0, 0x0, 0x0, 0x0, 0x80f64062, 0x0, 0x4057405c, 0x40580000, 0x0, 0x4056404e, 0x4316, 0x405e44e1, 0x4059, 0x0, 0x404f4064, 0x406c, 0x0, 0x40610000, 0x0, 0x44e4, 0x405b0000, 0x0, 0x4051, 0x405044e3, 0x44e2, 0x406d0000, 0x4063, 0x40544053, 0x404d, 0x40660000, 0x0, 0x0, 0x406a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x406e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42f3, 0x42f50000, 0x80f6, 0x42d742cd, 0x40334030, 0x42cf80f6, 0x403242cc, 0x42d180f6, 0x82d2402f, 0x40310000, 0x44c5, 0x0, 0x42370000, 0x0, 0x0, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42120000, 0x420a0000, 0x0, 0x420c0000, 0x0, 0x40a5, 0x0, 0x0, 0x0, 0x0, 0x42170000, 0x0, 0x849f0000, 0x0, 0x83f10000, 0x420b83f1, 0x0, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x423d0000, 0x0, 0x422e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42080000, 0x422c, 0x0, 0x423240b0, 0x0, 0x40a440a2, 0x420944bf, 0x40af, 0x40a3, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x0, 0x0, 0x0, 0x41d3, 0x0, 0x849f0000, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x423e, 0x42130000, 0x43350000, 0x45100000, 0x0, 0x420d0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x420c0000, 0x0, 0x0, 0x42380000, 0x4233, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43060000, 0x422d, 0x42d342d4, 0x42d580f6, 0x82d242d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433d, 0x44ed0000, 0x42ae, 0x0, 0x42c4, 0x0, 0x0, 0x0, 0x0, 0x44c60000, 0x422f, 0x0, 0x4239, 0x0, 0x44c044e6, 0x41ce0000, 0x0, 0x433b0000, 0x0, 0x434e0000, 0x0, 0x0, 0x446f, 0x849f, 0x431f, 0x432f, 0x0, 0x4214, 0x434c0000, 0x0, 0x43370000, 0x420e0000, 0x451e0000, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x40aa0000, 0x0, 0x42a50000, 0x40b240a9, 0x0, 0x407b0000, 0x40e5, 0x0, 0x438c, 0x40a8, 0x40a60000, 0x0, 0x0, 0x40b1, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x0, 0x40a70000, 0x42160000, 0x41d40000, 0x0, 0x0, 0x0, 0x433a, 0x0, 0x0, 0x420c, 0x0, 0x434f, 0x43510000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43310000, 0x0, 0x0, 0x447043bd, 0x0, 0x0, 0x0, 0x4215, 0x0, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x0, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x41d50000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x0, 0x406e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4308, 0x0, 0x430a4307, 0x0, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x4509, 0x0, 0x0, 0x45080000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42c6, 0x42c5, 0x42c00000, 0x40ae, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433e44ee, 0x0, 0x0, 0x438d, 0x0, 0x44c244e8, 0x44ea0000, 0x0, 0x45110000, 0x433c0000, 0x0, 0x43504352, 0x0, 0x0, 0x0, 0x43330000, 0x450a0000, 0x43be0000, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x0, 0x43380000, 0x4193, 0x44720000, 0x44bd4512, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a60000, 0x45130000, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x0, 0x0, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43b9, 0x43320000, 0x450d4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x0, 0x0, 0x450e0000, 0x0, 0x450f, 0x44bc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x430c4309, 0x0, 0x0, 0x430b0000, 0x0, 0x0, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42c7, 0x42c242c1, 0x0, 0x0, 0x0, 0x0, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42cb42c9, 0x0, 0x0, 0x43ba4334, 0x4319, 0x43e9, 0x4371, 0x0, 0x44be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x0, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x0, 0x43bf0000, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44730000, 0x0, 0x0, 0x0, 0x430d, 0x0, 0x42f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c70000, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4323, 0x43250000, 0x0, 0x0, 0x4368438b, 0x44ec41fb, 0x0, 0x41f0, 0x42f2, 0x43890000, 0x43c00000, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x0, 0x0, 0x0, 0x44740000, 0x0, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x0, 0x0, 0x4321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x0, 0x42da4387, 0x43bc0000, 0x4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43c9, 0x0, 0x0, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x0, 0x43ca0000, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x807d4055, 0x807d4068, 0x40524049, 0x40604006, 0x40484067, 0x807d807d, 0x403e406b, 0x4065807d, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x807d4043, 0x807d4042, 0x807d402d, 0x807d4062, 0x4040807d, 0x4057405c, 0x4058807d, 0x4047400c, 0x4056404e, 0x40034316, 0x405e8206, 0x404a4059, 0x4025807d, 0x404f4064, 0x4010406c, 0x4005807d, 0x40614029, 0x40264007, 0x807d44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x807d44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066807d, 0x807d807d, 0x402a4004, 0x807d406a, 0x4028807d, 0x81be81c7, 0x807d81ff, 0x40244022, 0x83cc807d, 0x40a1401c, 0x403c433f, 0x403a807d, 0x834d406e, 0x4039401e, 0x8224403b, 0x807d807d, 0x8224807d, 0x82244444, 0x8369807d, 0x807d807d, 0x44454442, 0x807d807d, 0x807d807d, 0x400940b6, 0x85f74021, 0x40b74019, 0x401b807d, 0x40b44036, 0x40174015, 0x807d807d, 0x403842f6, 0x807d42f3, 0x42f54037, 0x4440807d, 0x42d742cd, 0x40334030, 0x42cf807d, 0x403242cc, 0x42d1807d, 0x807d402f, 0x40314035, 0x44c344c5, 0x81be81be, 0x423782f5, 0x4013807d, 0x807d4011, 0x444a4348, 0x42ad807d, 0x42bf42bc, 0x42be42bd, 0x42bb42ba, 0x42af8427, 0x442c4023, 0x400b4455, 0x415b4002, 0x807d807d, 0x42128427, 0x420a401d, 0x807d81be, 0x420c401f, 0x445a4459, 0x807d40a5, 0x4457807d, 0x807d4454, 0x807d807d, 0x44438224, 0x4217807d, 0x822440b5, 0x8224807d, 0x40be40c0, 0x83f244c4, 0x420b83f2, 0x400a400e, 0x4293429a, 0x428d4288, 0x429b428b, 0x807d807d, 0x42894299, 0x423d807d, 0x444b442d, 0x422e8224, 0x4034440e, 0x81be81be, 0x81be81be, 0x81be81be, 0x81be81be, 0x420881be, 0x81be422c, 0x81be4525, 0x423240b0, 0x81be4081, 0x40a440a2, 0x420944bf, 0x408240af, 0x409040a3, 0x409f81be, 0x44e5807d, 0x81be807d, 0x842781be, 0x4339807d, 0x444881be, 0x807d434d, 0x44064340, 0x82244446, 0x44048224, 0x40b841d3, 0x82248224, 0x8224442a, 0x431e445b, 0x8224432d, 0x81be8224, 0x84b64458, 0x807d434b, 0x445640c6, 0x8224423e, 0x42134161, 0x43358224, 0x45108427, 0x40bf40c1, 0x420d807d, 0x42fc40b9, 0x440f4020, 0x44304449, 0x442e4447, 0x442b401a, 0x445c4410, 0x44124018, 0x420c4016, 0x44b140c7, 0x44b340f5, 0x423881be, 0x81be4233, 0x42f842f7, 0x42f942f4, 0x40ba445d, 0x81be81be, 0x81be40c4, 0x43064441, 0x42fd422d, 0x42d342d4, 0x42d5807d, 0x40bb42d8, 0x42ce42d0, 0x42d24300, 0x807d42e9, 0x807d42e5, 0x81be807d, 0x81be81be, 0x81be4014, 0x42fa4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x430142ae, 0x42b082f5, 0x807d42c4, 0x40bc40b3, 0x42fe40c2, 0x415742fb, 0x40bd807d, 0x44c681be, 0x81be422f, 0x82f540c3, 0x415c4239, 0x822481be, 0x44c044e6, 0x41ce42ff, 0x807d0000, 0x433b807d, 0x81be0000, 0x434e4408, 0x82248427, 0x44058224, 0x446f, 0x82248224, 0x431f, 0x8224432f, 0x81be8224, 0x84b64214, 0x434c0000, 0x4162, 0x43378224, 0x420e0000, 0x451e82f5, 0x4294428e, 0x428c428a, 0x428f0000, 0x4290, 0x807d4291, 0x40aa0000, 0x42a181be, 0x42a54150, 0x40b240a9, 0x8224, 0x407b8224, 0x81be40e5, 0x81be0000, 0x4083438c, 0x81be40a8, 0x40a681be, 0x4520407e, 0x407d4527, 0x409240b1, 0x81be81be, 0x44c140a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874521, 0x407f4080, 0x45264091, 0x81be407c, 0x868381be, 0x86838683, 0x86838683, 0x44e744e9, 0x81be416c, 0x40a781be, 0x42164463, 0x41d40000, 0x0, 0x81be81be, 0x81be0000, 0x4402433a, 0x42b1, 0x41c0, 0x420c, 0x41730000, 0x434f, 0x43514407, 0x807d, 0x81be8224, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a8224, 0x43314469, 0x0, 0x417b0000, 0x447043bd, 0x82248224, 0x81be, 0x82248224, 0x82244215, 0x441a0000, 0x41d2420f, 0x45070000, 0x0, 0x0, 0x0, 0x8224, 0x0, 0x0, 0x81be0000, 0x0, 0x0, 0x0, 0x431a, 0x431c81be, 0x81be43b1, 0x432e43b5, 0x43eb84b6, 0x4400, 0x87114401, 0x84b64372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x8224, 0x41d50000, 0x81be0000, 0x4184, 0x81be, 0x41b8, 0x81be, 0x87490000, 0x4192, 0x4471, 0x81be44bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x81be0000, 0x4308, 0x430281be, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x81be42ea, 0x42ed42e7, 0x81be4251, 0x427d81be, 0x81be0000, 0x432681be, 0x81be81be, 0x81be4509, 0x81be81be, 0x81be81be, 0x450881be, 0x81be82f5, 0x81be81be, 0x81be449b, 0x449e, 0x44a0449d, 0x81be44a5, 0x44a80000, 0x82f542c6, 0x42c5, 0x42c081be, 0x81be40ae, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x807d, 0x433e44ee, 0x44a64159, 0x4152, 0x81be438d, 0x81be0000, 0x44c244e8, 0x44ea416e, 0x44640000, 0x45114403, 0x433c42b2, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c8224, 0x4333446a, 0x450a417d, 0x43be441b, 0x4210, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x447281be, 0x44bd4512, 0x82f50000, 0x42a381be, 0x42a74292, 0x42a942a2, 0x42a64151, 0x451381be, 0x81be81be, 0x81be81be, 0x450c, 0x82f5450b, 0x81be4211, 0x81be0000, 0x416d0000, 0x81be0000, 0x42ca, 0x42c881be, 0x81be42b5, 0x0, 0x807d4174, 0x807d81be, 0x0, 0x440b43b9, 0x4332417c, 0x450d4318, 0x807d84b6, 0x84b60000, 0x43e70000, 0x43700000, 0x807d0000, 0x0, 0x41640000, 0x41850000, 0x450e81be, 0x81be, 0x81be450f, 0x81be44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881be, 0x81be81be, 0x81be81be, 0x40f34303, 0x430c4309, 0x430481be, 0x81be, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e881be, 0x42ee81be, 0x42b34322, 0x432481be, 0x81be, 0x81be0000, 0x81be81be, 0x81be, 0x44a2449f, 0x44a181be, 0x42c7, 0x42c242c1, 0x81be40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e81be, 0x81be0000, 0x432781be, 0x81be81be, 0x81be44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x807d4176, 0x81be440d, 0x43ba4334, 0x417e4319, 0x807d43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab42a4, 0x42a842aa, 0x4367, 0x81be0000, 0x81be81be, 0x81be81be, 0x81be438a, 0x81be44eb, 0x81be81be, 0x81be0000, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x81be0000, 0x42f1, 0x0, 0x43880000, 0x446b81be, 0x43bf441c, 0x0, 0x43c7, 0x81be81be, 0x43b34286, 0x4297, 0x43b743e8, 0x84b60000, 0x42060000, 0x43744355, 0x81be, 0x89e081be, 0x8749, 0x868389e0, 0x81be81be, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x81be81be, 0x81be81be, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c381be, 0x81be0000, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f481be, 0x42b44323, 0x432581be, 0x81be0000, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x81be43b4, 0x42874298, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x429d81be, 0x81be81be, 0x81be0000, 0x81be81be, 0x42b642d9, 0x807d81be, 0x81be4386, 0x43bb, 0x441e4149, 0x0, 0x81be, 0x4474443f, 0x81be4453, 0x0, 0x42dd4295, 0x425a4253, 0x81be81be, 0x81be44c8, 0x81be81be, 0x81be40e1, 0x4487, 0x40f24321, 0x425b4280, 0x81be81be, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x429f429e, 0x81be81be, 0x81be0000, 0x0, 0x807d, 0x81be81be, 0x441f0000, 0x43c9, 0x81be81be, 0x414a81be, 0x42de, 0x42e142df, 0x42db4255, 0x81be4257, 0x81be4296, 0x425c4254, 0x81be81be, 0x81be81be, 0x448b4421, 0x43ca414c, 0x42a081be, 0x81be81be, 0x41ed0000, 0x81be807d, 0x807d4204, 0x81be42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x81be4258, 0x81be41ee, 0x420581be, 0x81be807d, 0x807d81be, 0x42e4425e, 0x4262425f, 0x42824263, 0x428481be, 0x81be4349, 0x81be4260, 0x4264434a, 0x81be81be, 0x81be81be, 0x81be0000 },
                { 0x807e4055, 0x807e4068, 0x40524049, 0x40604006, 0x40484067, 0x807e807e, 0x403e406b, 0x4065807e, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x807e4043, 0x44964042, 0x807e402d, 0x44944062, 0x4040807e, 0x4057405c, 0x4058807e, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40254497, 0x404f4064, 0x4010406c, 0x4005807e, 0x40614029, 0x40264007, 0x807e44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x807e44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066807e, 0x807e807e, 0x402a4004, 0x4498406a, 0x40284499, 0x40f9, 0x449640fe, 0x40244022, 0x807e807e, 0x40a1401c, 0x403c807e, 0x403a4495, 0x40fb0000, 0x4039401e, 0x807e403b, 0x807e807e, 0x807e807e, 0x807e4444, 0x40fa807e, 0x807e807e, 0x44454442, 0x40fc807e, 0x40fd807e, 0x400940b6, 0x807e4021, 0x40b74019, 0x401b807e, 0x40b44036, 0x40174015, 0x807e807e, 0x4038807e, 0x42f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x807e0000, 0x4035, 0x44c344c5, 0x0, 0x807e, 0x4013807e, 0x807e4011, 0x444a807e, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b4455, 0x415b4002, 0x44960000, 0x807e, 0x401d, 0x0, 0x401f, 0x445a4459, 0x807e0000, 0x44570000, 0x807e4454, 0x807e807e, 0x44430000, 0x807e, 0x40b5, 0x84a00000, 0x40be40c0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x444b442d, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e540ff, 0x4122, 0x41030000, 0x43390000, 0x44480000, 0x4101434d, 0x44060000, 0x4446, 0x44040000, 0x40b80000, 0x0, 0x84a0442a, 0x431e445b, 0x432d, 0x0, 0x4458, 0x4100434b, 0x445640c6, 0x0, 0x4161, 0x43350000, 0x0, 0x40bf40c1, 0x4102, 0x410440b9, 0x440f4020, 0x44304449, 0x442e4447, 0x442b401a, 0x445c4410, 0x44124018, 0x4016, 0x44b140c7, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x40ba445d, 0x0, 0x40c4, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x40bb0000, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40bc40b3, 0x40c2, 0x41570000, 0x40bd0000, 0x44c60000, 0x0, 0x40c3, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x84a0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x4128412a, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x41210000, 0x0, 0x0, 0x0, 0x416d0000, 0x4127, 0x410542ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x412041fa, 0x418d4465, 0x41ef, 0x41294125, 0x4126, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed4123, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x807f0000, 0x82760000, 0x0, 0x4006, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40080000, 0x0, 0x4001, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400c, 0x0, 0x40030000, 0x44e1, 0x0, 0x0, 0x0, 0x0, 0x40050000, 0x0, 0x4007, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x807f, 0x81550000, 0x850b4004, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40130000, 0x4011, 0x0, 0x0, 0x0, 0x0, 0x84e60000, 0x0, 0x0, 0x400b0000, 0x4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x84a10000, 0x0, 0x0, 0x0, 0x400a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x84a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x84e6, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b30000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4408, 0x0, 0x44050000, 0x0, 0x84a1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x84e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4407, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x440a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44090000, 0x0, 0x440c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x440b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x440d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4437, 0x443e443b, 0x443d4419, 0x44ba0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x443f, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] value2 = null;

        protected static void value2Init()
        {
            value2 = new int[][]
            {
                { 0x80804055, 0x80804068, 0x40524049, 0x40604006, 0x40484067, 0x80808080, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80804043, 0x82a14042, 0x402d, 0x4062, 0x40408080, 0x4057405c, 0x405882a1, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668080, 0x81568080, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x81568080, 0x40a1401c, 0x403c8156, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x8080, 0x0, 0x0, 0x80808080, 0x0, 0x0, 0x0, 0x40090000, 0x81564021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81568080, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x40138156, 0x80804011, 0x8156, 0x42ad0000, 0x0, 0x0, 0x84e70000, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80800000, 0x0, 0x80800000, 0x81560000, 0x0, 0x0, 0x0, 0x84a20000, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x84a2442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x433584e7, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x84a2, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x433784e7, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80814055, 0x80814068, 0x40524049, 0x40604006, 0x40484067, 0x818d819a, 0x403e406b, 0x406580ff, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80814043, 0x81cc4042, 0x81cf402d, 0x80814062, 0x40408081, 0x4057405c, 0x4058810c, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x402581dd, 0x404f4064, 0x4010406c, 0x400581ed, 0x40614029, 0x40264007, 0x81f444e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x820744e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668081, 0x81578081, 0x402a4004, 0x8229406a, 0x40288199, 0x81c681cb, 0x823681e1, 0x81e6823a, 0x823b8251, 0x40a18252, 0x825b433f, 0x81e682dd, 0x8303406e, 0x82b38307, 0x8227830d, 0x832d8311, 0x8232832e, 0x82388298, 0x81cb8245, 0x833144d3, 0x44d98266, 0x81cb41cf, 0x81cb44dd, 0x40098346, 0x81574021, 0x833a4019, 0x401b8349, 0x83508352, 0x40174015, 0x81578081, 0x835b42f6, 0x808142f3, 0x42f581ed, 0x44408081, 0x42d742cd, 0x40334030, 0x42cf8081, 0x403242cc, 0x42d18081, 0x8157402f, 0x40318364, 0x836e44c5, 0x81c681c6, 0x837d82f8, 0x40138157, 0x80814011, 0x8385838a, 0x42ad8389, 0x83938394, 0x825b839e, 0x83c083c5, 0x841b8428, 0x84a783d1, 0x400b841b, 0x415b4002, 0x8505833e, 0x83d18428, 0x853e8445, 0x819981c6, 0x83da44cb, 0x846d853f, 0x81e68565, 0x82b3856b, 0x85704183, 0x81ed8478, 0x84ef837b, 0x85728576, 0x8382857a, 0x83878245, 0x44f3857d, 0x418c8266, 0x85e485f3, 0x400a400e, 0x84f844de, 0x428d4288, 0x8508428b, 0x44d18582, 0x428985a3, 0x831185af, 0x83b085ef, 0x825b83bf, 0x81998199, 0x81c681c6, 0x81c681c6, 0x81c681c6, 0x81c681c6, 0x875581c6, 0x81c68779, 0x81c64525, 0x879d87a2, 0x84044081, 0x87e387fc, 0x83d144bf, 0x408281ed, 0x409087fe, 0x409f81c6, 0x44e58644, 0x81c68508, 0x842881c6, 0x4339864c, 0x44ce81c6, 0x83d1434d, 0x44064340, 0x83bf8693, 0x44048466, 0x869641d3, 0x83bf83bf, 0x83bf8802, 0x431e85e7, 0x83bf432d, 0x81c683bf, 0x84c0830d, 0x83d1434b, 0x84c84378, 0x83bf857a, 0x857a4161, 0x433583bf, 0x87578428, 0x41948266, 0x826685a3, 0x88d588dc, 0x41f84020, 0x443086a5, 0x442e8696, 0x8757401a, 0x87554410, 0x44124018, 0x83da4016, 0x44b14203, 0x44b340f5, 0x83d181c6, 0x81c68932, 0x42f842f7, 0x42f942f4, 0x815741f9, 0x87518751, 0x81c68945, 0x43064441, 0x894883d1, 0x42d342d4, 0x42d58081, 0x815742d8, 0x42ce42d0, 0x42d283d1, 0x81ed42e9, 0x83d142e5, 0x81c683d1, 0x81c681c6, 0x81c64014, 0x857a4012, 0x449a80ff, 0x449c433d, 0x44ed44a4, 0x89bb42ae, 0x89bc82f8, 0x839e42c4, 0x41cf40b3, 0x83d18a07, 0x415783d1, 0x83d18a0e, 0x44c681c6, 0x81c683d1, 0x82f881ed, 0x415c833a, 0x83bf81c6, 0x44c044e6, 0x8a59877a, 0x8a5a864c, 0x433b8a07, 0x81c68851, 0x434e4408, 0x83bf8428, 0x4405861c, 0x8a0e446f, 0x83bf83bf, 0x8a07431f, 0x83bf432f, 0x81c683bf, 0x84c08862, 0x434c84c8, 0x8a0e4162, 0x433783bf, 0x88658266, 0x8a0782f8, 0x8a0e428e, 0x428c428a, 0x428f8650, 0x83d14290, 0x83d14291, 0x0, 0x81c6, 0x42a54150, 0x0, 0x819983bf, 0x83bf, 0x81c60000, 0x81c64523, 0x4083438c, 0x86760000, 0x81c6, 0x4520407e, 0x407d4527, 0x40920000, 0x81c681c6, 0x44c140a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874521, 0x407f4080, 0x45264091, 0x81c6407c, 0x88a38404, 0x0, 0x0, 0x44e744e9, 0x81c6416c, 0x81c6, 0x4463, 0x41d40000, 0x412a, 0x81c681c6, 0x81c60000, 0x4402433a, 0x44df41da, 0x41db41c0, 0x83da, 0x41730000, 0x434f, 0x43514407, 0x8199, 0x81c683bf, 0x41d841d6, 0x819941aa, 0x41ac0000, 0x41b10000, 0x440a83bf, 0x43314469, 0x8851, 0x417b0000, 0x447043bd, 0x83bf83bf, 0x81c6, 0x83bf83bf, 0x83bf0000, 0x441a4202, 0x0, 0x83d141dd, 0x0, 0x88a3, 0x83d1, 0x83bf, 0x0, 0x8696, 0x81c60000, 0x0, 0x0, 0x0, 0x8199431a, 0x431c81c6, 0x81c643b1, 0x432e43b5, 0x43eb84c0, 0x84c04400, 0x84c04401, 0x84c04372, 0x84c88199, 0x81994376, 0x84c84163, 0x0, 0x43368199, 0x81998199, 0x81998199, 0x819983bf, 0x41d58199, 0x81c68199, 0x81994184, 0x83d181c6, 0x41d941b8, 0x41d781c6, 0x87518266, 0x41954192, 0x82664471, 0x81c644bb, 0x4432, 0x406e442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x81c60000, 0x4308, 0x430281c6, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x81c642ea, 0x42ed42e7, 0x81c64251, 0x427d81c6, 0x81c60000, 0x432681c6, 0x81c681c6, 0x81c60000, 0x81c681c6, 0x81c681c6, 0x81c6, 0x81c682f8, 0x81c681c6, 0x81c6449b, 0x449e, 0x44a0449d, 0x81c644a5, 0x44a80000, 0x82f842c6, 0x42c5, 0x42c081c6, 0x81c60000, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x81c6438d, 0x86760000, 0x44c244e8, 0x44ea416e, 0x44640000, 0x83d14403, 0x433c83d1, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c83bf, 0x4333446a, 0x83d1417d, 0x43be441b, 0x8aa7, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x84c84165, 0x43384186, 0x41b94193, 0x447281c6, 0x44bd0000, 0x82f80000, 0x420081c6, 0x42a74292, 0x42a94201, 0x42a64151, 0x81c6, 0x86768404, 0x840481c6, 0x0, 0x82f80000, 0x81c60000, 0x81c641dc, 0x416d0000, 0x81c60000, 0x42ca, 0x42c881c6, 0x81c60000, 0x0, 0x80814174, 0x81998676, 0x0, 0x440b43b9, 0x4332417c, 0x83d14318, 0x84c0, 0x84c084c0, 0x43e70000, 0x437084c8, 0x81994379, 0x437a437b, 0x41640000, 0x41850000, 0x81c6, 0x419681c6, 0x81c68266, 0x81c644bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b881c6, 0x81c681c6, 0x81c681c6, 0x40f34303, 0x430c4309, 0x430481c6, 0x81c6, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e881c6, 0x42ee81c6, 0x4322, 0x432481c6, 0x81c6, 0x81c60000, 0x81c681c6, 0x81c6, 0x44a2449f, 0x44a181c6, 0x42c7, 0x42c242c1, 0x81c640dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e81c6, 0x81c60000, 0x432781c6, 0x81c681c6, 0x81c644a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80814176, 0x8676440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x81c60000, 0x81c681c6, 0x81c681c6, 0x8676438a, 0x867644eb, 0x81c681c6, 0x81c60000, 0x41fa, 0x418d4465, 0x41ef, 0x41290000, 0x81c60000, 0x42f1, 0x44e18157, 0x43880000, 0x446b81c6, 0x43bf441c, 0x0, 0x43c7, 0x867681c6, 0x43b34286, 0x0, 0x43b743e8, 0x84c00000, 0x420684c8, 0x43744355, 0x81c6, 0x875181c6, 0x41978751, 0x8751, 0x81c681c6, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x81c681c6, 0x81c681c6, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c381c6, 0x81c60000, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f481c6, 0x4323, 0x432581c6, 0x81c60000, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x867643b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x81c6, 0x81c68676, 0x86760000, 0x81c681c6, 0x42d9, 0x81c6, 0x81c64386, 0x43bb, 0x441e4149, 0x0, 0x81c6, 0x4474443f, 0x81c64453, 0x0, 0x42dd0000, 0x425a4253, 0x81c681c6, 0x81c644c8, 0x81c681c6, 0x81c640e1, 0x4487, 0x40f24321, 0x425b4280, 0x81c681c6, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x81c681c6, 0x81c60000, 0x0, 0x0, 0x81c681c6, 0x441f0000, 0x43c9, 0x81c681c6, 0x414a81c6, 0x42de, 0x42e142df, 0x42db4255, 0x81c64257, 0x81c60000, 0x425c4254, 0x81c681c6, 0x81c681c6, 0x448b4421, 0x43ca414c, 0x81c6, 0x81c681c6, 0x41ed0000, 0x81c68081, 0x4204, 0x81c642e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x81c64258, 0x81c641ee, 0x420581c6, 0x81c68081, 0x81c6, 0x42e4425e, 0x4262425f, 0x42824263, 0x428481c6, 0x81c64349, 0x81c64260, 0x4264434a, 0x81c681c6, 0x81c681c6, 0x81c60000 },
                { 0x80824055, 0x80824068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80824043, 0x4042, 0x402d, 0x80824062, 0x40408082, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668082, 0x80828082, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x80824021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x80828082, 0x42f6, 0x808242f3, 0x42f50000, 0x44408082, 0x42d742cd, 0x40334030, 0x42cf8082, 0x403242cc, 0x42d18082, 0x8082402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138082, 0x80824011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x80820000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d58082, 0x808242d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80824174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80824176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18082, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x8082, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8082, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80834055, 0x80834068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x4043, 0x4042, 0x402d, 0x4062, 0x40400000, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x808344e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668083, 0x81588083, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81580000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81584021, 0x4019, 0x401b0000, 0x4036, 0x40174015, 0x0, 0x40380000, 0x42f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x4035, 0x44c5, 0x0, 0x0, 0x40130000, 0x4011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x40340000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed0000, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x41580000, 0x40f6, 0x0, 0x433e44ee, 0x4159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x0, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x0, 0x415a, 0x4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df0000, 0x0, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x0, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x0, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x0, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40b6, 0x0, 0x40b70000, 0x8322, 0x40b40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x85b7, 0x40b5, 0x0, 0x40be40c0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40c6, 0x0, 0x0, 0x0, 0x0, 0x40bf40c1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40c7, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40c4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40c5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40c2, 0x0, 0x0, 0x0, 0x0, 0x40c3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x81d0823d, 0x82778280, 0x411b4111, 0x828f8304, 0x83278328, 0x820f820f, 0x832a4160, 0x415d415f, 0x44d48353, 0x838b4154, 0x415683b5, 0x44cf439d, 0x83c8439f, 0x840b840c, 0x841344d7, 0x41708419, 0x83dc841d, 0x81314467, 0x44d28420, 0x8421842e, 0x44df8432, 0x44da4221, 0x453f8433, 0x844344cc, 0x41778446, 0x8449844a, 0x4541844b, 0x44d08373, 0x845a41ae, 0x845c446d, 0x846a828a, 0x44d6846e, 0x417f8472, 0x439e4354, 0x848484a5, 0x432984a9, 0x84ab453c, 0x84ae84b0, 0x423f4380, 0x84cb4381, 0x416a4169, 0x416784d2, 0x453e84d7, 0x418884ed, 0x41ba84f1, 0x850c4546, 0x4218850f, 0x8518421a, 0x81c081c9, 0x81dc8201, 0x82fe41ca, 0x81eb424b, 0x40a181f7, 0x843d41c2, 0x8204412d, 0x834f406e, 0x845e8305, 0x4382830b, 0x412e822f, 0x41cd8235, 0x45434113, 0x836b4353, 0x824c8250, 0x44d9423a, 0x851e41cf, 0x455144dd, 0x8527828a, 0x44f84114, 0x8275854f, 0x453a8283, 0x42244230, 0x85664538, 0x82fe856a, 0x828e856e, 0x454d8594, 0x859982b7, 0x859a85a7, 0x453740ce, 0x85b085c0, 0x85c18373, 0x85c385d0, 0x85d685e5, 0x85ec85f1, 0x85f482d8, 0x82dc85f9, 0x85fa85fc, 0x42374115, 0x85ff8600, 0x86028603, 0x83024348, 0x86054234, 0x42224223, 0x8520856d, 0x4117421b, 0x4285858d, 0x831785f8, 0x860841c9, 0x8609860f, 0x85044249, 0x42128342, 0x420a4179, 0x4245834c, 0x420c44cb, 0x4181835a, 0x861240a5, 0x83614243, 0x83664183, 0x836d4225, 0x418a4247, 0x42178380, 0x860d8384, 0x86118613, 0x83908392, 0x418c8614, 0x420b4241, 0x86168617, 0x41cb44de, 0x861f8621, 0x85078623, 0x44d1862b, 0x86258618, 0x423d40d7, 0x83af83b3, 0x422e862c, 0x862e8630, 0x86328638, 0x863a863c, 0x86408641, 0x86574533, 0x42088659, 0x4534422c, 0x865a869c, 0x423240b0, 0x86ad86ae, 0x40a440a2, 0x420986b8, 0x41c340af, 0x41b040a3, 0x454286ce, 0x86cf8643, 0x86d18646, 0x453d86d6, 0x87058438, 0x44ce4328, 0x871243a2, 0x87164240, 0x43a08464, 0x43a3421c, 0x846941d3, 0x4242424a, 0x872284a4, 0x873585e6, 0x42448761, 0x41cc4246, 0x876485ea, 0x4547876a, 0x84c74378, 0x4548423e, 0x4213876e, 0x87714544, 0x45104248, 0x419484fc, 0x420d8772, 0x87758520, 0x850a4552, 0x87768512, 0x41128516, 0x851a4116, 0x851d4118, 0x87838788, 0x85238789, 0x878e8526, 0x453b8792, 0x4238858d, 0x454a4233, 0x445e8797, 0x44608799, 0x879c8545, 0x8547430f, 0x879f854e, 0x453987a4, 0x8558422d, 0x87a887c8, 0x454e87c9, 0x454f87d5, 0x87d84536, 0x40d38560, 0x856487e8, 0x856944f4, 0x87ec8575, 0x415e87fa, 0x880b880c, 0x8590880d, 0x880e8593, 0x881187f5, 0x8819881f, 0x87f98820, 0x42264119, 0x85a58822, 0x85aa8824, 0x85bb8825, 0x88274231, 0x423b882f, 0x88298834, 0x8837422f, 0x411e8839, 0x883c4239, 0x883f8847, 0x8849884a, 0x41ce44f7, 0x45354155, 0x884d860c, 0x43a8421d, 0x885b43a5, 0x43a4861a, 0x885f4219, 0x861e446f, 0x88608875, 0x862a8878, 0x424c4171, 0x4468887a, 0x887d4214, 0x88848637, 0x417841af, 0x446e889f, 0x420e8642, 0x451e41c5, 0x41c44180, 0x88a788a8, 0x4422864f, 0x865288ad, 0x41b688b1, 0x40aa8656, 0x41b788b6, 0x88b788bf, 0x40b240a9, 0x42364384, 0x407b4383, 0x88c240e5, 0x41684235, 0x416b88c7, 0x418940a8, 0x40a641bb, 0x88e84553, 0x88ea88ed, 0x454940b1, 0x88ee88ef, 0x88f08902, 0x89038906, 0x8909454b, 0x890c411a, 0x88ff890a, 0x89148918, 0x8929892a, 0x892b892c, 0x45508931, 0x40cf893b, 0x893a893e, 0x44f54461, 0x89444462, 0x42278947, 0x451f423c, 0x422b8917, 0x8943894a, 0x894e895e, 0x40a7895f, 0x42168961, 0x41d48928, 0x892d412a, 0x89628963, 0x89648965, 0x89668968, 0x896a8972, 0x89758971, 0x8977897a, 0x897344f7, 0x44f6897b, 0x897d897e, 0x897f8983, 0x8988898d, 0x89b989bd, 0x89c0432c, 0x89eb4554, 0x417a40d8, 0x89ef40d6, 0x89f489fb, 0x89fd421f, 0x4182421e, 0x44708a14, 0x45148a17, 0x8a1d4172, 0x8a218a27, 0x8a284215, 0x8a2989fe, 0x41d2420f, 0x45078a03, 0x8a2c41c6, 0x41b34229, 0x42284517, 0x8a0a4345, 0x8a208a2a, 0x8a2d8a2e, 0x43a18a32, 0x8a338a34, 0x8a3b414d, 0x89d5414f, 0x418b8a42, 0x45328a55, 0x8a5f8a62, 0x8a75411f, 0x8a768a77, 0x8a878a78, 0x8a7d8a89, 0x8a948a96, 0x8a8a8a8c, 0x8a988a99, 0x8a9e445f, 0x8aad8aba, 0x8ac08ac1, 0x40d240d5, 0x8ac38a16, 0x8ac48a19, 0x41d58ac7, 0x8a394220, 0x8ac8454c, 0x45028acc, 0x8acd8ad6, 0x8ace43a7, 0x8ad88ae4, 0x419543aa, 0x8aed8af0, 0x8af78afd, 0x8ad20000, 0x87664423, 0x87694427, 0x4424876d, 0x45458770, 0x44294426, 0x44e08a67, 0x41bc8774, 0x0, 0x43a6, 0x41b48785, 0x87870000, 0x4503, 0x40d940d1, 0x87908a83, 0x879543a9, 0x8a9b, 0x879b0000, 0x45040000, 0x414e0000, 0x4251, 0x427d0000, 0x87a7, 0x44f90000, 0x41b50000, 0x411d4509, 0x44e0, 0x8acb, 0x45080000, 0x4425432a, 0x442840d0, 0x43ab, 0x87c743ad, 0x43af43ac, 0x43ae, 0x43b0411c, 0x0, 0x87d70000, 0x0, 0x40ae, 0x422a87e6, 0x87eb, 0x0, 0x45190000, 0x45188801, 0x0, 0x0, 0x432b0000, 0x0, 0x4516, 0x0, 0x0, 0x0, 0x45110000, 0x451b, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x450a0000, 0x0, 0x40d44210, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4512, 0x8846, 0x8ae50000, 0x0, 0x8ae8, 0x0, 0x45130000, 0x0, 0x0, 0x8aec450c, 0x450b, 0x4211, 0x8aef, 0x451d, 0x8afb, 0x8afc0000, 0x0, 0x42b5, 0x451c0000, 0x0, 0x889a, 0x0, 0x0, 0x0, 0x450d0000, 0x88b50000, 0x0, 0x0, 0x0, 0x4379, 0x437a437b, 0x0, 0x0, 0x450e0000, 0x41960000, 0x450f, 0x0, 0x0, 0x88ec0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x430e0000, 0x0, 0x89080000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x893d0000, 0x0, 0x0, 0x4252, 0x427e0000, 0x894d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x896f0000, 0x0, 0x0, 0x89790000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x898c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41290000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x89cb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41970000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x89f90000, 0x4259427f, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8a360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8a41, 0x0, 0x0, 0x0, 0x0, 0x42b60000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8a728a74, 0x0, 0x425a4253, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x425b4280, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8abf0000, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x0, 0x0, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x0, 0x4260, 0x42640000, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4444, 0x0, 0x0, 0x44454442, 0x82c5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4455, 0x0, 0x0, 0x8554, 0x0, 0x0, 0x0, 0x4459, 0x0, 0x44570000, 0x4454, 0x0, 0x44430000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4458, 0x0, 0x44560000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x445c0000, 0x0, 0x0, 0x44b10000, 0x44b30000, 0x0, 0x0, 0x0, 0x0, 0x445d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449a0000, 0x449c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44ba0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80844055, 0x80844068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80844043, 0x4042, 0x402d, 0x80844062, 0x40408084, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668084, 0x80848084, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x80844021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x80848084, 0x42f6, 0x808442f3, 0x42f50000, 0x44408084, 0x42d742cd, 0x40334030, 0x42cf8084, 0x403242cc, 0x42d18084, 0x8084402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138084, 0x80844011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x80840000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d58084, 0x808442d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80844174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80844176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18084, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x8084, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8084, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80854055, 0x80854068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80854043, 0x4042, 0x402d, 0x80854062, 0x40408085, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668085, 0x81598085, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81594021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81598085, 0x42f6, 0x808542f3, 0x42f50000, 0x44408085, 0x42d742cd, 0x40334030, 0x42cf8085, 0x403242cc, 0x42d18085, 0x8159402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138159, 0x80854011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81590000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d58085, 0x815942d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80854174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80854176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18159, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x8085, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8085, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x818e4055, 0x83474068, 0x4052856f, 0x406087fb, 0x4067, 0x0, 0x406b, 0x406580dc, 0x404c0000, 0x405d, 0x405f, 0x405a, 0x4069, 0x404b4317, 0x0, 0x0, 0x82ae0000, 0x0, 0x4062, 0x0, 0x4057405c, 0x40580000, 0x0, 0x4056404e, 0x4316, 0x405e44e1, 0x4059, 0x0, 0x404f4064, 0x406c, 0x0, 0x40610000, 0x0, 0x44e4, 0x405b0000, 0x0, 0x4051, 0x405044e3, 0x44e2, 0x406d0000, 0x4063, 0x40544053, 0x404d, 0x40660000, 0x0, 0x0, 0x406a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42f6, 0x80dc42f3, 0x42f50000, 0x0, 0x42cd, 0x40334030, 0x42cf0000, 0x403242cc, 0x42d10000, 0x402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x0, 0x43350000, 0x0, 0x0, 0x0, 0x42fc0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x82ae0000, 0x0, 0x0, 0x43060000, 0x42fd0000, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d24300, 0x42e9, 0x0, 0x0, 0x0, 0x0, 0x42fa0000, 0x0, 0x433d, 0x44ed0000, 0x430142ae, 0x0, 0x42c4, 0x0, 0x42fe0000, 0x42fb, 0x0, 0x44c60000, 0x0, 0x0, 0x0, 0x0, 0x44c044e6, 0x42ff, 0x0, 0x433b0000, 0x0, 0x434e0000, 0x0, 0x0, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x0, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a50000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433a, 0x0, 0x0, 0x0, 0x0, 0x434f, 0x43510000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43310000, 0x0, 0x0, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x0, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x0, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433e44ee, 0x0, 0x0, 0x438d, 0x0, 0x44c244e8, 0x44ea0000, 0x0, 0x0, 0x433c0000, 0x0, 0x43504352, 0x0, 0x0, 0x0, 0x43330000, 0x0, 0x43be0000, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x0, 0x43380000, 0x4193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a60000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43b9, 0x43320000, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4303, 0x430c4309, 0x43040000, 0x0, 0x430b0000, 0x0, 0x0, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42c7, 0x42c242c1, 0x0, 0x0, 0x0, 0x0, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42cb42c9, 0x0, 0x0, 0x43ba4334, 0x4319, 0x43e9, 0x4371, 0x0, 0x44be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x0, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x0, 0x43bf0000, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44730000, 0x0, 0x0, 0x0, 0x4305430d, 0x0, 0x42f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c70000, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4323, 0x43250000, 0x0, 0x0, 0x4368438b, 0x44ec41fb, 0x0, 0x41f0, 0x42f2, 0x43890000, 0x43c00000, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x0, 0x0, 0x0, 0x44740000, 0x0, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x0, 0x0, 0x4321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x0, 0x4387, 0x43bc0000, 0x4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43c9, 0x0, 0x0, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x0, 0x43ca0000, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80864055, 0x80864068, 0x40524049, 0x40604006, 0x40484067, 0x80868086, 0x403e406b, 0x40658086, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80864043, 0x80864042, 0x8086402d, 0x4062, 0x40408086, 0x4057405c, 0x40588086, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668086, 0x80868086, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x80868086, 0x40a1401c, 0x403c8086, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x8086, 0x0, 0x0, 0x80868086, 0x0, 0x0, 0x0, 0x40090000, 0x80864021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x80868086, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x40138086, 0x80864011, 0x8086, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80860000, 0x0, 0x80860000, 0x80860000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x44c98299, 0x85848586, 0x411b4111, 0x82908597, 0x43464369, 0x436a865c, 0x832b4160, 0x415d415f, 0x44d441fc, 0x40f74154, 0x415683b6, 0x44cf439d, 0x83c9439f, 0x4107410d, 0x41068697, 0x4170841a, 0x80d4841e, 0x419a4467, 0x44d2410a, 0x4108410c, 0x410e410f, 0x41104221, 0x8299418f, 0x869a41f1, 0x41778447, 0x86a1412c, 0x86b186bb, 0x4391419b, 0x845b41ae, 0x86bc446d, 0x846b84fa, 0x44d6846f, 0x417f86de, 0x439e4354, 0x848586df, 0x43c243cc, 0x43cb86e9, 0x86df86df, 0x423f4380, 0x84cc4381, 0x416a4169, 0x416784d3, 0x8714871c, 0x418884ee, 0x41ba84f2, 0x437e871d, 0x42184354, 0x4357421a, 0x419840f9, 0x426740fe, 0x81e341ca, 0x4279424b, 0x40a18685, 0x843e41c2, 0x8687412d, 0x40fb406e, 0x845f8306, 0x438240ea, 0x412e8230, 0x41cd868e, 0x412b4113, 0x40fa4353, 0x4353427b, 0x44d9423a, 0x40fc41cf, 0x40fd4270, 0x872480d4, 0x44f84114, 0x44d84358, 0x41bf4273, 0x42244230, 0x873941a1, 0x41a0419e, 0x42778758, 0x877b4142, 0x413a4275, 0x877e413d, 0x877f40ce, 0x85b14144, 0x40f8878a, 0x832b4310, 0x82908597, 0x832b83b6, 0x85f54278, 0x43ed8597, 0x870d43f5, 0x42374115, 0x841a841e, 0x44af44ac, 0x43fc4348, 0x87d04234, 0x42224223, 0x43f74353, 0x4117421b, 0x43fb43ff, 0x43fd831c, 0x44b041c9, 0x8586448e, 0x80d44249, 0x42124491, 0x420a4179, 0x424540c8, 0x420c44cb, 0x4181413f, 0x835d40a5, 0x43124243, 0x42714183, 0x42664225, 0x418a4247, 0x42178230, 0x843e87e1, 0x844740ca, 0x87f1427a, 0x418c8584, 0x420b4241, 0x845b8586, 0x41cb8597, 0x846b846f, 0x865c8697, 0x869a44dc, 0x8626845f, 0x423d40d7, 0x881b86bc, 0x422e86df, 0x86e986df, 0x86df8639, 0x84d38835, 0x84ee84f2, 0x87248584, 0x42088586, 0x436d422c, 0x83b6841a, 0x423240b0, 0x436c41d1, 0x40a440a2, 0x42098447, 0x41c340af, 0x41b040a3, 0x436f41e1, 0x885240ff, 0x447a4122, 0x4103846f, 0x41e4436e, 0x44ce8856, 0x410143a2, 0x87174240, 0x43a041fe, 0x43a3421c, 0x41ff41d3, 0x4242424a, 0x84d38230, 0x84ee8306, 0x42448856, 0x41cc4246, 0x886c41e3, 0x410041f4, 0x886b8881, 0x8894423e, 0x4213439b, 0x43934395, 0x45104248, 0x419484fd, 0x420d4102, 0x410480d4, 0x41f84396, 0x439741f3, 0x411241f6, 0x41a24116, 0x41d04118, 0x82904394, 0x420c4398, 0x439c4203, 0x439986df, 0x42388894, 0x88564233, 0x445e8798, 0x446043c5, 0x43c441f9, 0x439a43c6, 0x43d943de, 0x43d643e6, 0x43e5422d, 0x43e043db, 0x43d343da, 0x43d743d0, 0x43ce43d8, 0x40d343d1, 0x43d443e1, 0x43dd43e2, 0x85b18230, 0x415e4265, 0x43d243dc, 0x43e4832b, 0x43e343d5, 0x83b643df, 0x841a88c1, 0x859e435b, 0x42264119, 0x43cf8447, 0x88af435a, 0x43ef85be, 0x43624231, 0x423b85d4, 0x846f43f6, 0x8717422f, 0x411e84d3, 0x84ee4239, 0x43f9870d, 0x88d188e3, 0x41ce44f7, 0x43664155, 0x44774365, 0x43a8421d, 0x4482885c, 0x885a4363, 0x87d04219, 0x435f446f, 0x435e88d1, 0x823088d1, 0x424c4171, 0x4468841e, 0x899e4214, 0x8856435d, 0x417841af, 0x446e846b, 0x420e84fd, 0x451e41c5, 0x41c44180, 0x426a88a9, 0x44224364, 0x436044ca, 0x41b64347, 0x40aa4361, 0x41b741a3, 0x44ad8586, 0x40b240a9, 0x42364384, 0x407b4383, 0x871740e5, 0x41684235, 0x416b8934, 0x418940a8, 0x40a641bb, 0x40704077, 0x41a888d7, 0x407840b1, 0x41a54076, 0x889440ab, 0x407340ad, 0x407940ac, 0x4199411a, 0x82908798, 0x87e140e9, 0x40ec89b3, 0x40714072, 0x419f4109, 0x40cf4272, 0x85b1410b, 0x448f4461, 0x42744462, 0x42278290, 0x451f423c, 0x422b435c, 0x85978597, 0x87f186df, 0x40a788e3, 0x42168584, 0x41d4406f, 0x4128412a, 0x8586436b, 0x858440a2, 0x841e8586, 0x407b41da, 0x41db846b, 0x447e420c, 0x88a944f7, 0x44f688d1, 0x41fd8717, 0x43144269, 0x41904250, 0x41d841d6, 0x426d41f2, 0x88d14276, 0x417a40d8, 0x439040d6, 0x438f4392, 0x427c421f, 0x4182421e, 0x447043c1, 0x451441e6, 0x899d4172, 0x40e840eb, 0x41914215, 0x43c34202, 0x41d2420f, 0x450741dd, 0x41e041c6, 0x41b34229, 0x42284517, 0x43cd43f1, 0x43f243fa, 0x43fe41f7, 0x43a1437c, 0x437d437f, 0x4359414d, 0x89d6414f, 0x418b89d9, 0x41bd88d1, 0x88d14478, 0x447f411f, 0x413b43ee, 0x868e4400, 0x43f84401, 0x413f413c, 0x41394141, 0x41434311, 0x4315445f, 0x424e426c, 0x44ae424f, 0x40d240d5, 0x40c985b1, 0x44908290, 0x41d58a31, 0x89d64220, 0x88d1447b, 0x450241e2, 0x41d9447c, 0x41d743a7, 0x447d41e8, 0x419543aa, 0x41ec88d1, 0x875d88d1, 0x859e88d1, 0x406e4423, 0x41f54427, 0x442443f4, 0x8a604075, 0x44294426, 0x44e089d6, 0x41bc406f, 0x8a694479, 0x448343a6, 0x41b441a7, 0x40a24140, 0x41384503, 0x40d940d1, 0x40e385b1, 0x41a443a9, 0x40cb89d6, 0x407442b8, 0x45048ab0, 0x414e41be, 0x41a64481, 0x88d141a9, 0x41de8230, 0x44f941df, 0x41b540e2, 0x411d4509, 0x88d144e0, 0x426885b1, 0x45088ad7, 0x44258a63, 0x442840d0, 0x88d143ab, 0x448043ad, 0x43af43ac, 0x87cf43ae, 0x43b0411c, 0x424d0000, 0x426f0000, 0x419c, 0x40ed40ae, 0x422a41ea, 0x41eb, 0x0, 0x45190000, 0x45188230, 0x0, 0x0, 0x85d40000, 0x40ef0000, 0x419d4516, 0x0, 0x0, 0x43a6, 0x45110000, 0x451b, 0x0, 0x0, 0x0, 0x0, 0x8abd, 0x0, 0x450a0000, 0x0, 0x40d44210, 0x0, 0x0, 0x0, 0x0, 0x41e90000, 0x0, 0x0, 0x875d, 0x4512, 0x8adc8add, 0x420040ee, 0x0, 0x4201, 0x0, 0x451341e5, 0x40f00000, 0x0, 0x4121450c, 0x8ae2450b, 0x4211, 0x41dc, 0x451d, 0x4127, 0x41050000, 0x0, 0x40e4, 0x451c8af2, 0x0, 0x8af30000, 0x42b98af9, 0x0, 0x0, 0x450d0000, 0x8b0243f0, 0x8b048b06, 0x8b09, 0x8b0b, 0x0, 0x0, 0x0, 0x0, 0x450e0000, 0x41960000, 0x450f, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4137, 0x0, 0x0, 0x0, 0x0, 0x4313, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x426b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8230, 0x0, 0x0, 0x87cf0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41e7, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41200000, 0x0, 0x0, 0x41294125, 0x4126, 0x0, 0x43480000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43f30000, 0x0, 0x0, 0x0, 0x0, 0x41970000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x426e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4123, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80874055, 0x80874068, 0x40524049, 0x40604006, 0x40484067, 0x80878087, 0x403e406b, 0x40658087, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80874043, 0x44964042, 0x8087402d, 0x44944062, 0x40408087, 0x4057405c, 0x40588087, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40254497, 0x404f4064, 0x4010406c, 0x40058087, 0x40614029, 0x40264007, 0x808744e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x808744e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668087, 0x80878087, 0x402a4004, 0x4498406a, 0x40284499, 0x40f9, 0x449640fe, 0x40244022, 0x80878087, 0x40a1401c, 0x403c8087, 0x403a4495, 0x40fb0000, 0x4039401e, 0x8087403b, 0x80878087, 0x80878087, 0x80874444, 0x40fa8087, 0x80878087, 0x44454442, 0x40fc8087, 0x40fd8087, 0x400940b6, 0x80874021, 0x40b74019, 0x401b8087, 0x40b44036, 0x40174015, 0x80878087, 0x40388087, 0x42f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x80870000, 0x4035, 0x44c344c5, 0x0, 0x8087, 0x40138087, 0x80874011, 0x444a8087, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b4455, 0x415b4002, 0x44960000, 0x8087, 0x401d, 0x0, 0x401f, 0x445a4459, 0x80870000, 0x44570000, 0x80874454, 0x80878087, 0x44430000, 0x8087, 0x40b5, 0x0, 0x40be40c0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x444b442d, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e540ff, 0x4122, 0x41030000, 0x43390000, 0x44480000, 0x4101434d, 0x44060000, 0x4446, 0x44040000, 0x40b80000, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x4458, 0x4100434b, 0x445640c6, 0x0, 0x4161, 0x43350000, 0x0, 0x40bf40c1, 0x4102, 0x410440b9, 0x440f4020, 0x44304449, 0x442e4447, 0x442b401a, 0x445c4410, 0x44124018, 0x4016, 0x44b140c7, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x40ba445d, 0x0, 0x40c4, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x40bb0000, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40bc40b3, 0x40c2, 0x41570000, 0x40bd0000, 0x44c60000, 0x0, 0x40c3, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x4128412a, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x41210000, 0x0, 0x0, 0x0, 0x416d0000, 0x4127, 0x410542ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x412041fa, 0x418d4465, 0x41ef, 0x41294125, 0x4126, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed4123, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80884055, 0x80884068, 0x40524049, 0x40604006, 0x40484067, 0x80888088, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80884043, 0x4042, 0x402d, 0x4062, 0x40408088, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668088, 0x815a8088, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x815a8088, 0x40a1401c, 0x403c815a, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x8088, 0x0, 0x0, 0x80888088, 0x0, 0x0, 0x0, 0x40090000, 0x815a4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x815a8088, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x4013815a, 0x80884011, 0x815a, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80880000, 0x0, 0x80880000, 0x815a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80894055, 0x80894068, 0x40524049, 0x40604006, 0x40484067, 0x80898089, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80894043, 0x4042, 0x402d, 0x4062, 0x40408089, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668089, 0x815b8089, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x815b8089, 0x40a1401c, 0x403c815b, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x8089, 0x0, 0x0, 0x80898089, 0x0, 0x0, 0x0, 0x40090000, 0x815b4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x815b8089, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x4013815b, 0x80894011, 0x815b, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80890000, 0x0, 0x80890000, 0x815b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] value3 = null;

        protected static void value3Init()
        {
            value3 = new int[][]
            {
                { 0x0, 0x0, 0x4049, 0x0, 0x40480000, 0x80b40000, 0x403e0000, 0x0, 0x4045, 0x0, 0x402e0000, 0x40410000, 0x0, 0x0, 0x40460000, 0x4043, 0x4042, 0x402d, 0x0, 0x40400000, 0x0, 0x0, 0x40470000, 0x0, 0x0, 0x44e1, 0x404a0000, 0x40250000, 0x0, 0x0, 0x0, 0x4029, 0x40260000, 0x0, 0x4044, 0x402b0000, 0x403f0000, 0x0, 0x0, 0x403d, 0x402c0000, 0x0, 0x40270000, 0x0, 0x0, 0x402a0000, 0x0, 0x40280000, 0x0, 0x0, 0x40244022, 0x826e80b4, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x0, 0x415b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x442d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x442a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4161, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x442b0000, 0x0, 0x0, 0x0, 0x0, 0x40f5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4441, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40b3, 0x0, 0x41570000, 0x0, 0x0, 0x0, 0x0, 0x415c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4162, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41c0, 0x0, 0x41730000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x0, 0x4469, 0x0, 0x417b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43eb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4163, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x0, 0x44bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x444c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40da0000, 0x41580000, 0x40f6, 0x0, 0x0, 0x4159, 0x4152, 0x0, 0x0, 0x0, 0x416e, 0x44640000, 0x0, 0x0, 0x41c14175, 0x0, 0x41ab, 0x41ad41b2, 0x0, 0x446a, 0x417d, 0x441b, 0x0, 0x0, 0x0, 0x43ec, 0x0, 0x4165, 0x4186, 0x41b94193, 0x0, 0x44bd0000, 0x0, 0x0, 0x0, 0x0, 0x4151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x0, 0x417c, 0x0, 0x0, 0x0, 0x43e70000, 0x0, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40f30000, 0x0, 0x0, 0x0, 0x444e, 0x444d, 0x44500000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40dc, 0x40db, 0x40de0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415a, 0x4153, 0x416f0000, 0x0, 0x4176, 0x0, 0x0, 0x417e0000, 0x43e9, 0x0, 0x41660000, 0x418744be, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418d4465, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x446b0000, 0x441c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43e8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40f1, 0x0, 0x444f4452, 0x44510000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40dd40e0, 0x40df0000, 0x0, 0x40f40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418e4466, 0x0, 0x0, 0x446c, 0x441d, 0x0, 0x0, 0x0, 0x43ea, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441e4149, 0x0, 0x0, 0x0, 0x4453, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40e1, 0x0, 0x40f20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4420, 0x414b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x0, 0x0, 0x414a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4421, 0x414c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x808a4055, 0x808a4068, 0x40524049, 0x40604006, 0x40484067, 0x80b580c1, 0x403e406b, 0x406580dd, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80cd4043, 0x82af4042, 0x402d, 0x80f74062, 0x40408105, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x4025811b, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x812944e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066808a, 0x815c808a, 0x402a4004, 0x8184406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x826f80b5, 0x40a1401c, 0x403c827a, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x82860000, 0x832980c1, 0x4444, 0x0, 0x828682c1, 0x44454442, 0x82c6, 0x82c1, 0x400940b6, 0x815c4021, 0x40b74019, 0x401b8323, 0x40b44036, 0x40174015, 0x828780cd, 0x403842f6, 0x82ad42f3, 0x42f54037, 0x444080f7, 0x42d742cd, 0x40334030, 0x42cf80f7, 0x403242cc, 0x42d180f7, 0x82d3402f, 0x40314035, 0x44c344c5, 0x0, 0x851f, 0x401382fb, 0x81054011, 0x851f, 0x42ad811b, 0x42bf42bc, 0x42be42bd, 0x42bb42ba, 0x42af8314, 0x442c4023, 0x400b4455, 0x415b4002, 0x0, 0x8555, 0x401d, 0x0, 0x401f, 0x445a4459, 0x82860000, 0x44570000, 0x82c14454, 0x85538553, 0x44430000, 0x85b8, 0x40b5, 0x0, 0x40be40c0, 0x44c4, 0x0, 0x400a400e, 0x4293429a, 0x428d4288, 0x429b428b, 0x81848184, 0x42894299, 0x83a58184, 0x442d, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x4458, 0x434b, 0x445640c6, 0x0, 0x4161, 0x43350000, 0x0, 0x40bf40c1, 0x0, 0x42fc0000, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x445c4410, 0x44124018, 0x4016, 0x44b140c7, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x8541445d, 0x0, 0x40c4, 0x43064441, 0x42fd0000, 0x42d342d4, 0x42d580f7, 0x82d342d8, 0x42ce42d0, 0x42d24300, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x42fa4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x430142ae, 0x42b00000, 0x42c4, 0x40b3, 0x42fe40c2, 0x415742fb, 0x0, 0x44c60000, 0x0, 0x40c3, 0x415c0000, 0x0, 0x44c044e6, 0x42ff, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x4294428e, 0x428c428a, 0x428f0000, 0x4290, 0x81844291, 0x83a50000, 0x42a10000, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x42b1, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c42b2, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x42a30000, 0x42a74292, 0x42a942a2, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x88924174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x42b34322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x88924176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab42a4, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e189b7, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x4297, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x42b44323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42874298, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x429d0000, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd4295, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x429f429e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x4296, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x42a00000, 0x0, 0x41ed0000, 0x8892, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8892, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x808b0000, 0x0, 0x4049, 0x4006, 0x40480000, 0x808b, 0x403e0000, 0x80c6, 0x4045, 0x0, 0x402e0000, 0x40410000, 0x40080000, 0x0, 0x40464001, 0x4043, 0x44964042, 0x808b402d, 0x44940000, 0x40400000, 0x0, 0x0, 0x4047400c, 0x0, 0x40030000, 0x44e1, 0x404a0000, 0x40254497, 0x0, 0x0, 0x4005828b, 0x4029, 0x40264007, 0x808b0000, 0x4044, 0x402b0000, 0x403f0000, 0x0, 0x808b0000, 0x403d, 0x402c0000, 0x0, 0x40270000, 0x808b, 0x808b0000, 0x402a4004, 0x44980000, 0x40284499, 0x40f9, 0x449640fe, 0x0, 0x0, 0x40a1401c, 0x403c808b, 0x403a4495, 0x40fb0000, 0x4039401e, 0x808b403b, 0x808b0000, 0x808b808b, 0x808b4444, 0x40fa0000, 0x0, 0x44454442, 0x40fc808b, 0x40fd808b, 0x4009828b, 0x4021, 0x4019, 0x401b0000, 0x4036, 0x40174015, 0x0, 0x40380000, 0x0, 0x4037, 0x44400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x808b0000, 0x4035, 0x44c30000, 0x0, 0x808b, 0x40130000, 0x4011, 0x444a0000, 0x0, 0x0, 0x85210000, 0x0, 0x0, 0x0, 0x400b4455, 0x415b4002, 0x44960000, 0x808b, 0x401d, 0x0, 0x401f, 0x4459, 0x0, 0x44570000, 0x4454, 0x808b, 0x44430000, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x444b0000, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40ff, 0x4122, 0x41030000, 0x0, 0x44480000, 0x41010000, 0x44060000, 0x4446, 0x44040000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4458, 0x41000000, 0x44560000, 0x0, 0x4161, 0x0, 0x0, 0x0, 0x4102, 0x41048521, 0x440f4020, 0x44304449, 0x442e4447, 0x401a, 0x445c4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x0, 0x445d, 0x0, 0x0, 0x4441, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c0000, 0x0, 0x0, 0x0, 0x0, 0x40b3, 0x0, 0x41570000, 0x0, 0x0, 0x0, 0x0, 0x415c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4408, 0x0, 0x44050000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4162, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416c, 0x0, 0x4463, 0x0, 0x4128412a, 0x0, 0x0, 0x0, 0x0, 0x41c0, 0x0, 0x41730000, 0x0, 0x4407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x4469, 0x0, 0x417b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43eb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4163, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x0, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x0, 0x0, 0x0, 0x444c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40da0000, 0x41580000, 0x40f6, 0x0, 0x0, 0x4159, 0x4152, 0x0, 0x0, 0x0, 0x416e, 0x44640000, 0x0, 0x0, 0x41c14175, 0x0, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x446a, 0x417d, 0x441b, 0x0, 0x0, 0x0, 0x43ec, 0x0, 0x4165, 0x4186, 0x41b94193, 0x0, 0x44bd0000, 0x0, 0x0, 0x0, 0x0, 0x4151, 0x0, 0x0, 0x0, 0x41210000, 0x0, 0x0, 0x0, 0x416d0000, 0x4127, 0x41050000, 0x0, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b0000, 0x417c, 0x0, 0x0, 0x0, 0x43e70000, 0x0, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x0, 0x0, 0x0, 0x444e, 0x444d, 0x44500000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x0, 0x0, 0x40dc, 0x40db, 0x40de0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415a, 0x4153, 0x416f0000, 0x0, 0x4176, 0x440d, 0x0, 0x417e0000, 0x43e9, 0x0, 0x41660000, 0x418744be, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41200000, 0x418d4465, 0x0, 0x41294125, 0x4126, 0x0, 0x0, 0x0, 0x446b0000, 0x441c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43e8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x0, 0x444f4452, 0x44510000, 0x0, 0x0, 0x0, 0x0, 0x44a3, 0x0, 0x0, 0x0, 0x0, 0x40dd40e0, 0x40df0000, 0x0, 0x40f40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418e4466, 0x0, 0x0, 0x446c, 0x441d, 0x0, 0x0, 0x0, 0x43ea, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441e4149, 0x0, 0x0, 0x443f, 0x4453, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40e1, 0x0, 0x40f20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4420, 0x414b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x0, 0x0, 0x414a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4421, 0x414c, 0x0, 0x0, 0x4123, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x4055, 0x4068, 0x40524049, 0x40600000, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x4069, 0x404b4317, 0x40460000, 0x80ce4043, 0x4042, 0x402d, 0x4062, 0x40400000, 0x4057405c, 0x40580000, 0x40470000, 0x4056404e, 0x4316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x0, 0x40614029, 0x40260000, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40660000, 0x0, 0x402a0000, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80ce0000, 0x0, 0x0, 0x0, 0x80ce0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4036, 0x40174015, 0x828880ce, 0x40380000, 0x42f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x4035, 0x44c5, 0x0, 0x8288, 0x40130000, 0x4011, 0x8288, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x445a0000, 0x80ce0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x40340000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e0000, 0x0, 0x0, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43510000, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x0, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x41580000, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x0, 0x433c0000, 0x41c14175, 0x43504352, 0x41ab, 0x41ad41b2, 0x0, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x0, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x0, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44730000, 0x0, 0x0, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df0000, 0x0, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x0, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x44740000, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x0, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x0, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x808c4055, 0x808c4068, 0x40524049, 0x40604006, 0x40484067, 0x808c808c, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x808c4043, 0x82a24042, 0x402d, 0x4062, 0x4040808c, 0x4057405c, 0x405882a2, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066808c, 0x815d808c, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x815d808c, 0x40a1401c, 0x403c815d, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x808c, 0x0, 0x0, 0x808c808c, 0x0, 0x0, 0x0, 0x40090000, 0x815d4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x815d808c, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x4013815d, 0x808c4011, 0x815d, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x808c0000, 0x0, 0x808c0000, 0x815d0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x808d4055, 0x808d4068, 0x40524049, 0x40604006, 0x40484067, 0x808d808d, 0x403e406b, 0x406585b5, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x808d4043, 0x82a34042, 0x85b5402d, 0x85b54062, 0x4040808d, 0x4057405c, 0x405882a3, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066808d, 0x815e808d, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x815e808d, 0x40a1401c, 0x403c815e, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x808d, 0x0, 0x0, 0x808d808d, 0x0, 0x0, 0x0, 0x40090000, 0x815e4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x815e808d, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x4013815e, 0x808d4011, 0x815e, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x808d0000, 0x0, 0x808d0000, 0x815e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x808e0000, 0x0, 0x4049, 0x4006, 0x40480000, 0x808e, 0x403e0000, 0x808e, 0x4045, 0x0, 0x402e0000, 0x40410000, 0x40080000, 0x0, 0x40464001, 0x4043, 0x44964042, 0x808e402d, 0x44940000, 0x40400000, 0x0, 0x0, 0x4047400c, 0x0, 0x40030000, 0x44e1, 0x404a0000, 0x40254497, 0x0, 0x0, 0x40050000, 0x4029, 0x40264007, 0x808e0000, 0x4044, 0x402b0000, 0x403f0000, 0x0, 0x808e0000, 0x403d, 0x402c0000, 0x0, 0x40270000, 0x808e, 0x808e0000, 0x402a4004, 0x44980000, 0x40284499, 0x40f9, 0x449640fe, 0x0, 0x0, 0x40a1401c, 0x403c808e, 0x403a4495, 0x40fb0000, 0x4039401e, 0x808e403b, 0x808e0000, 0x808e808e, 0x808e4444, 0x40fa0000, 0x0, 0x44454442, 0x40fc808e, 0x40fd808e, 0x40090000, 0x4021, 0x4019, 0x401b0000, 0x4036, 0x40174015, 0x0, 0x40380000, 0x0, 0x4037, 0x44400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x808e0000, 0x4035, 0x44c30000, 0x0, 0x808e, 0x40130000, 0x4011, 0x444a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b4455, 0x415b4002, 0x44960000, 0x808e, 0x401d, 0x0, 0x401f, 0x4459, 0x0, 0x44570000, 0x4454, 0x808e, 0x44430000, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x444b0000, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40ff, 0x4122, 0x41030000, 0x0, 0x44480000, 0x41010000, 0x44060000, 0x4446, 0x44040000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4458, 0x41000000, 0x44560000, 0x0, 0x4161, 0x0, 0x0, 0x0, 0x4102, 0x41040000, 0x440f4020, 0x44304449, 0x442e4447, 0x401a, 0x445c4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x0, 0x445d, 0x0, 0x0, 0x4441, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c0000, 0x0, 0x0, 0x0, 0x0, 0x40b3, 0x0, 0x41570000, 0x0, 0x0, 0x0, 0x0, 0x415c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4408, 0x0, 0x44050000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4162, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416c, 0x0, 0x4463, 0x0, 0x4128412a, 0x0, 0x0, 0x0, 0x0, 0x41c0, 0x0, 0x41730000, 0x0, 0x4407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x4469, 0x0, 0x417b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43eb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4163, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x0, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x0, 0x0, 0x0, 0x444c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40da0000, 0x41580000, 0x40f6, 0x0, 0x0, 0x4159, 0x4152, 0x0, 0x0, 0x0, 0x416e, 0x44640000, 0x0, 0x0, 0x41c14175, 0x0, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x446a, 0x417d, 0x441b, 0x0, 0x0, 0x0, 0x43ec, 0x0, 0x4165, 0x4186, 0x41b94193, 0x0, 0x44bd0000, 0x0, 0x0, 0x0, 0x0, 0x4151, 0x0, 0x0, 0x0, 0x41210000, 0x0, 0x0, 0x0, 0x416d0000, 0x4127, 0x41050000, 0x0, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b0000, 0x417c, 0x0, 0x0, 0x0, 0x43e70000, 0x0, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x0, 0x0, 0x0, 0x444e, 0x444d, 0x44500000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x0, 0x0, 0x40dc, 0x40db, 0x40de0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415a, 0x4153, 0x416f0000, 0x0, 0x4176, 0x440d, 0x0, 0x417e0000, 0x43e9, 0x0, 0x41660000, 0x418744be, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41200000, 0x418d4465, 0x0, 0x41294125, 0x4126, 0x0, 0x0, 0x0, 0x446b0000, 0x441c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43e8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x0, 0x444f4452, 0x44510000, 0x0, 0x0, 0x0, 0x0, 0x44a3, 0x0, 0x0, 0x0, 0x0, 0x40dd40e0, 0x40df0000, 0x0, 0x40f40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418e4466, 0x0, 0x0, 0x446c, 0x441d, 0x0, 0x0, 0x0, 0x43ea, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441e4149, 0x0, 0x0, 0x443f, 0x4453, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40e1, 0x0, 0x40f20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4420, 0x414b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x0, 0x0, 0x414a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4421, 0x414c, 0x0, 0x0, 0x4123, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x808f4055, 0x808f4068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x808f4043, 0x4042, 0x402d, 0x808f4062, 0x4040808f, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066808f, 0x815f808f, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x815f4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x815f808f, 0x42f6, 0x808f42f3, 0x42f50000, 0x4440808f, 0x42d742cd, 0x40334030, 0x42cf808f, 0x403242cc, 0x42d1808f, 0x815f402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x4013815f, 0x808f4011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x815f0000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d5808f, 0x815f42d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x808f4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x808f4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e1815f, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x808f, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x808f, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80904055, 0x80904068, 0x40524049, 0x40604006, 0x40484067, 0x80908090, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80904043, 0x4042, 0x402d, 0x4062, 0x40408090, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668090, 0x81608090, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x81608090, 0x40a1401c, 0x403c8160, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x8090, 0x0, 0x0, 0x80908090, 0x0, 0x0, 0x0, 0x40090000, 0x81604021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81608090, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x40138160, 0x80904011, 0x8160, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80900000, 0x0, 0x80900000, 0x81600000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80914055, 0x80914068, 0x40524049, 0x40604006, 0x40484067, 0x80918091, 0x403e406b, 0x40658091, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80914043, 0x80914042, 0x8091402d, 0x4062, 0x40408091, 0x4057405c, 0x40588091, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40058122, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668091, 0x80918091, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x80918091, 0x40a1401c, 0x403c8091, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x8325, 0x8091, 0x0, 0x0, 0x80918091, 0x0, 0x0, 0x0, 0x400940b6, 0x80914021, 0x40b74019, 0x401b8122, 0x40b40000, 0x40174015, 0x80918091, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x40138091, 0x80914011, 0x8091, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80910000, 0x0, 0x80910000, 0x80910000, 0x0, 0x8325, 0x40b5, 0x0, 0x40be40c0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x40b80000, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x40c6, 0x0, 0x4161, 0x43350000, 0x0, 0x40bf40c1, 0x0, 0x40b9, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b140c7, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x40ba0000, 0x0, 0x40c4, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x40bb0000, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40bc40b3, 0x40c2, 0x41570000, 0x40bd0000, 0x44c60000, 0x0, 0x40c3, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80924055, 0x80924068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80924043, 0x4042, 0x402d, 0x80924062, 0x40408092, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668092, 0x81618092, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81614021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81618092, 0x42f6, 0x809242f3, 0x42f50000, 0x44408092, 0x42d742cd, 0x40334030, 0x42cf8092, 0x403242cc, 0x42d18092, 0x8161402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138161, 0x80924011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81610000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d58092, 0x816142d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80924174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80924176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18161, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x8092, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8092, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80934055, 0x80934068, 0x40524049, 0x40604006, 0x40484067, 0x82118211, 0x403e406b, 0x40658211, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80934043, 0x82e54042, 0x85df402d, 0x80934062, 0x40408093, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40258375, 0x404f4064, 0x4010406c, 0x40058211, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x837544e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668093, 0x81628093, 0x402a4004, 0x406a, 0x40288375, 0x0, 0x0, 0x82110000, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81624021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81628093, 0x42f6, 0x809342f3, 0x42f50000, 0x44408093, 0x42d742cd, 0x40334030, 0x42cf8093, 0x403242cc, 0x42d18093, 0x8162402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138162, 0x80934011, 0x0, 0x42ad0000, 0x0, 0x83750000, 0x0, 0x8375, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81620000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d58093, 0x816242d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80944055, 0x80944068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80944043, 0x4042, 0x402d, 0x80944062, 0x40408094, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668094, 0x81638094, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81634021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81638094, 0x42f6, 0x809442f3, 0x42f50000, 0x44408094, 0x42d742cd, 0x40334030, 0x42cf8094, 0x403242cc, 0x42d18094, 0x8163402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138163, 0x80944011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81630000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d58094, 0x816342d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80944174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80944176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18163, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x8094, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8094, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80954055, 0x80954068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80954043, 0x4042, 0x402d, 0x80954062, 0x40408095, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668095, 0x81648095, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81644021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81648095, 0x42f6, 0x809542f3, 0x42f50000, 0x44408095, 0x42d742cd, 0x40334030, 0x42cf8095, 0x403242cc, 0x42d18095, 0x8164402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138164, 0x80954011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81640000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d58095, 0x816442d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80964055, 0x80964068, 0x40524049, 0x40604006, 0x40484067, 0x83ac82e6, 0x403e406b, 0x406582e6, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80964043, 0x82e64042, 0x85e0402d, 0x80964062, 0x40408096, 0x4057405c, 0x4058864d, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40258096, 0x404f4064, 0x4010406c, 0x400585e0, 0x40614029, 0x40264007, 0x85e044e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668096, 0x81658096, 0x402a4004, 0x8096406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81654021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81658096, 0x42f6, 0x809642f3, 0x42f50000, 0x44408096, 0x42d742cd, 0x40334030, 0x42cf8096, 0x403242cc, 0x42d18096, 0x8165402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138165, 0x80964011, 0x0, 0x42ad8096, 0x42bf42bc, 0x42be42bd, 0x42bb42ba, 0x42af8165, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x4293429a, 0x428d4288, 0x429b428b, 0x80968096, 0x42894299, 0x81658096, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81650000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d58096, 0x816542d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x42b00000, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x4294428e, 0x428c428a, 0x428f0000, 0x4290, 0x80964291, 0x81650000, 0x42a10000, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x42b1, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c42b2, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x42a30000, 0x42a74292, 0x42a942a2, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x88934174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x42b34322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x88934176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab42a4, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e189b8, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x4297, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x42b44323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42874298, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x429d0000, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd4295, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x429f429e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x4296, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x42a00000, 0x0, 0x41ed0000, 0x8893, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8893, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] value4 = null;

        protected static void value4Init()
        {
            value4 = new int[][]
            {
                { 0x80974055, 0x80974068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80974043, 0x4042, 0x402d, 0x80974062, 0x40408097, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40668097, 0x81668097, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81664021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x81668097, 0x42f6, 0x809742f3, 0x42f50000, 0x44408097, 0x42d742cd, 0x40334030, 0x42cf8097, 0x403242cc, 0x42d18097, 0x8166402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138166, 0x80974011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81660000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d58097, 0x816642d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80974174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80974176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18166, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x8097, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x8097, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81850000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4293429a, 0x428d4288, 0x429b428b, 0x81858185, 0x42894299, 0x83a60000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4294428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a50000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4251, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a60000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4252, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ab0000, 0x42a842aa, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4286, 0x4297, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42590000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42874298, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ac0000, 0x429d0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4295, 0x425a4253, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x425b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x429f429e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4255, 0x4257, 0x4296, 0x425c4254, 0x0, 0x0, 0x0, 0x0, 0x42a00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x425d0000, 0x42610000, 0x4256, 0x4258, 0x0, 0x0, 0x0, 0x0, 0x425e, 0x4262425f, 0x4263, 0x0, 0x0, 0x4260, 0x42640000, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x81860000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4293429a, 0x428d4288, 0x429b428b, 0x81868186, 0x42894299, 0x83a78186, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4294428e, 0x428c428a, 0x428f0000, 0x4290, 0x81864291, 0x83a70000, 0x42a10000, 0x42a50000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4251, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42a30000, 0x42a74292, 0x42a942a2, 0x42a60000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4252, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ab42a4, 0x42a842aa, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4286, 0x4297, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42590000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42874298, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ac0000, 0x429d0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4295, 0x425a4253, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x425b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x429f429e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4255, 0x4257, 0x4296, 0x425c4254, 0x0, 0x0, 0x0, 0x0, 0x42a00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x425d0000, 0x42610000, 0x4256, 0x4258, 0x0, 0x0, 0x0, 0x0, 0x425e, 0x4262425f, 0x4263, 0x0, 0x0, 0x4260, 0x42640000, 0x0, 0x0, 0x0 },
                { 0x80980000, 0x0, 0x4049, 0x4006, 0x40480000, 0x8098, 0x403e0000, 0x8098, 0x4045, 0x0, 0x402e0000, 0x40410000, 0x40080000, 0x0, 0x40464001, 0x4043, 0x44964042, 0x8098402d, 0x44940000, 0x40400000, 0x0, 0x0, 0x4047400c, 0x0, 0x40030000, 0x44e1, 0x404a0000, 0x40254497, 0x0, 0x0, 0x40050000, 0x4029, 0x40264007, 0x80980000, 0x4044, 0x402b0000, 0x403f0000, 0x0, 0x80980000, 0x403d, 0x402c0000, 0x0, 0x40270000, 0x8098, 0x80980000, 0x402a4004, 0x44980000, 0x40284499, 0x40f9, 0x449640fe, 0x0, 0x0, 0x40a1401c, 0x403c8098, 0x403a4495, 0x40fb0000, 0x4039401e, 0x8098403b, 0x80980000, 0x80988098, 0x80984444, 0x40fa0000, 0x0, 0x44454442, 0x40fc8098, 0x40fd8098, 0x40090000, 0x4021, 0x4019, 0x401b0000, 0x4036, 0x40174015, 0x0, 0x40380000, 0x0, 0x4037, 0x44400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80980000, 0x4035, 0x44c30000, 0x0, 0x8098, 0x40130000, 0x4011, 0x444a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b4455, 0x415b4002, 0x44960000, 0x8098, 0x401d, 0x0, 0x401f, 0x4459, 0x0, 0x44570000, 0x4454, 0x8098, 0x44430000, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x444b0000, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40ff, 0x4122, 0x41030000, 0x0, 0x44480000, 0x41010000, 0x44060000, 0x4446, 0x44040000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4458, 0x41000000, 0x44560000, 0x0, 0x4161, 0x0, 0x0, 0x0, 0x4102, 0x41040000, 0x440f4020, 0x44304449, 0x442e4447, 0x401a, 0x445c4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x0, 0x445d, 0x0, 0x0, 0x4441, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c0000, 0x0, 0x0, 0x0, 0x0, 0x40b3, 0x0, 0x41570000, 0x0, 0x0, 0x0, 0x0, 0x415c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4408, 0x0, 0x44050000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4162, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416c, 0x0, 0x4463, 0x0, 0x4128412a, 0x0, 0x0, 0x0, 0x0, 0x41c0, 0x0, 0x41730000, 0x0, 0x4407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x4469, 0x0, 0x417b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43eb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4163, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x0, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x0, 0x0, 0x0, 0x444c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40da0000, 0x41580000, 0x40f6, 0x0, 0x0, 0x4159, 0x4152, 0x0, 0x0, 0x0, 0x416e, 0x44640000, 0x0, 0x0, 0x41c14175, 0x0, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x446a, 0x417d, 0x441b, 0x0, 0x0, 0x0, 0x43ec, 0x0, 0x4165, 0x4186, 0x41b94193, 0x0, 0x44bd0000, 0x0, 0x0, 0x0, 0x0, 0x4151, 0x0, 0x0, 0x0, 0x41210000, 0x0, 0x0, 0x0, 0x416d0000, 0x4127, 0x41050000, 0x0, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b0000, 0x417c, 0x0, 0x0, 0x0, 0x43e70000, 0x0, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x0, 0x0, 0x0, 0x444e, 0x444d, 0x44500000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x0, 0x0, 0x40dc, 0x40db, 0x40de0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415a, 0x4153, 0x416f0000, 0x0, 0x4176, 0x440d, 0x0, 0x417e0000, 0x43e9, 0x0, 0x41660000, 0x418744be, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41200000, 0x418d4465, 0x0, 0x41294125, 0x4126, 0x0, 0x0, 0x0, 0x446b0000, 0x441c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43e8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x0, 0x444f4452, 0x44510000, 0x0, 0x0, 0x0, 0x0, 0x44a3, 0x0, 0x0, 0x0, 0x0, 0x40dd40e0, 0x40df0000, 0x0, 0x40f40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418e4466, 0x0, 0x0, 0x446c, 0x441d, 0x0, 0x0, 0x0, 0x43ea, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441e4149, 0x0, 0x0, 0x443f, 0x4453, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40e1, 0x0, 0x40f20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4420, 0x414b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x0, 0x0, 0x414a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4421, 0x414c, 0x0, 0x0, 0x4123, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x4049, 0x0, 0x40480000, 0x80b60000, 0x403e0000, 0x0, 0x4045, 0x0, 0x402e0000, 0x40410000, 0x0, 0x0, 0x40460000, 0x4043, 0x4042, 0x402d, 0x0, 0x40400000, 0x0, 0x0, 0x40470000, 0x0, 0x0, 0x44e1, 0x404a0000, 0x40250000, 0x0, 0x0, 0x0, 0x4029, 0x40260000, 0x0, 0x4044, 0x402b0000, 0x403f0000, 0x0, 0x0, 0x403d, 0x402c0000, 0x0, 0x40270000, 0x0, 0x0, 0x402a0000, 0x0, 0x40280000, 0x0, 0x0, 0x40244022, 0x827080b6, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x0, 0x415b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x442d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x442a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4161, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x442b0000, 0x0, 0x0, 0x0, 0x0, 0x40f5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4441, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40b3, 0x0, 0x41570000, 0x0, 0x0, 0x0, 0x0, 0x415c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4162, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41c0, 0x0, 0x41730000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x0, 0x4469, 0x0, 0x417b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43eb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4163, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x0, 0x44bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x444c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40da0000, 0x41580000, 0x40f6, 0x0, 0x0, 0x4159, 0x4152, 0x0, 0x0, 0x0, 0x416e, 0x44640000, 0x0, 0x0, 0x41c14175, 0x0, 0x41ab, 0x41ad41b2, 0x0, 0x446a, 0x417d, 0x441b, 0x0, 0x0, 0x0, 0x43ec, 0x0, 0x4165, 0x4186, 0x41b94193, 0x0, 0x44bd0000, 0x0, 0x0, 0x0, 0x0, 0x4151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x0, 0x417c, 0x0, 0x0, 0x0, 0x43e70000, 0x0, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40f30000, 0x0, 0x0, 0x0, 0x444e, 0x444d, 0x44500000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40dc, 0x40db, 0x40de0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415a, 0x4153, 0x416f0000, 0x0, 0x4176, 0x0, 0x0, 0x417e0000, 0x43e9, 0x0, 0x41660000, 0x418744be, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418d4465, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x446b0000, 0x441c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43e8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40f1, 0x0, 0x444f4452, 0x44510000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40dd40e0, 0x40df0000, 0x0, 0x40f40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418e4466, 0x0, 0x0, 0x446c, 0x441d, 0x0, 0x0, 0x0, 0x43ea, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441e4149, 0x0, 0x0, 0x0, 0x4453, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40e1, 0x0, 0x40f20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4420, 0x414b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x0, 0x0, 0x414a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4421, 0x414c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80990000, 0x0, 0x4049, 0x4006, 0x40480000, 0x8099, 0x403e0000, 0x80c7, 0x4045, 0x0, 0x402e0000, 0x40410000, 0x40080000, 0x0, 0x40464001, 0x4043, 0x44964042, 0x8099402d, 0x44940000, 0x40400000, 0x0, 0x0, 0x4047400c, 0x0, 0x40030000, 0x44e1, 0x404a0000, 0x40254497, 0x0, 0x0, 0x40050000, 0x4029, 0x40264007, 0x80990000, 0x4044, 0x402b0000, 0x403f0000, 0x0, 0x80990000, 0x403d, 0x402c0000, 0x0, 0x40270000, 0x8099, 0x80990000, 0x402a4004, 0x44980000, 0x40284499, 0x40f9, 0x449640fe, 0x82ff0000, 0x0, 0x40a1401c, 0x403c8099, 0x403a4495, 0x40fb0000, 0x4039401e, 0x8099403b, 0x80990000, 0x80998099, 0x80994444, 0x40fa0000, 0x0, 0x44454442, 0x40fc8099, 0x40fd8099, 0x40090000, 0x4021, 0x4019, 0x401b0000, 0x4036, 0x40174015, 0x82ff0000, 0x40380000, 0x0, 0x4037, 0x44400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80990000, 0x4035, 0x44c30000, 0x0, 0x8099, 0x40130000, 0x4011, 0x444a0000, 0x0, 0x0, 0x0, 0x0, 0x858e, 0x0, 0x400b4455, 0x415b4002, 0x44960000, 0x8099, 0x401d, 0x0, 0x401f, 0x4459, 0x0, 0x44570000, 0x4454, 0x8099, 0x44430000, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x444b0000, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40ff, 0x4122, 0x41030000, 0x0, 0x44480000, 0x41010000, 0x44060000, 0x4446, 0x44040000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4458, 0x41000000, 0x44560000, 0x0, 0x4161, 0x0, 0x0, 0x0, 0x4102, 0x41040000, 0x440f4020, 0x44304449, 0x442e4447, 0x401a, 0x445c4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x858e, 0x0, 0x0, 0x0, 0x445d, 0x0, 0x0, 0x4441, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c0000, 0x0, 0x0, 0x0, 0x0, 0x40b3, 0x0, 0x41570000, 0x0, 0x0, 0x0, 0x0, 0x415c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4408, 0x0, 0x44050000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4162, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416c, 0x0, 0x4463, 0x0, 0x4128412a, 0x0, 0x0, 0x0, 0x0, 0x41c0, 0x0, 0x41730000, 0x0, 0x4407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x4469, 0x0, 0x417b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43eb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4163, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x0, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x0, 0x0, 0x0, 0x444c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40da0000, 0x41580000, 0x40f6, 0x0, 0x0, 0x4159, 0x4152, 0x0, 0x0, 0x0, 0x416e, 0x44640000, 0x0, 0x0, 0x41c14175, 0x0, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x446a, 0x417d, 0x441b, 0x0, 0x0, 0x0, 0x43ec, 0x0, 0x4165, 0x4186, 0x41b94193, 0x0, 0x44bd0000, 0x0, 0x0, 0x0, 0x0, 0x4151, 0x0, 0x0, 0x0, 0x41210000, 0x0, 0x0, 0x0, 0x416d0000, 0x4127, 0x41050000, 0x0, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b0000, 0x417c, 0x0, 0x0, 0x0, 0x43e70000, 0x0, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x0, 0x0, 0x0, 0x444e, 0x444d, 0x44500000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x0, 0x0, 0x40dc, 0x40db, 0x40de0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415a, 0x4153, 0x416f0000, 0x0, 0x4176, 0x440d, 0x0, 0x417e0000, 0x43e9, 0x0, 0x41660000, 0x418744be, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41200000, 0x418d4465, 0x0, 0x41294125, 0x4126, 0x0, 0x0, 0x0, 0x446b0000, 0x441c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43e8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x0, 0x444f4452, 0x44510000, 0x0, 0x0, 0x0, 0x0, 0x44a3, 0x0, 0x0, 0x0, 0x0, 0x40dd40e0, 0x40df0000, 0x0, 0x40f40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418e4466, 0x0, 0x0, 0x446c, 0x441d, 0x0, 0x0, 0x0, 0x43ea, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441e4149, 0x0, 0x0, 0x443f, 0x4453, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40e1, 0x0, 0x40f20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4420, 0x414b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x0, 0x0, 0x414a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4421, 0x414c, 0x0, 0x0, 0x4123, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x809a4055, 0x809a4068, 0x40524049, 0x40604006, 0x40484067, 0x809a809a, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x809a4043, 0x4042, 0x402d, 0x809a4062, 0x4040809a, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066809a, 0x8167809a, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x8167809a, 0x40a1401c, 0x403c8167, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x809a, 0x0, 0x0, 0x809a809a, 0x0, 0x0, 0x0, 0x40090000, 0x81674021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x8167809a, 0x42f6, 0x809a42f3, 0x42f50000, 0x4440809a, 0x42d742cd, 0x40334030, 0x42cf809a, 0x403242cc, 0x42d1809a, 0x8167402f, 0x40310000, 0x44c344c5, 0x0, 0x0, 0x40138167, 0x809a4011, 0x8167, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x809a0000, 0x0, 0x809a0000, 0x81670000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81670000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d5809a, 0x816742d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x809b4055, 0x809b4068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x809b4043, 0x4042, 0x402d, 0x809b4062, 0x4040809b, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066809b, 0x8168809b, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81684021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x8168809b, 0x42f6, 0x809b42f3, 0x42f50000, 0x4440809b, 0x42d742cd, 0x40334030, 0x42cf809b, 0x403242cc, 0x42d1809b, 0x8168402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138168, 0x809b4011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81680000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d5809b, 0x816842d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x809b4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x809b4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18168, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x809b, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x809b, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x809c4055, 0x809c4068, 0x40524049, 0x40604006, 0x40484067, 0x809c809c, 0x403e406b, 0x4065809c, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x809c4043, 0x44964042, 0x809c402d, 0x44944062, 0x4040809c, 0x4057405c, 0x4058809c, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40254497, 0x404f4064, 0x4010406c, 0x4005809c, 0x40614029, 0x40264007, 0x809c44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x809c44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066809c, 0x809c809c, 0x402a4004, 0x4498406a, 0x40284499, 0x40f9, 0x449640fe, 0x40244022, 0x809c809c, 0x40a1401c, 0x403c809c, 0x403a4495, 0x40fb0000, 0x4039401e, 0x809c403b, 0x809c809c, 0x809c809c, 0x809c4444, 0x40fa809c, 0x809c809c, 0x44454442, 0x40fc809c, 0x40fd809c, 0x400940b6, 0x809c4021, 0x40b74019, 0x401b809c, 0x40b44036, 0x40174015, 0x809c809c, 0x4038809c, 0x42f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x809c0000, 0x4035, 0x44c344c5, 0x0, 0x809c, 0x4013809c, 0x809c4011, 0x444a809c, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b4455, 0x415b4002, 0x44960000, 0x809c, 0x401d, 0x0, 0x401f, 0x445a4459, 0x809c0000, 0x44570000, 0x809c4454, 0x809c809c, 0x44430000, 0x809c, 0x40b5, 0x0, 0x40be40c0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x444b442d, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e540ff, 0x4122, 0x41030000, 0x43390000, 0x44480000, 0x4101434d, 0x44060000, 0x4446, 0x44040000, 0x40b80000, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x4458, 0x4100434b, 0x445640c6, 0x0, 0x4161, 0x43350000, 0x0, 0x40bf40c1, 0x4102, 0x410440b9, 0x440f4020, 0x44304449, 0x442e4447, 0x442b401a, 0x445c4410, 0x44124018, 0x4016, 0x44b140c7, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x40ba445d, 0x0, 0x40c4, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x40bb0000, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40bc40b3, 0x40c2, 0x41570000, 0x40bd0000, 0x44c60000, 0x0, 0x40c3, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x4128412a, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x41210000, 0x0, 0x0, 0x0, 0x416d0000, 0x4127, 0x410542ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x412041fa, 0x418d4465, 0x41ef, 0x41294125, 0x4126, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed4123, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x809d0000, 0x0, 0x4049, 0x4006, 0x40480000, 0x809d, 0x403e0000, 0x809d, 0x4045, 0x0, 0x402e0000, 0x40410000, 0x40080000, 0x0, 0x40464001, 0x4043, 0x44964042, 0x809d402d, 0x44940000, 0x40400000, 0x0, 0x0, 0x4047400c, 0x0, 0x40030000, 0x44e1, 0x404a0000, 0x40254497, 0x0, 0x0, 0x40050000, 0x4029, 0x40264007, 0x809d0000, 0x4044, 0x402b0000, 0x403f0000, 0x0, 0x809d0000, 0x403d, 0x402c0000, 0x0, 0x40270000, 0x809d, 0x809d0000, 0x402a4004, 0x44980000, 0x40284499, 0x40f9, 0x449640fe, 0x0, 0x0, 0x40a1401c, 0x403c809d, 0x403a4495, 0x40fb0000, 0x4039401e, 0x809d403b, 0x809d0000, 0x809d809d, 0x809d4444, 0x40fa0000, 0x0, 0x44454442, 0x40fc809d, 0x40fd809d, 0x40090000, 0x4021, 0x4019, 0x401b0000, 0x4036, 0x40174015, 0x0, 0x40380000, 0x0, 0x4037, 0x44400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x809d0000, 0x4035, 0x44c30000, 0x0, 0x809d, 0x40130000, 0x4011, 0x444a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b4455, 0x415b4002, 0x44960000, 0x809d, 0x401d, 0x0, 0x401f, 0x4459, 0x0, 0x44570000, 0x4454, 0x809d, 0x44430000, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x444b0000, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40ff, 0x4122, 0x41030000, 0x0, 0x44480000, 0x41010000, 0x44060000, 0x4446, 0x44040000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4458, 0x41000000, 0x44560000, 0x0, 0x4161, 0x0, 0x0, 0x0, 0x4102, 0x41040000, 0x440f4020, 0x44304449, 0x442e4447, 0x401a, 0x445c4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x0, 0x445d, 0x0, 0x0, 0x4441, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c0000, 0x0, 0x0, 0x0, 0x0, 0x40b3, 0x0, 0x41570000, 0x0, 0x0, 0x0, 0x0, 0x415c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4408, 0x0, 0x44050000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4162, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416c, 0x0, 0x4463, 0x0, 0x4128412a, 0x0, 0x0, 0x0, 0x0, 0x41c0, 0x0, 0x41730000, 0x0, 0x4407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x4469, 0x0, 0x417b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43eb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4163, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x0, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x0, 0x0, 0x0, 0x444c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40da0000, 0x41580000, 0x40f6, 0x0, 0x0, 0x4159, 0x4152, 0x0, 0x0, 0x0, 0x416e, 0x44640000, 0x0, 0x0, 0x41c14175, 0x0, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x446a, 0x417d, 0x441b, 0x0, 0x0, 0x0, 0x43ec, 0x0, 0x4165, 0x4186, 0x41b94193, 0x0, 0x44bd0000, 0x0, 0x0, 0x0, 0x0, 0x4151, 0x0, 0x0, 0x0, 0x41210000, 0x0, 0x0, 0x0, 0x416d0000, 0x4127, 0x41050000, 0x0, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b0000, 0x417c, 0x0, 0x0, 0x0, 0x43e70000, 0x0, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x0, 0x0, 0x0, 0x444e, 0x444d, 0x44500000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x0, 0x0, 0x40dc, 0x40db, 0x40de0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415a, 0x4153, 0x416f0000, 0x0, 0x4176, 0x440d, 0x0, 0x417e0000, 0x43e9, 0x0, 0x41660000, 0x418744be, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41200000, 0x418d4465, 0x0, 0x41294125, 0x4126, 0x0, 0x0, 0x0, 0x446b0000, 0x441c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43e8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x0, 0x444f4452, 0x44510000, 0x0, 0x0, 0x0, 0x0, 0x44a3, 0x0, 0x0, 0x0, 0x0, 0x40dd40e0, 0x40df0000, 0x0, 0x40f40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418e4466, 0x0, 0x0, 0x446c, 0x441d, 0x0, 0x0, 0x0, 0x43ea, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441e4149, 0x0, 0x0, 0x443f, 0x4453, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40e1, 0x0, 0x40f20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4420, 0x414b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x0, 0x0, 0x414a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4421, 0x414c, 0x0, 0x0, 0x4123, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x4055, 0x4068, 0x40520000, 0x40600000, 0x4067, 0x0, 0x406b, 0x40650000, 0x404c0000, 0x405d, 0x405f, 0x405a, 0x4069, 0x404b4317, 0x0, 0x0, 0x0, 0x0, 0x80f84062, 0x0, 0x4057405c, 0x40580000, 0x0, 0x4056404e, 0x4316, 0x405e44e1, 0x4059, 0x0, 0x404f4064, 0x406c, 0x0, 0x40610000, 0x0, 0x44e4, 0x405b0000, 0x0, 0x4051, 0x405044e3, 0x44e2, 0x406d0000, 0x4063, 0x40544053, 0x404d, 0x40660000, 0x0, 0x0, 0x406a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42f3, 0x42f50000, 0x80f8, 0x42d742cd, 0x40334030, 0x42cf0000, 0x403242cc, 0x42d180f8, 0x82d4402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x0, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43060000, 0x0, 0x42d342d4, 0x42d50000, 0x42d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433d, 0x44ed0000, 0x42ae, 0x0, 0x42c4, 0x0, 0x0, 0x0, 0x0, 0x44c60000, 0x0, 0x0, 0x0, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e0000, 0x0, 0x0, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x0, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a50000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433a, 0x0, 0x0, 0x0, 0x0, 0x434f, 0x43510000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43310000, 0x0, 0x0, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x0, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4308, 0x0, 0x430a4307, 0x0, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433e44ee, 0x0, 0x0, 0x438d, 0x0, 0x44c244e8, 0x44ea0000, 0x0, 0x0, 0x433c0000, 0x0, 0x43504352, 0x0, 0x0, 0x0, 0x43330000, 0x0, 0x43be0000, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x0, 0x43380000, 0x4193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a60000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43b9, 0x43320000, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x430c4309, 0x0, 0x0, 0x430b0000, 0x0, 0x0, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42c7, 0x42c242c1, 0x0, 0x0, 0x0, 0x0, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42cb42c9, 0x0, 0x0, 0x43ba4334, 0x4319, 0x43e9, 0x4371, 0x0, 0x44be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x0, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x0, 0x43bf0000, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44730000, 0x0, 0x0, 0x0, 0x430d, 0x0, 0x42f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c70000, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4323, 0x43250000, 0x0, 0x0, 0x4368438b, 0x44ec41fb, 0x0, 0x41f0, 0x42f2, 0x43890000, 0x43c00000, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x0, 0x0, 0x0, 0x44740000, 0x0, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x0, 0x0, 0x4321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x0, 0x42da4387, 0x43bc0000, 0x4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43c9, 0x0, 0x0, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x0, 0x43ca0000, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x44c986a7, 0x85858587, 0x86ac8709, 0x870b8598, 0x43464369, 0x436a865d, 0x87334131, 0x412f8677, 0x413041fc, 0x4134877c, 0x41454096, 0x44cf439d, 0x4098439f, 0x4099409e, 0x87ef8698, 0x409d409a, 0x4097409b, 0x8678409c, 0x409587f0, 0x88318833, 0x44df883b, 0x884b4221, 0x8863418f, 0x869b41f1, 0x8879887f, 0x86a2412c, 0x86b2438e, 0x43918690, 0x88ae4132, 0x86bd88cd, 0x88f186b7, 0x89008901, 0x89158936, 0x439e4354, 0x897686e0, 0x43c243cc, 0x43cb86ea, 0x8708870a, 0x423f8989, 0x89b242b7, 0x89e689f0, 0x41474146, 0x87158a0f, 0x8a138a1e, 0x8a528a64, 0x437e871e, 0x42188720, 0x4357421a, 0x8a85429c, 0x426787a1, 0x87c58815, 0x4279424b, 0x40a186e1, 0x8821884e, 0x886e8880, 0x88b0406e, 0x890e890f, 0x89128913, 0x896089a8, 0x41cd868f, 0x412b89f5, 0x89fa4353, 0x86e1427b, 0x8a1b423a, 0x8a1c41cf, 0x8a534270, 0x87258a86, 0x8a958a9a, 0x44d84358, 0x41bf4273, 0x42244230, 0x8ab44148, 0x8ae10000, 0x42770000, 0x41334142, 0x413a4275, 0x4135413d, 0x41360000, 0x4144, 0x878b, 0x4310, 0x87ca, 0x0, 0x4278, 0x43ed87cb, 0x454043f5, 0x423782f7, 0x0, 0x44af44ac, 0x43fc4348, 0x87d14234, 0x42224223, 0x43f7871f, 0x8aae421b, 0x43fb43ff, 0x43fd0000, 0x44b08ab3, 0x87d3448e, 0x8ae04249, 0x42124491, 0x420a8b10, 0x42450000, 0x420c8b13, 0x8b14413f, 0x8b1540a5, 0x43124243, 0x42718b16, 0x42664225, 0x8b174247, 0x42170000, 0x0, 0x0, 0x87f2427a, 0x8806, 0x420b4241, 0x8807, 0x41cb880a, 0x0, 0x88128817, 0x881844dc, 0x0, 0x423d0000, 0x881c8823, 0x422e882a, 0x882d8830, 0x88320000, 0x8836, 0x0, 0x883a8840, 0x42088841, 0x436d422c, 0x0, 0x423240b0, 0x436c41d1, 0x40a440a2, 0x42090000, 0x40af, 0x409040a3, 0x436f41e1, 0x88530000, 0x447a0000, 0x0, 0x41e4436e, 0x8857, 0x0, 0x4240, 0x41fe, 0x421c, 0x41ff41d3, 0x4242424a, 0x0, 0x0, 0x42448866, 0x41cc4246, 0x84be41e3, 0x41f4, 0x868f8882, 0x8895423e, 0x4213439b, 0x43934395, 0x45104248, 0x0, 0x420d0000, 0x0, 0x41f84396, 0x439741f3, 0x41f6, 0x0, 0x41d00000, 0x4394, 0x420c4398, 0x439c4203, 0x4399889e, 0x423888a0, 0x88a44233, 0x0, 0x43c5, 0x43c441f9, 0x439a43c6, 0x43d943de, 0x43d643e6, 0x43e5422d, 0x43e043db, 0x43d343da, 0x43d743d0, 0x43ce43d8, 0x43d1, 0x43d443e1, 0x43dd43e2, 0x0, 0x4265, 0x43d243dc, 0x43e40000, 0x43e343d5, 0x43df, 0x0, 0x85a0435b, 0x42260000, 0x43cf0000, 0x435a, 0x43ef0000, 0x43624231, 0x423b85d5, 0x43f6, 0x422f, 0x82f70000, 0x4239, 0x43f988bc, 0x88d288e4, 0x41ce0000, 0x43660000, 0x44774365, 0x421d, 0x44820000, 0x4363, 0x88e94219, 0x435f446f, 0x435e8919, 0x891b, 0x424c0000, 0x0, 0x86344214, 0x891e435d, 0x0, 0x0, 0x420e0000, 0x451e0000, 0x0, 0x426a0000, 0x4364, 0x436044ca, 0x4347, 0x40aa4361, 0x0, 0x44ad8930, 0x40b240a9, 0x42360000, 0x407b0000, 0x40e5, 0x86714235, 0x408340cc, 0x40a8, 0x40a60000, 0x407e, 0x407d0000, 0x409240b1, 0x86710000, 0x40cd0000, 0x4094, 0x40930000, 0x408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874272, 0x407f4080, 0x448f4091, 0x4274407c, 0x42270000, 0x451f423c, 0x422b435c, 0x89588959, 0x895c8970, 0x40a78982, 0x42168986, 0x41d40000, 0x0, 0x8987436b, 0x89a90000, 0x89aa, 0x0, 0x0, 0x447e420c, 0x0, 0x89ab, 0x41fd0000, 0x43144269, 0x41904250, 0x41d841d6, 0x426d41f2, 0x89ad4276, 0x0, 0x43900000, 0x438f4392, 0x427c421f, 0x421e, 0x447043c1, 0x451441e6, 0x868f8671, 0x0, 0x41914215, 0x43c34202, 0x41d2420f, 0x450741dd, 0x41e00000, 0x4229, 0x42284517, 0x43cd43f1, 0x43f243fa, 0x43fe41f7, 0x437c, 0x437d437f, 0x43590000, 0x0, 0x89da, 0x41bd89e7, 0x89e84478, 0x447f0000, 0x413b43ee, 0x4400, 0x43f84401, 0x413f413c, 0x41394141, 0x41434311, 0x43150000, 0x424e426c, 0x44ae424f, 0x0, 0x0, 0x44900000, 0x41d50000, 0x4220, 0x8a43447b, 0x450241e2, 0x41d9447c, 0x41d70000, 0x447d41e8, 0x0, 0x41ec8a4a, 0x87608a4b, 0x87638a5e, 0x406e0000, 0x41f50000, 0x43f4, 0x0, 0x0, 0x0, 0x0, 0x8a6a4479, 0x44830000, 0x0, 0x4140, 0x41384503, 0x0, 0x40e30000, 0x0, 0x0, 0x42b8, 0x45040000, 0x41be, 0x4481, 0x8ac50000, 0x41de0000, 0x41df, 0x0, 0x4509, 0x8ac90000, 0x42680000, 0x45088671, 0x0, 0x0, 0x8af40000, 0x44800000, 0x0, 0x0, 0x0, 0x424d0000, 0x426f0000, 0x0, 0x40ae, 0x422a41ea, 0x41eb, 0x0, 0x45190000, 0x45180000, 0x0, 0x0, 0x88100000, 0x0, 0x4516, 0x0, 0x0, 0x89a8, 0x45110000, 0x451b, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x450a0000, 0x0, 0x4210, 0x0, 0x0, 0x0, 0x0, 0x41e90000, 0x0, 0x0, 0x883e, 0x4512, 0x0, 0x42000000, 0x0, 0x4201, 0x0, 0x451341e5, 0x0, 0x0, 0x450c, 0x450b, 0x4211, 0x41dc, 0x451d, 0x0, 0x0, 0x0, 0x40e4, 0x451c0000, 0x0, 0x0, 0x42b98afa, 0x0, 0x0, 0x450d0000, 0x8b0343f0, 0x8b058b07, 0x8b0a, 0x8b0c, 0x0, 0x0, 0x0, 0x0, 0x450e0000, 0x0, 0x450f, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4137, 0x0, 0x0, 0x0, 0x0, 0x4313, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x426b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41e7, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43f30000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x426e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x809e4055, 0x809e4068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x809e4043, 0x4042, 0x402d, 0x809e4062, 0x4040809e, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066809e, 0x8169809e, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81694021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x8169809e, 0x42f6, 0x809e42f3, 0x42f50000, 0x4440809e, 0x42d742cd, 0x40334030, 0x42cf809e, 0x403242cc, 0x42d1809e, 0x8169402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138169, 0x809e4011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81690000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d5809e, 0x816942d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x809e4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x809e4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18169, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x809e, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x809e, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x809f4055, 0x809f4068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x809f4043, 0x4042, 0x402d, 0x809f4062, 0x4040809f, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x4066809f, 0x816a809f, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x816a4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x816a809f, 0x42f6, 0x809f42f3, 0x42f50000, 0x4440809f, 0x42d742cd, 0x40334030, 0x42cf809f, 0x403242cc, 0x42d1809f, 0x816a402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x4013816a, 0x809f4011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x816a0000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d5809f, 0x816a42d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x809f4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x809f4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e1816a, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x809f, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x809f, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80a04055, 0x80a04068, 0x40524049, 0x40604006, 0x40484067, 0x80a080a0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80a04043, 0x82a44042, 0x402d, 0x4062, 0x404080a0, 0x4057405c, 0x405882a4, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680a0, 0x816b80a0, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x816b80a0, 0x40a1401c, 0x403c816b, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x80a0, 0x0, 0x0, 0x80a080a0, 0x0, 0x0, 0x0, 0x40090000, 0x816b4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x816b80a0, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x4013816b, 0x80a04011, 0x816b, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80a00000, 0x0, 0x80a00000, 0x816b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] value5 = null;

        protected static void value5Init()
        {
            value5 = new int[][]
            {
                { 0x4055, 0x4068, 0x40520000, 0x40600000, 0x4067, 0x0, 0x406b, 0x40650000, 0x404c0000, 0x405d, 0x405f, 0x405a, 0x4069, 0x404b4317, 0x0, 0x0, 0x0, 0x0, 0x80f94062, 0x0, 0x4057405c, 0x40580000, 0x0, 0x4056404e, 0x4316, 0x405e44e1, 0x4059, 0x0, 0x404f4064, 0x406c, 0x0, 0x40610000, 0x0, 0x44e4, 0x405b0000, 0x0, 0x4051, 0x405044e3, 0x44e2, 0x406d0000, 0x4063, 0x40544053, 0x404d, 0x40660000, 0x0, 0x0, 0x406a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42f3, 0x42f50000, 0x80f9, 0x42d742cd, 0x40334030, 0x42cf0000, 0x403242cc, 0x42d180f9, 0x82d5402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x0, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43060000, 0x0, 0x42d342d4, 0x42d50000, 0x42d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433d, 0x44ed0000, 0x42ae, 0x0, 0x42c4, 0x0, 0x0, 0x0, 0x0, 0x44c60000, 0x0, 0x0, 0x0, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e0000, 0x0, 0x0, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x0, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a50000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433a, 0x0, 0x0, 0x0, 0x0, 0x434f, 0x43510000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43310000, 0x0, 0x0, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x0, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4308, 0x0, 0x430a4307, 0x0, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433e44ee, 0x0, 0x0, 0x438d, 0x0, 0x44c244e8, 0x44ea0000, 0x0, 0x0, 0x433c0000, 0x0, 0x43504352, 0x0, 0x0, 0x0, 0x43330000, 0x0, 0x43be0000, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x0, 0x43380000, 0x4193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a60000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43b9, 0x43320000, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x430c4309, 0x0, 0x0, 0x430b0000, 0x0, 0x0, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42c7, 0x42c242c1, 0x0, 0x0, 0x0, 0x0, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42cb42c9, 0x0, 0x0, 0x43ba4334, 0x4319, 0x43e9, 0x4371, 0x0, 0x44be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x0, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x0, 0x43bf0000, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44730000, 0x0, 0x0, 0x0, 0x430d, 0x0, 0x42f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c70000, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4323, 0x43250000, 0x0, 0x0, 0x4368438b, 0x44ec41fb, 0x0, 0x41f0, 0x42f2, 0x43890000, 0x43c00000, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x0, 0x0, 0x0, 0x44740000, 0x0, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x0, 0x0, 0x4321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x0, 0x42da4387, 0x43bc0000, 0x4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43c9, 0x0, 0x0, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x0, 0x43ca0000, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80a14055, 0x80a14068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80a14043, 0x4042, 0x402d, 0x80a14062, 0x404080a1, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680a1, 0x816c80a1, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x816c4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x816c80a1, 0x42f6, 0x80a142f3, 0x42f50000, 0x444080a1, 0x42d742cd, 0x40334030, 0x42cf80a1, 0x403242cc, 0x42d180a1, 0x816c402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x4013816c, 0x80a14011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x816c0000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d580a1, 0x816c42d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80a14174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80a14176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e1816c, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x80a1, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x80a1, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80a24055, 0x80a24068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80a24043, 0x4042, 0x402d, 0x80a24062, 0x404080a2, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680a2, 0x816d80a2, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x816d4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x816d80a2, 0x42f6, 0x80a242f3, 0x42f50000, 0x444080a2, 0x42d742cd, 0x40334030, 0x42cf80a2, 0x403242cc, 0x42d180a2, 0x816d402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x4013816d, 0x80a24011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x816d0000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d580a2, 0x816d42d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80a24174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80a24176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e1816d, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x80a2, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x80a2, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80a34055, 0x80a34068, 0x40524049, 0x40604006, 0x40484067, 0x80a380a3, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80a34043, 0x82a54042, 0x402d, 0x4062, 0x404080a3, 0x4057405c, 0x405882a5, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680a3, 0x816e80a3, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x816e80a3, 0x40a1401c, 0x403c816e, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x80a3, 0x0, 0x0, 0x80a380a3, 0x0, 0x0, 0x0, 0x40090000, 0x816e4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x816e80a3, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x4013816e, 0x80a34011, 0x816e, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80a30000, 0x0, 0x80a30000, 0x816e0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80a44055, 0x80a44068, 0x40524049, 0x40604006, 0x40484067, 0x80a480a4, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80a44043, 0x82a64042, 0x402d, 0x4062, 0x404080a4, 0x4057405c, 0x405882a6, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680a4, 0x816f80a4, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x816f80a4, 0x40a1401c, 0x403c816f, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x80a4, 0x0, 0x0, 0x80a480a4, 0x0, 0x0, 0x0, 0x40090000, 0x816f4021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x816f80a4, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x4013816f, 0x80a44011, 0x816f, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80a40000, 0x0, 0x80a40000, 0x816f0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x4055, 0x4068, 0x40524049, 0x40600000, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x4069, 0x404b4317, 0x40460000, 0x4043, 0x4042, 0x402d, 0x4062, 0x40408106, 0x4057405c, 0x40580000, 0x40470000, 0x4056404e, 0x4316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x0, 0x40614029, 0x40260000, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x40660000, 0x0, 0x402a0000, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8106, 0x0, 0x0, 0x8106, 0x0, 0x0, 0x0, 0x0, 0x4036, 0x40174015, 0x0, 0x40380000, 0x42f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x4035, 0x44c5, 0x0, 0x0, 0x401382fc, 0x81064011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x445a0000, 0x0, 0x0, 0x81060000, 0x82fc82fc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x40340000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed0000, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e0000, 0x0, 0x0, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43510000, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x0, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x4159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x0, 0x433c0000, 0x41c14175, 0x43504352, 0x41ab, 0x41ad41b2, 0x0, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x0, 0x415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x0, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44730000, 0x0, 0x0, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x44740000, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x4055, 0x4068, 0x40520000, 0x40600000, 0x4067, 0x0, 0x406b, 0x406580de, 0x404c0000, 0x405d, 0x405f, 0x405a, 0x4069, 0x404b4317, 0x0, 0x0, 0x82b00000, 0x0, 0x4062, 0x0, 0x4057405c, 0x40580000, 0x0, 0x4056404e, 0x4316, 0x405e44e1, 0x4059, 0x0, 0x404f4064, 0x406c, 0x0, 0x40610000, 0x0, 0x44e4, 0x405b0000, 0x0, 0x4051, 0x405044e3, 0x44e2, 0x406d0000, 0x4063, 0x40544053, 0x404d, 0x40660000, 0x0, 0x0, 0x406a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42f6, 0x80de42f3, 0x42f50000, 0x0, 0x42cd, 0x40334030, 0x42cf0000, 0x403242cc, 0x42d10000, 0x402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x0, 0x43350000, 0x0, 0x0, 0x0, 0x42fc0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x82b00000, 0x0, 0x0, 0x43060000, 0x42fd0000, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d24300, 0x42e9, 0x0, 0x0, 0x0, 0x0, 0x42fa0000, 0x0, 0x433d, 0x44ed0000, 0x430142ae, 0x0, 0x42c4, 0x0, 0x42fe0000, 0x42fb, 0x0, 0x44c60000, 0x0, 0x0, 0x0, 0x0, 0x44c044e6, 0x42ff, 0x0, 0x433b0000, 0x0, 0x434e0000, 0x0, 0x0, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x0, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a50000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433a, 0x0, 0x0, 0x0, 0x0, 0x434f, 0x43510000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43310000, 0x0, 0x0, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x0, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x0, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x433e44ee, 0x0, 0x0, 0x438d, 0x0, 0x44c244e8, 0x44ea0000, 0x0, 0x0, 0x433c0000, 0x0, 0x43504352, 0x0, 0x0, 0x0, 0x43330000, 0x0, 0x43be0000, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x0, 0x43380000, 0x4193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a60000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43b9, 0x43320000, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4303, 0x430c4309, 0x43040000, 0x0, 0x430b0000, 0x0, 0x0, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42c7, 0x42c242c1, 0x0, 0x0, 0x0, 0x0, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x42cb42c9, 0x0, 0x0, 0x43ba4334, 0x4319, 0x43e9, 0x4371, 0x0, 0x44be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x0, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x0, 0x43bf0000, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44730000, 0x0, 0x0, 0x0, 0x4305430d, 0x0, 0x42f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c70000, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4323, 0x43250000, 0x0, 0x0, 0x4368438b, 0x44ec41fb, 0x0, 0x41f0, 0x42f2, 0x43890000, 0x43c00000, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x0, 0x0, 0x0, 0x44740000, 0x0, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x0, 0x0, 0x4321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x0, 0x4387, 0x43bc0000, 0x4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43c9, 0x0, 0x0, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x0, 0x43ca0000, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80a54055, 0x80a54068, 0x40524049, 0x40604006, 0x40484067, 0x80a580a5, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80a54043, 0x4042, 0x402d, 0x4062, 0x404080a5, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680a5, 0x817080a5, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x817080a5, 0x40a1401c, 0x403c8170, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x80a5, 0x0, 0x0, 0x80a580a5, 0x0, 0x0, 0x0, 0x40090000, 0x81704021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x817080a5, 0x0, 0x42f3, 0x42f50000, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x0, 0x0, 0x44c344c5, 0x0, 0x0, 0x40138170, 0x80a54011, 0x8170, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80a50000, 0x0, 0x80a50000, 0x81700000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x0, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x0, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80a64055, 0x80a64068, 0x40524049, 0x40604006, 0x40484067, 0x80a680a6, 0x403e406b, 0x40658279, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80a64043, 0x850e4042, 0x8513402d, 0x80a64062, 0x404080a6, 0x4057405c, 0x40588767, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680a6, 0x817180a6, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x817180a6, 0x40a1401c, 0x403c8171, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x80a6, 0x0, 0x0, 0x80a680a6, 0x0, 0x0, 0x0, 0x40090000, 0x81714021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x817180a6, 0x42f6, 0x80a642f3, 0x42f50000, 0x444080a6, 0x42d742cd, 0x40334030, 0x42cf80a6, 0x403242cc, 0x42d180a6, 0x8171402f, 0x40310000, 0x44c344c5, 0x0, 0x0, 0x40138171, 0x80a64011, 0x8171, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80a60000, 0x0, 0x80a60000, 0x81710000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81710000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d580a6, 0x817142d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80a74055, 0x80a74068, 0x40524049, 0x40604006, 0x40484067, 0x80a780a7, 0x403e406b, 0x406580a7, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80a74043, 0x44964042, 0x80a7402d, 0x44944062, 0x404080a7, 0x4057405c, 0x405880a7, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40254497, 0x404f4064, 0x4010406c, 0x400580a7, 0x40614029, 0x40264007, 0x80a744e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x80a744e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680a7, 0x80a780a7, 0x402a4004, 0x4498406a, 0x40284499, 0x40f9, 0x449640fe, 0x40244022, 0x80a780a7, 0x40a1401c, 0x403c80a7, 0x403a4495, 0x40fb0000, 0x4039401e, 0x80a7403b, 0x80a780a7, 0x80a780a7, 0x80a74444, 0x40fa80a7, 0x80a780a7, 0x44454442, 0x40fc80a7, 0x40fd80a7, 0x400940b6, 0x80a74021, 0x40b74019, 0x401b80a7, 0x40b44036, 0x40174015, 0x80a780a7, 0x403880a7, 0x42f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x80a70000, 0x4035, 0x44c344c5, 0x0, 0x80a7, 0x401380a7, 0x80a74011, 0x444a80a7, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b4455, 0x415b4002, 0x44960000, 0x80a7, 0x401d, 0x0, 0x401f, 0x445a4459, 0x80a70000, 0x44570000, 0x80a74454, 0x80a780a7, 0x44430000, 0x80a7, 0x40b5, 0x0, 0x40be40c0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x444b442d, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e540ff, 0x4122, 0x41030000, 0x43390000, 0x44480000, 0x4101434d, 0x44060000, 0x4446, 0x44040000, 0x40b80000, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x4458, 0x4100434b, 0x445640c6, 0x0, 0x4161, 0x43350000, 0x0, 0x40bf40c1, 0x4102, 0x410440b9, 0x440f4020, 0x44304449, 0x442e4447, 0x442b401a, 0x445c4410, 0x44124018, 0x4016, 0x44b140c7, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x40ba445d, 0x0, 0x40c4, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x40bb0000, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40bc40b3, 0x40c2, 0x41570000, 0x40bd0000, 0x44c60000, 0x0, 0x40c3, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x4128412a, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x41210000, 0x0, 0x0, 0x0, 0x416d0000, 0x4127, 0x410542ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x412041fa, 0x418d4465, 0x41ef, 0x41294125, 0x4126, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed4123, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x80a84055, 0x80a84068, 0x40524049, 0x40604006, 0x40484067, 0x0, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80a84043, 0x4042, 0x402d, 0x80a84062, 0x404080a8, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680a8, 0x817280a8, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40090000, 0x81724021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x817280a8, 0x42f6, 0x80a842f3, 0x42f50000, 0x444080a8, 0x42d742cd, 0x40334030, 0x42cf80a8, 0x403242cc, 0x42d180a8, 0x8172402f, 0x40310000, 0x44c5, 0x0, 0x0, 0x40138172, 0x80a84011, 0x0, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x0, 0x431e0000, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x4020, 0x44300000, 0x442e0000, 0x401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81720000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d580a8, 0x817242d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x80a84174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x80a84176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x44e18172, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x80a8, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x80a8, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x4049, 0x0, 0x40480000, 0x80c2, 0x403e0000, 0x0, 0x4045, 0x0, 0x402e0000, 0x40410000, 0x0, 0x0, 0x40460000, 0x4043, 0x4042, 0x402d, 0x0, 0x40400000, 0x0, 0x0, 0x40470000, 0x0, 0x0, 0x44e1, 0x404a0000, 0x40250000, 0x0, 0x0, 0x0, 0x4029, 0x40260000, 0x0, 0x4044, 0x402b0000, 0x403f0000, 0x0, 0x0, 0x403d, 0x402c0000, 0x0, 0x40270000, 0x0, 0x0, 0x402a0000, 0x0, 0x40280000, 0x0, 0x0, 0x0, 0x0, 0x40a1401c, 0x403c827b, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x80c2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40174015, 0x0, 0x0, 0x0, 0x0, 0x44400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c30000, 0x0, 0x0, 0x40130000, 0x4011, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415b0000, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4161, 0x0, 0x0, 0x0, 0x0, 0x0, 0x440f0000, 0x0, 0x0, 0x0, 0x0, 0x4018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4441, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c0000, 0x0, 0x0, 0x0, 0x0, 0x40b3, 0x0, 0x41570000, 0x0, 0x0, 0x0, 0x0, 0x415c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4162, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41c0, 0x0, 0x41730000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x0, 0x4469, 0x0, 0x417b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43eb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4163, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x0, 0x44bb, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x0, 0x0, 0x0, 0x444c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40da0000, 0x41580000, 0x40f6, 0x0, 0x0, 0x4159, 0x4152, 0x0, 0x0, 0x0, 0x416e, 0x44640000, 0x0, 0x0, 0x41c14175, 0x0, 0x41ab, 0x41ad41b2, 0x0, 0x446a, 0x417d, 0x441b, 0x0, 0x0, 0x0, 0x43ec, 0x0, 0x4165, 0x4186, 0x41b94193, 0x0, 0x44bd0000, 0x0, 0x0, 0x0, 0x0, 0x4151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x0, 0x417c, 0x0, 0x0, 0x0, 0x43e70000, 0x0, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x0, 0x0, 0x0, 0x444e, 0x444d, 0x44500000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x0, 0x0, 0x40dc, 0x40db, 0x40de0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x415a, 0x4153, 0x416f0000, 0x0, 0x4176, 0x0, 0x0, 0x417e0000, 0x43e9, 0x0, 0x41660000, 0x418744be, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418d4465, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x446b0000, 0x441c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x43e8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44ba40f1, 0x0, 0x444f4452, 0x44510000, 0x0, 0x0, 0x0, 0x0, 0x44a3, 0x0, 0x0, 0x0, 0x0, 0x40dd40e0, 0x40df0000, 0x0, 0x40f40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x418e4466, 0x0, 0x0, 0x446c, 0x441d, 0x0, 0x0, 0x0, 0x43ea, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441e4149, 0x0, 0x0, 0x0, 0x4453, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40e1, 0x0, 0x40f20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4420, 0x414b0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x0, 0x0, 0x414a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4421, 0x414c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e1, 0x0, 0x0, 0x0, 0x0, 0x8123, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40a10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8326, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40b6, 0x0, 0x40b70000, 0x8123, 0x40b40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8326, 0x40b5, 0x0, 0x40be40c0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40b80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40c6, 0x0, 0x0, 0x0, 0x0, 0x40bf40c1, 0x0, 0x40b9, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40c7, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40ba0000, 0x0, 0x40c4, 0x0, 0x0, 0x0, 0x0, 0x40bb0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40c5, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40bc0000, 0x40c2, 0x0, 0x40bd0000, 0x0, 0x0, 0x40c3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80a94055, 0x80a94068, 0x40524049, 0x40604006, 0x40484067, 0x810e8137, 0x403e406b, 0x406581ce, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80a94043, 0x80d64042, 0x8135402d, 0x80a94062, 0x404080a9, 0x4057405c, 0x4058810b, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x402580a9, 0x404f4064, 0x4010406c, 0x4005818c, 0x40614029, 0x40264007, 0x810b44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x81d944e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680a9, 0x80a980a9, 0x402a4004, 0x80a9406a, 0x402880a9, 0x80a940f9, 0x813540fe, 0x81e581e8, 0x80a9810b, 0x40a181f8, 0x81fe433f, 0x8205412d, 0x40fb44aa, 0x8215821b, 0x80a980a9, 0x412e810b, 0x80a98205, 0x80a94492, 0x40fa80a9, 0x810b80a9, 0x825880a9, 0x40fc8269, 0x40fd826c, 0x400980d6, 0x80a94021, 0x82054019, 0x401b8205, 0x44ab4493, 0x40174015, 0x80a980a9, 0x820542f6, 0x80a942f3, 0x42f58205, 0x444080a9, 0x42d742cd, 0x40334030, 0x42cf80a9, 0x403242cc, 0x42d180a9, 0x80a9402f, 0x40318205, 0x820544c5, 0x80a980a9, 0x80a980a9, 0x401380a9, 0x80a94011, 0x82054348, 0x42ad80a9, 0x42bf42bc, 0x42be42bd, 0x42bb42ba, 0x42af80a9, 0x8205831e, 0x400b8135, 0x415b4002, 0x80d6810b, 0x810e8135, 0x8137818c, 0x80a980a9, 0x81ce810b, 0x81d98135, 0x81e581e8, 0x80a9810b, 0x81f881fe, 0x82058215, 0x821b80a9, 0x80a9810b, 0x80a98205, 0x80a980a9, 0x810b80a9, 0x825880a9, 0x8269826c, 0x400a400e, 0x4293429a, 0x428d4288, 0x429b428b, 0x80a980a9, 0x42894299, 0x80a980a9, 0x82058205, 0x818c80a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a98417, 0x80a94525, 0x810b810b, 0x80a94081, 0x81e580a9, 0x810b44bf, 0x408281fe, 0x40908461, 0x409f80a9, 0x44e540ff, 0x80a94122, 0x410380a9, 0x433980a9, 0x844180a9, 0x4101434d, 0x44064340, 0x80a98205, 0x440480a9, 0x8205821b, 0x80a980a9, 0x80a9810b, 0x431e810b, 0x80a9432d, 0x80a980a9, 0x84bf810b, 0x4100434b, 0x80a94378, 0x80a9810b, 0x84414161, 0x433580a9, 0x825880a9, 0x419480a9, 0x810b4102, 0x410480d6, 0x82054020, 0x44308205, 0x442e8205, 0x8205401a, 0x82054410, 0x44124018, 0x82054016, 0x44b18205, 0x44b340f5, 0x80d680a9, 0x80a980d6, 0x42f842f7, 0x42f942f4, 0x80a98205, 0x80a9430f, 0x80a98205, 0x43064441, 0x8205855d, 0x42d342d4, 0x42d580a9, 0x80a942d8, 0x42ce42d0, 0x42d28205, 0x820542e9, 0x820542e5, 0x80a9810b, 0x80a980a9, 0x80a94014, 0x82054012, 0x449a8205, 0x449c433d, 0x44ed44a4, 0x818c42ae, 0x42b080a9, 0x820542c4, 0x820540b3, 0x81e5831e, 0x41578135, 0x44fc818c, 0x44c680a9, 0x80a980a9, 0x80a98135, 0x415c818c, 0x80a980a9, 0x44c044e6, 0x80a98417, 0x810b810b, 0x433b80a9, 0x80a981fe, 0x434e4408, 0x80a98205, 0x440580a9, 0x8205821b, 0x80a980a9, 0x810b431f, 0x80a9432f, 0x80a980a9, 0x84bf810b, 0x434c80a9, 0x810b4162, 0x433780a9, 0x825880a9, 0x80a980a9, 0x4294428e, 0x428c428a, 0x428f8205, 0x82054290, 0x80a94291, 0x80a98205, 0x42a180a9, 0x42a54150, 0x818c44fb, 0x80a980a9, 0x810b80a9, 0x80a980a9, 0x80a94523, 0x4083438c, 0x80a9810b, 0x80a980a9, 0x4520407e, 0x407d4527, 0x409280a9, 0x80a980a9, 0x44c140a0, 0x45284094, 0x40934522, 0x4524408a, 0x40884089, 0x408e4084, 0x408c408d, 0x408f408b, 0x40854086, 0x40874521, 0x407f4080, 0x45264091, 0x80a9407c, 0x868480a9, 0x86848684, 0x86848684, 0x44e744e9, 0x80a9416c, 0x810b80a9, 0x44fe4463, 0x81d981d9, 0x4128412a, 0x80a980a9, 0x80a981e5, 0x4402433a, 0x80a942b1, 0x810b41c0, 0x810b41c7, 0x417381fe, 0x44ff434f, 0x43514407, 0x80a980a9, 0x80a980a9, 0x810b8461, 0x80a941aa, 0x41ac8461, 0x41b1810b, 0x440a80a9, 0x43314469, 0x82158215, 0x417b821b, 0x450043bd, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a9810b, 0x441a86e8, 0x810b810b, 0x810b810b, 0x810b810b, 0x810b810b, 0x810b810b, 0x810b80a9, 0x810b810b, 0x810b810b, 0x80a9810b, 0x810b810b, 0x810b810b, 0x810b810b, 0x80a9431a, 0x431c80a9, 0x80a943b1, 0x432e43b5, 0x43eb84bf, 0x84bf4400, 0x84bf4401, 0x84bf4372, 0x80a980a9, 0x80a94376, 0x80a94163, 0x810b44fd, 0x433680a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x810b80a9, 0x80a980a9, 0x80a94184, 0x825880a9, 0x450141b8, 0x80a980a9, 0x874f80a9, 0x41954192, 0x80a94471, 0x80a944bb, 0x818c4432, 0x8205442f, 0x82054431, 0x44348205, 0x44388205, 0x44114414, 0x44164413, 0x44b28205, 0x44b544b7, 0x44b480d6, 0x80a980a9, 0x82054308, 0x430280a9, 0x430a4307, 0x8205444c, 0x8205855d, 0x42d642eb, 0x820542e6, 0x80a942ea, 0x42ed42e7, 0x80a94251, 0x427d80a9, 0x80a9810b, 0x432680a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a9449b, 0x8205449e, 0x44a0449d, 0x80a944a5, 0x44a8818c, 0x80a942c6, 0x820542c5, 0x42c080a9, 0x80a9831e, 0x831e81e5, 0x40da81e5, 0x4158448c, 0x818c40f6, 0x80d6810b, 0x433e44ee, 0x44a64159, 0x818c4152, 0x80a9438d, 0x80a9810b, 0x44c244e8, 0x44ea416e, 0x446481d9, 0x81e54403, 0x433c42b2, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c80a9, 0x4333446a, 0x8215417d, 0x43be441b, 0x86e8810b, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x80a94165, 0x43384186, 0x41b94193, 0x447280a9, 0x44bd80a9, 0x80a98205, 0x42a380a9, 0x42a74292, 0x42a942a2, 0x42a64151, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x412180a9, 0x80a980a9, 0x80a980a9, 0x80a98417, 0x416d810b, 0x80a94127, 0x410542ca, 0x42c880a9, 0x80a942b5, 0x810b41c8, 0x80a94174, 0x80a980a9, 0x8461810b, 0x440b43b9, 0x4332417c, 0x80a94318, 0x80a984bf, 0x84bf84bf, 0x43e7810b, 0x437080a9, 0x80a94379, 0x437a437b, 0x4164818c, 0x418580a9, 0x80a980a9, 0x419680a9, 0x80a988e2, 0x80a944bc, 0x44334436, 0x8205443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b880a9, 0x80a980a9, 0x80a980a9, 0x40f34303, 0x430c4309, 0x430480a9, 0x430e80a9, 0x430b444e, 0x8205444d, 0x4450855d, 0x42ec42ef, 0x42e880a9, 0x42ee80a9, 0x42b34322, 0x432480a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x44a2449f, 0x44a180a9, 0x818c42c7, 0x42c242c1, 0x80a940dc, 0x81e540db, 0x40de831e, 0x818c4484, 0x80d64252, 0x427e80a9, 0x80a9810b, 0x432780a9, 0x80a980a9, 0x80a944a7, 0x44a9415a, 0x448d4153, 0x416f810b, 0x42cb42c9, 0x80a94176, 0x80a9440d, 0x43ba4334, 0x417e4319, 0x80a943e9, 0x810b4371, 0x4166818c, 0x418744be, 0x42ab42a4, 0x42a842aa, 0x82054367, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x80a9438a, 0x80a944eb, 0x80a980a9, 0x89a589a7, 0x412041fa, 0x418d4465, 0x80a941ef, 0x41294125, 0x80a94126, 0x81e542f1, 0x44e180a9, 0x4388810b, 0x446b80a9, 0x43bf441c, 0x86e889cc, 0x89ce43c7, 0x80a980a9, 0x43b34286, 0x80a94297, 0x43b743e8, 0x84bf80a9, 0x420689a5, 0x43744355, 0x818c80a9, 0x874f80a9, 0x4197874f, 0x8684874f, 0x80a980a9, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x82054320, 0x4259427f, 0x80a980a9, 0x80a980a9, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c380a9, 0x80a9831e, 0x40dd40e0, 0x40df4486, 0x448580a9, 0x40f480a9, 0x42b44323, 0x432580a9, 0x80a980a9, 0x80a94488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x80a941f0, 0x81e542f2, 0x4389446c, 0x43c0441d, 0x89cc43c8, 0x80a943b4, 0x42874298, 0x43b843ea, 0x80a94207, 0x43754356, 0x818c4475, 0x80a980a9, 0x42ac8205, 0x429d80a9, 0x80a989a5, 0x89a580a9, 0x80a980a9, 0x42b642d9, 0x80a980a9, 0x80a94386, 0x810b43bb, 0x441e4149, 0x818c44fa, 0x80a980a9, 0x4474443f, 0x80a94453, 0x82058205, 0x42dd4295, 0x425a4253, 0x80a980a9, 0x80a944c8, 0x80a980a9, 0x80a940e1, 0x831e4487, 0x40f24321, 0x425b4280, 0x80a980a9, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x80a980a9, 0x429f429e, 0x80a980a9, 0x80a980a9, 0x80a980a9, 0x412480a9, 0x80a980a9, 0x441f89cc, 0x89ce43c9, 0x80a980a9, 0x414a80a9, 0x820542de, 0x42e142df, 0x42db4255, 0x80a94257, 0x80a94296, 0x425c4254, 0x80a980a9, 0x80a980a9, 0x448b4421, 0x43ca414c, 0x42a080a9, 0x80a980a9, 0x41ed4123, 0x80a980a9, 0x80a94204, 0x80a942e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x80a94258, 0x80a941ee, 0x420580a9, 0x80a980a9, 0x80a980a9, 0x42e4425e, 0x4262425f, 0x42824263, 0x428480a9, 0x80a94349, 0x80a94260, 0x4264434a, 0x80a980a9, 0x80a980a9, 0x80a90000 },
                { 0x80aa4055, 0x80aa4068, 0x40524049, 0x40604006, 0x40484067, 0x80aa80aa, 0x403e406b, 0x406580aa, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80aa4043, 0x44964042, 0x80aa402d, 0x44944062, 0x404080aa, 0x4057405c, 0x405880aa, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40254497, 0x404f4064, 0x4010406c, 0x400580aa, 0x40614029, 0x40264007, 0x80aa44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x80aa44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680aa, 0x80aa80aa, 0x402a4004, 0x4498406a, 0x40284499, 0x40f9, 0x449640fe, 0x40244022, 0x80aa80aa, 0x40a1401c, 0x403c80aa, 0x403a4495, 0x40fb0000, 0x4039401e, 0x80aa403b, 0x80aa80aa, 0x80aa80aa, 0x80aa4444, 0x40fa80aa, 0x80aa80aa, 0x44454442, 0x40fc80aa, 0x40fd80aa, 0x400940b6, 0x80aa4021, 0x40b74019, 0x401b80aa, 0x40b44036, 0x40174015, 0x80aa80aa, 0x403880aa, 0x42f3, 0x42f54037, 0x44400000, 0x42cd, 0x0, 0x42cf0000, 0x42cc, 0x42d10000, 0x80aa0000, 0x4035, 0x44c344c5, 0x0, 0x80aa, 0x401380aa, 0x80aa4011, 0x444a80aa, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b4455, 0x415b4002, 0x44960000, 0x80aa, 0x401d, 0x0, 0x401f, 0x445a4459, 0x80aa0000, 0x44570000, 0x80aa4454, 0x80aa80aa, 0x44430000, 0x80aa, 0x40b5, 0x0, 0x40be40c0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x444b442d, 0x0, 0x4034440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e540ff, 0x4122, 0x41030000, 0x43390000, 0x44480000, 0x4101434d, 0x44060000, 0x4446, 0x44040000, 0x40b80000, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x4458, 0x4100434b, 0x445640c6, 0x0, 0x4161, 0x43350000, 0x0, 0x40bf40c1, 0x4102, 0x410440b9, 0x440f4020, 0x44304449, 0x442e4447, 0x442b401a, 0x445c4410, 0x44124018, 0x4016, 0x44b140c7, 0x44b340f5, 0x0, 0x0, 0x0, 0x42f4, 0x40ba445d, 0x0, 0x40c4, 0x43064441, 0x0, 0x42d342d4, 0x42d50000, 0x40bb0000, 0x42ce42d0, 0x42d20000, 0x42e9, 0x0, 0x0, 0x0, 0x4014, 0x4012, 0x449a40c5, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40bc40b3, 0x40c2, 0x41570000, 0x40bd0000, 0x44c60000, 0x0, 0x40c3, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x4128412a, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x0, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x0, 0x42ea, 0x42ed0000, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x41210000, 0x0, 0x0, 0x0, 0x416d0000, 0x4127, 0x410542ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f30000, 0x430c4309, 0x0, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x0, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x412041fa, 0x418d4465, 0x41ef, 0x41294125, 0x4126, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x0, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41240000, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x0, 0x0, 0x4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed4123, 0x0, 0x4204, 0x0, 0x0, 0x425d4281, 0x42614283, 0x4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] value6 = null;

        protected static void value6Init()
        {
            value6 = new int[][]
            {
                { 0x80ab4055, 0x80ab4068, 0x40524049, 0x40604006, 0x40484067, 0x80ab80ab, 0x403e406b, 0x40650000, 0x404c4045, 0x400f405d, 0x402e405f, 0x4041405a, 0x40084069, 0x404b4317, 0x40464001, 0x80ab4043, 0x4042, 0x402d, 0x80ab4062, 0x404080ab, 0x4057405c, 0x40580000, 0x4047400c, 0x4056404e, 0x40034316, 0x405e44e1, 0x404a4059, 0x40250000, 0x404f4064, 0x4010406c, 0x40050000, 0x40614029, 0x40264007, 0x44e4, 0x405b4044, 0x402b400d, 0x403f4051, 0x405044e3, 0x44e2, 0x406d403d, 0x402c4063, 0x40544053, 0x4027404d, 0x406680ab, 0x817380ab, 0x402a4004, 0x406a, 0x40280000, 0x0, 0x0, 0x40244022, 0x817380ab, 0x40a1401c, 0x403c8173, 0x403a0000, 0x0, 0x4039401e, 0x403b, 0x0, 0x80ab, 0x0, 0x0, 0x80ab80ab, 0x0, 0x0, 0x0, 0x40090000, 0x81734021, 0x4019, 0x401b0000, 0x0, 0x40174015, 0x817380ab, 0x42f6, 0x80ab42f3, 0x42f50000, 0x444080ab, 0x42d742cd, 0x40334030, 0x42cf80ab, 0x403242cc, 0x42d180ab, 0x8173402f, 0x40310000, 0x44c344c5, 0x0, 0x0, 0x40138173, 0x80ab4011, 0x8173, 0x42ad0000, 0x0, 0x0, 0x0, 0x0, 0x442c4023, 0x400b0000, 0x415b4002, 0x0, 0x0, 0x401d, 0x0, 0x401f, 0x445a0000, 0x80ab0000, 0x0, 0x80ab0000, 0x81730000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c4, 0x0, 0x400a400e, 0x0, 0x428d4288, 0x428b, 0x0, 0x42890000, 0x0, 0x442d, 0x0, 0x440e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44bf, 0x0, 0x0, 0x0, 0x44e50000, 0x0, 0x0, 0x43390000, 0x0, 0x434d, 0x44060000, 0x0, 0x44040000, 0x0, 0x0, 0x442a, 0x431e445b, 0x432d, 0x0, 0x0, 0x434b, 0x0, 0x0, 0x4161, 0x43350000, 0x0, 0x0, 0x0, 0x0, 0x440f4020, 0x44300000, 0x442e0000, 0x442b401a, 0x4410, 0x44124018, 0x4016, 0x44b10000, 0x44b340f5, 0x0, 0x0, 0x42f842f7, 0x42f942f4, 0x81730000, 0x0, 0x0, 0x43064441, 0x0, 0x42d342d4, 0x42d580ab, 0x817342d8, 0x42ce42d0, 0x42d20000, 0x42e9, 0x42e5, 0x0, 0x0, 0x4014, 0x4012, 0x449a0000, 0x449c433d, 0x44ed44a4, 0x42ae, 0x0, 0x42c4, 0x40b3, 0x0, 0x41570000, 0x0, 0x44c60000, 0x0, 0x0, 0x415c0000, 0x0, 0x44c044e6, 0x0, 0x0, 0x433b0000, 0x0, 0x434e4408, 0x0, 0x44050000, 0x0, 0x0, 0x431f, 0x432f, 0x0, 0x0, 0x434c0000, 0x4162, 0x43370000, 0x0, 0x0, 0x428e, 0x428c428a, 0x428f0000, 0x4290, 0x4291, 0x0, 0x0, 0x42a54150, 0x0, 0x0, 0x0, 0x0, 0x0, 0x438c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44c10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44e744e9, 0x416c, 0x0, 0x4463, 0x0, 0x0, 0x0, 0x0, 0x4402433a, 0x0, 0x41c0, 0x0, 0x41730000, 0x434f, 0x43514407, 0x0, 0x0, 0x0, 0x41aa, 0x41ac0000, 0x41b10000, 0x440a0000, 0x43314469, 0x0, 0x417b0000, 0x43bd, 0x0, 0x0, 0x0, 0x0, 0x441a0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x431a, 0x431c0000, 0x43b1, 0x432e43b5, 0x43eb0000, 0x0, 0x0, 0x4372, 0x0, 0x4376, 0x4163, 0x0, 0x43360000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4184, 0x0, 0x41b8, 0x0, 0x0, 0x4192, 0x4471, 0x44bb, 0x4432, 0x442f, 0x4431, 0x44340000, 0x44380000, 0x44114414, 0x44164413, 0x44b20000, 0x44b544b7, 0x44b40000, 0x0, 0x4308, 0x43020000, 0x430a4307, 0x444c, 0x0, 0x42d642eb, 0x42e6, 0x42ea, 0x42ed42e7, 0x4251, 0x427d0000, 0x0, 0x43260000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x449b, 0x449e, 0x44a0449d, 0x44a5, 0x44a80000, 0x42c6, 0x42c5, 0x42c00000, 0x0, 0x0, 0x40da0000, 0x4158448c, 0x40f6, 0x0, 0x433e44ee, 0x44a64159, 0x4152, 0x438d, 0x0, 0x44c244e8, 0x44ea416e, 0x44640000, 0x4403, 0x433c0000, 0x41c14175, 0x43504352, 0x440941ab, 0x41ad41b2, 0x440c0000, 0x4333446a, 0x417d, 0x43be441b, 0x0, 0x431b431d, 0x43b24330, 0x43b643ec, 0x43734377, 0x4165, 0x43384186, 0x41b94193, 0x44720000, 0x44bd0000, 0x0, 0x0, 0x42a74292, 0x42a90000, 0x42a64151, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x416d0000, 0x0, 0x42ca, 0x42c80000, 0x0, 0x0, 0x4174, 0x0, 0x0, 0x440b43b9, 0x4332417c, 0x4318, 0x0, 0x0, 0x43e70000, 0x43700000, 0x0, 0x0, 0x41640000, 0x41850000, 0x0, 0x0, 0x0, 0x44bc, 0x44334436, 0x443a, 0x4435443c, 0x44394418, 0x44154417, 0x44b944b6, 0x44b80000, 0x0, 0x0, 0x40f34303, 0x430c4309, 0x43040000, 0x0, 0x430b444e, 0x444d, 0x44500000, 0x42ec42ef, 0x42e80000, 0x42ee0000, 0x4322, 0x43240000, 0x0, 0x0, 0x0, 0x0, 0x44a2449f, 0x44a10000, 0x42c7, 0x42c242c1, 0x40dc, 0x40db, 0x40de0000, 0x4484, 0x4252, 0x427e0000, 0x0, 0x43270000, 0x0, 0x44a7, 0x44a9415a, 0x448d4153, 0x416f0000, 0x42cb42c9, 0x4176, 0x440d, 0x43ba4334, 0x417e4319, 0x43e9, 0x4371, 0x41660000, 0x418744be, 0x42ab0000, 0x42a842aa, 0x4367, 0x0, 0x0, 0x0, 0x438a, 0x44eb, 0x0, 0x0, 0x41fa, 0x418d4465, 0x41ef, 0x0, 0x0, 0x42f1, 0x0, 0x43880000, 0x446b0000, 0x43bf441c, 0x0, 0x43c7, 0x0, 0x43b34286, 0x0, 0x43b743e8, 0x0, 0x42060000, 0x43744355, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44734437, 0x443e443b, 0x443d4419, 0x44ba40f1, 0x4305430d, 0x444f4452, 0x445142f0, 0x4320, 0x4259427f, 0x0, 0x0, 0x44c744a3, 0x44f144ef, 0x43434341, 0x42c30000, 0x0, 0x40dd40e0, 0x40df4486, 0x44850000, 0x40f40000, 0x4323, 0x43250000, 0x0, 0x4488, 0x4368438b, 0x44ec41fb, 0x418e4466, 0x41f0, 0x42f2, 0x4389446c, 0x43c0441d, 0x43c8, 0x43b4, 0x42870000, 0x43b843ea, 0x4207, 0x43754356, 0x4475, 0x0, 0x42ac0000, 0x0, 0x0, 0x0, 0x0, 0x42d9, 0x0, 0x4386, 0x43bb, 0x441e4149, 0x0, 0x0, 0x4474443f, 0x4453, 0x0, 0x42dd0000, 0x425a4253, 0x0, 0x44c8, 0x0, 0x40e1, 0x4487, 0x40f24321, 0x425b4280, 0x0, 0x44f244f0, 0x43444342, 0x448a4489, 0x42da4387, 0x43bc4420, 0x414b4476, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x441f0000, 0x43c9, 0x0, 0x414a0000, 0x42de, 0x42e142df, 0x42db4255, 0x4257, 0x0, 0x425c4254, 0x0, 0x0, 0x448b4421, 0x43ca414c, 0x0, 0x0, 0x41ed0000, 0x0, 0x4204, 0x42e2, 0x42e042e3, 0x425d4281, 0x42614283, 0x42dc4256, 0x4258, 0x41ee, 0x42050000, 0x0, 0x0, 0x42e4425e, 0x4262425f, 0x42824263, 0x42840000, 0x4349, 0x4260, 0x4264434a, 0x0, 0x0, 0x0 }
            };
        }

        protected static short lookupValue(int row, int col)
        {
            if (row <= 14)
                return (short)(col % 2 == 0 ? value[row][col/2] >>> 16 : value[row][col/2] & 0xFFFF);
            else if (row >= 15 && row <= 29)
                return (short)(col % 2 == 0 ? value1[row-15][col/2] >>> 16 : value1[row-15][col/2] & 0xFFFF);
            else if (row >= 30 && row <= 44)
                return (short)(col % 2 == 0 ? value2[row-30][col/2] >>> 16 : value2[row-30][col/2] & 0xFFFF);
            else if (row >= 45 && row <= 59)
                return (short)(col % 2 == 0 ? value3[row-45][col/2] >>> 16 : value3[row-45][col/2] & 0xFFFF);
            else if (row >= 60 && row <= 74)
                return (short)(col % 2 == 0 ? value4[row-60][col/2] >>> 16 : value4[row-60][col/2] & 0xFFFF);
            else if (row >= 75 && row <= 89)
                return (short)(col % 2 == 0 ? value5[row-75][col/2] >>> 16 : value5[row-75][col/2] & 0xFFFF);
            else if (row >= 90)
                return (short)(col % 2 == 0 ? value6[row-90][col/2] >>> 16 : value6[row-90][col/2] & 0xFFFF);
            else
                throw new Error("Unexpected location requested in value6 lookup");
        }

        static
        {
            sigmapInit();
            sigmap1Init();
            valueInit();
            value1Init();
            value2Init();
            value3Init();
            value4Init();
            value5Init();
            value6Init();
        }
    }

    /**
     * Looks up a row in the GOTO table
     */
    private static short lookupGoTo(int row, int col) { return GoToTable.get(row, col); }

    protected static class GoToTable
    {
        protected static final int[] rowmap = { 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 1, 1, 0, 0, 1, 0, 3, 0, 0, 0, 0, 0, 0, 1, 0, 4, 0, 0, 0, 0, 5, 2, 0, 1, 0, 0, 6, 0, 3, 0, 2, 0, 7, 0, 1, 0, 8, 0, 9, 10, 0, 11, 1, 4, 12, 0, 4, 4, 13, 0, 14, 0, 15, 16, 0, 5, 17, 18, 0, 0, 0, 0, 1, 0, 19, 20, 0, 0, 0, 2, 2, 1, 0, 1, 21, 0, 6, 22, 0, 0, 0, 23, 2, 0, 24, 0, 1, 5, 1, 25, 5, 0, 7, 26, 27, 0, 0, 0, 0, 1, 28, 2, 6, 29, 4, 1, 1, 30, 2, 3, 8, 4, 2, 4, 0, 0, 1, 31, 32, 0, 0, 0, 33, 1, 0, 34, 5, 9, 0, 35, 0, 0, 10, 36, 2, 37, 0, 38, 11, 39, 2, 0, 0, 40, 41, 0, 42, 1, 6, 0, 0, 1, 43, 6, 0, 5, 12, 2, 3, 0, 2, 6, 3, 0, 2, 1, 2, 0, 0, 1, 1, 0, 44, 4, 45, 5, 6, 1, 46, 47, 48, 49, 0, 0, 1, 2, 13, 0, 3, 0, 50, 2, 3, 6, 51, 7, 14, 1, 0, 5, 52, 1, 53, 5, 2, 0, 0, 3, 54, 0, 7, 0, 1, 1, 0, 0, 0, 55, 7, 1, 3, 1, 0, 0, 56, 1, 7, 2, 0, 0, 57, 0, 5, 2, 58, 3, 59, 15, 0, 1, 0, 2, 60, 61, 1, 7, 3, 1, 0, 0, 1, 4, 3, 62, 3, 16, 0, 0, 0, 0, 0, 1, 63, 1, 0, 0, 64, 1, 1, 2, 65, 0, 17, 18, 9, 1, 2, 4, 0, 2, 0, 66, 67, 19, 0, 20, 2, 4, 0, 0, 0, 68, 69, 4, 5, 0, 5, 0, 10, 3, 21, 4, 70, 71, 5, 5, 22, 0, 0, 4, 72, 23, 6, 2, 3, 0, 24, 1, 3, 73, 1, 0, 74, 10, 0, 3, 0, 5, 1, 7, 75, 25, 11, 76, 77, 0, 1, 26, 0, 0, 0, 5, 0, 78, 4, 0, 3, 27, 0, 79, 80, 81, 28, 8, 4, 3, 0, 29, 0, 30, 0, 6, 82, 0, 4, 0, 0, 2, 0, 83, 4, 31, 4, 5, 2, 7, 0, 2, 0, 6, 84, 85, 0, 0, 0, 86, 1, 14, 5, 87, 88, 32, 1, 4, 0, 2, 0, 1, 0, 0, 33, 34, 0, 89, 1, 35, 3, 90, 2, 91, 0, 92, 93, 1, 94, 0, 0, 0, 0, 7, 8, 0, 7, 1, 4, 95, 2, 0, 4, 96, 97, 1, 3, 36, 1, 8, 7, 4, 1, 0, 16, 98, 3, 0, 2, 3, 5, 99, 37, 13, 4, 0, 9, 0, 7 };
        protected static final int[] columnmap = { 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 3, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 4, 0, 6, 7, 0, 8, 0, 1, 0, 9, 0, 0, 0, 10, 0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12, 0, 0, 0, 0, 0, 13, 0, 0, 0, 0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15, 16, 0, 0, 0, 0, 17, 0, 0, 0, 0, 0, 0, 18, 6, 11, 0, 0, 19, 0, 0, 20, 0, 0, 21, 0, 4, 17, 0, 22, 0, 23, 0, 0, 24, 25, 10, 0, 26, 6, 27, 28, 29, 30, 0, 0, 0, 31, 32, 0, 0, 33, 0, 0, 0, 0, 34, 35, 0, 0, 36, 37, 0, 22, 2, 0, 0, 38, 23, 39, 0, 27, 0, 0, 0, 0, 40, 0, 0, 41, 42, 43, 0, 0, 44, 45, 46, 47, 48, 0, 49, 50, 0, 51, 10, 0, 0, 0, 8, 52, 53, 0, 0, 0, 0, 0, 0, 0, 0, 12, 0, 0, 28, 0, 54, 0, 0, 0, 0, 0, 0, 0, 55, 16, 29, 0, 56, 0, 0, 0, 57, 0, 58, 0, 0, 0, 59, 0, 0, 0, 60, 0, 61, 0, 62, 63, 0, 0, 64, 0, 19, 0, 65, 0, 0, 0, 0, 0, 66, 0, 0, 0, 67, 3, 0, 0, 68, 0, 14, 0, 0, 31, 69, 0, 15, 70, 0, 71, 20, 35, 0, 0, 72, 0, 0, 0, 0, 0, 73, 74, 0, 0, 75, 0, 0, 0, 0, 0, 0, 0, 0, 4, 76, 0, 0, 77, 0, 78, 0, 21, 0, 0, 0, 0, 0, 22, 0, 0, 0, 23, 0, 0, 79, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 80, 81, 82, 83, 84, 85, 86, 0, 0, 87, 0, 16, 40, 0, 0, 88, 0, 0, 89, 90, 43, 0, 12, 91, 92, 93, 94, 0, 95, 96, 0, 97, 0, 98, 99, 0, 100, 101, 47, 7, 0, 102, 48, 103, 0, 49, 0, 0, 0, 104, 105, 106, 0, 0, 107, 108, 109, 110, 111, 0, 0, 112, 0, 0, 0, 0, 0, 0, 0, 113, 114, 0, 0, 13, 115, 0, 116, 117, 0, 118, 0, 0, 0, 0, 119, 0, 0, 14, 120, 121, 0, 0, 0, 0, 0, 0, 122, 50, 0, 25, 123, 55, 58, 0, 2, 0, 124, 125, 26, 0, 60, 33, 0, 34, 0, 2, 0, 126, 10, 2, 127, 36, 0, 74, 39, 0, 81, 0, 0, 0, 128, 0, 0, 0, 71, 0, 0, 0, 0, 0, 0, 83, 0, 0, 0, 0, 85, 129, 0, 0, 130, 0, 0, 131, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 132, 0, 0, 133, 134, 0, 4, 0, 0, 13, 0, 0, 0, 0, 0, 24, 135, 0, 136, 73, 0, 0, 137, 88, 0, 0, 0, 0, 91, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 138, 0, 0, 0, 0, 0, 139, 140, 0, 93, 141, 0, 0, 0, 0, 0, 0, 142, 0, 0, 0, 0, 0, 143, 144, 145, 45, 0, 14, 0, 0, 0, 146, 0, 147, 148, 0, 0, 0, 0, 0, 75, 0, 149, 0, 0, 0, 95, 0, 150, 0, 151, 152, 0, 0, 153, 0, 0, 154, 155, 0, 0, 156, 0, 0, 0, 0, 157, 158, 0, 0, 0, 0, 0, 159, 160, 92, 0, 0, 0, 0, 0, 0, 0, 99, 161, 0, 0, 0, 0, 0, 0, 0, 162, 82, 163, 0, 0, 0, 0, 164, 0, 165, 166, 0, 0, 0, 0, 0, 0, 0, 167, 0, 0, 168, 0, 169, 0, 0, 0, 170, 0, 0, 0, 0, 0, 24, 0, 0, 171, 0, 0, 172, 0, 0, 0, 0, 173, 0, 0, 174, 175, 110, 0, 176, 177, 178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 179, 0, 0, 0, 0, 30, 0, 180, 0, 181, 32, 0, 182, 183, 0, 0, 0, 0, 0, 0, 32, 25, 0, 37, 0, 26, 0, 0, 0, 27, 184, 0, 0, 185, 0, 0, 0, 111, 37, 0, 0, 0, 0, 186, 0, 106, 0, 0, 0, 0, 187, 0, 0, 0, 121, 188, 189, 0, 0, 0, 190, 0, 191, 0, 192, 193, 15, 0, 0, 0, 112, 0, 0, 0, 53, 0, 0, 4, 0, 124, 0, 125, 0, 30, 0, 194, 0, 0, 195, 0, 0, 0, 196, 0, 0, 0, 197, 0, 0, 0, 0, 0, 198, 0, 0, 0, 199, 0, 200, 0, 0, 0, 0, 201, 202, 0, 0, 203, 0, 0, 0, 0, 33, 204, 0, 205, 0, 0, 0, 0, 0, 0, 126, 0, 206, 0, 0, 0, 207, 0, 0, 0, 0, 127, 208, 0, 118, 209, 0, 0, 0, 210, 211, 0, 212, 0, 0, 213, 0, 214, 0, 0, 133, 0, 215, 0, 0, 0, 0, 216, 0, 217, 0, 218, 0, 0, 0, 0, 0, 0, 219, 0, 7, 0, 0, 0, 0, 42, 131, 137, 0, 0, 135, 0, 0, 0, 0, 0, 0, 220, 0, 0, 0, 221, 0, 139, 222, 0, 0, 0, 223, 0, 224, 0, 225, 226, 0, 0, 0, 227, 0, 0, 145, 0, 228, 0, 229, 230, 0, 231, 0, 0, 232, 0, 0, 0, 233, 234, 154, 0, 0, 235, 0, 236, 0, 0, 136, 237, 0, 0, 0, 0, 0, 0, 0, 238, 239, 0, 240, 0, 38, 241, 0, 0, 0, 155, 0, 0, 0, 0, 0, 242, 0, 0, 0, 0, 243, 20, 158, 244, 54, 160, 0, 0, 0, 152, 245, 0, 0, 0, 246, 247, 248, 6, 249, 0, 0, 164, 176, 0, 0, 0, 4, 0, 0, 0, 178, 250, 180, 0, 0, 188, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 56, 0, 0, 189, 0, 33, 3, 251, 252, 0, 0, 0, 190, 253, 254, 0, 255, 256, 257, 0, 0, 258, 193, 0, 0, 196, 0, 0, 0, 0, 0, 0, 0, 0, 259, 260, 0, 0, 197, 261, 0, 0, 0, 0, 0, 262, 0, 0, 263, 264, 0, 202, 0, 265, 0, 0, 14, 0, 0, 0, 0, 0, 0, 44, 0, 0, 0, 0, 266, 0, 267, 0, 0, 0, 268, 0, 0, 2, 0, 204, 0, 0, 0, 0, 0, 0, 0, 210, 269, 211, 30, 43, 215, 270, 44, 271, 272, 0, 0, 213, 0, 273, 0, 274, 0, 0, 0, 0, 0, 0, 275, 0, 0, 0, 0, 276, 277, 3, 0, 278, 0, 0, 0, 52, 61, 0, 0, 218, 279, 68, 0, 280, 72, 0, 0, 0, 34, 51, 234, 281, 0, 282, 0, 0, 0, 283, 284, 285, 286, 287, 288, 289, 290, 291, 292, 293, 57, 236, 294, 295, 296, 297, 247, 298, 299, 300, 301, 302, 303, 304, 305, 0, 0, 0, 248, 249, 0, 0, 0, 0, 306, 0, 0, 0, 0, 0, 0, 12, 307, 0, 15, 0, 0, 0, 0, 20, 0, 0, 0, 0, 308, 0, 230, 0, 0, 0, 0, 0, 309, 55, 71, 0, 310, 0, 0, 0, 38, 0, 0, 311, 0, 0, 0, 0, 0, 0, 0, 312, 313, 314, 315, 316, 317, 76, 252, 318, 319, 257, 320, 321, 0, 0, 0, 0, 0, 0, 240, 322, 0, 0, 323, 0, 0, 0, 0, 51, 324, 325, 326, 0, 0, 327, 0, 0, 0, 242, 258, 0, 0, 328, 0, 0, 329, 0, 0, 330, 0, 0, 331, 0, 0, 0, 0, 332, 0, 0, 0, 333, 0, 0, 0, 0, 0, 0, 0, 0, 0, 334, 0, 0, 0, 0, 0, 0, 0, 0, 0, 335, 336, 337, 0, 0, 0, 0, 0, 0, 338, 0, 0, 0, 0, 0, 0, 0, 0, 339, 340, 341, 0, 0, 57, 342, 0, 0, 0, 0, 0, 343, 0, 264, 0, 0, 0, 0, 0, 0, 0, 0, 344, 0, 0, 0, 0, 345, 346, 0, 0, 0, 0, 0, 0, 347, 0, 0, 0, 268, 0, 0, 0, 0, 0, 271, 348, 0, 0, 0, 273, 0, 276, 349, 0, 0, 306, 0, 323, 350, 351, 0, 62, 352, 90, 336, 77, 63, 353, 259, 354, 266, 355, 274, 97, 356, 337, 338, 357, 0, 358, 0, 0, 0, 0, 0, 0, 0, 0, 79, 359, 0, 0, 0, 0, 0, 360, 0, 0, 0, 0, 361, 0, 0, 362, 0, 0, 0, 0, 0, 341, 0, 363, 364, 0, 365, 0, 0, 0, 0, 366, 367, 0, 0, 0, 0, 368, 0, 369, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 370, 371, 0, 0, 372, 373, 0, 0, 0, 374, 0, 375, 0, 376, 0, 0, 377, 0, 0, 378, 0, 0, 0, 0, 0, 0, 0, 379, 0, 0, 0, 343, 0, 380, 381, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 382, 0, 0, 0, 0, 0, 21, 0, 0, 0, 0, 0, 0, 383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 384, 0, 0, 0, 385, 0, 0, 0, 0, 0, 386, 0, 387, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 388, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 389, 0, 0, 390, 0, 0, 0, 278, 391, 392, 0, 0, 348, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 393, 0, 394, 0, 0, 0, 307, 308, 0, 43, 395, 356, 396, 0, 0, 397, 398, 0, 0, 0, 357, 0, 399, 0, 0, 0, 400, 0, 0, 0, 401, 402, 0, 403, 0, 43, 0, 0, 0, 0, 0, 363, 0, 371, 404, 0, 0, 0, 0, 0, 373, 0, 0, 405, 64, 406, 0, 73, 407, 408, 0, 374, 409, 0, 0, 0, 0, 0, 0, 410, 0, 0, 0, 411, 0, 0, 412, 0, 0, 0, 0, 0, 0, 0, 413, 414, 0, 0, 415, 0, 0, 367, 0, 0, 0, 416, 22, 417, 0, 0, 418, 0, 0, 0, 0, 0, 0, 419, 0, 0, 0, 0, 420, 421, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 360, 422, 0, 0, 423, 23, 24, 0, 0, 0, 0, 0, 0, 0, 0, 0, 424, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 425, 0, 0, 0, 0, 0, 0, 0, 426, 0, 427, 428, 0, 80, 0, 0, 429, 430, 0, 0, 0, 25, 0, 62, 63, 0, 0, 377, 0, 0, 0, 0, 431, 26, 0, 0, 0, 0, 0, 0, 27, 0, 432, 433, 434, 435, 436, 437, 0, 0, 0, 438, 439, 440, 441, 442, 0, 0, 0, 0, 0, 443, 76, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 2, 0, 0, 0, 0, 0, 381, 0, 0, 0, 444, 0, 445, 446, 0, 447, 448, 0, 383, 84, 0, 0, 0, 0, 0, 0, 0, 0, 449, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 450, 393, 0, 394, 451, 395, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 397, 0, 398, 452, 0, 0, 0, 453, 0, 0, 0, 0, 0, 0, 454, 0, 0, 0, 0, 399, 455, 0, 456, 0, 402, 457, 0, 458, 0, 0, 0, 0, 404, 459, 0, 0, 78, 0, 0, 368, 460, 405, 0, 0, 0, 0, 0, 412, 370, 52, 0, 418, 0, 419, 378, 0, 420, 461, 462, 0, 0, 0, 0, 0, 0, 0, 0, 0, 463, 424, 464, 0, 0, 410, 0, 0, 0, 0, 0, 465, 0, 466, 0, 0, 0, 0, 0, 467, 0, 468, 0, 0, 0, 469, 0, 0, 0, 0, 0, 0, 470, 0, 0, 471, 472, 0, 0, 0, 0, 473, 0, 0, 0, 0, 425, 0, 426, 474, 0, 0, 427, 0, 428, 475, 415, 421, 0, 0, 476, 0, 0, 0, 0, 0, 477, 0, 0, 478, 0, 0, 0, 0, 0, 0, 0, 479, 0, 0, 0, 0, 0, 429, 0, 480, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 86, 0, 0, 0, 73, 0, 0, 0, 0, 0, 481, 0, 0, 0, 0, 430, 432, 482, 0, 0, 0, 0, 0, 0, 0, 0, 483, 0, 0, 0, 484, 0, 485, 79, 0, 433, 434, 486, 0, 487, 435, 488, 0, 489, 490, 0, 0, 0, 0, 491, 0, 0, 492, 493, 494, 0, 495, 0, 436, 6, 496, 0, 0, 0, 0, 0, 0, 0, 0, 497, 0, 0, 0, 0, 0, 437, 0, 0, 0, 0, 438, 0, 33, 0, 498, 0, 0, 0, 0, 0, 499, 0, 0, 82, 0, 500, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 501, 0, 0, 0, 0, 0, 502, 0, 503, 504, 0, 0, 0, 0, 439, 505, 34, 0, 0, 506, 440, 0, 0, 0, 0, 84, 0, 0, 0, 0, 0, 507, 0, 0, 441, 447, 0, 0, 448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 508, 0, 0, 0, 0, 0, 509, 510, 0, 452, 511, 0, 512, 0, 513, 454, 0, 0, 514, 0, 0, 0, 0, 0, 455, 0, 456, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 515, 0, 0, 0, 516, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 517, 0, 0, 0, 0, 0, 0, 0, 518, 0, 0, 457, 519, 0, 0, 0, 0, 520, 0, 0, 521, 0, 0, 458, 522, 0, 459, 523, 524, 460, 525, 461, 526, 527, 528, 0, 0, 0, 0, 0, 0, 529, 0, 462, 0, 0, 0, 463, 465, 530, 0, 531, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 89, 0, 0, 466, 532, 0, 533, 0, 0, 0, 0, 467, 534, 0, 0, 472, 464, 0, 469, 0, 0, 0, 535, 536, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 477, 0, 36, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 537, 0, 538, 0, 0, 0, 0, 0, 478, 0, 0, 0, 0, 539, 0, 0, 471, 479, 487, 490, 0, 540, 0, 0, 0, 0, 0, 0, 0, 0, 541, 542, 494, 0, 0, 495, 543, 544, 0, 545, 0, 546, 547, 0, 0, 0, 0, 496, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 548, 497, 0, 549, 0, 89, 550, 551, 0, 0, 0, 0, 91, 498, 499, 0, 0, 552, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 553, 0, 0, 0, 0, 0, 554, 0, 0, 500, 0, 0, 0, 0, 0, 0, 0, 0, 555, 0, 0, 0, 0, 0, 0, 514, 0, 0, 556, 92, 0, 0, 557, 515, 0, 0, 0, 0, 501, 507, 0, 0, 0, 0, 0, 0, 0, 0, 0, 558, 559, 0, 0, 516, 560, 561, 562, 563, 0, 0, 517, 564, 565, 0, 0, 566, 0, 0, 0, 567, 0, 0, 0, 0, 568, 0, 0, 0, 0, 569, 0, 0, 518, 570, 571, 519, 572, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 94, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 573, 574, 520, 522, 0, 0, 0, 0, 523, 575, 576, 0, 577, 0, 578, 579, 580, 525, 526, 94, 96, 98, 0, 0, 581, 0, 0, 0, 100, 528, 529, 582, 0, 0, 583, 0, 584, 38, 0, 530, 0, 532, 0, 0, 0, 533, 0, 585, 0, 0, 0, 0, 534, 0, 0, 0, 0, 0, 0, 0, 0, 535, 586, 0, 0, 0, 536, 587, 0, 0, 588, 0, 0, 0, 0, 0, 0, 0, 0, 589, 0, 0, 541, 590, 591, 0, 542, 592, 0, 0, 0, 0, 0, 0, 0, 0, 593, 594, 0, 595, 543, 544, 0, 0, 0, 0, 0, 596, 597, 598, 0, 0, 0, 0, 0, 545, 96, 599, 0, 548, 101, 0, 0, 0, 0, 600, 0, 601, 602, 550, 0, 0, 0, 0, 603, 0, 0, 604, 605, 0, 0, 606, 0, 0, 0, 0, 0, 551, 607, 0, 0, 553, 608, 0, 0, 98, 0, 609, 610, 0, 555, 556, 611, 0, 558, 103, 559, 612, 0, 0, 0, 0, 0, 0, 0, 613, 0, 0, 614, 0, 0, 0, 562, 563, 615, 616, 617, 0, 618, 566, 0, 0, 0, 0, 619, 620, 0, 0, 568, 0, 569, 0, 570, 0, 621, 575, 0, 576, 0, 622, 623, 624, 0, 625, 626, 0, 0, 0, 0, 0 };

        public static short get(int row, int col)
        {
            if (isErrorEntry(row, col))
                return (short)-1;
            else
                return lookupValue(rowmap[row], columnmap[col]);
        }

        protected static boolean isErrorEntry(int row, int col)
        {
            final int INT_BITS = 32;
            int sigmapRow = row;

            int sigmapCol = col / INT_BITS;
            int bitNumberFromLeft = col % INT_BITS;
            int sigmapMask = 0x1 << (INT_BITS - bitNumberFromLeft - 1);

            return (lookupSigmap(sigmapRow, sigmapCol) & sigmapMask) == 0;
        }

        protected static int[][] sigmap = null;

        protected static void sigmapInit()
        {
            sigmap = new int[][]
            {
                { 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x802000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x20, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80000000, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000020, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x24, 0x28000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x500, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4200, 0x24000000, 0x0, 0x10000000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1020000, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x20, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x21, 0x1000910, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20004, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x10900, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x12000000, 0x40000, 0x0, 0x0, 0x0, 0x1008000, 0x1402080, 0x4004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x20814, 0x21001000, 0x4, 0x0, 0x0, 0x0, 0x0, 0x800, 0x2000000, 0x20000400, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x20, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x2, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x4000, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20004, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80000000, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x900000, 0x10901, 0x0, 0x0, 0x19, 0x0, 0x0, 0x24000001, 0x12200000, 0x48069, 0x5a02d10, 0x2100000, 0x40000800, 0x1008000, 0x41402280, 0x4004000, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x4000, 0x320, 0x2400000, 0x8020814, 0x25001002, 0x8015, 0x84404030, 0x80, 0x8020010, 0x940000, 0xc880, 0x20018c8, 0x20000420, 0x23000000, 0x60001f, 0xaf010080, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xaab00080, 0x10090000, 0x0, 0x1400, 0x8, 0x0, 0x8088000, 0x8200, 0x5021240, 0x400c014, 0x0, 0x0, 0x8, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5002d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x8080, 0x0, 0x0, 0x20000000, 0x1f, 0x8b000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x8000, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x6c00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x960, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80000000, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80000000, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x2, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x10, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x80084, 0x0, 0x8008040, 0x0, 0x0, 0x0, 0x10000, 0x100, 0x0, 0x0, 0x880000, 0x24, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x100000, 0x1000000, 0x0, 0x0, 0x40, 0x0, 0x100, 0x40, 0x0, 0x0, 0x0, 0x20, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x2, 0x40000000, 0x0, 0x0, 0x0, 0x500, 0x1000008, 0xc000000, 0x10000000, 0x40, 0x0, 0x0, 0x3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x20, 0x10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x20, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x2010, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x101000, 0x0, 0x0, 0x0, 0x2000, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0xa, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80100000, 0x10000000, 0x2008006, 0x0, 0x0, 0x4, 0x80200000, 0xa000000, 0x40000000, 0x1100000, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x8000000, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000020, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400010, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10008000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x180000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1002080, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x21000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x10900, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x12000000, 0x40000, 0x0, 0x0, 0x0, 0x1008000, 0x41402080, 0x4004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x20814, 0x21001000, 0x5, 0x0, 0x0, 0x0, 0x0, 0x800, 0x2000008, 0x20000400, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x10, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80080, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x880000, 0x24, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x2, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x1000008, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5a02d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x80, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x40100c0, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x904504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4d20, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000000, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x920, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x20000, 0x0, 0x0, 0x2, 0x0, 0x60000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x240000, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x1000004, 0x0, 0x8000000, 0x3f16000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400010, 0x5, 0x20000000, 0x0, 0x0, 0x80080, 0x822000, 0x10008000, 0x0, 0x0, 0x3, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x2, 0x10, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x20, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x8000, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80000000, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] sigmap1 = null;

        protected static void sigmap1Init()
        {
            sigmap1 = new int[][]
            {
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x80, 0x822000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80100000, 0x10000000, 0x2008004, 0x0, 0x0, 0x0, 0x80200000, 0x8000000, 0x40000000, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x822000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x21, 0x10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0xc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0310002, 0x10400000, 0x42008007, 0x0, 0x800000, 0x23004, 0x80380000, 0xca000000, 0x70000001, 0x41100000, 0x20, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x1a, 0x88000000, 0x20, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xa000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x24, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x180000, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x21, 0x910, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x11000000, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x9000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x2, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x33, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400010, 0x7, 0x20900040, 0x18901, 0x42020000, 0x180080, 0x82301b, 0x30008040, 0x0, 0x24004001, 0x92284003, 0x30048069, 0x5a02d10, 0x2180000, 0x400c0800, 0x1008000, 0x41402282, 0x4004010, 0x8000000, 0x480, 0x85000, 0xc0004, 0x4280, 0x4000, 0x360, 0x2408000, 0x28020814, 0x25081086, 0x508015, 0x8c40c070, 0x80, 0x8020010, 0x940100, 0x1c880, 0x20019c8, 0x20000420, 0x23000000, 0xe8001f, 0xaf0101a4, 0x4000000, 0x2, 0x20100000, 0x410a2ef, 0xeab00080, 0x10090000, 0x0, 0x1400, 0x8, 0x40, 0x8188000, 0x1008200, 0x5023740, 0x400c114, 0x40, 0x0, 0x400108, 0x60, 0x4504, 0x37, 0x280004, 0x20221065, 0xa0010000, 0x1000000a, 0x2c200000, 0x50004000, 0x6e68, 0x24100008, 0x40020000, 0x10030802, 0x40004400, 0x14000000, 0x9, 0x28000c00, 0x4808d00, 0x1000208, 0xc000906, 0x10014038, 0x40, 0x404000, 0x70e0000, 0x24000003, 0x807c0e0, 0x14400042, 0x33000, 0xc1011804, 0x1100c, 0xe0003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x20, 0x10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404020, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0xa0300002, 0x4400000, 0x5, 0x0, 0x0, 0x1000, 0x80080080, 0xc0822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80100000, 0x10000000, 0x2008004, 0x0, 0x0, 0x0, 0x80200000, 0x8000000, 0x40000000, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x2008000, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x40000000, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x107ff3df, 0xc0000000, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x50000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x20, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x1000d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20004, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x4000, 0x8000000, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x2, 0x0, 0x800, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x4000, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x280000, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x7, 0x0, 0x0, 0x20000, 0x180080, 0x823000, 0x30000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0xc0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x4000, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20004, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x8080, 0x0, 0x0, 0x20000000, 0x1f, 0x8b000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80100000, 0x10000000, 0x2008006, 0x0, 0x0, 0x4, 0x80200000, 0xa000000, 0x40000000, 0x1100000, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x8000000, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000000, 0x0, 0x2000005, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0xa0000002, 0x4400010, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10008000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x880000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x4000000, 0x40000, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x2, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x21, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x840000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80000000, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x40000, 0x0, 0xa00000, 0x1001, 0x0, 0x0, 0x0, 0x80000, 0x10, 0x28000000, 0x20200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x11000000, 0x2, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0xb200, 0x0, 0x40000800, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x1, 0x9000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x110, 0x0, 0x0, 0x8, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x40000, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x9000, 0x100, 0x80000000, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x5000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5a02d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x80, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x40100c0, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x904504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4d20, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2040000, 0x0, 0x0, 0x0, 0x11000, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80110000, 0x10000000, 0x2008006, 0x0, 0x0, 0x4, 0x80200000, 0xa000000, 0x40000000, 0x1100000, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x8000000, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x10900, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x12000000, 0x40000, 0x0, 0x0, 0x0, 0x1008000, 0x41402080, 0x4004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x20814, 0x21001000, 0x5, 0x0, 0x0, 0x0, 0x0, 0x800, 0x2001848, 0x20000400, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x200000, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x34440010, 0x42008005, 0x20a00000, 0x1001, 0x1000, 0x80280080, 0xc8822000, 0x50088001, 0x10, 0x28000000, 0x20200203, 0x10000000, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x80000, 0x4000000, 0x11000000, 0x2, 0x0, 0x20000000, 0x0, 0x400008, 0x0, 0x0, 0x0, 0x0, 0xb200, 0x0, 0x40000800, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x1, 0x9000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x110, 0x0, 0x0, 0x400008, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1020, 0x0, 0x40000, 0x0, 0x4000, 0x80, 0x0, 0x0, 0x9800, 0x100, 0xc0000000, 0x0 },
                { 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] sigmap2 = null;

        protected static void sigmap2Init()
        {
            sigmap2 = new int[][]
            {
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000020, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x200000, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x104, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x80240000, 0x400000, 0x20000000, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x40800, 0x400000, 0x80000, 0x480000, 0x0, 0x40, 0x200, 0x20000, 0x0, 0x0, 0x2082010, 0x804000, 0x800000, 0x80000000, 0x8, 0x3000000, 0x8000, 0x10, 0x0, 0x40002, 0x94102000, 0x18020040, 0x0, 0x1000, 0x0, 0x84, 0x1440200, 0x4000000, 0x0, 0x4, 0x220, 0x4044204, 0x95020000, 0x504010, 0x0, 0x30d0, 0x20000, 0x8000, 0x20080000, 0x60c00021, 0x10000000, 0x6800810, 0x100021, 0x108001, 0xc04058, 0x22008862, 0x10120683, 0x876000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400010, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10008000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x920, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x21, 0x910, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x200, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x28, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x208000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x802000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1002080, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x21000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x8000, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5002d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000008, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80100000, 0x0, 0x2008004, 0x0, 0x0, 0x0, 0x80200000, 0x0, 0x40000000, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x2, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x30, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x2000000, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x4000, 0x29, 0x5a02d10, 0x2100000, 0x40000800, 0x0, 0x202, 0x10, 0x8000000, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4080080, 0x8000, 0x84404030, 0x80, 0x8020010, 0x940000, 0x14000, 0x80, 0x20, 0x3000000, 0x880000, 0x40100a4, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x40, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x100, 0x40, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100008, 0x40000000, 0x10030002, 0x40004400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x1000208, 0x906, 0x14018, 0x40, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x802000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x24, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x4400000, 0x5, 0x0, 0x0, 0x1000, 0x80080080, 0xc0822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5802d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x80, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x108, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x920, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80000000, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x400000, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x8040, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x8080, 0x0, 0x0, 0x20000000, 0x60001f, 0x8b000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x19, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5a02d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x4000, 0x320, 0x400000, 0x8000000, 0x4000002, 0x8000, 0x84404030, 0x80, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xaab00080, 0x10090000, 0x0, 0x1400, 0x0, 0x0, 0x8088000, 0x8200, 0x5021240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x20, 0x10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x6c00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x2, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x34, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x21, 0x1000910, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x40000, 0x0, 0x2010, 0x1000, 0x80000000, 0x0, 0x0, 0x0, 0x10, 0x101000, 0x20100000, 0x0, 0x0, 0x2004, 0x10000, 0x0, 0x208010, 0x200000, 0x2000000, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x810, 0x2, 0x210000, 0x2, 0x80000, 0x0, 0x20, 0xa, 0x401, 0x0, 0x107ff3df, 0xc0000000, 0x1000, 0x10000000, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x1, 0x100, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x40000800, 0x40000, 0x0, 0x800, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x1, 0x10, 0x400, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x10000400, 0x800, 0x20020002, 0x280, 0x2000020, 0x0, 0x80, 0x8000000, 0x6000000, 0x0, 0x0, 0x0, 0x24000, 0x0, 0x0, 0x20251, 0x900000, 0x8800, 0x4002840, 0x9, 0x1080, 0x0, 0x0, 0x2400, 0x0, 0x3080000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x121100, 0x40000000, 0x10000840, 0x40000000, 0x20000, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x300, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x400000, 0x5, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4004020, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x24, 0x28000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x500, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4200, 0x24000000, 0x0, 0x10000000, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1020000, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] sigmap3 = null;

        protected static void sigmap3Init()
        {
            sigmap3 = new int[][]
            {
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x880000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x920, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x1000910, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x802000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400010, 0x5, 0x20000000, 0x0, 0x0, 0x80080, 0x822000, 0x10008000, 0x0, 0x0, 0x3, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x5000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x3000000, 0x0, 0xc0000000, 0x2000000, 0x0, 0x8000000, 0x2000, 0x0, 0x40, 0x8000, 0x0, 0x0, 0x3000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x300002, 0x0, 0x0, 0x0, 0x4400000, 0x800, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x1000d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x40000, 0x8, 0x1200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x21, 0x110, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0100002, 0x400000, 0x5, 0x0, 0x0, 0x0, 0x80080000, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x7, 0x0, 0x0, 0x23004, 0x80180800, 0xc0000000, 0x30000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x40, 0x1a, 0x80000000, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5802d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x80, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x920, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x10900, 0x0, 0x0, 0x19, 0x0, 0x0, 0x4000001, 0x12000000, 0x40029, 0x1000d10, 0x2100000, 0x40000800, 0x1008000, 0x1402280, 0x4004000, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x4000, 0x320, 0x2400000, 0x8020814, 0x25001002, 0x8005, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4800, 0x2000080, 0x20000420, 0x3000000, 0x0, 0x24010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x2a300080, 0x10090000, 0x0, 0x1400, 0x8, 0x0, 0x8088000, 0x8200, 0x5021240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000004, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5a02d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x80, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x40100c0, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4d20, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x10900, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x12000000, 0x40000, 0x0, 0x0, 0x0, 0x1008000, 0x41402080, 0x4004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x20814, 0x21001000, 0x5, 0x0, 0x0, 0x0, 0x0, 0x800, 0x2000848, 0x20000400, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x107ff3df, 0xc0000000, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x300000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x21, 0x110, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x20, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80100000, 0x10000000, 0x2008006, 0x0, 0x0, 0x4, 0x80200000, 0xa000000, 0x40000000, 0x1100000, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x8000000, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400010, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10008000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x1000000, 0x800024, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20010000, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x42000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xa00000c, 0x80000, 0x0, 0xc, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x820000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400010, 0x5, 0x20000000, 0x0, 0x0, 0x80080, 0x822000, 0x10008000, 0x0, 0x0, 0x3, 0x10000000, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x1800, 0x0, 0xc0000000, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x1000910, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x880000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x4000000, 0x40000, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x2, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5a02d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x80, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x21, 0x10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x34400010, 0x42008005, 0x20000000, 0x0, 0x1000, 0x80280080, 0xc8822000, 0x50008001, 0x0, 0x0, 0x203, 0x10000000, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x1800, 0x0, 0xc0000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x400000, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x107ff3df, 0xc0000000, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x200, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x20, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x800000, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x200000, 0x8040, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x8080, 0x0, 0x0, 0x20000000, 0x60001f, 0x8b000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x60000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x30000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x400000, 0x5, 0x0, 0x0, 0x1000, 0x80080000, 0xc0000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x80100000, 0x0, 0x2008004, 0x0, 0x0, 0x0, 0x80200000, 0x0, 0x40000000, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400010, 0x5, 0x20000000, 0x0, 0x0, 0x80080, 0x822000, 0x10008000, 0x0, 0x0, 0x3, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x21, 0x910, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x280, 0x0, 0x40300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x0, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e5, 0x28300080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x10900, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x12000000, 0x40000, 0x0, 0x0, 0x0, 0x1008000, 0x1402080, 0x4004000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x20814, 0x21001000, 0x5, 0x0, 0x0, 0x0, 0x0, 0x800, 0x2000000, 0x20000400, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
            };
        }

        protected static int[][] sigmap4 = null;

        protected static void sigmap4Init()
        {
            sigmap4 = new int[][]
            {
                { 0x0, 0x0, 0x0, 0x10, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xa, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x80, 0x822000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0000002, 0x4400000, 0x5, 0x0, 0x0, 0x0, 0x80080, 0x822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000, 0x0, 0x80000000, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x40000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x200000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x0, 0x0, 0x0, 0x0, 0x40, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x100000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0xa0300002, 0x4400000, 0x5, 0x0, 0x0, 0x1000, 0x80080080, 0xc0822000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x12, 0x80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x100000, 0x0, 0x0, 0x0, 0x18, 0x0, 0x0, 0x4000000, 0x0, 0x29, 0x5802d10, 0x2100000, 0x40000800, 0x0, 0x200, 0x0, 0x0, 0x480, 0x1000, 0x4, 0x4280, 0x0, 0x300, 0x400000, 0x8000000, 0x4000000, 0x8000, 0x84404030, 0x80, 0x8020010, 0x940000, 0x4000, 0x80, 0x20, 0x3000000, 0x0, 0x4010080, 0x4000000, 0x2, 0x100000, 0x410a2e7, 0xa8b00080, 0x90000, 0x0, 0x1400, 0x0, 0x0, 0x8008000, 0x8200, 0x5020240, 0x400c004, 0x0, 0x0, 0x0, 0x0, 0x4504, 0x37, 0x280004, 0x20201045, 0x20010000, 0x1000000a, 0x2c200000, 0x10000000, 0x4e68, 0x24100000, 0x40000000, 0x10030000, 0x4400, 0x14000000, 0x9, 0x28000c00, 0x808800, 0x200, 0x906, 0x14018, 0x0, 0x404000, 0x70e0000, 0x24000000, 0x8060060, 0x14400042, 0x33000, 0xc1010004, 0x1100c, 0x20003040, 0x2a500000 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000020, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x24, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x200, 0x4000000, 0x0, 0x0, 0x0, 0x0, 0x500, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x24000000, 0x0, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1020000, 0x0, 0x0, 0x0, 0x0, 0x1000000, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xa10, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xc4000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x4000, 0x20, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2000000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x80000, 0x0, 0x1000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x0, 0x0, 0x800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
                { 0x0, 0x0, 0x0, 0x20000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 }
            };
        }

        protected static int lookupSigmap(int row, int col)
        {
            if (row <= 111)
                return sigmap[row][col];
            else if (row >= 112 && row <= 223)
                return sigmap1[row-112][col];
            else if (row >= 224 && row <= 335)
                return sigmap2[row-224][col];
            else if (row >= 336 && row <= 447)
                return sigmap3[row-336][col];
            else if (row >= 448)
                return sigmap4[row-448][col];
            else
                throw new Error("Unexpected location requested in sigmap4 lookup");
        }

        protected static int[][] value = null;

        protected static void valueInit()
        {
            value = new int[][]
            {
                { 0x200fa, 0xb000b8, 0xcf00c9, 0xd700df, 0xe80104, 0x107010d, 0x10f011c, 0x124012a, 0x12c0136, 0x20177, 0x1870190, 0x1ca01cd, 0x1d101d6, 0x1de01e7, 0x1ea01ec, 0x1ee01f6, 0x1f90203, 0x209020c, 0x2120216, 0x22b0233, 0x23c023e, 0x246024f, 0x2530259, 0x25e0267, 0x268026a, 0x26b00cf, 0x2710274, 0x2820124, 0x28d0291, 0x29b02a8, 0x2b102b5, 0x2b802bf, 0xdf00e8, 0x2ca00e8, 0x2d602e0, 0x2e80124, 0x3010308, 0x30a030e, 0x3150319, 0x11c032c, 0xcf0335, 0x33b033f, 0x3400343, 0x1870190, 0x35101d1, 0x356035f, 0x3600362, 0x1ee0365, 0x367036c, 0x36f0371, 0x2120216, 0x37f0383, 0x38c0246, 0x3910395, 0x259025e, 0x399039a, 0x39b039f, 0x1770177, 0x3ad03b1, 0x18703b9, 0x3c203c6, 0x19003ce, 0x3d803db, 0x3e503f9, 0x401040f, 0x41401d1, 0x4220426, 0x42a043a, 0x1ee043f, 0x44e0460, 0x4630467, 0x2160473, 0x475047a, 0x47e0486, 0x4a304aa, 0x4af04b1, 0x4c104c4, 0x4c904ca, 0x4cf0246, 0x4d504d9, 0x4e804f5, 0x25e04fe, 0x50600cf, 0x5090511, 0x5150519, 0x51c0522, 0x52500cf, 0x2910291, 0x52f0530, 0x53c00cf, 0x5420544, 0x548054c, 0x556055b, 0x55f0567, 0x3db03db, 0x5710574, 0x577057b, 0x58f0592, 0x5950596, 0x59c02e8, 0x5a405a8, 0x5ab05b2, 0x5b90319, 0x5c405d1, 0x5da02e8, 0x5e805e9, 0x5ed0187, 0x3b903db, 0x40f0414, 0x1d10422, 0x6150619, 0x61d0216, 0x6240486, 0x629062d, 0x63104b1, 0x6350636, 0x246063d, 0x63e025e, 0x64502e8, 0x64e0651, 0x39f0654, 0x65b0190, 0x6640669, 0x67303c6, 0x3db03e5, 0x6690682, 0x3db068a, 0x68b068c, 0x68d03db, 0x3db069d, 0x53006a0, 0x6a306a6, 0x3db06a9, 0x1ee01ee, 0x3db0190, 0x6c406c6, 0x19006c9, 0x6ca06d0, 0x6d20212, 0x2160669, 0x47306e4, 0x6e506eb, 0x6ec06ed, 0x6ee06ef, 0x6f006f1, 0x6f206f3, 0x6f406f5, 0x6f806f9, 0x6fa06fb, 0x6fd06fe, 0x6ff0700, 0x7010702, 0x7030190, 0x3db070e, 0x4b10718, 0x71b0246, 0x1900190, 0x1900190, 0x1900190, 0x72e0190, 0x1900190, 0x5300738, 0x669073d, 0x25e025e, 0x759059c, 0x7650768, 0x76c076f, 0x7730291, 0x77803db, 0x3db00cf, 0x5300782, 0x78603db, 0x78f0793, 0x55b079a, 0x3db07a6, 0x7ac0669, 0x6690669, 0x7b702e8, 0x3db03db, 0x7c607cd, 0x7d202e8, 0x7d607db, 0x7dc0319, 0x31907e5, 0x7e905b2, 0x7f307f4, 0xcf03db, 0x7fd0800, 0x8030808, 0x80905d1, 0x67303db, 0x81a03db, 0x8260212, 0x82b082e, 0x838083d, 0x57702e8, 0x845084c, 0x3c603db, 0x85003d8, 0x3db03db, 0x6730861, 0x3db0864, 0x3db03db, 0x2e80669, 0x8740877, 0x87b0669, 0x3db0886, 0x88701ee, 0x1900673, 0x3db03c6, 0x89c089d, 0x8a20473, 0x8ac03db, 0x3c603db, 0x3db08b8, 0x8b90190, 0x3c603c6, 0x3c603c6, 0x3c603c6, 0x3c603db, 0x3c603c6, 0x8c808ce, 0x6690669, 0x4fe03d8, 0x3db08eb, 0x53008fb, 0x3db0907, 0x90b03db, 0x3db03db, 0x3db03db, 0x3db03db, 0x3db03db, 0x92f03db, 0x9330937, 0x7dc093c, 0x3190941, 0x94200cf, 0x94c0952, 0x95a095d, 0x3db087b, 0x67308c8, 0x803098a, 0x98e0669, 0x99503db, 0x99a099c, 0x3db0669, 0x66909a2, 0x3db03db, 0x3db03db, 0x3db00fa, 0x9ba09be, 0x9c10669, 0x9c409c7, 0x67309d2, 0x8c80995, 0x9db09e1, 0x9e3068b, 0x3db03db, 0x3db03db, 0x3db03db, 0x3db09ff, 0x3db03db, 0xa0403db, 0x3db0a0b, 0x3db03db, 0xa100319, 0x3db03db, 0x3db03db, 0xa2b09c7, 0x67308c8, 0xa4003db, 0x3db03db, 0x3db03db, 0x99a099c, 0x3db0a56, 0x3350a58, 0xa5d03db, 0xa6603db, 0x3db0a6d, 0x3db03db, 0xa7b0669, 0x3db03db, 0xa800669, 0xa820319, 0x3db03db, 0x3db0a8d, 0xa910a04, 0xa0b03db, 0x3db03db, 0xaa20aa5, 0xaa603db, 0x3db0ab6, 0xab703db, 0x3db03db, 0xaca03db, 0xad103db, 0xad50a8d, 0xa9103db, 0x66903db, 0x3db03db, 0x3db03db, 0x3db03db, 0xae703db, 0xaea03db, 0x3db03db, 0x3db03db, 0x3db03db, 0x3db0b00, 0x3db03db, 0x3db03db, 0x3db03db, 0x3db03db, 0x3db03db, 0x3db0000 },
                { 0x10001, 0xb100ba, 0xd00001, 0xd800e2, 0x10001, 0x1da0001, 0x110011f, 0x2da02e7, 0x12d019c, 0x10179, 0x1880192, 0x21c0231, 0x1d201d7, 0x1df0237, 0xba0001, 0x1ef019c, 0x1fa0202, 0x20a0001, 0x2130217, 0x22c0234, 0x19c0240, 0x2480358, 0x254025a, 0x25c037a, 0x3810386, 0x19c00d0, 0x2720273, 0x28103d2, 0x28c0292, 0x19c0001, 0x19c02b6, 0x2bb02be, 0xe20001, 0x10001, 0x2d7019c, 0x2e90407, 0x300043c, 0x19c04a8, 0x316031a, 0x31f012d, 0xd0019c, 0x407019c, 0x12d019c, 0x3440192, 0x19c0354, 0x1d701fa, 0x2540465, 0x1ef019c, 0x1fa0202, 0x20a025c, 0x3760378, 0x22c0234, 0x2400248, 0x5620254, 0x397025c, 0x19c0001, 0x58003a0, 0x17903a8, 0x3ae03b2, 0x5e103b8, 0x3c3019c, 0x3ca03cf, 0x19c019c, 0x12d019c, 0x19c0411, 0x415043c, 0x423019c, 0x19c01fa, 0x647019c, 0x44c061b, 0x4620468, 0x4700474, 0x476019c, 0x47d0487, 0x22c047d, 0x47d04b2, 0x668025c, 0x254019c, 0x4cd0248, 0x66004d8, 0x19c04f6, 0x25c04ff, 0x67a00d0, 0x2730510, 0x5140281, 0x51b028c, 0x5240528, 0x529052a, 0x19c019c, 0x67d00d0, 0x19c02b6, 0x19c054d, 0x557055c, 0x2d70568, 0x3d203d2, 0x19c022c, 0x4f6019c, 0x3000591, 0x694069f, 0x59d02e9, 0x31605a9, 0x19c05b3, 0x6ba031a, 0x5c505d3, 0x19c02e9, 0x25c019c, 0x12d06c8, 0x3b806c3, 0x4110415, 0x6da0604, 0x44c0462, 0x4680622, 0x47d0487, 0x22c047d, 0x47d0633, 0x6a4025c, 0x24804d8, 0x6e3025c, 0x6f702e9, 0x3ae03b2, 0x7360655, 0x754065e, 0x661019c, 0x19c019c, 0x19c019c, 0x19c0681, 0x72d0681, 0x6810681, 0x681019c, 0x75603d2, 0x19c01d7, 0x6a4019c, 0x19c019c, 0x7b806b4, 0x19c0192, 0x661019c, 0x6c7019c, 0x7be047d, 0x19c06d3, 0x842019c, 0x4740661, 0x6e60661, 0x6610661, 0x6610661, 0x6610661, 0x6610661, 0x6610661, 0x6610661, 0x6610661, 0x6610661, 0x6610661, 0x6610661, 0x6610704, 0x19c0754, 0x756025c, 0x25c0721, 0x7260727, 0x7280729, 0x72a072b, 0x661072f, 0x7310732, 0x19c019c, 0x19c073e, 0x25c025c, 0x19c0762, 0x5100514, 0x76b051b, 0x5240777, 0x19c019c, 0x19c00d0, 0x19c019c, 0x54d019c, 0x5570794, 0x7960568, 0x19c022c, 0x19c019c, 0x19c019c, 0x19c07bb, 0x84f019c, 0x591019c, 0x8a102e9, 0x5a9019c, 0x19c07e0, 0x31a0001, 0x3d207ed, 0x921019c, 0xd0019c, 0x19c022c, 0x4f6025c, 0x926080f, 0x19c019c, 0x1d7019c, 0x47d0828, 0x6e60661, 0x25c019c, 0x4f602e9, 0x655019c, 0x19c019c, 0x4f6019c, 0x19c019c, 0x19c04f6, 0x19c04f6, 0x19c019c, 0x2e9019c, 0x4110415, 0x7be019c, 0x19c019c, 0x19c0953, 0x192019c, 0x19c019c, 0x19c0956, 0x4f60474, 0x19c019c, 0x19c019c, 0x19c0001, 0x7b80192, 0x7be019c, 0x19c019c, 0x19c025c, 0x19c09c5, 0x19c06a4, 0x8c9025c, 0x19c019c, 0x4ff025c, 0x25c076b, 0x19c019c, 0x9de0794, 0x9de019c, 0x19c09f8, 0x19c019c, 0x19c019c, 0x956019c, 0xa24019c, 0x19c019c, 0x19c0a26, 0x31a0a3c, 0x19c00d0, 0x22c019c, 0x19c025c, 0x25c0a3d, 0x19c0980, 0x4f6098b, 0x4f6019c, 0x19c0953, 0x19c019c, 0x956019c, 0x19c09a1, 0x19c019c, 0xa4c019c, 0x19c019c, 0x25c0a71, 0x4f6019c, 0x9c309c6, 0x19c0a73, 0x9d7019c, 0x9dc019c, 0x73e0681, 0x19c019c, 0x19c019c, 0x19c019c, 0xa9c0a00, 0xa9d019c, 0x4f6019c, 0x19c04f6, 0x19c019c, 0x19c031a, 0x19c019c, 0xabe019c, 0x19c09c6, 0x19c0a3a, 0x98b019c, 0x19c019c, 0x19c019c, 0x19c019c, 0x1019c, 0x19c019c, 0xffff019c, 0xffff019c, 0x19c0a6e, 0x1019c, 0x19c019c, 0x19c019c, 0x19c019c, 0x19c031a, 0x19c019c, 0x19c04f6, 0x4f604f6, 0x4f6019c, 0x19cffff, 0xaa30aa4, 0x4f6ffff, 0xffff0ab5, 0x9c6ffff, 0xffffffff, 0xffffffff, 0x19cffff, 0x19c04f6, 0x4f6ffff, 0x19cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xae6ffff, 0xae9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0b01, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x30003, 0xb300bc, 0xd10003, 0xd900e3, 0x30003, 0x3f30003, 0x118011e, 0x47c057e, 0x12e019e, 0x3017b, 0x1890194, 0x21e021e, 0x1d3019e, 0x3de021e, 0x27d0003, 0x1f0019e, 0x1fb03f3, 0x47c0003, 0x3fb0218, 0x47c019e, 0x19e0241, 0x249047c, 0x255047b, 0x25d021e, 0x21e021e, 0x19e00d1, 0x57e0583, 0x21effff, 0xffff0293, 0x19e0003, 0x19effff, 0xffff08bd, 0x2c20003, 0x30003, 0xffff019e, 0x2eaffff, 0x6dd08bd, 0x19e030f, 0xffff031b, 0x11effff, 0x332019e, 0xffff019e, 0x341019e, 0x1890348, 0x19e0355, 0x7b101fb, 0x4ec021e, 0x363019e, 0x1fbffff, 0xffff04f3, 0xffff0379, 0xffffffff, 0x241038e, 0xffff0396, 0xffff025d, 0x19e0003, 0xffffffff, 0x3a3ffff, 0xffffffff, 0x189021e, 0xffff019e, 0xffffffff, 0x19e019e, 0xffff019e, 0x19e0412, 0x4160418, 0x425019e, 0x19e01fb, 0xffff019e, 0x21e021e, 0xffffffff, 0xffffffff, 0xffff019e, 0x21e021e, 0xffff021e, 0x21e04b3, 0x4c204c3, 0x63f019e, 0x21e04d4, 0x21e021e, 0x19effff, 0x25d0500, 0xffff0332, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff052b, 0x19e019e, 0xffff0540, 0x19effff, 0x19effff, 0xffffffff, 0xffffffff, 0xffffffff, 0x19effff, 0xffff019e, 0xffffffff, 0xffffffff, 0x59b02ea, 0xffffffff, 0x19effff, 0x1fb05bd, 0x5c605d2, 0x19e02ea, 0x6be019e, 0xffff0189, 0x21e021e, 0x5fb05fd, 0x5fe0425, 0x21effff, 0xffffffff, 0x21e021e, 0xffff021e, 0x21e04b3, 0x4c204c3, 0x63b021e, 0x21e025d, 0x21e02ea, 0xffffffff, 0xffffffff, 0x189ffff, 0xffff019e, 0x19e019e, 0x19e019e, 0x19effff, 0x21effff, 0xffffffff, 0xffff019e, 0xffffffff, 0x19effff, 0xffff019e, 0x19e019e, 0xffffffff, 0x19e06bf, 0xffff019e, 0xffff019e, 0xffff021e, 0x19effff, 0xffff019e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x19effff, 0x71304c3, 0x4c3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x19e019e, 0x19e0740, 0x25d025d, 0x19e059b, 0xffffffff, 0xffffffff, 0xffffffff, 0x19e019e, 0x19e0780, 0x19e019e, 0xffff019e, 0xffffffff, 0xffffffff, 0x19effff, 0x19e019e, 0x19e019e, 0x19e02ea, 0xffff019e, 0xffff019e, 0x59b02ea, 0xffff019e, 0x19effff, 0x7e40003, 0xffffffff, 0x5d2019e, 0x7f8019e, 0x19effff, 0xffff04c3, 0xffff05d2, 0x19e019e, 0xffff019e, 0x21effff, 0xffffffff, 0x4c3019e, 0xffff02ea, 0xffff019e, 0x19e019e, 0xffff019e, 0x19e019e, 0x19effff, 0x19effff, 0x19e019e, 0x2ea019e, 0xffffffff, 0x425019e, 0x19e019e, 0x19effff, 0x897019e, 0x19e019e, 0x19effff, 0xffffffff, 0x19e019e, 0x19e019e, 0x19e0003, 0xffff08c4, 0xffff019e, 0x19e019e, 0x19e09ac, 0x19effff, 0x19effff, 0x8ca04f3, 0x19e019e, 0x8e109d3, 0x4c3ffff, 0x19e019e, 0x740ffff, 0x740019e, 0x19effff, 0x19e019e, 0x19e019e, 0xffff019e, 0x59b019e, 0x19e019e, 0x19effff, 0x94005d2, 0x19e0946, 0xffff019e, 0x19e0a2f, 0xa380425, 0x19e08ca, 0xffffffff, 0xffff019e, 0x19effff, 0x19e019e, 0xffff019e, 0x19effff, 0x19e019e, 0xffff019e, 0x19e019e, 0x4f3ffff, 0xffff019e, 0xffff09c8, 0x19effff, 0x8ca019e, 0xffff019e, 0x740ffff, 0x19e019e, 0x19e019e, 0x19e019e, 0xffffffff, 0xffff019e, 0xffff019e, 0x19effff, 0x19e019e, 0x19e0a12, 0x19e019e, 0xffff019e, 0x19e0a35, 0x19e08ca, 0xffff019e, 0x19e019e, 0x19e019e, 0x19e019e, 0x3019e, 0x19e019e, 0xffff019e, 0x8ca019e, 0x19effff, 0x3019e, 0x19e019e, 0x19e019e, 0x19e019e, 0x19e0a84, 0x19e019e, 0x19effff, 0xffffffff, 0xffff019e, 0x19effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x19effff, 0x19effff, 0xffffffff, 0x19effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x40004, 0x3e200bf, 0xd20004, 0x67900e4, 0x40004, 0x10a0004, 0x70c0120, 0x18f0816, 0x12f019d, 0x4017d, 0x18a018f, 0x21d021d, 0x1d4019d, 0x19d021d, 0x27e0004, 0x1f1019d, 0x1fc019d, 0x19d0004, 0x19d0219, 0x22d019d, 0x19d018f, 0x24703e2, 0x256021d, 0x260021d, 0x21d021d, 0x19d00d2, 0x21d019d, 0x21d018f, 0x19d0294, 0x19d0004, 0x19d021d, 0x70c0424, 0x2c30004, 0x40004, 0x21d019d, 0x2eb018f, 0x21d0424, 0x19d022d, 0xffff019d, 0x320012f, 0xd2019d, 0x33d019d, 0x12f019d, 0x345018f, 0x19d01d4, 0x19d0444, 0x256021d, 0x1f1019d, 0x368019d, 0xffff04f4, 0xffff0219, 0x22dffff, 0x18f0247, 0xffff0256, 0xffff0260, 0x19d0004, 0xffff03a1, 0x17dffff, 0xffffffff, 0x3b4021d, 0xffff019d, 0x18fffff, 0x19d019d, 0x5c2019d, 0x19dffff, 0xffff01d4, 0x424019d, 0x19d0610, 0x1f1019d, 0x21d021d, 0xffffffff, 0x219ffff, 0xffff019d, 0x21d021d, 0x22d021d, 0x21d04b4, 0x42404c5, 0x256019d, 0x21d0247, 0x21d021d, 0x19dffff, 0x2600501, 0xffff00d2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff00d2, 0xffffffff, 0x19d019d, 0xffff00d2, 0x19dffff, 0x19dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x19d022d, 0xffff019d, 0xffffffff, 0xffffffff, 0xffff02eb, 0xffffffff, 0x19dffff, 0xffffffff, 0x5c7ffff, 0x19d02eb, 0xffff019d, 0x12f05f0, 0x21d021d, 0xffffffff, 0x1d40424, 0x21dffff, 0xffff0219, 0x21d021d, 0x22d021d, 0x21d04b4, 0x42404c5, 0x247021d, 0x21d0260, 0x21d02eb, 0xffffffff, 0x256ffff, 0xffff018f, 0x662019d, 0x19d019d, 0x19d019d, 0x19dffff, 0x21dffff, 0xffffffff, 0xffff019d, 0xffff01d4, 0x19dffff, 0xffff019d, 0x19d019d, 0x1f101f1, 0x19d018f, 0x662019d, 0x18f019d, 0x6cb021d, 0x19dffff, 0x6d5019d, 0xffff0662, 0xffff0662, 0x6620662, 0x6620662, 0x6620662, 0x6620662, 0x6620662, 0x6620662, 0x6620662, 0x6620662, 0x6620662, 0x6620662, 0x662018f, 0x19dffff, 0x4b404c5, 0x4c50247, 0x18f018f, 0x18f018f, 0x18f018f, 0x662018f, 0x18f018f, 0x19d019d, 0x19d0741, 0xffff0260, 0x19dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x19d019d, 0x19d00d2, 0x19d019d, 0xffff019d, 0xffffffff, 0xffffffff, 0x19d022d, 0x19d019d, 0x19d019d, 0x19d02eb, 0xffff019d, 0xffff019d, 0xffff02eb, 0xffff019d, 0x19dffff, 0xffff0004, 0xffffffff, 0xffff019d, 0xd2019d, 0x19d022d, 0xffff08c0, 0xffffffff, 0x19d019d, 0xffff019d, 0x21dffff, 0xffff0662, 0x4c5019d, 0xffff02eb, 0xffff019d, 0x19d019d, 0xffff019d, 0x19d019d, 0x19dffff, 0x19dffff, 0x19d019d, 0x2eb019d, 0xffffffff, 0x424019d, 0x19d019d, 0x19d01f1, 0x18f019d, 0x19d019d, 0x19d06cb, 0xffffffff, 0x19d019d, 0x19d019d, 0x19d0004, 0x8ba018f, 0xffff019d, 0x19d019d, 0x19dffff, 0x19dffff, 0x19dffff, 0x8cb08cf, 0x19d019d, 0x501ffff, 0x8c0ffff, 0x19d019d, 0x741ffff, 0x741019d, 0x19dffff, 0x19d019d, 0x19d019d, 0xffff019d, 0xffff019d, 0x19d019d, 0x19dffff, 0xffffffff, 0x19d00d2, 0x22d019d, 0x19dffff, 0xffff0424, 0x19d0981, 0xffffffff, 0xffff019d, 0x19dffff, 0x19d019d, 0xffff019d, 0x19dffff, 0x19d019d, 0xffff019d, 0x19d019d, 0xa6806cb, 0xffff019d, 0xffff09c9, 0x19dffff, 0xffff019d, 0xffff019d, 0x741ffff, 0x19d019d, 0x19d019d, 0x19d019d, 0xffffffff, 0xffff019d, 0xffff019d, 0x19dffff, 0x19d019d, 0x19dffff, 0x19d019d, 0xffff019d, 0x19d09c9, 0x19dffff, 0xffff019d, 0x19d019d, 0x19d019d, 0x19d019d, 0x4019d, 0x19d019d, 0x6cb019d, 0xffff019d, 0x19dffff, 0x4019d, 0x19d019d, 0x19d019d, 0x19d019d, 0x19dffff, 0x19d019d, 0x19dffff, 0xffffffff, 0xffff019d, 0x19dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x19dffff, 0x19dffff, 0xffffffff, 0x19dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x50005, 0x50005, 0xd30005, 0x1e200e5, 0xea0005, 0x1db0005, 0x35c0431, 0x6ab070f, 0x130019f, 0x5017e, 0x710081e, 0x89808a5, 0x8a608bb, 0x8c508c6, 0x50005, 0x1f2019f, 0x509b0, 0x9cd0005, 0xa300005, 0xab8ffff, 0x19f0243, 0x24a0359, 0xffffffff, 0x262ffff, 0xffffffff, 0x19f00d3, 0x5ffff, 0xffffffff, 0xffffffff, 0xffff02a9, 0xffffffff, 0x50005, 0xffff00ea, 0x2cb02cf, 0xffff019f, 0x2ecffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0130, 0x333ffff, 0xffff019f, 0x130019f, 0xffffffff, 0x19fffff, 0xffffffff, 0xffffffff, 0x1f2019f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x38d038f, 0xffffffff, 0xffff0262, 0x19f055e, 0xffffffff, 0x17e03a9, 0xffffffff, 0xffff03ba, 0xffff019f, 0xffffffff, 0x19f019f, 0x130019f, 0x19fffff, 0xffffffff, 0xffff019f, 0x19fffff, 0x43b019f, 0x44dffff, 0xffffffff, 0xffffffff, 0xffff019f, 0x47f0488, 0xffff047f, 0x47f04b5, 0xffff04c6, 0xffff019f, 0x4ceffff, 0xffff04da, 0x19fffff, 0x2620502, 0xffff0333, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x19f019f, 0xffffffff, 0x19fffff, 0x19fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x19fffff, 0xffff019f, 0xffffffff, 0xffffffff, 0xffff02ec, 0xffffffff, 0x19fffff, 0x5baffff, 0x5c8ffff, 0x19f02ec, 0xffff019f, 0x130ffff, 0x5f2ffff, 0xffffffff, 0xffffffff, 0x44dffff, 0xffffffff, 0x47f0627, 0xffff047f, 0x47f04b5, 0xffff04c6, 0xffff04da, 0xffff0262, 0xffff02ec, 0xffffffff, 0xffffffff, 0xffffffff, 0x663019f, 0x19f019f, 0x19f019f, 0x19fffff, 0xffffffff, 0xffffffff, 0xffff019f, 0xffffffff, 0x19fffff, 0xffff019f, 0x19f019f, 0x6b306b5, 0x19fffff, 0x663019f, 0xffff019f, 0xffff047f, 0x19fffff, 0xffff019f, 0xffff0663, 0xffff0663, 0x6630663, 0x6630663, 0x6630663, 0x6630663, 0x6630663, 0x6630663, 0x6630663, 0x6630663, 0x6630663, 0x6630663, 0x663ffff, 0x19f04b5, 0x4b504c6, 0x4c6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x663ffff, 0xffffffff, 0x19f019f, 0x19f0742, 0x7520262, 0x19fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x19f019f, 0x19fffff, 0x19f019f, 0xffff019f, 0xffffffff, 0xffffffff, 0x19fffff, 0x19f019f, 0x19f019f, 0x19f07bc, 0xffff019f, 0xffff019f, 0xffff02ec, 0xffff019f, 0x19fffff, 0xffff05ba, 0x7eaffff, 0xffff019f, 0xffff019f, 0x19fffff, 0xffff04c6, 0xffffffff, 0x19f019f, 0xffff019f, 0x47fffff, 0xffff0663, 0x4c6019f, 0xffff02ec, 0xffff019f, 0x19f019f, 0xffff019f, 0x19f019f, 0x19fffff, 0x19fffff, 0x19f019f, 0x2ec019f, 0xffffffff, 0xffff019f, 0x19f019f, 0x19f0889, 0x896019f, 0x19f019f, 0x19fffff, 0xffffffff, 0x19f019f, 0x19f019f, 0x19f04b5, 0x4b508c3, 0xffff019f, 0x19f019f, 0x19fffff, 0x19fffff, 0x19fffff, 0xffffffff, 0x19f019f, 0x502ffff, 0x4c6ffff, 0x19f019f, 0x742ffff, 0x742019f, 0x19fffff, 0x19f019f, 0x19f019f, 0xffff019f, 0xffff019f, 0x19f019f, 0x19f07ea, 0xffffffff, 0x19fffff, 0xffff019f, 0x19fffff, 0xffffffff, 0x19fffff, 0xffffffff, 0xffff019f, 0x19fffff, 0x19f019f, 0xffff019f, 0x19fffff, 0x19f019f, 0xffff019f, 0x19f019f, 0xffffffff, 0xffff019f, 0xffff09ca, 0x19f04b5, 0xffff019f, 0xffff019f, 0x742ffff, 0x19f019f, 0x19f019f, 0x19f019f, 0xffffffff, 0xffff019f, 0xffff019f, 0x19fffff, 0x19f019f, 0x19fffff, 0x19f019f, 0xffff019f, 0x19f09ca, 0x19fffff, 0xffff019f, 0x19f019f, 0x19f019f, 0x19f019f, 0xffff019f, 0x19f019f, 0xffff019f, 0xffff019f, 0x19fffff, 0xffff019f, 0x19f019f, 0x19f019f, 0x19f019f, 0x19fffff, 0x19f019f, 0x19fffff, 0xffffffff, 0xffff019f, 0x19fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x9caffff, 0xffffffff, 0xffffffff, 0x19fffff, 0x19fffff, 0xffffffff, 0x19fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x60006, 0x3d600c0, 0x3d60006, 0x9a6ffff, 0x60006, 0xffff0006, 0xffffffff, 0x3c1ffff, 0xffffffff, 0x6017f, 0xffff0195, 0x2200220, 0xffffffff, 0xffff0220, 0xffff0006, 0xffffffff, 0xffffffff, 0xffff0006, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0220, 0x2630220, 0x2200220, 0xffffffff, 0x6d7ffff, 0x2200719, 0xffffffff, 0xffff0006, 0xffff0220, 0xffffffff, 0xffff0006, 0x60006, 0x220ffff, 0x2ed071a, 0x220ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0195, 0xffffffff, 0xffffffff, 0xffff0220, 0xffff04ad, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0263, 0xffff0006, 0xffff017f, 0x17fffff, 0xffffffff, 0xffff0220, 0xffffffff, 0x195ffff, 0xffffffff, 0xffffffff, 0x402ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2200220, 0xffffffff, 0xffffffff, 0xffff062f, 0x2200220, 0xffff0220, 0x220ffff, 0xffffffff, 0xffff0658, 0x220ffff, 0x2200220, 0xffffffff, 0x4fb0503, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0531, 0xffffffff, 0xffffffff, 0x54affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff02ed, 0xffffffff, 0xffffffff, 0xffffffff, 0x5c9ffff, 0xffff02ed, 0xffffffff, 0xffffffff, 0x2200220, 0xffffffff, 0xffffffff, 0x220ffff, 0xffffffff, 0x2200220, 0xffff0220, 0x220ffff, 0xffffffff, 0xffff0220, 0x22004fb, 0x22002ed, 0xffffffff, 0xffffffff, 0xffff0195, 0xffff066a, 0x674ffff, 0xffffffff, 0x66affff, 0x220ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x531ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0195, 0xffffffff, 0x195ffff, 0xffff0220, 0xffffffff, 0xffff066a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0195, 0x7a9ffff, 0xffffffff, 0xffffffff, 0x1950195, 0x1950195, 0x1950195, 0xffff0195, 0x1950195, 0x531ffff, 0xffff0743, 0xffff0753, 0x75cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x531ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x848ffff, 0xffffffff, 0xffff066a, 0xffff02ed, 0xffffffff, 0xffffffff, 0xffff02ed, 0xffffffff, 0x7deffff, 0xffff0006, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x814ffff, 0xffff08e6, 0x220ffff, 0xffffffff, 0xffff075c, 0xffff02ed, 0xffffffff, 0xffffffff, 0xffff0904, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2edffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x888ffff, 0x1950899, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff094f, 0xffff0006, 0xffff0195, 0xffffffff, 0xffffffff, 0x9a4ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x8e0ffff, 0x9d4ffff, 0x531ffff, 0xffffffff, 0xffff09ea, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x939ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x96effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0a5c, 0xffffffff, 0xffffffff, 0xffffffff, 0x9cfffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0ab2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa37ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa470a49, 0x6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x708d8, 0x8dbffff, 0x19b00c8, 0xffffffff, 0xe90100, 0x108ffff, 0xffffffff, 0xffffffff, 0x34a01a0, 0x70180, 0x65f0193, 0x2220222, 0xffff01a0, 0x1a00222, 0xffff0289, 0x6c101a0, 0x55101a0, 0x1a002fd, 0x1a00552, 0x22a01a0, 0x1a0023f, 0xffff06c2, 0x6c50222, 0x2640222, 0x2220222, 0x1a006e2, 0xffff01a0, 0x2220284, 0x1a006f6, 0x1a002aa, 0x1a006d8, 0x2ba02bd, 0xffff00e9, 0xe902ce, 0x6dc01a0, 0x2ef02f9, 0x2220309, 0x1a0ffff, 0x72c01a0, 0xffff07cc, 0xffff01a0, 0x33c01a0, 0x8e701a0, 0xffff0193, 0x1a00a57, 0x1a0ffff, 0xa6c0222, 0xa9701a0, 0xadf01a0, 0xaf1ffff, 0xffffffff, 0x37effff, 0x23fffff, 0xffffffff, 0xffff0398, 0x1a002ce, 0xffff03a2, 0x18003aa, 0xffffffff, 0xffff0222, 0xffff01a0, 0xffffffff, 0x1a001a0, 0x3e601a0, 0x1a0ffff, 0xffffffff, 0xffff01a0, 0x1a0ffff, 0xffff01a0, 0x2220222, 0xffffffff, 0xffffffff, 0xffff01a0, 0x2220222, 0xffff0222, 0x222ffff, 0xffffffff, 0xffff01a0, 0x222ffff, 0x2220222, 0x1a0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a001a0, 0xffffffff, 0x1a0ffff, 0x1a0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a00573, 0xffff01a0, 0xffffffff, 0x3090309, 0xffff02ef, 0xffffffff, 0x1a0ffff, 0xffffffff, 0x5caffff, 0x1a002ef, 0x30901a0, 0xffffffff, 0x2220222, 0xffffffff, 0xffffffff, 0x222ffff, 0xffffffff, 0x2220222, 0xffff0222, 0x222ffff, 0xffffffff, 0xffff0222, 0x222ffff, 0x22202ef, 0xffffffff, 0x653ffff, 0xffffffff, 0xffff01a0, 0x1a001a0, 0x1a001a0, 0x1a0ffff, 0x222ffff, 0xffffffff, 0xffff01a0, 0xffffffff, 0x1a0ffff, 0xffff01a0, 0x1a001a0, 0xffffffff, 0x1a00193, 0xffff01a0, 0xffff01a0, 0xffff0222, 0x1a0ffff, 0xffff01a0, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a001a0, 0x1a0073f, 0xffffffff, 0x1a0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a001a0, 0x1a0ffff, 0x1a001a0, 0xffff01a0, 0xffffffff, 0xffffffff, 0x1a0ffff, 0x1a001a0, 0x1a001a0, 0x1a002ef, 0xffff01a0, 0xffff01a0, 0xffff02ef, 0xffff01a0, 0x1a0ffff, 0xffff088c, 0xffffffff, 0xffff01a0, 0xffff01a0, 0x1a007ff, 0xffff0309, 0x309ffff, 0x1a001a0, 0xffff01a0, 0x222ffff, 0xffffffff, 0xffff01a0, 0xffff02ef, 0xffff01a0, 0x1a001a0, 0xffff01a0, 0x1a001a0, 0x1a0ffff, 0x1a0ffff, 0x1a001a0, 0x2ef01a0, 0xffffffff, 0xffff01a0, 0x1a001a0, 0x1a0ffff, 0x19301a0, 0x1a001a0, 0x1a0ffff, 0xffffffff, 0x1a001a0, 0x1a001a0, 0x1a0096c, 0xffff0193, 0xffff01a0, 0x1a001a0, 0x1a0ffff, 0x1a0ffff, 0x1a0ffff, 0xffffffff, 0x1a001a0, 0xffffffff, 0xffffffff, 0x1a001a0, 0x73fffff, 0x73f01a0, 0x1a0ffff, 0x1a001a0, 0x1a001a0, 0xffff01a0, 0xffff01a0, 0x1a001a0, 0x1a0ffff, 0xffffffff, 0x1a0ffff, 0xffff01a0, 0x1a0ffff, 0xffffffff, 0x1a0ffff, 0xffffffff, 0xffff01a0, 0x1a0ffff, 0x1a001a0, 0xffff01a0, 0x1a0ffff, 0x1a001a0, 0xffff01a0, 0x1a001a0, 0xffffffff, 0xffff01a0, 0xffffffff, 0x1a0ffff, 0xffff01a0, 0xffff01a0, 0x73fffff, 0x1a001a0, 0x1a001a0, 0x1a001a0, 0xffffffff, 0xffff01a0, 0xffff01a0, 0x1a0ffff, 0x1a001a0, 0x1a0ffff, 0x1a001a0, 0xffff01a0, 0x1a0ffff, 0x1a0ffff, 0xffff01a0, 0x1a001a0, 0x1a001a0, 0x1a001a0, 0x88c01a0, 0x1a001a0, 0xffff01a0, 0xffff01a0, 0x1a0ffff, 0x96c01a0, 0x1a001a0, 0x1a001a0, 0x1a001a0, 0x1a0ffff, 0x1a001a0, 0x1a0ffff, 0xffffffff, 0xffff01a0, 0x1a0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a0ffff, 0x1a0ffff, 0xffffffff, 0x1a0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x80008, 0x80008, 0x5880008, 0xdaffff, 0xed0008, 0xffff0008, 0xffffffff, 0x196ffff, 0xffffffff, 0x80183, 0xffff0196, 0x2230223, 0xffffffff, 0xffff0223, 0x80008, 0xffffffff, 0x8ffff, 0xffff0008, 0xffff0008, 0xffffffff, 0xffff0242, 0xffffffff, 0xffffffff, 0xffff0223, 0x2230223, 0xffffffff, 0x8ffff, 0x6650196, 0xffffffff, 0xffff02ab, 0xffffffff, 0x80008, 0xffff00ed, 0xffff02d0, 0xffffffff, 0x2f20196, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0991, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0196, 0xffffffff, 0xffffffff, 0xffff0223, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x242ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0183, 0x183ffff, 0xffffffff, 0xffff0223, 0xffffffff, 0x196ffff, 0xffffffff, 0xffff03f8, 0x403ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2230223, 0xffffffff, 0xffffffff, 0xffff0479, 0x4800223, 0xffff0480, 0x480ffff, 0xffffffff, 0xffffffff, 0x4d0ffff, 0x2230223, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0532, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x689ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff02f2, 0xffffffff, 0xffffffff, 0xffffffff, 0x5cbffff, 0xffff05e2, 0xffffffff, 0xffffffff, 0x2230223, 0xffffffff, 0xffffffff, 0x223ffff, 0xffffffff, 0x4800223, 0xffff0480, 0x480ffff, 0xffffffff, 0xffff0223, 0x223ffff, 0x2230648, 0xffffffff, 0xffffffff, 0xffff0196, 0xffff066d, 0x675ffff, 0xffffffff, 0x66dffff, 0x223ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x69effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0196, 0xffffffff, 0x196ffff, 0xffff0480, 0xffffffff, 0xffff066d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0196, 0xffffffff, 0xffffffff, 0xffffffff, 0x1960196, 0x1960196, 0x1960196, 0xffff0196, 0x1960196, 0x737ffff, 0xffff0744, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x781ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff066d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff05e2, 0xffff0868, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff086a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x675ffff, 0xffffffff, 0x480ffff, 0xffffffff, 0xffffffff, 0xffff0843, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x85effff, 0xffffffff, 0xffffffff, 0x870ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1960675, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0196, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x8f2ffff, 0x744ffff, 0x744ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x675ffff, 0xffffffff, 0xffffffff, 0x996ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x675ffff, 0xffffffff, 0xffffffff, 0x744ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x675ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x90009, 0x3f603f6, 0xffff0009, 0xffffffff, 0x90009, 0xffff0009, 0xffffffff, 0xffffffff, 0xffff01a1, 0x9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0009, 0xffff01a1, 0xffffffff, 0xffff0009, 0xffffffff, 0xffffffff, 0x1a1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3d50009, 0xffffffff, 0xffffffff, 0xffff0009, 0x90009, 0xffff01a1, 0x2f3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a1, 0xffff01a1, 0xffffffff, 0x1a1ffff, 0xffffffff, 0xffffffff, 0xffff01a1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a10009, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a1, 0xffffffff, 0x1a101a1, 0xffff01a1, 0x1a1ffff, 0xffffffff, 0xffff01a1, 0x1a1ffff, 0xffff01a1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a1, 0xffffffff, 0xffffffff, 0x1a1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a101a1, 0xffffffff, 0x1a1ffff, 0x1a1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a1ffff, 0xffff01a1, 0xffffffff, 0xffffffff, 0xffff05a1, 0xffffffff, 0x1a1ffff, 0xffffffff, 0x5cdffff, 0x1a105e3, 0xffff01a1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0649, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a1, 0x1a101a1, 0x1a101a1, 0x1a1ffff, 0xffffffff, 0xffffffff, 0xffff01a1, 0xffffffff, 0x1a1ffff, 0xffff01a1, 0x1a101a1, 0xffffffff, 0x1a1ffff, 0xffff01a1, 0xffff01a1, 0xffffffff, 0x1a1ffff, 0xffff01a1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a101a1, 0x1a10745, 0xffffffff, 0x1a1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a101a1, 0x1a1ffff, 0x1a101a1, 0xffff01a1, 0xffffffff, 0xffffffff, 0x1a1ffff, 0x1a101a1, 0x1a101a1, 0x1a1ffff, 0xffff01a1, 0xffff01a1, 0xffff07d4, 0xffff01a1, 0x1a1ffff, 0xffff0009, 0xffffffff, 0xffff01a1, 0xffff01a1, 0x1a1ffff, 0xffffffff, 0xffffffff, 0x1a101a1, 0xffff01a1, 0xffffffff, 0xffffffff, 0xffff01a1, 0xffff0844, 0xffff01a1, 0x1a101a1, 0xffff01a1, 0x1a101a1, 0x1a1ffff, 0x1a1ffff, 0x1a101a1, 0x87101a1, 0xffffffff, 0xffff01a1, 0x1a101a1, 0x1a1ffff, 0xffff01a1, 0x1a101a1, 0x1a1ffff, 0xffffffff, 0x1a101a1, 0x1a101a1, 0x1a10009, 0xffffffff, 0xffff01a1, 0x1a101a1, 0x1a1ffff, 0x1a1ffff, 0x1a1ffff, 0xffffffff, 0x1a101a1, 0xffffffff, 0xffffffff, 0x1a101a1, 0x9dfffff, 0x9e501a1, 0x1a1ffff, 0x1a101a1, 0x1a101a1, 0xffff01a1, 0xffff01a1, 0x1a101a1, 0x1a1ffff, 0xffffffff, 0x1a1ffff, 0xffff01a1, 0x1a1ffff, 0xffffffff, 0x1a1ffff, 0xffffffff, 0xffff01a1, 0x1a1ffff, 0x1a101a1, 0xffff01a1, 0x1a1ffff, 0x1a101a1, 0xffff01a1, 0x1a101a1, 0xffffffff, 0xffff01a1, 0xffffffff, 0x1a1ffff, 0xffff01a1, 0xffff01a1, 0x745ffff, 0x1a101a1, 0x1a101a1, 0x1a101a1, 0xffffffff, 0xffff01a1, 0xffff01a1, 0x1a1ffff, 0x1a101a1, 0x1a1ffff, 0x1a101a1, 0xffff01a1, 0x1a1ffff, 0x1a1ffff, 0xffff01a1, 0x1a101a1, 0x1a101a1, 0x1a101a1, 0x901a1, 0x1a101a1, 0xffff01a1, 0xffff01a1, 0x1a1ffff, 0x901a1, 0x1a101a1, 0x1a101a1, 0x1a101a1, 0x1a1ffff, 0x1a101a1, 0x1a1ffff, 0xffffffff, 0xffff01a1, 0x1a1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a1ffff, 0x1a1ffff, 0xffffffff, 0x1a1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0xaffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xeeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a2, 0xaffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffff00ee, 0xffff00ee, 0xffff01a2, 0x2f4ffff, 0xffffffff, 0x3fcffff, 0xffffffff, 0xffffffff, 0xffff01a2, 0xffff01a2, 0xffff01a2, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffff01a2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a2, 0xffffffff, 0x1a201a2, 0xffff01a2, 0x1a2ffff, 0xffffffff, 0xffff01a2, 0x1a2ffff, 0xffff01a2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a2, 0xffffffff, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a201a2, 0xffffffff, 0x1a2ffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a2ffff, 0xffff01a2, 0xffffffff, 0xffffffff, 0xffff02f4, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0x1a202f4, 0xffff01a2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff02f4, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a2, 0x1a201a2, 0x1a201a2, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffff01a2, 0xffffffff, 0x1a2ffff, 0xffff01a2, 0x1a201a2, 0xffffffff, 0x1a2ffff, 0xffff01a2, 0xffff01a2, 0xffffffff, 0x1a2ffff, 0xffff01a2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a201a2, 0x1a2ffff, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a201a2, 0x1a2ffff, 0x1a201a2, 0xffff01a2, 0xffffffff, 0xffffffff, 0x1a2ffff, 0x1a201a2, 0x1a201a2, 0x1a202f4, 0xffff01a2, 0xffff01a2, 0xffff02f4, 0xffff01a2, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffff01a2, 0xffff01a2, 0x1a2ffff, 0xffffffff, 0xffffffff, 0x1a201a2, 0xffff01a2, 0xffffffff, 0xffffffff, 0xffff01a2, 0xffff02f4, 0xffff01a2, 0x1a201a2, 0xffff01a2, 0x1a201a2, 0x1a2ffff, 0x1a2ffff, 0x1a201a2, 0x2f401a2, 0xffffffff, 0xffff01a2, 0x1a201a2, 0x1a2ffff, 0xffff01a2, 0x1a201a2, 0x1a2ffff, 0xffffffff, 0x1a201a2, 0x1a201a2, 0x1a2ffff, 0xffffffff, 0xffff01a2, 0x1a201a2, 0x1a2ffff, 0x1a2ffff, 0x1a2ffff, 0xffffffff, 0x1a201a2, 0xffffffff, 0xffffffff, 0x1a201a2, 0xffffffff, 0xffff01a2, 0x1a2ffff, 0x1a201a2, 0x1a201a2, 0xffff01a2, 0xffff01a2, 0x1a201a2, 0x1a2ffff, 0xffffffff, 0x1a2ffff, 0xffff01a2, 0x1a2ffff, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffff01a2, 0x1a2ffff, 0x1a201a2, 0xffff01a2, 0x1a2ffff, 0x1a201a2, 0xffff01a2, 0x1a201a2, 0xffffffff, 0xffff01a2, 0xffffffff, 0x1a2ffff, 0xffff01a2, 0xffff01a2, 0xffffffff, 0x1a201a2, 0x1a201a2, 0x1a201a2, 0xffffffff, 0xffff01a2, 0xffff01a2, 0x1a2ffff, 0x1a201a2, 0x1a2ffff, 0x1a201a2, 0xffff01a2, 0x1a2ffff, 0x1a2ffff, 0xffff01a2, 0x1a201a2, 0x1a201a2, 0x1a201a2, 0xffff01a2, 0x1a201a2, 0xffff01a2, 0xffff01a2, 0x1a2ffff, 0xffff01a2, 0x1a201a2, 0x1a201a2, 0x1a201a2, 0x1a2ffff, 0x1a201a2, 0x1a2ffff, 0xffffffff, 0xffff01a2, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a2ffff, 0x1a2ffff, 0xffffffff, 0x1a2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0xbffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff02c9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x67c0533, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff066e, 0xffffffff, 0xffffffff, 0x66effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x533ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff066e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x533ffff, 0x66effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x533ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff066e, 0x66e066e, 0x66effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x7ddffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff08f3, 0xffff08fd, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff066e, 0xffffffff, 0xffff066e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x66e066e, 0xffffffff, 0xffffffff, 0x53308f9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff07dd, 0xffffffff, 0xffffffff, 0x7ddffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff066e, 0x66effff, 0xffffffff, 0xffff066e, 0x66effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff066e, 0xffffffff, 0xffffffff, 0xffff066e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff066e, 0xffffffff, 0xffff066e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x66effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0xc000c, 0xc000c, 0xffff000c, 0xffffffff, 0xf1000c, 0xffff000c, 0xffffffff, 0xffffffff, 0xffff01a4, 0xcffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc000c, 0xffff02de, 0xcffff, 0xffff000c, 0xffff000c, 0xffffffff, 0x34bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3cbffff, 0xcffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc000c, 0xffff00f1, 0xffff02d1, 0xffff02df, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff040a, 0xffff040d, 0xffffffff, 0x442ffff, 0xffffffff, 0xffffffff, 0xffff04ac, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x52dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff056c, 0xffffffff, 0x589058c, 0xffff05d9, 0x400ffff, 0xffffffff, 0xffff05f6, 0x429ffff, 0xffff060e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff04ac, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff04ac, 0xffffffff, 0xffffffff, 0x666ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x67b0534, 0xffffffff, 0xffffffff, 0x549ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x688ffff, 0xffff0691, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x6a8ffff, 0xffffffff, 0xffffffff, 0x2dfffff, 0xffff06c0, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff066c, 0x67206fc, 0x7060707, 0x66cffff, 0xffffffff, 0xffffffff, 0xffff0730, 0xffffffff, 0x534ffff, 0xffff0429, 0x78d0429, 0xffffffff, 0x79effff, 0xffff07a0, 0xffff07a3, 0xffffffff, 0x7a5ffff, 0xffff066c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4acffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x53407ab, 0x66cffff, 0xffffffff, 0x75bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x7b207c3, 0x7c4ffff, 0x53407d9, 0xffff0813, 0xffffffff, 0xffffffff, 0x4acffff, 0x2df066c, 0x66c066c, 0x66cffff, 0xffff0858, 0xffff07ce, 0xffffffff, 0xffff0867, 0x534ffff, 0xffffffff, 0xffffffff, 0xffff0869, 0xffff086d, 0x876ffff, 0xffffffff, 0xffffffff, 0x67208dd, 0xffff075b, 0xffffffff, 0xffffffff, 0xffff075b, 0xffffffff, 0xffff0534, 0x8f608fa, 0xffff0549, 0x905090d, 0x85dffff, 0x911ffff, 0x916091d, 0xffff066c, 0xffffffff, 0xffff066c, 0x9200885, 0x4acffff, 0xffff0672, 0x9230925, 0x927ffff, 0xffffffff, 0x92e0949, 0x94b04ac, 0x951ffff, 0xffffffff, 0xffff0992, 0x9930998, 0x86dffff, 0x9aeffff, 0x9d0ffff, 0xffffffff, 0x66c066c, 0xffffffff, 0xffffffff, 0x53408fa, 0xffffffff, 0xffff075b, 0x9ecffff, 0xa010a02, 0xa080a09, 0xffff0534, 0xffff08f6, 0xa1a0a1f, 0x534ffff, 0xffffffff, 0xa23ffff, 0xffff02df, 0x95bffff, 0xffffffff, 0x672ffff, 0xffffffff, 0xffff066c, 0x66cffff, 0x999099b, 0xffff066c, 0x66cffff, 0xa450a46, 0xffff0a54, 0xa5b04ac, 0xffffffff, 0xffff066c, 0xffffffff, 0x672ffff, 0xffff066c, 0xffff09e2, 0xffffffff, 0xa6b08f6, 0xa7a0a7f, 0xa8b0a90, 0xffffffff, 0xffff0a9f, 0xffff0aa8, 0xaa9ffff, 0xab104ac, 0xab9ffff, 0xabb0abc, 0xffff0ac2, 0xac6ffff, 0x672ffff, 0xffff0ad0, 0xad40ad9, 0xada0ade, 0xa48099b, 0xffff0885, 0xae30aeb, 0xffff0aee, 0xffff0af5, 0xaf6ffff, 0xffff0af8, 0x2df066c, 0xafe0aff, 0x2df066c, 0xb08ffff, 0xb0d0b0e, 0xb0fffff, 0xffffffff, 0xffff0b11, 0xb12ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2dfffff, 0x2dfffff, 0xffffffff, 0x66cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0xd000d, 0xffffffff, 0xffff000d, 0xffffffff, 0xd000d, 0xffff000d, 0xffffffff, 0xffffffff, 0xffff01a5, 0xdffff, 0xffffffff, 0xffffffff, 0xffff01a5, 0xffffffff, 0xffff000d, 0xffff01a5, 0xffffffff, 0xffff000d, 0xffffffff, 0xffff01a5, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a5000d, 0x1a5ffff, 0xffffffff, 0xffff000d, 0xd000d, 0xffff01a5, 0xffffffff, 0xffffffff, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffff01a5, 0xffff01a5, 0xffff01a5, 0xffffffff, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffff01a5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a5000d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a5, 0xffffffff, 0x1a501a5, 0xffff01a5, 0x1a5ffff, 0xffffffff, 0xffff01a5, 0x1a5ffff, 0xffff01a5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a5, 0xffffffff, 0xffffffff, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a501a5, 0xffffffff, 0x1a5ffff, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a5ffff, 0xffff01a5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a5ffff, 0xffffffff, 0xffffffff, 0x1a5ffff, 0xffff01a5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a5, 0x1a501a5, 0x1a501a5, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffff01a5, 0xffffffff, 0x1a5ffff, 0xffff01a5, 0x1a501a5, 0xffffffff, 0x1a5ffff, 0xffff01a5, 0xffff01a5, 0xffffffff, 0x1a5ffff, 0xffff01a5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a501a5, 0x1a5ffff, 0xffffffff, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a501a5, 0x1a5ffff, 0x1a501a5, 0xffff01a5, 0xffffffff, 0xffffffff, 0x1a5ffff, 0x1a501a5, 0x1a501a5, 0x1a5ffff, 0xffff01a5, 0xffff01a5, 0xffffffff, 0xffff01a5, 0x1a5ffff, 0xffff000d, 0xffffffff, 0xffff01a5, 0xffff01a5, 0x1a5ffff, 0xffffffff, 0xffffffff, 0x1a501a5, 0xffff01a5, 0xffffffff, 0xffffffff, 0xffff01a5, 0xffffffff, 0xffff01a5, 0x1a501a5, 0xffff01a5, 0x1a501a5, 0x1a5ffff, 0x1a5ffff, 0x1a501a5, 0xffff01a5, 0xffffffff, 0xffff01a5, 0x1a501a5, 0x1a5ffff, 0xffff01a5, 0x1a501a5, 0x1a5ffff, 0xffffffff, 0x1a501a5, 0x1a501a5, 0x1a5000d, 0xffffffff, 0xffff01a5, 0x1a501a5, 0x1a5ffff, 0x1a5ffff, 0x1a5ffff, 0xffffffff, 0x1a501a5, 0xffffffff, 0xffffffff, 0x1a501a5, 0xffffffff, 0xffff01a5, 0x1a5ffff, 0x1a501a5, 0x1a501a5, 0xffff01a5, 0xffff01a5, 0x1a501a5, 0x1a5ffff, 0xffffffff, 0x1a5ffff, 0xffff01a5, 0x1a5ffff, 0xffffffff, 0x1a5ffff, 0xffffffff, 0xffff01a5, 0x1a5ffff, 0x1a501a5, 0xffff01a5, 0x1a5ffff, 0x1a501a5, 0xffff01a5, 0x1a501a5, 0xffffffff, 0xffff01a5, 0xffffffff, 0x1a5ffff, 0xffff01a5, 0xffff01a5, 0xffffffff, 0x1a501a5, 0x1a501a5, 0x1a501a5, 0xffffffff, 0xffff01a5, 0xffff01a5, 0x1a5ffff, 0x1a501a5, 0x1a5ffff, 0x1a501a5, 0xffff01a5, 0x1a5ffff, 0x1a5ffff, 0xffff01a5, 0x1a501a5, 0x1a501a5, 0x1a501a5, 0xd01a5, 0x1a501a5, 0xffff01a5, 0xffff01a5, 0x1a5ffff, 0xd01a5, 0x1a501a5, 0x1a501a5, 0x1a501a5, 0x1a5ffff, 0x1a501a5, 0x1a5ffff, 0xffffffff, 0xffff01a5, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a5ffff, 0x1a5ffff, 0xffffffff, 0x1a5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0xe000e, 0x5350535, 0x8fe000e, 0xffffffff, 0xe000e, 0xffff000e, 0xffffffff, 0xffffffff, 0xffffffff, 0xeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff000e, 0xffffffff, 0xffffffff, 0xffff000e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff000e, 0xffffffff, 0xffffffff, 0xffff000e, 0xe000e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff000e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0535, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff066f, 0xffffffff, 0xffffffff, 0x67effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x535ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff06db, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x535ffff, 0x73cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x535ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff07ad, 0x7b007b6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x535ffff, 0xffff000e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff08fe, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0873, 0xffffffff, 0xffff087e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff000e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x8de08df, 0xffffffff, 0xffffffff, 0x535ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x535ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff098f, 0xffffffff, 0xffffffff, 0xffff099f, 0x9a0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff09c2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xeffff, 0xffff0a7c, 0xffffffff, 0xffff0a81, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xadbffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0xf01a6, 0x1a6000f, 0xffffffff, 0xf000f, 0xf2ffff, 0xffffffff, 0xfffff, 0xf000f, 0xffff01a6, 0xf000f, 0xffffffff, 0xf000f, 0xf01a6, 0x1a6000f, 0xfffff, 0xf01a6, 0xffff01a6, 0x1a6ffff, 0x1a6ffff, 0xffff01a6, 0x1a6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a6ffff, 0xffff01a6, 0xffff000f, 0x1a6ffff, 0x1a6ffff, 0x1a6ffff, 0xffffffff, 0xf00f2, 0x2ccffff, 0xffff01a6, 0xffff000f, 0xffffffff, 0x1a6ffff, 0xffff01a6, 0xffffffff, 0xffff01a6, 0xffff01a6, 0xffff01a6, 0xffffffff, 0x1a6ffff, 0x1a6ffff, 0xffffffff, 0xffff01a6, 0xffff01a6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a6ffff, 0xfffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a6, 0xffffffff, 0x1a601a6, 0xffff01a6, 0x1a6ffff, 0xffffffff, 0xffff01a6, 0x1a6ffff, 0xffff01a6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a6, 0xffffffff, 0xffffffff, 0x1a6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a601a6, 0xffffffff, 0x1a6ffff, 0x1a6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a6ffff, 0xffff01a6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a6ffff, 0xffffffff, 0xffffffff, 0x1a6ffff, 0xffff01a6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a6, 0x1a601a6, 0x1a601a6, 0x1a6ffff, 0xffffffff, 0xffffffff, 0xffff01a6, 0xffffffff, 0x1a6ffff, 0xffff01a6, 0x1a601a6, 0xffffffff, 0x1a6ffff, 0xffff01a6, 0xffff01a6, 0xffffffff, 0x1a6ffff, 0xffff01a6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a601a6, 0x1a601a6, 0xffffffff, 0x1a6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a601a6, 0x1a6ffff, 0x1a601a6, 0xffff01a6, 0xffffffff, 0xffffffff, 0x1a6ffff, 0x1a601a6, 0x1a601a6, 0x1a6ffff, 0xffff01a6, 0xffff01a6, 0xffffffff, 0xffff01a6, 0x1a6ffff, 0xffffffff, 0xffffffff, 0xffff01a6, 0xffff01a6, 0x1a6ffff, 0xffffffff, 0xffffffff, 0x1a601a6, 0xffff01a6, 0xffffffff, 0xffffffff, 0xffff01a6, 0xffffffff, 0xffff01a6, 0x1a601a6, 0xffff01a6, 0x1a601a6, 0x1a6ffff, 0x1a6ffff, 0x1a601a6, 0xffff01a6, 0xffffffff, 0xffff01a6, 0x1a601a6, 0x1a6ffff, 0xffff01a6, 0x1a601a6, 0x1a6ffff, 0xffffffff, 0x1a601a6, 0x1a601a6, 0x1a6ffff, 0xffffffff, 0xffff01a6, 0x1a601a6, 0x1a6ffff, 0x1a6ffff, 0x1a6ffff, 0xffffffff, 0x1a601a6, 0xffffffff, 0xffffffff, 0x1a601a6, 0x1a6ffff, 0x1a601a6, 0x1a6ffff, 0x1a601a6, 0x1a601a6, 0xffff01a6, 0xffff01a6, 0x1a601a6, 0x1a6ffff, 0xffffffff, 0x1a6ffff, 0xffff01a6, 0x1a6ffff, 0xffffffff, 0x1a6ffff, 0xffffffff, 0xffff01a6, 0x1a6ffff, 0x1a601a6, 0xffff01a6, 0x1a6ffff, 0x1a601a6, 0xffff01a6, 0x1a601a6, 0xffffffff, 0xffff01a6, 0xffffffff, 0x1a6ffff, 0xffff01a6, 0xffff01a6, 0x1a6ffff, 0x1a601a6, 0x1a601a6, 0x1a601a6, 0xffffffff, 0xffff01a6, 0xffff01a6, 0x1a6ffff, 0x1a601a6, 0x1a6ffff, 0x1a601a6, 0xffff01a6, 0x1a6ffff, 0x1a6ffff, 0xffff01a6, 0x1a601a6, 0x1a601a6, 0x1a601a6, 0xffff01a6, 0x1a601a6, 0xffff01a6, 0xffff01a6, 0x1a6ffff, 0xffff01a6, 0x1a601a6, 0x1a601a6, 0x1a601a6, 0x1a6ffff, 0x1a601a6, 0x1a6ffff, 0xffffffff, 0xffff01a6, 0x1a6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a6ffff, 0x1a6ffff, 0xffffffff, 0x1a6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x100010, 0xffffffff, 0xffff0010, 0xffffffff, 0x100010, 0xffff0010, 0xffffffff, 0xffffffff, 0xffff01a7, 0x10ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0010, 0xffff01a7, 0xffffffff, 0xffff0010, 0xffffffff, 0xffffffff, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a70010, 0xffffffff, 0xffffffff, 0xffff0010, 0x100010, 0xffff01a7, 0xffffffff, 0xffffffff, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffff01a7, 0xffff01a7, 0xffff01a7, 0xffffffff, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffff01a7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a70010, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a7, 0xffffffff, 0x1a701a7, 0xffff01a7, 0x1a7ffff, 0xffffffff, 0xffff01a7, 0x1a7ffff, 0xffff01a7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a7, 0xffffffff, 0xffffffff, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a701a7, 0xffffffff, 0x1a7ffff, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a7ffff, 0xffff01a7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a7ffff, 0xffffffff, 0xffffffff, 0x1a7ffff, 0xffff01a7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a7, 0x1a701a7, 0x1a701a7, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffff01a7, 0xffffffff, 0x1a7ffff, 0xffff01a7, 0x1a701a7, 0xffffffff, 0x1a7ffff, 0xffff01a7, 0xffff01a7, 0xffffffff, 0x1a7ffff, 0xffff01a7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a701a7, 0x1a7ffff, 0xffffffff, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a701a7, 0x1a7ffff, 0x1a701a7, 0xffff01a7, 0xffffffff, 0xffffffff, 0x1a7ffff, 0x1a701a7, 0x1a701a7, 0x1a7ffff, 0xffff01a7, 0xffff01a7, 0xffffffff, 0xffff01a7, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffff01a7, 0xffff01a7, 0x1a7ffff, 0xffffffff, 0xffffffff, 0x1a701a7, 0xffff01a7, 0xffffffff, 0xffffffff, 0xffff01a7, 0xffffffff, 0xffff01a7, 0x1a701a7, 0xffff01a7, 0x1a701a7, 0x1a7ffff, 0x1a7ffff, 0x1a701a7, 0xffff01a7, 0xffffffff, 0xffff01a7, 0x1a701a7, 0x1a7ffff, 0xffff01a7, 0x1a701a7, 0x1a7ffff, 0xffffffff, 0x1a701a7, 0x1a701a7, 0x1a7ffff, 0xffffffff, 0xffff01a7, 0x1a701a7, 0x1a7ffff, 0x1a7ffff, 0x1a7ffff, 0xffffffff, 0x1a701a7, 0xffffffff, 0xffffffff, 0x1a701a7, 0xffffffff, 0xffff01a7, 0x1a7ffff, 0x1a701a7, 0x1a701a7, 0xffff01a7, 0xffff01a7, 0x1a701a7, 0x1a7ffff, 0xffffffff, 0x1a7ffff, 0xffff01a7, 0x1a7ffff, 0xffffffff, 0x1a7ffff, 0xffffffff, 0xffff01a7, 0x1a7ffff, 0x1a701a7, 0xffff01a7, 0x1a7ffff, 0x1a701a7, 0xffff01a7, 0x1a701a7, 0xffffffff, 0xffff01a7, 0xffffffff, 0x1a7ffff, 0xffff01a7, 0xffff01a7, 0xffffffff, 0x1a701a7, 0x1a701a7, 0x1a701a7, 0xffffffff, 0xffff01a7, 0xffff01a7, 0x1a7ffff, 0x1a701a7, 0x1a7ffff, 0x1a701a7, 0xffff01a7, 0x1a7ffff, 0x1a7ffff, 0xffff01a7, 0x1a701a7, 0x1a701a7, 0x1a701a7, 0xffff01a7, 0x1a701a7, 0xffff01a7, 0xffff01a7, 0x1a7ffff, 0xffff01a7, 0x1a701a7, 0x1a701a7, 0x1a701a7, 0x1a7ffff, 0x1a701a7, 0x1a7ffff, 0xffffffff, 0xffff01a7, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a7ffff, 0x1a7ffff, 0xffffffff, 0x1a7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x110011, 0x110011, 0xffff0011, 0xffffffff, 0xf40011, 0xffff0011, 0xffffffff, 0xffffffff, 0xffff01a8, 0x11ffff, 0xffffffff, 0xffffffff, 0xffff01a8, 0x1a8ffff, 0x110011, 0xffff01a8, 0x11ffff, 0xffff0011, 0x1a80011, 0xffff01a8, 0x1a8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a8ffff, 0x1101a8, 0xffffffff, 0xffffffff, 0x1a800f4, 0x1a8ffff, 0x110011, 0xffff00f4, 0xf400f4, 0xffff01a8, 0xffffffff, 0xffffffff, 0x1a8ffff, 0xffffffff, 0xffffffff, 0xffff01a8, 0xffff01a8, 0xffff01a8, 0xffffffff, 0x1a8ffff, 0x1a8ffff, 0xffffffff, 0xffff01a8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a800f4, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a8, 0xffffffff, 0x1a801a8, 0xffff01a8, 0x1a8ffff, 0xffffffff, 0xffff01a8, 0x1a8ffff, 0xffff01a8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a8, 0xffffffff, 0xffffffff, 0x1a8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a801a8, 0xffffffff, 0x1a8ffff, 0x1a8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a8ffff, 0xffff01a8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a8ffff, 0xffffffff, 0xffffffff, 0x1a8ffff, 0xffff01a8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a8, 0x1a801a8, 0x1a801a8, 0x1a8ffff, 0xffffffff, 0xffffffff, 0xffff01a8, 0xffffffff, 0x1a8ffff, 0xffff01a8, 0x1a801a8, 0xffffffff, 0x1a8ffff, 0xffff01a8, 0xffff01a8, 0xffffffff, 0x1a8ffff, 0xffff01a8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a801a8, 0x1a8ffff, 0xffffffff, 0x1a8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a801a8, 0x1a8ffff, 0x1a801a8, 0xffff01a8, 0xffffffff, 0xffffffff, 0x1a8ffff, 0x1a801a8, 0x1a801a8, 0x1a8ffff, 0xffff01a8, 0xffff01a8, 0xffffffff, 0xffff01a8, 0x1a8ffff, 0xffffffff, 0xffffffff, 0xffff01a8, 0xffff01a8, 0x1a8ffff, 0xffffffff, 0xffffffff, 0x1a801a8, 0xffff01a8, 0xffffffff, 0xffffffff, 0xffff01a8, 0xffffffff, 0xffff01a8, 0x1a801a8, 0xffff01a8, 0x1a801a8, 0x1a8ffff, 0x1a8ffff, 0x1a801a8, 0xffff01a8, 0xffffffff, 0xffff01a8, 0x1a801a8, 0x1a8ffff, 0xffff01a8, 0x1a801a8, 0x1a8ffff, 0xffffffff, 0x1a801a8, 0x1a801a8, 0x1a8ffff, 0xffffffff, 0xffff01a8, 0x1a801a8, 0x1a8ffff, 0x1a8ffff, 0x1a8ffff, 0xffffffff, 0x1a801a8, 0xffffffff, 0xffffffff, 0x1a801a8, 0xffffffff, 0xffff01a8, 0x1a8ffff, 0x1a801a8, 0x1a801a8, 0xffff01a8, 0xffff01a8, 0x1a801a8, 0x1a8ffff, 0xffffffff, 0x1a8ffff, 0xffff01a8, 0x1a8ffff, 0xffffffff, 0x1a8ffff, 0xffffffff, 0xffff01a8, 0x1a8ffff, 0x1a801a8, 0xffff01a8, 0x1a8ffff, 0x1a801a8, 0xffff01a8, 0x1a801a8, 0xffffffff, 0xffff01a8, 0xffffffff, 0x1a8ffff, 0xffff01a8, 0xffff01a8, 0xffffffff, 0x1a801a8, 0x1a801a8, 0x1a801a8, 0xffffffff, 0xffff01a8, 0xffff01a8, 0x1a8ffff, 0x1a801a8, 0x1a8ffff, 0x1a801a8, 0xffff01a8, 0x1a8ffff, 0x1a8ffff, 0xffff01a8, 0x1a801a8, 0x1a801a8, 0x1a801a8, 0xffff01a8, 0x1a801a8, 0xffff01a8, 0xffff01a8, 0x1a8ffff, 0xffff01a8, 0x1a801a8, 0x1a801a8, 0x1a801a8, 0x1a8ffff, 0x1a801a8, 0x1a8ffff, 0xffffffff, 0xffff01a8, 0x1a8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a8ffff, 0x1a8ffff, 0xffffffff, 0x1a8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x120012, 0xaf00b7, 0xffff0012, 0xffffffff, 0xffff0012, 0xffff0012, 0xffffffff, 0xffffffff, 0xffffffff, 0x12ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x27c0012, 0xffffffff, 0x550ffff, 0xffff0012, 0xffff0550, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xafffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2b902b9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0536, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x536ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x536ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x536ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x536ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff08f4, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x536ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff08f4, 0xffffffff, 0xffffffff, 0x536ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x130013, 0xffffffff, 0xffff0013, 0xffffffff, 0x130013, 0xffff0013, 0x111ffff, 0xffffffff, 0xffff01a9, 0x13ffff, 0xffffffff, 0xffffffff, 0xffff01a9, 0x1a9ffff, 0xffff0013, 0xffff01a9, 0xffff01a9, 0x1a90013, 0x1a9ffff, 0xffff01a9, 0x1a9ffff, 0xffffffff, 0xffffffff, 0x261ffff, 0xffffffff, 0x1a9ffff, 0xffff01a9, 0xffffffff, 0x1a9ffff, 0x1a90013, 0x1a9ffff, 0xffffffff, 0xffff0013, 0x130013, 0xffff01a9, 0xffffffff, 0xffffffff, 0x1a90111, 0xffff01a9, 0xffffffff, 0xffff01a9, 0xffff01a9, 0xffff01a9, 0xffffffff, 0x1a9ffff, 0x1a9ffff, 0xffffffff, 0xffff01a9, 0xffff01a9, 0xffff0261, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0261, 0x1a90013, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a9, 0xffffffff, 0x1a901a9, 0xffff01a9, 0x1a9ffff, 0xffffffff, 0xffff01a9, 0x1a9ffff, 0xffff01a9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0261, 0xffff01a9, 0xffffffff, 0xffffffff, 0x1a9ffff, 0x261ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a901a9, 0xffffffff, 0x1a9ffff, 0x1a9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a9ffff, 0xffff01a9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a9ffff, 0xffffffff, 0xffffffff, 0x1a9ffff, 0x26101a9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0261, 0xffffffff, 0xffff0261, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01a9, 0x1a901a9, 0x1a901a9, 0x1a9ffff, 0xffffffff, 0xffffffff, 0xffff01a9, 0xffffffff, 0x1a9ffff, 0xffff01a9, 0x1a901a9, 0xffffffff, 0x1a9ffff, 0xffff01a9, 0xffff01a9, 0xffffffff, 0x1a9ffff, 0xffff01a9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a9ffff, 0xffff0261, 0x261ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a901a9, 0x1a9ffff, 0x2610261, 0x1a9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a901a9, 0x1a9ffff, 0x1a901a9, 0xffff01a9, 0xffffffff, 0xffffffff, 0x1a9ffff, 0x1a901a9, 0x1a901a9, 0x1a9ffff, 0xffff01a9, 0xffff01a9, 0xffffffff, 0xffff01a9, 0x1a9ffff, 0xffff0013, 0xffffffff, 0xffff01a9, 0xffff01a9, 0x1a9ffff, 0xffff0261, 0xffffffff, 0x1a901a9, 0xffff01a9, 0xffffffff, 0xffffffff, 0x26101a9, 0xffffffff, 0xffff01a9, 0x1a901a9, 0xffff01a9, 0x1a901a9, 0x1a9ffff, 0x1a9ffff, 0x1a901a9, 0xffff01a9, 0xffffffff, 0xffff01a9, 0x1a901a9, 0x1a9ffff, 0xffff01a9, 0x1a901a9, 0x1a9ffff, 0xffffffff, 0x1a901a9, 0x1a901a9, 0x1a90013, 0xffffffff, 0xffff01a9, 0x1a901a9, 0x1a90261, 0x1a9ffff, 0x1a9ffff, 0xffff0261, 0x1a901a9, 0xffff0261, 0x261ffff, 0x1a901a9, 0xffffffff, 0xffff01a9, 0x1a9ffff, 0x1a901a9, 0x1a901a9, 0xffff01a9, 0xffff01a9, 0x1a901a9, 0x1a9ffff, 0xffffffff, 0x1a9ffff, 0xffff01a9, 0x1a90261, 0x261ffff, 0x1a9ffff, 0xffffffff, 0xffff01a9, 0x1a9ffff, 0x1a901a9, 0xffff01a9, 0x1a9ffff, 0x1a901a9, 0xffff01a9, 0x1a901a9, 0x261ffff, 0xffff01a9, 0xffffffff, 0x1a9ffff, 0xffff01a9, 0xffff01a9, 0xffffffff, 0x1a901a9, 0x1a901a9, 0x1a901a9, 0xffffffff, 0xffff01a9, 0xffff01a9, 0x1a9ffff, 0x1a901a9, 0x1a9ffff, 0x1a901a9, 0xffff01a9, 0x1a9ffff, 0x1a9ffff, 0xffff01a9, 0x1a901a9, 0x1a901a9, 0x1a901a9, 0x1301a9, 0x1a901a9, 0xffff01a9, 0xffff01a9, 0x1a9ffff, 0x1301a9, 0x1a901a9, 0x1a901a9, 0x1a901a9, 0x1a9ffff, 0x1a901a9, 0x1a9ffff, 0xffffffff, 0xffff01a9, 0x1a9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a9ffff, 0x1a9ffff, 0xffffffff, 0x1a9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x140014, 0x140014, 0xffff0014, 0xffffffff, 0xffff0014, 0xffff0014, 0xffffffff, 0xffffffff, 0xffffffff, 0x14ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x140014, 0xffffffff, 0x14ffff, 0xffff0014, 0xffff0014, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x14ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x140014, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0537, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x537ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x537ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x537ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x537ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x150015, 0xffffffff, 0xffff0015, 0xffffffff, 0x150015, 0xffff0015, 0xffffffff, 0xffffffff, 0xffff01aa, 0x15ffff, 0xffffffff, 0xffffffff, 0xffff01aa, 0x1aaffff, 0xffff0015, 0xffff01aa, 0xffff01aa, 0xffff0015, 0x1aaffff, 0xffff01aa, 0x1aaffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aaffff, 0xffff01aa, 0xffffffff, 0x1aaffff, 0x1aa0015, 0x1aaffff, 0xffffffff, 0xffff0015, 0x150015, 0xffff01aa, 0xffffffff, 0xffffffff, 0x1aaffff, 0xffff01aa, 0xffffffff, 0xffff01aa, 0xffff01aa, 0xffff01aa, 0xffffffff, 0x1aaffff, 0x1aaffff, 0xffffffff, 0xffff01aa, 0xffff01aa, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aa0015, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01aa, 0xffffffff, 0x1aa01aa, 0xffff01aa, 0x1aaffff, 0xffffffff, 0xffff01aa, 0x1aaffff, 0xffff01aa, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01aa, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01aa, 0xffffffff, 0xffffffff, 0x1aaffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aa01aa, 0xffffffff, 0x1aaffff, 0x1aaffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aaffff, 0xffff01aa, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aaffff, 0xffffffff, 0xffffffff, 0x1aaffff, 0xffff01aa, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01aa, 0x1aa01aa, 0x1aa01aa, 0x1aaffff, 0xffffffff, 0xffffffff, 0xffff01aa, 0xffffffff, 0x1aaffff, 0xffff01aa, 0x1aa01aa, 0xffffffff, 0x1aaffff, 0xffff01aa, 0xffff01aa, 0xffffffff, 0x1aaffff, 0xffff01aa, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aaffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aa01aa, 0x1aaffff, 0xffffffff, 0x1aaffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aa01aa, 0x1aaffff, 0x1aa01aa, 0xffff01aa, 0xffffffff, 0xffffffff, 0x1aaffff, 0x1aa01aa, 0x1aa01aa, 0x1aaffff, 0xffff01aa, 0xffff01aa, 0xffffffff, 0xffff01aa, 0x1aaffff, 0xffff0015, 0xffffffff, 0xffff01aa, 0xffff01aa, 0x1aaffff, 0xffffffff, 0xffffffff, 0x1aa01aa, 0xffff01aa, 0xffffffff, 0xffffffff, 0xffff01aa, 0xffffffff, 0xffff01aa, 0x1aa01aa, 0xffff01aa, 0x1aa01aa, 0x1aaffff, 0x1aaffff, 0x1aa01aa, 0xffff01aa, 0xffffffff, 0xffff01aa, 0x1aa01aa, 0x1aaffff, 0xffff01aa, 0x1aa01aa, 0x1aaffff, 0xffffffff, 0x1aa01aa, 0x1aa01aa, 0x1aa0015, 0xffffffff, 0xffff01aa, 0x1aa01aa, 0x1aaffff, 0x1aaffff, 0x1aaffff, 0xffffffff, 0x1aa01aa, 0xffffffff, 0xffffffff, 0x1aa01aa, 0xffffffff, 0xffff01aa, 0x1aaffff, 0x1aa01aa, 0x1aa01aa, 0xffff01aa, 0xffff01aa, 0x1aa01aa, 0x1aaffff, 0xffffffff, 0x1aaffff, 0xffff01aa, 0x1aaffff, 0xffffffff, 0x1aaffff, 0xffffffff, 0xffff01aa, 0x1aaffff, 0x1aa01aa, 0xffff01aa, 0x1aaffff, 0x1aa01aa, 0xffff01aa, 0x1aa01aa, 0xffffffff, 0xffff01aa, 0xffffffff, 0x1aaffff, 0xffff01aa, 0xffff01aa, 0xffffffff, 0x1aa01aa, 0x1aa01aa, 0x1aa01aa, 0xffffffff, 0xffff01aa, 0xffff01aa, 0x1aaffff, 0x1aa01aa, 0x1aaffff, 0x1aa01aa, 0xffff01aa, 0x1aaffff, 0x1aaffff, 0xffff01aa, 0x1aa01aa, 0x1aa01aa, 0x1aa01aa, 0x1501aa, 0x1aa01aa, 0xffff01aa, 0xffff01aa, 0x1aaffff, 0x1501aa, 0x1aa01aa, 0x1aa01aa, 0x1aa01aa, 0x1aaffff, 0x1aa01aa, 0x1aaffff, 0xffffffff, 0xffff01aa, 0x1aaffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aaffff, 0x1aaffff, 0xffffffff, 0x1aaffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x160016, 0x160016, 0xffff0016, 0xffffffff, 0xffff0016, 0xffff0016, 0xffffffff, 0xffffffff, 0xffff01ab, 0x16ffff, 0xffffffff, 0xffffffff, 0xffff01ab, 0x1abffff, 0x160016, 0xffff01ab, 0x1601ab, 0x1ab0016, 0x1ab0016, 0xffff01ab, 0x1abffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1abffff, 0x1601ab, 0xffffffff, 0x1abffff, 0x1abffff, 0x1abffff, 0x160016, 0xffffffff, 0xffffffff, 0xffff01ab, 0x2f0ffff, 0xffffffff, 0x1abffff, 0xffff01ab, 0xffffffff, 0xffff01ab, 0xffff01ab, 0xffff01ab, 0xffffffff, 0x1abffff, 0x1abffff, 0xffffffff, 0xffff01ab, 0xffff01ab, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1abffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ab, 0xffffffff, 0x1ab01ab, 0xffff01ab, 0x1abffff, 0xffffffff, 0xffff01ab, 0x1abffff, 0xffff01ab, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ab, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ab, 0xffffffff, 0xffffffff, 0x1abffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ab01ab, 0xffffffff, 0x1abffff, 0x1abffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1abffff, 0xffff01ab, 0xffffffff, 0xffffffff, 0xffff02f0, 0xffffffff, 0x1abffff, 0xffffffff, 0xffffffff, 0x1ab02f0, 0xffff01ab, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff02f0, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ab, 0x1ab01ab, 0x1ab01ab, 0x1abffff, 0xffffffff, 0xffffffff, 0xffff01ab, 0xffffffff, 0x1abffff, 0xffff01ab, 0x1ab01ab, 0xffffffff, 0x1abffff, 0xffff01ab, 0xffff01ab, 0xffffffff, 0x1abffff, 0xffff01ab, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1abffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ab01ab, 0x1abffff, 0xffffffff, 0x1abffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ab01ab, 0x1abffff, 0x1ab01ab, 0xffff01ab, 0xffffffff, 0xffffffff, 0x1abffff, 0x1ab01ab, 0x1ab01ab, 0x1ab02f0, 0xffff01ab, 0xffff01ab, 0xffff02f0, 0xffff01ab, 0x1abffff, 0xffffffff, 0xffffffff, 0xffff01ab, 0xffff01ab, 0x1abffff, 0xffffffff, 0xffffffff, 0x1ab01ab, 0xffff01ab, 0xffffffff, 0xffffffff, 0xffff01ab, 0xffff02f0, 0xffff01ab, 0x1ab01ab, 0xffff01ab, 0x1ab01ab, 0x1abffff, 0x1abffff, 0x1ab01ab, 0x2f001ab, 0xffffffff, 0xffff01ab, 0x1ab01ab, 0x1abffff, 0xffff01ab, 0x1ab01ab, 0x1abffff, 0xffffffff, 0x1ab01ab, 0x1ab01ab, 0x1abffff, 0xffffffff, 0xffff01ab, 0x1ab01ab, 0x1abffff, 0x1abffff, 0x1abffff, 0xffffffff, 0x1ab01ab, 0xffffffff, 0xffffffff, 0x1ab01ab, 0xffffffff, 0xffff01ab, 0x1abffff, 0x1ab01ab, 0x1ab01ab, 0xffff01ab, 0xffff01ab, 0x1ab01ab, 0x1abffff, 0xffffffff, 0x1abffff, 0xffff01ab, 0x1abffff, 0xffffffff, 0x1abffff, 0xffffffff, 0xffff01ab, 0x1abffff, 0x1ab01ab, 0xffff01ab, 0x1abffff, 0x1ab01ab, 0xffff01ab, 0x1ab01ab, 0xffffffff, 0xffff01ab, 0xffffffff, 0x1abffff, 0xffff01ab, 0xffff01ab, 0xffffffff, 0x1ab01ab, 0x1ab01ab, 0x1ab01ab, 0xffffffff, 0xffff01ab, 0xffff01ab, 0x1abffff, 0x1ab01ab, 0x1abffff, 0x1ab01ab, 0xffff01ab, 0x1abffff, 0x1abffff, 0xffff01ab, 0x1ab01ab, 0x1ab01ab, 0x1ab01ab, 0xffff01ab, 0x1ab01ab, 0xffff01ab, 0xffff01ab, 0x1abffff, 0xffff01ab, 0x1ab01ab, 0x1ab01ab, 0x1ab01ab, 0x1abffff, 0x1ab01ab, 0x1abffff, 0xffffffff, 0xffff01ab, 0x1abffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1abffff, 0x1abffff, 0xffffffff, 0x1abffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x170017, 0xffffffff, 0xffff0017, 0xffffffff, 0x170017, 0xffff0017, 0xffffffff, 0xffffffff, 0xffff01ac, 0x17ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0017, 0xffff01ac, 0xffffffff, 0xffff0017, 0xffffffff, 0xffffffff, 0x1acffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1acffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0017, 0xffffffff, 0xffffffff, 0xffff0017, 0x170017, 0xffff01ac, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ac, 0xffff01ac, 0xffffffff, 0x1acffff, 0xffffffff, 0xffffffff, 0xffff01ac, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ac0017, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ac, 0xffffffff, 0x1ac01ac, 0xffff01ac, 0x1acffff, 0xffffffff, 0xffff01ac, 0x1acffff, 0xffff01ac, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ac, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ac, 0xffffffff, 0xffffffff, 0x1acffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ac01ac, 0xffffffff, 0x67fffff, 0x1acffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1acffff, 0xffff01ac, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1acffff, 0xffffffff, 0xffffffff, 0x1acffff, 0xffff01ac, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ac, 0x1ac01ac, 0x1ac01ac, 0x1acffff, 0xffffffff, 0xffffffff, 0xffff01ac, 0xffffffff, 0x1acffff, 0xffff01ac, 0x1ac01ac, 0xffffffff, 0x1acffff, 0xffff01ac, 0xffff01ac, 0xffffffff, 0x1acffff, 0xffff01ac, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1acffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ac01ac, 0x1acffff, 0xffffffff, 0x1acffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ac01ac, 0x1acffff, 0x1ac01ac, 0xffff01ac, 0xffffffff, 0xffffffff, 0x1acffff, 0x1ac01ac, 0x1ac01ac, 0x1acffff, 0xffff01ac, 0xffff01ac, 0xffffffff, 0xffff01ac, 0x1acffff, 0xffff0017, 0xffffffff, 0xffff01ac, 0xffff01ac, 0x1acffff, 0xffffffff, 0xffffffff, 0x1ac01ac, 0xffff01ac, 0xffffffff, 0xffffffff, 0xffff01ac, 0xffffffff, 0xffff01ac, 0x1ac01ac, 0xffff01ac, 0x1ac01ac, 0x1acffff, 0x1acffff, 0x1ac01ac, 0xffff01ac, 0xffffffff, 0xffff01ac, 0x1ac01ac, 0x1acffff, 0xffff01ac, 0x1ac01ac, 0x1acffff, 0xffffffff, 0x1ac01ac, 0x1ac01ac, 0x1ac0017, 0xffffffff, 0xffff01ac, 0x1ac01ac, 0x1acffff, 0x1acffff, 0x1acffff, 0xffffffff, 0x1ac01ac, 0xffffffff, 0xffffffff, 0x1ac01ac, 0xffffffff, 0xffff01ac, 0x1acffff, 0x1ac01ac, 0x1ac01ac, 0xffff01ac, 0xffff01ac, 0x1ac01ac, 0x1acffff, 0xffffffff, 0x1acffff, 0xffff01ac, 0x1acffff, 0xffffffff, 0x1acffff, 0xffffffff, 0xffff01ac, 0x1acffff, 0x1ac01ac, 0xffff01ac, 0x1acffff, 0x1ac01ac, 0xffff01ac, 0x1ac01ac, 0xffffffff, 0xffff01ac, 0xffffffff, 0x1acffff, 0xffff01ac, 0xffff01ac, 0xffffffff, 0x1ac01ac, 0x1ac01ac, 0x1ac01ac, 0xffffffff, 0xffff01ac, 0xffff01ac, 0x1acffff, 0x1ac01ac, 0x1acffff, 0x1ac01ac, 0xffff01ac, 0x1acffff, 0x1acffff, 0xffff01ac, 0x1ac01ac, 0x1ac01ac, 0x1ac01ac, 0x1701ac, 0x1ac01ac, 0xffff01ac, 0xffff01ac, 0x1acffff, 0x1701ac, 0x1ac01ac, 0x1ac01ac, 0x1ac01ac, 0x1acffff, 0x1ac01ac, 0x1acffff, 0xffffffff, 0xffff01ac, 0x1acffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1acffff, 0x1acffff, 0xffffffff, 0x1acffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x1800ac, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x18ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0538, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x538ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x538ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x538ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x538ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0538, 0x8f7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x538ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0538, 0xffff08f7, 0xffffffff, 0x538ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff08f7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x190019, 0xffffffff, 0xffff0019, 0xffffffff, 0x190019, 0xffff0019, 0x112ffff, 0xffffffff, 0xffff01ad, 0x190178, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0019, 0xffff01ad, 0xffffffff, 0xffff0019, 0xffffffff, 0xffffffff, 0x1adffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1adffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ad0019, 0x3dfffff, 0xffffffff, 0xffff0019, 0x190019, 0xffff01ad, 0xffffffff, 0xffffffff, 0x1ad0112, 0xffffffff, 0xffffffff, 0xffff01ad, 0xffff01ad, 0xffff01ad, 0xffffffff, 0x1adffff, 0xffffffff, 0xffffffff, 0xffff01ad, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ad0019, 0xffff0178, 0x1780178, 0xffffffff, 0xffffffff, 0xffff01ad, 0xffffffff, 0x1ad01ad, 0xffff01ad, 0x1adffff, 0xffffffff, 0xffff01ad, 0x1adffff, 0xffff01ad, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ad, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ad, 0xffffffff, 0xffffffff, 0x1adffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ad01ad, 0xffffffff, 0x1adffff, 0x1adffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1adffff, 0xffff01ad, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1adffff, 0xffffffff, 0xffffffff, 0x1adffff, 0xffff01ad, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x178ffff, 0xffffffff, 0xffff01ad, 0x1ad01ad, 0x1ad01ad, 0x1adffff, 0xffffffff, 0xffffffff, 0xffff01ad, 0xffffffff, 0x1adffff, 0xffff01ad, 0x1ad01ad, 0xffffffff, 0x1adffff, 0xffff01ad, 0xffff01ad, 0xffffffff, 0x1adffff, 0xffff01ad, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1adffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ad01ad, 0x1adffff, 0xffffffff, 0x1adffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ad01ad, 0x1adffff, 0x1ad01ad, 0xffff01ad, 0xffffffff, 0xffffffff, 0x1adffff, 0x1ad01ad, 0x1ad01ad, 0x1adffff, 0xffff01ad, 0xffff01ad, 0xffffffff, 0xffff01ad, 0x1adffff, 0xffff0019, 0xffffffff, 0xffff01ad, 0xffff01ad, 0x1adffff, 0xffffffff, 0xffffffff, 0x1ad01ad, 0xffff01ad, 0xffffffff, 0xffffffff, 0xffff01ad, 0xffffffff, 0xffff01ad, 0x1ad01ad, 0xffff01ad, 0x1ad01ad, 0x1adffff, 0x1adffff, 0x1ad01ad, 0xffff01ad, 0xffffffff, 0xffff01ad, 0x1ad01ad, 0x1adffff, 0xffff01ad, 0x1ad01ad, 0x1adffff, 0xffffffff, 0x1ad01ad, 0x1ad01ad, 0x1ad0019, 0xffffffff, 0xffff01ad, 0x1ad01ad, 0x1adffff, 0x1adffff, 0x1adffff, 0xffffffff, 0x1ad01ad, 0xffffffff, 0xffffffff, 0x1ad01ad, 0xffffffff, 0xffff01ad, 0x1adffff, 0x1ad01ad, 0x1ad01ad, 0xffff01ad, 0xffff01ad, 0x1ad01ad, 0x1adffff, 0xffffffff, 0x1adffff, 0xffff01ad, 0x1adffff, 0xffffffff, 0x1adffff, 0xffffffff, 0xffff01ad, 0x1adffff, 0x1ad01ad, 0xffff01ad, 0x1adffff, 0x1ad01ad, 0xffff01ad, 0x1ad01ad, 0xffffffff, 0xffff01ad, 0xffffffff, 0x1adffff, 0xffff01ad, 0xffff01ad, 0xffffffff, 0x1ad01ad, 0x1ad01ad, 0x1ad01ad, 0xffffffff, 0xffff01ad, 0xffff01ad, 0x1adffff, 0x1ad01ad, 0x1adffff, 0x1ad01ad, 0xffff01ad, 0x1adffff, 0x1adffff, 0xffff01ad, 0x1ad01ad, 0x1ad01ad, 0x1ad01ad, 0x1901ad, 0x1ad01ad, 0xffff01ad, 0xffff01ad, 0x1adffff, 0x1901ad, 0x1ad01ad, 0x1ad01ad, 0x1ad01ad, 0x1adffff, 0x1ad01ad, 0x1adffff, 0xffffffff, 0xffff01ad, 0x1adffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1adffff, 0x1adffff, 0xffffffff, 0x1adffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x1a001a, 0xffffffff, 0xffff001a, 0xffffffff, 0x1a001a, 0xffff001a, 0xffffffff, 0xffffffff, 0xffff01ae, 0x1affff, 0xffffffff, 0xffffffff, 0xffff03cd, 0xffffffff, 0xffff001a, 0xffff01ae, 0xffffffff, 0xffff001a, 0xffffffff, 0xffffffff, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ae001a, 0x1aeffff, 0xffffffff, 0xffff001a, 0x1a001a, 0xffff01ae, 0xffffffff, 0xffffffff, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffff01ae, 0xffff01ae, 0xffff01ae, 0xffffffff, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffff01ae, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ae001a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ae, 0xffffffff, 0x1ae01ae, 0xffff01ae, 0x1aeffff, 0xffffffff, 0xffff01ae, 0x1aeffff, 0xffff01ae, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ae, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ae, 0xffffffff, 0xffffffff, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ae01ae, 0xffffffff, 0x1aeffff, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aeffff, 0xffff01ae, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aeffff, 0xffffffff, 0xffffffff, 0x1aeffff, 0xffff01ae, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01ae, 0x1ae01ae, 0x1ae01ae, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffff01ae, 0xffffffff, 0x1aeffff, 0xffff01ae, 0x1ae01ae, 0xffffffff, 0x1aeffff, 0xffff01ae, 0xffff01ae, 0xffffffff, 0x1aeffff, 0xffff01ae, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ae01ae, 0x1aeffff, 0xffffffff, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1ae01ae, 0x1aeffff, 0x1ae01ae, 0xffff01ae, 0xffffffff, 0xffffffff, 0x1aeffff, 0x1ae01ae, 0x1ae01ae, 0x1aeffff, 0xffff01ae, 0xffff01ae, 0xffffffff, 0xffff01ae, 0x1aeffff, 0xffff088a, 0xffffffff, 0xffff01ae, 0xffff01ae, 0x1aeffff, 0xffffffff, 0xffffffff, 0x1ae01ae, 0xffff01ae, 0xffffffff, 0xffffffff, 0xffff01ae, 0xffffffff, 0xffff01ae, 0x1ae01ae, 0xffff01ae, 0x1ae01ae, 0x1aeffff, 0x1aeffff, 0x1ae01ae, 0xffff01ae, 0xffffffff, 0xffff01ae, 0x1ae01ae, 0x1aeffff, 0xffff01ae, 0x1ae01ae, 0x1aeffff, 0xffffffff, 0x1ae01ae, 0x1ae01ae, 0x1ae096b, 0xffffffff, 0xffff01ae, 0x1ae01ae, 0x1aeffff, 0x1aeffff, 0x1aeffff, 0xffffffff, 0x1ae01ae, 0xffffffff, 0xffffffff, 0x1ae01ae, 0xffffffff, 0xffff01ae, 0x1aeffff, 0x1ae01ae, 0x1ae01ae, 0xffff01ae, 0xffff01ae, 0x1ae01ae, 0x1aeffff, 0xffffffff, 0x1aeffff, 0xffff01ae, 0x1aeffff, 0xffffffff, 0x1aeffff, 0xffffffff, 0xffff01ae, 0x1aeffff, 0x1ae01ae, 0xffff01ae, 0x1aeffff, 0x1ae01ae, 0xffff01ae, 0x1ae01ae, 0xffffffff, 0xffff01ae, 0xffffffff, 0x1aeffff, 0xffff01ae, 0xffff01ae, 0xffffffff, 0x1ae01ae, 0x1ae01ae, 0x1ae01ae, 0xffffffff, 0xffff01ae, 0xffff01ae, 0x1aeffff, 0x1ae01ae, 0x1aeffff, 0x1ae01ae, 0xffff01ae, 0x1aeffff, 0x1aeffff, 0xffff01ae, 0x1ae01ae, 0x1ae01ae, 0x1ae01ae, 0x88a01ae, 0x1ae01ae, 0xffff01ae, 0xffff01ae, 0x1aeffff, 0x96b01ae, 0x1ae01ae, 0x1ae01ae, 0x1ae01ae, 0x1aeffff, 0x1ae01ae, 0x1aeffff, 0xffffffff, 0xffff01ae, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1aeffff, 0x1aeffff, 0xffffffff, 0x1aeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x1b001b, 0xffffffff, 0xffff001b, 0xffffffff, 0x1b001b, 0xffff001b, 0xffffffff, 0xffffffff, 0xffff01af, 0x1bffff, 0xffffffff, 0xffffffff, 0xffff01af, 0xffffffff, 0xffff001b, 0xffff01af, 0xffffffff, 0xffff001b, 0xffffffff, 0xffff03fd, 0x1afffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1afffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1af001b, 0x1afffff, 0xffffffff, 0xffff001b, 0x1b001b, 0xffff01af, 0xffffffff, 0xffffffff, 0x1afffff, 0xffffffff, 0xffffffff, 0xffff01af, 0xffff01af, 0xffff01af, 0xffffffff, 0x1afffff, 0xffffffff, 0xffffffff, 0xffff01af, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1af001b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01af, 0xffffffff, 0x1af01af, 0xffff01af, 0x1afffff, 0xffffffff, 0xffff01af, 0x1afffff, 0xffff01af, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01af, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01af, 0xffffffff, 0xffffffff, 0x1afffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1af01af, 0xffffffff, 0x1afffff, 0x1afffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1afffff, 0xffff01af, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1afffff, 0xffffffff, 0xffffffff, 0x1afffff, 0xffff01af, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01af, 0x1af01af, 0x1af01af, 0x1afffff, 0xffffffff, 0xffffffff, 0xffff01af, 0xffffffff, 0x1afffff, 0xffff01af, 0x1af01af, 0xffffffff, 0x1afffff, 0xffff01af, 0xffff01af, 0xffffffff, 0x1afffff, 0xffff01af, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1afffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1af01af, 0x1afffff, 0xffffffff, 0x1afffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1af01af, 0x1afffff, 0x1af01af, 0xffff01af, 0xffffffff, 0xffffffff, 0x1afffff, 0x1af01af, 0x1af01af, 0x1afffff, 0xffff01af, 0xffff01af, 0xffffffff, 0xffff01af, 0x1afffff, 0xffffffff, 0xffffffff, 0xffff01af, 0xffff01af, 0x1afffff, 0xffffffff, 0xffffffff, 0x1af01af, 0xffff01af, 0xffffffff, 0xffffffff, 0xffff01af, 0xffffffff, 0xffff01af, 0x1af01af, 0xffff01af, 0x1af01af, 0x1afffff, 0x1afffff, 0x1af01af, 0xffff01af, 0xffffffff, 0xffff01af, 0x1af01af, 0x1afffff, 0xffff01af, 0x1af01af, 0x1afffff, 0xffffffff, 0x1af01af, 0x1af01af, 0x1afffff, 0xffffffff, 0xffff01af, 0x1af01af, 0x1afffff, 0x1afffff, 0x1afffff, 0xffffffff, 0x1af01af, 0xffffffff, 0xffffffff, 0x1af01af, 0xffffffff, 0xffff01af, 0x1afffff, 0x1af01af, 0x1af01af, 0xffff01af, 0xffff01af, 0x1af01af, 0x1afffff, 0xffffffff, 0x1afffff, 0xffff01af, 0x1afffff, 0xffffffff, 0x1afffff, 0xffffffff, 0xffff01af, 0x1afffff, 0x1af01af, 0xffff01af, 0x1afffff, 0x1af01af, 0xffff01af, 0x1af01af, 0xffffffff, 0xffff01af, 0xffffffff, 0x1afffff, 0xffff01af, 0xffff01af, 0xffffffff, 0x1af01af, 0x1af01af, 0x1af01af, 0xffffffff, 0xffff01af, 0xffff01af, 0x1afffff, 0x1af01af, 0x1afffff, 0x1af01af, 0xffff01af, 0x1afffff, 0x1afffff, 0xffff01af, 0x1af01af, 0x1af01af, 0x1af01af, 0xffff01af, 0x1af01af, 0xffff01af, 0xffff01af, 0x1afffff, 0xffff01af, 0x1af01af, 0x1af01af, 0x1af01af, 0x1afffff, 0x1af01af, 0x1afffff, 0xffffffff, 0xffff01af, 0x1afffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1afffff, 0x1afffff, 0xffffffff, 0x1afffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x1c001c, 0x1c001c, 0xffff001c, 0xffffffff, 0xffff001c, 0xffff001c, 0xffffffff, 0xffffffff, 0xffff01b0, 0x1cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1c001c, 0xffff01b0, 0x1cffff, 0xffff001c, 0xffff001c, 0xffffffff, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b0ffff, 0x1cffff, 0xffffffff, 0xffffffff, 0x1b0ffff, 0xffffffff, 0x1c001c, 0xffffffff, 0xffffffff, 0xffff01b0, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff03fe, 0xffff01b0, 0xffff01b0, 0xffffffff, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffff01b0, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b0, 0xffffffff, 0x1b001b0, 0xffff01b0, 0x1b0ffff, 0xffffffff, 0xffff01b0, 0x1b0ffff, 0xffff01b0, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b0, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b0, 0xffffffff, 0xffffffff, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b001b0, 0xffffffff, 0x1b0ffff, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b0ffff, 0xffff01b0, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b0ffff, 0xffffffff, 0xffffffff, 0x1b0ffff, 0xffff01b0, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b0, 0x1b001b0, 0x1b001b0, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffff01b0, 0xffffffff, 0x1b0ffff, 0xffff01b0, 0x1b001b0, 0xffffffff, 0x1b0ffff, 0xffff01b0, 0xffff01b0, 0xffffffff, 0x1b0ffff, 0xffff01b0, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b001b0, 0x1b0ffff, 0xffffffff, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b001b0, 0x1b0ffff, 0x1b001b0, 0xffff01b0, 0xffffffff, 0xffffffff, 0x1b0ffff, 0x1b001b0, 0x1b001b0, 0x1b0ffff, 0xffff01b0, 0xffff01b0, 0xffffffff, 0xffff01b0, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffff01b0, 0xffff01b0, 0x1b0ffff, 0xffffffff, 0xffffffff, 0x1b001b0, 0xffff01b0, 0xffffffff, 0xffffffff, 0xffff01b0, 0xffffffff, 0xffff01b0, 0x1b001b0, 0xffff01b0, 0x1b001b0, 0x1b0ffff, 0x1b0ffff, 0x1b001b0, 0xffff01b0, 0xffffffff, 0xffff01b0, 0x1b001b0, 0x1b0ffff, 0xffff01b0, 0x1b001b0, 0x1b0ffff, 0xffffffff, 0x1b001b0, 0x1b001b0, 0x1b0ffff, 0xffffffff, 0xffff01b0, 0x1b001b0, 0x1b0ffff, 0x1b0ffff, 0x1b0ffff, 0xffffffff, 0x1b001b0, 0xffffffff, 0xffffffff, 0x1b001b0, 0xffffffff, 0xffff01b0, 0x1b0ffff, 0x1b001b0, 0x1b001b0, 0xffff01b0, 0xffff01b0, 0x1b001b0, 0x1b0ffff, 0xffffffff, 0x1b0ffff, 0xffff01b0, 0x1b0ffff, 0xffffffff, 0x1b0ffff, 0xffffffff, 0xffff01b0, 0x1b0ffff, 0x1b001b0, 0xffff01b0, 0x1b0ffff, 0x1b001b0, 0xffff01b0, 0x1b001b0, 0xffffffff, 0xffff01b0, 0xffffffff, 0x1b0ffff, 0xffff01b0, 0xffff01b0, 0xffffffff, 0x1b001b0, 0x1b001b0, 0x1b001b0, 0xffffffff, 0xffff01b0, 0xffff01b0, 0x1b0ffff, 0x1b001b0, 0x1b0ffff, 0x1b001b0, 0xffff01b0, 0x1b0ffff, 0x1b0ffff, 0xffff01b0, 0x1b001b0, 0x1b001b0, 0x1b001b0, 0xffff01b0, 0x1b001b0, 0xffff01b0, 0xffff01b0, 0x1b0ffff, 0xffff01b0, 0x1b001b0, 0x1b001b0, 0x1b001b0, 0x1b0ffff, 0x1b001b0, 0x1b0ffff, 0xffffffff, 0xffff01b0, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b0ffff, 0x1b0ffff, 0xffffffff, 0x1b0ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x1dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b1, 0x138ffff, 0xffffffff, 0xffffffff, 0xffff01b1, 0x1b1ffff, 0xffffffff, 0xffff01b1, 0xffff01b1, 0xffffffff, 0x1b1ffff, 0xffff01b1, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b1ffff, 0xffff01b1, 0xffffffff, 0x1b1ffff, 0x1b1ffff, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b1, 0xffffffff, 0xffffffff, 0x1b1ffff, 0xffff01b1, 0xffffffff, 0xffff01b1, 0xffff01b1, 0xffff01b1, 0xffffffff, 0x1b1ffff, 0x1b1ffff, 0xffffffff, 0xffff01b1, 0xffff01b1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b1, 0xffffffff, 0x1b101b1, 0xffff01b1, 0x1b1ffff, 0xffffffff, 0xffff01b1, 0x1b1ffff, 0xffff01b1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b1, 0xffffffff, 0xffffffff, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b101b1, 0xffffffff, 0x1b1ffff, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b1ffff, 0xffff01b1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b1ffff, 0xffffffff, 0xffffffff, 0x1b1ffff, 0xffff01b1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b1, 0x1b101b1, 0x1b101b1, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffff01b1, 0xffffffff, 0x1b1ffff, 0xffff01b1, 0x1b101b1, 0xffffffff, 0x1b1ffff, 0xffff01b1, 0xffff01b1, 0xffffffff, 0x1b1ffff, 0xffff01b1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b101b1, 0x1b1ffff, 0xffffffff, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b101b1, 0x1b1ffff, 0x1b101b1, 0xffff01b1, 0xffffffff, 0xffffffff, 0x1b1ffff, 0x1b101b1, 0x1b101b1, 0x1b1ffff, 0xffff01b1, 0xffff01b1, 0xffffffff, 0xffff01b1, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffff01b1, 0xffff01b1, 0x1b1ffff, 0xffffffff, 0xffffffff, 0x1b101b1, 0xffff01b1, 0xffffffff, 0xffffffff, 0xffff01b1, 0xffffffff, 0xffff01b1, 0x1b101b1, 0xffff01b1, 0x1b101b1, 0x1b1ffff, 0x1b1ffff, 0x1b101b1, 0xffff01b1, 0xffffffff, 0xffff01b1, 0x1b101b1, 0x1b1ffff, 0xffff01b1, 0x1b101b1, 0x1b1ffff, 0xffffffff, 0x1b101b1, 0x1b101b1, 0x1b1ffff, 0xffffffff, 0xffff01b1, 0x1b101b1, 0x1b1ffff, 0x1b1ffff, 0x1b1ffff, 0xffffffff, 0x1b101b1, 0xffffffff, 0xffffffff, 0x1b101b1, 0xffffffff, 0xffff01b1, 0x1b1ffff, 0x1b101b1, 0x1b101b1, 0xffff01b1, 0xffff01b1, 0x1b101b1, 0x1b1ffff, 0xffffffff, 0x1b1ffff, 0xffff01b1, 0x1b1ffff, 0xffffffff, 0x1b1ffff, 0xffffffff, 0xffff01b1, 0x1b1ffff, 0x1b101b1, 0xffff01b1, 0x1b1ffff, 0x1b101b1, 0xffff01b1, 0x1b101b1, 0xffffffff, 0xffff01b1, 0xffffffff, 0x1b1ffff, 0xffff01b1, 0xffff01b1, 0xffffffff, 0x1b101b1, 0x1b101b1, 0x1b101b1, 0xffffffff, 0xffff01b1, 0xffff01b1, 0x1b1ffff, 0x1b101b1, 0x1b1ffff, 0x1b101b1, 0xffff01b1, 0x1b1ffff, 0x1b1ffff, 0xffff01b1, 0x1b101b1, 0x1b101b1, 0x1b101b1, 0xffff01b1, 0x1b101b1, 0xffff01b1, 0xffff01b1, 0x1b1ffff, 0xffff01b1, 0x1b101b1, 0x1b101b1, 0x1b101b1, 0x1b1ffff, 0x1b101b1, 0x1b1ffff, 0xffffffff, 0xffff01b1, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b1ffff, 0x1b1ffff, 0xffffffff, 0x1b1ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x1effff, 0xffff001e, 0xffffffff, 0xffff00e0, 0xffffffff, 0xffffffff, 0xffffffff, 0x1e001e, 0xffff01b2, 0x1effff, 0xffffffff, 0x2210221, 0xffff01b2, 0xffff0221, 0x1effff, 0xffff01b2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b2, 0x1b2ffff, 0xffffffff, 0xffffffff, 0xffff0221, 0x2210221, 0x1b2ffff, 0xffffffff, 0x221001e, 0xffffffff, 0x1b2ffff, 0x1b2ffff, 0xffffffff, 0xe0ffff, 0xffffffff, 0xffff01b2, 0x2f1001e, 0xffffffff, 0x1b2ffff, 0xffffffff, 0xffffffff, 0xffff01b2, 0xffff01b2, 0xffff01b2, 0xffffffff, 0x1b2ffff, 0xffffffff, 0xffff0221, 0xffff01b2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b2ffff, 0x1effff, 0xffffffff, 0xffffffff, 0xffff0221, 0xffff01b2, 0xffffffff, 0x1b201b2, 0xffff01b2, 0x1b2ffff, 0xffffffff, 0xffff01b2, 0x1b2ffff, 0xffff01b2, 0x2210221, 0xffffffff, 0xffffffff, 0xffff01b2, 0x2210221, 0xffff0221, 0x221ffff, 0xffffffff, 0xffff01b2, 0x221ffff, 0x2210221, 0x1b2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b201b2, 0xffffffff, 0x1b2ffff, 0x1b2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b2ffff, 0xffff01b2, 0xffffffff, 0xffffffff, 0xffff02f1, 0xffffffff, 0x1b2ffff, 0xffffffff, 0xffffffff, 0x1b202f1, 0xffff01b2, 0xffffffff, 0x2210221, 0xffffffff, 0xffffffff, 0x221ffff, 0xffffffff, 0x2210221, 0xffff0221, 0x221ffff, 0xffffffff, 0xffff0221, 0x221ffff, 0x22102f1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b2, 0x1b201b2, 0x1b201b2, 0x1b2ffff, 0x221ffff, 0xffffffff, 0xffff01b2, 0xffffffff, 0x1b2ffff, 0xffff01b2, 0x1b201b2, 0xffffffff, 0x1b2ffff, 0xffff01b2, 0xffff01b2, 0xffff0221, 0x1b2ffff, 0xffff01b2, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b201b2, 0x1b2ffff, 0xffffffff, 0x1b2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b201b2, 0x1b2ffff, 0x1b201b2, 0xffff01b2, 0xffffffff, 0xffffffff, 0x1b2ffff, 0x1b201b2, 0x1b201b2, 0x1b202f1, 0xffff01b2, 0xffff01b2, 0xffff02f1, 0xffff01b2, 0x1b2ffff, 0xffffffff, 0xffffffff, 0xffff01b2, 0xffff01b2, 0x1b2ffff, 0xffffffff, 0xffffffff, 0x1b201b2, 0xffff01b2, 0x221ffff, 0xffffffff, 0xffff01b2, 0xffff02f1, 0xffff01b2, 0x1b201b2, 0xffff01b2, 0x1b201b2, 0x1b2ffff, 0x1b2ffff, 0x1b201b2, 0x2f101b2, 0xffffffff, 0xffff01b2, 0x1b201b2, 0x1b2ffff, 0xffff01b2, 0x1b201b2, 0x1b2ffff, 0xffffffff, 0x1b201b2, 0x1b201b2, 0x1b2ffff, 0xffffffff, 0xffff01b2, 0x1b201b2, 0x1b2ffff, 0x1b2ffff, 0x1b2ffff, 0xffffffff, 0x1b201b2, 0xffffffff, 0xffffffff, 0x1b201b2, 0xffffffff, 0xffff01b2, 0x1b2ffff, 0x1b201b2, 0x1b201b2, 0xffff01b2, 0xffff01b2, 0x1b201b2, 0x1b2ffff, 0xffffffff, 0x1b2ffff, 0xffff01b2, 0x1b2ffff, 0xffffffff, 0x1b2ffff, 0xffffffff, 0xffff01b2, 0x1b2ffff, 0x1b201b2, 0xffff01b2, 0x1b2ffff, 0x1b201b2, 0xffff01b2, 0x1b201b2, 0xffffffff, 0xffff01b2, 0xffffffff, 0x1b2ffff, 0xffff01b2, 0xffff01b2, 0xffffffff, 0x1b201b2, 0x1b201b2, 0x1b201b2, 0xffffffff, 0xffff01b2, 0xffff01b2, 0x1b2ffff, 0x1b201b2, 0x1b2ffff, 0x1b201b2, 0xffff01b2, 0x1b2ffff, 0x1b2ffff, 0xffff01b2, 0x1b201b2, 0x1b201b2, 0x1b201b2, 0xffff01b2, 0x1b201b2, 0xffff01b2, 0xffff01b2, 0x1b2ffff, 0xffff01b2, 0x1b201b2, 0x1b201b2, 0x1b201b2, 0x1b2ffff, 0x1b201b2, 0x1b2ffff, 0xffffffff, 0xffff01b2, 0x1b2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b2ffff, 0x1b2ffff, 0xffffffff, 0x1b2ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x1f001f, 0x1f001f, 0xffff001f, 0xffffffff, 0xffff001f, 0xffff001f, 0xffffffff, 0xffffffff, 0xffff01b3, 0x1fffff, 0xffffffff, 0xffffffff, 0xffff01b3, 0x1b3ffff, 0x1f001f, 0xffff01b3, 0x1f01b3, 0x1b3001f, 0x1b3001f, 0xffff01b3, 0x1b3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b3ffff, 0x1f01b3, 0xffffffff, 0x1b3ffff, 0x1b3ffff, 0x1b3ffff, 0x1f001f, 0xffffffff, 0xffffffff, 0xffff01b3, 0xffffffff, 0xffffffff, 0x1b3ffff, 0xffff01b3, 0xffffffff, 0xffff01b3, 0xffff01b3, 0xffff01b3, 0xffffffff, 0x1b3ffff, 0x1b3ffff, 0xffffffff, 0xffff01b3, 0xffff01b3, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b3, 0xffffffff, 0x1b301b3, 0xffff01b3, 0x1b3ffff, 0xffffffff, 0xffff01b3, 0x1b3ffff, 0xffff01b3, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b3, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b3, 0xffffffff, 0xffffffff, 0x1b3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b301b3, 0xffffffff, 0x1b3ffff, 0x1b3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b3ffff, 0xffff01b3, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b3ffff, 0xffffffff, 0xffffffff, 0x1b3ffff, 0xffff01b3, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b3, 0x1b301b3, 0x1b301b3, 0x1b3ffff, 0xffffffff, 0xffffffff, 0xffff01b3, 0xffffffff, 0x1b3ffff, 0xffff01b3, 0x1b301b3, 0xffffffff, 0x1b3ffff, 0xffff01b3, 0xffff01b3, 0xffffffff, 0x1b3ffff, 0xffff01b3, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b301b3, 0x1b30746, 0xffffffff, 0x1b3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b301b3, 0x1b3ffff, 0x1b301b3, 0xffff01b3, 0xffffffff, 0xffffffff, 0x1b3ffff, 0x1b301b3, 0x1b301b3, 0x1b3ffff, 0xffff01b3, 0xffff01b3, 0xffffffff, 0xffff01b3, 0x1b3ffff, 0xffffffff, 0xffffffff, 0xffff01b3, 0xffff01b3, 0x1b3ffff, 0xffffffff, 0xffffffff, 0x1b301b3, 0xffff01b3, 0xffffffff, 0xffffffff, 0xffff01b3, 0xffffffff, 0xffff01b3, 0x1b301b3, 0xffff01b3, 0x1b301b3, 0x1b3ffff, 0x1b3ffff, 0x1b301b3, 0xffff01b3, 0xffffffff, 0xffff01b3, 0x1b301b3, 0x1b3ffff, 0xffff01b3, 0x1b301b3, 0x1b3ffff, 0xffffffff, 0x1b301b3, 0x1b301b3, 0x1b3ffff, 0xffffffff, 0xffff01b3, 0x1b301b3, 0x1b3ffff, 0x1b3ffff, 0x1b3ffff, 0xffffffff, 0x1b301b3, 0xffffffff, 0xffffffff, 0x1b301b3, 0x746ffff, 0x74601b3, 0x1b3ffff, 0x1b301b3, 0x1b301b3, 0xffff01b3, 0xffff01b3, 0x1b301b3, 0x1b3ffff, 0xffffffff, 0x1b3ffff, 0xffff01b3, 0x1b3ffff, 0xffffffff, 0x1b3ffff, 0xffffffff, 0xffff01b3, 0x1b3ffff, 0x1b301b3, 0xffff01b3, 0x1b3ffff, 0x1b301b3, 0xffff01b3, 0x1b301b3, 0xffffffff, 0xffff01b3, 0xffffffff, 0x1b3ffff, 0xffff01b3, 0xffff01b3, 0x746ffff, 0x1b301b3, 0x1b301b3, 0x1b301b3, 0xffffffff, 0xffff01b3, 0xffff01b3, 0x1b3ffff, 0x1b301b3, 0x1b3ffff, 0x1b301b3, 0xffff01b3, 0x1b3ffff, 0x1b3ffff, 0xffff01b3, 0x1b301b3, 0x1b301b3, 0x1b301b3, 0xffff01b3, 0x1b301b3, 0xffff01b3, 0xffff01b3, 0x1b3ffff, 0xffff01b3, 0x1b301b3, 0x1b301b3, 0x1b301b3, 0x1b3ffff, 0x1b301b3, 0x1b3ffff, 0xffffffff, 0xffff01b3, 0x1b3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b3ffff, 0x1b3ffff, 0xffffffff, 0x1b3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
            };
        }

        protected static int[][] value1 = null;

        protected static void value1Init()
        {
            value1 = new int[][]
            {
                { 0x2000ad, 0xad0020, 0xc300ad, 0x13900c3, 0x26d00ad, 0x26d00ad, 0x139011d, 0xc300c3, 0x26d01b4, 0x2000c3, 0x26d0324, 0xc300c3, 0xc3026d, 0x26d00c3, 0x2000ad, 0xc301b4, 0xad0324, 0x20800ad, 0xffff00ad, 0xffffffff, 0x1b4ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b4ffff, 0xadffff, 0xffff00c3, 0xffffffff, 0x1b4ffff, 0xffffffff, 0xad00ad, 0xc3ffff, 0xffffffff, 0xffff01b4, 0xffff00c3, 0xffffffff, 0xffffffff, 0xffffffff, 0x11dffff, 0xffff01b4, 0xffff01b4, 0xffff01b4, 0xffffffff, 0x1b4ffff, 0xffffffff, 0xffffffff, 0xffff01b4, 0xffffffff, 0x208ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b4ffff, 0xc3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b4, 0xffffffff, 0x1b401b4, 0xffff01b4, 0x1b4ffff, 0xffffffff, 0xffff01b4, 0x1b4ffff, 0xffff01b4, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b4, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b4, 0xffffffff, 0xffffffff, 0x1b4ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b401b4, 0xffffffff, 0x1b4ffff, 0x1b4ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b4ffff, 0xffff01b4, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b4ffff, 0xffffffff, 0xffffffff, 0x1b4ffff, 0xffff01b4, 0xffff0208, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b4, 0x1b401b4, 0x1b401b4, 0x1b4ffff, 0xffffffff, 0xffffffff, 0xffff01b4, 0xffffffff, 0x1b4ffff, 0xffff01b4, 0x1b401b4, 0xffffffff, 0x1b4ffff, 0xffff01b4, 0xffff01b4, 0xffffffff, 0x1b4ffff, 0xffff01b4, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b4ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b401b4, 0x1b4ffff, 0xffffffff, 0x1b4ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b401b4, 0x1b4ffff, 0x1b401b4, 0xffff01b4, 0xffffffff, 0xffffffff, 0x1b4ffff, 0x1b401b4, 0x1b401b4, 0x1b4ffff, 0xffff01b4, 0xffff01b4, 0xffffffff, 0xffff01b4, 0x1b4ffff, 0xffffffff, 0xffffffff, 0xffff01b4, 0xffff01b4, 0x1b4ffff, 0xffffffff, 0xffffffff, 0x1b401b4, 0xffff01b4, 0xffffffff, 0xffffffff, 0xffff01b4, 0xffffffff, 0xffff01b4, 0x1b401b4, 0xffff01b4, 0x1b401b4, 0x1b4ffff, 0x1b4ffff, 0x1b401b4, 0xffff01b4, 0xffffffff, 0xffff01b4, 0x1b401b4, 0x1b4ffff, 0xffff01b4, 0x1b401b4, 0x1b4ffff, 0xffffffff, 0x1b401b4, 0x1b401b4, 0x1b4ffff, 0xffffffff, 0xffff01b4, 0x1b401b4, 0x1b4ffff, 0x1b4ffff, 0x1b4ffff, 0xffffffff, 0x1b401b4, 0xffffffff, 0xffffffff, 0x1b401b4, 0xffffffff, 0xffff01b4, 0x1b4ffff, 0x1b401b4, 0x1b401b4, 0xffff01b4, 0xffff01b4, 0x1b401b4, 0x1b4ffff, 0xffffffff, 0x1b4ffff, 0xffff01b4, 0x1b4ffff, 0xffffffff, 0x1b4ffff, 0xffffffff, 0xffff01b4, 0x1b4ffff, 0x1b401b4, 0xffff01b4, 0x1b4ffff, 0x1b401b4, 0xffff01b4, 0x1b401b4, 0xffffffff, 0xffff01b4, 0xffffffff, 0x1b4ffff, 0xffff01b4, 0xffff01b4, 0xffffffff, 0x1b401b4, 0x1b401b4, 0x1b401b4, 0xffffffff, 0xffff01b4, 0xffff01b4, 0x1b4ffff, 0x1b401b4, 0x1b4ffff, 0x1b401b4, 0xffff01b4, 0x1b4ffff, 0x1b4ffff, 0xffff01b4, 0x1b401b4, 0x1b401b4, 0x1b401b4, 0xffff01b4, 0x1b401b4, 0xffff01b4, 0xffff01b4, 0x1b4ffff, 0xffff01b4, 0x1b401b4, 0x1b401b4, 0x1b401b4, 0x1b4ffff, 0x1b401b4, 0x1b4ffff, 0xffffffff, 0xffff01b4, 0x1b4ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b4ffff, 0x1b4ffff, 0xffffffff, 0x1b4ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x210021, 0x210021, 0xffff0021, 0xffffffff, 0xffff0021, 0xffff0021, 0xffffffff, 0xffffffff, 0xffff01b5, 0x21ffff, 0xffffffff, 0xffffffff, 0xffff01b5, 0xffffffff, 0x210021, 0xffff01b5, 0x21ffff, 0xffff0021, 0xffff0021, 0xffffffff, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b5ffff, 0x21ffff, 0xffffffff, 0xffffffff, 0x1b5ffff, 0x1b5ffff, 0x210021, 0xffffffff, 0xffffffff, 0xffff01b5, 0xffffffff, 0xffffffff, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffff01b5, 0xffff01b5, 0xffff01b5, 0xffffffff, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffff01b5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b5, 0xffffffff, 0x1b501b5, 0xffff01b5, 0x1b5ffff, 0xffffffff, 0xffff01b5, 0x1b5ffff, 0xffff01b5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b5, 0xffffffff, 0xffffffff, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b501b5, 0xffffffff, 0x1b5ffff, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b5ffff, 0xffff01b5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b5ffff, 0xffffffff, 0xffffffff, 0x1b5ffff, 0xffff01b5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b5, 0x1b501b5, 0x1b501b5, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffff01b5, 0xffffffff, 0x1b5ffff, 0xffff01b5, 0x1b501b5, 0xffffffff, 0x1b5ffff, 0xffff01b5, 0xffff01b5, 0xffffffff, 0x1b5ffff, 0xffff01b5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b501b5, 0x1b5ffff, 0xffffffff, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b501b5, 0x1b5ffff, 0x1b501b5, 0xffff01b5, 0xffffffff, 0xffffffff, 0x1b5ffff, 0x1b501b5, 0x1b501b5, 0x1b5ffff, 0xffff01b5, 0xffff01b5, 0xffffffff, 0xffff01b5, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffff01b5, 0xffff01b5, 0x1b5ffff, 0xffffffff, 0xffffffff, 0x1b501b5, 0xffff01b5, 0xffffffff, 0xffffffff, 0xffff01b5, 0xffffffff, 0xffff01b5, 0x1b501b5, 0xffff01b5, 0x1b501b5, 0x1b5ffff, 0x1b5ffff, 0x1b501b5, 0xffff01b5, 0xffffffff, 0xffff01b5, 0x1b501b5, 0x1b5ffff, 0xffff01b5, 0x1b501b5, 0x1b5ffff, 0xffffffff, 0x1b501b5, 0x1b501b5, 0x1b5ffff, 0xffffffff, 0xffff01b5, 0x1b501b5, 0x1b5ffff, 0x1b5ffff, 0x1b5ffff, 0xffffffff, 0x1b501b5, 0xffffffff, 0xffffffff, 0x1b501b5, 0xffffffff, 0xffff01b5, 0x1b5ffff, 0x1b501b5, 0x1b501b5, 0xffff01b5, 0xffff01b5, 0x1b501b5, 0x1b5ffff, 0xffffffff, 0x1b5ffff, 0xffff01b5, 0x1b5ffff, 0xffffffff, 0x1b5ffff, 0xffffffff, 0xffff01b5, 0x1b5ffff, 0x1b501b5, 0xffff01b5, 0x1b5ffff, 0x1b501b5, 0xffff01b5, 0x1b501b5, 0xffffffff, 0xffff01b5, 0xffffffff, 0x1b5ffff, 0xffff01b5, 0xffff01b5, 0xffffffff, 0x1b501b5, 0x1b501b5, 0x1b501b5, 0xffffffff, 0xffff01b5, 0xffff01b5, 0x1b5ffff, 0x1b501b5, 0x1b5ffff, 0x1b501b5, 0xffff01b5, 0x1b5ffff, 0x1b5ffff, 0xffff01b5, 0x1b501b5, 0x1b501b5, 0x1b501b5, 0xffff01b5, 0x1b501b5, 0xffff01b5, 0xffff01b5, 0x1b5ffff, 0xffff01b5, 0x1b501b5, 0x1b501b5, 0x1b501b5, 0x1b5ffff, 0x1b501b5, 0x1b5ffff, 0xffffffff, 0xffff01b5, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b5ffff, 0x1b5ffff, 0xffffffff, 0x1b5ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x220022, 0x6b906d4, 0xffff0022, 0xffffffff, 0x220022, 0xffff0022, 0xffffffff, 0xffffffff, 0xffffffff, 0x22ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0022, 0xffffffff, 0xffffffff, 0xffff0022, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0022, 0xffffffff, 0xffffffff, 0xffff0022, 0x220022, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0022, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0539, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x539ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x539ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x539ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x7dfffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x539ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x7dfffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x230023, 0x230023, 0xffff0023, 0xffffffff, 0xffff0023, 0xffff0023, 0xffffffff, 0xffffffff, 0xffff01b6, 0x23ffff, 0xffffffff, 0xffffffff, 0xffff01b6, 0x1b6ffff, 0x230023, 0xffff01b6, 0x2301b6, 0x3e10023, 0x1b60023, 0xffff01b6, 0x1b6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b6ffff, 0x2301b6, 0xffffffff, 0x1b6ffff, 0x1b6ffff, 0x1b6ffff, 0x230023, 0xffffffff, 0xffffffff, 0xffff01b6, 0xffffffff, 0xffffffff, 0x1b6ffff, 0xffff01b6, 0xffffffff, 0xffff01b6, 0xffff01b6, 0xffff01b6, 0xffffffff, 0x1b6ffff, 0x1b6ffff, 0xffffffff, 0xffff01b6, 0xffff01b6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b6, 0xffffffff, 0x1b601b6, 0xffff01b6, 0x1b6ffff, 0xffffffff, 0xffff01b6, 0x1b6ffff, 0xffff01b6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b6, 0xffffffff, 0xffffffff, 0x1b6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b601b6, 0xffffffff, 0x1b6ffff, 0x1b6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b6ffff, 0xffff01b6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b6ffff, 0xffffffff, 0xffffffff, 0x1b6ffff, 0xffff01b6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b6, 0x1b601b6, 0x1b601b6, 0x1b6ffff, 0xffffffff, 0xffffffff, 0xffff01b6, 0xffffffff, 0x1b6ffff, 0xffff01b6, 0x1b601b6, 0xffffffff, 0x1b6ffff, 0xffff01b6, 0xffff01b6, 0xffffffff, 0x1b6ffff, 0xffff01b6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b601b6, 0x1b6ffff, 0xffffffff, 0x1b6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b601b6, 0x1b6ffff, 0x1b601b6, 0xffff01b6, 0xffffffff, 0xffffffff, 0x1b6ffff, 0x1b601b6, 0x1b601b6, 0x1b6ffff, 0xffff01b6, 0xffff01b6, 0xffffffff, 0xffff01b6, 0x1b6ffff, 0xffffffff, 0xffffffff, 0xffff01b6, 0xffff01b6, 0x1b6ffff, 0xffffffff, 0xffffffff, 0x1b601b6, 0xffff01b6, 0xffffffff, 0xffffffff, 0xffff01b6, 0xffffffff, 0xffff01b6, 0x1b601b6, 0xffff01b6, 0x1b601b6, 0x1b6ffff, 0x1b6ffff, 0x1b601b6, 0xffff01b6, 0xffffffff, 0xffff01b6, 0x1b601b6, 0x1b6ffff, 0xffff01b6, 0x1b601b6, 0x1b6ffff, 0xffffffff, 0x1b601b6, 0x1b601b6, 0x1b6ffff, 0xffffffff, 0xffff01b6, 0x1b601b6, 0x1b6ffff, 0x1b6ffff, 0x1b6ffff, 0xffffffff, 0x1b601b6, 0xffffffff, 0xffffffff, 0x1b601b6, 0xffffffff, 0xffff01b6, 0x1b6ffff, 0x1b601b6, 0x1b601b6, 0xffff01b6, 0xffff01b6, 0x1b601b6, 0x1b6ffff, 0xffffffff, 0x1b6ffff, 0xffff01b6, 0x1b6ffff, 0xffffffff, 0x1b6ffff, 0xffffffff, 0xffff01b6, 0x1b6ffff, 0x1b601b6, 0xffff01b6, 0x1b6ffff, 0x1b601b6, 0xffff01b6, 0x1b601b6, 0xffffffff, 0xffff01b6, 0xffffffff, 0x1b6ffff, 0xffff01b6, 0xffff01b6, 0xffffffff, 0x1b601b6, 0x1b601b6, 0x1b601b6, 0xffffffff, 0xffff01b6, 0xffff01b6, 0x1b6ffff, 0x1b601b6, 0x1b6ffff, 0x1b601b6, 0xffff01b6, 0x1b6ffff, 0x1b6ffff, 0xffff01b6, 0x1b601b6, 0x1b601b6, 0x1b601b6, 0xffff01b6, 0x1b601b6, 0xffff01b6, 0xffff01b6, 0x1b6ffff, 0xffff01b6, 0x1b601b6, 0x1b601b6, 0x1b601b6, 0x1b6ffff, 0x1b601b6, 0x1b6ffff, 0xffffffff, 0xffff01b6, 0x1b6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b6ffff, 0x1b6ffff, 0xffffffff, 0x1b6ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x240024, 0x240024, 0xffff0024, 0xffffffff, 0xffff0024, 0xffff0024, 0xffffffff, 0xffffffff, 0xffff01b7, 0x24ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x240024, 0xffff01b7, 0x24ffff, 0xffff0024, 0xffff0024, 0xffffffff, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b7ffff, 0x24ffff, 0xffffffff, 0xffffffff, 0x1b7ffff, 0xffffffff, 0x240024, 0xffffffff, 0xffffffff, 0xffff01b7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b7, 0xffff01b7, 0xffffffff, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffff01b7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b7, 0xffffffff, 0x1b701b7, 0xffff01b7, 0x1b7ffff, 0xffffffff, 0xffff01b7, 0x1b7ffff, 0xffff01b7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b7, 0xffffffff, 0xffffffff, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b701b7, 0xffffffff, 0x1b7ffff, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b7ffff, 0xffff01b7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b7ffff, 0xffffffff, 0xffffffff, 0x1b7ffff, 0xffff01b7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b7, 0x1b701b7, 0x1b701b7, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffff01b7, 0xffffffff, 0x1b7ffff, 0xffff01b7, 0x1b701b7, 0xffffffff, 0x1b7ffff, 0xffff01b7, 0xffff01b7, 0xffffffff, 0x1b7ffff, 0xffff01b7, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b701b7, 0x1b7ffff, 0xffffffff, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b701b7, 0x1b7ffff, 0x1b701b7, 0xffff01b7, 0xffffffff, 0xffffffff, 0x1b7ffff, 0x1b701b7, 0x1b701b7, 0x1b7ffff, 0xffff01b7, 0xffff01b7, 0xffffffff, 0xffff01b7, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffff01b7, 0xffff01b7, 0x1b7ffff, 0xffffffff, 0xffffffff, 0x1b701b7, 0xffff01b7, 0xffffffff, 0xffffffff, 0xffff01b7, 0xffffffff, 0xffff01b7, 0x1b701b7, 0xffff01b7, 0x1b701b7, 0x1b7ffff, 0x1b7ffff, 0x1b701b7, 0xffff01b7, 0xffffffff, 0xffff01b7, 0x1b701b7, 0x1b7ffff, 0xffff01b7, 0x1b701b7, 0x1b7ffff, 0xffffffff, 0x1b701b7, 0x1b701b7, 0x1b7ffff, 0xffffffff, 0xffff01b7, 0x1b701b7, 0x1b7ffff, 0x1b7ffff, 0x1b7ffff, 0xffffffff, 0x1b701b7, 0xffffffff, 0xffffffff, 0x1b701b7, 0xffffffff, 0xffff01b7, 0x1b7ffff, 0x1b701b7, 0x1b701b7, 0xffff01b7, 0xffff01b7, 0x1b701b7, 0x1b7ffff, 0xffffffff, 0x1b7ffff, 0xffff01b7, 0x1b7ffff, 0xffffffff, 0x1b7ffff, 0xffffffff, 0xffff01b7, 0x1b7ffff, 0x1b701b7, 0xffff01b7, 0x1b7ffff, 0x1b701b7, 0xffff01b7, 0x1b701b7, 0xffffffff, 0xffff01b7, 0xffffffff, 0x1b7ffff, 0xffff01b7, 0xffff01b7, 0xffffffff, 0x1b701b7, 0x1b701b7, 0x1b701b7, 0xffffffff, 0xffff01b7, 0xffff01b7, 0x1b7ffff, 0x1b701b7, 0x1b7ffff, 0x1b701b7, 0xffff01b7, 0x1b7ffff, 0x1b7ffff, 0xffff01b7, 0x1b701b7, 0x1b701b7, 0x1b701b7, 0xffff01b7, 0x1b701b7, 0xffff01b7, 0xffff01b7, 0x1b7ffff, 0xffff01b7, 0x1b701b7, 0x1b701b7, 0x1b701b7, 0x1b7ffff, 0x1b701b7, 0x1b7ffff, 0xffffffff, 0xffff01b7, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b7ffff, 0x1b7ffff, 0xffffffff, 0x1b7ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x250025, 0xffffffff, 0xffff0025, 0xffffffff, 0x250025, 0xffff0025, 0xffffffff, 0xffffffff, 0xffff01b8, 0x25ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0025, 0xffff01b8, 0xffffffff, 0xffff0025, 0xffffffff, 0xffffffff, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b80025, 0xffffffff, 0xffffffff, 0xffff0025, 0x250025, 0xffff01b8, 0xffffffff, 0xffffffff, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffff01b8, 0xffff01b8, 0xffff01b8, 0xffffffff, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffff01b8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b80025, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b8, 0xffffffff, 0x1b801b8, 0xffff01b8, 0x1b8ffff, 0xffffffff, 0xffff01b8, 0x1b8ffff, 0xffff01b8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b8, 0xffffffff, 0xffffffff, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b801b8, 0xffffffff, 0x1b8ffff, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b8ffff, 0xffff01b8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b8ffff, 0xffffffff, 0x5ccffff, 0x1b8ffff, 0xffff01b8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b8, 0x1b801b8, 0x1b801b8, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffff01b8, 0xffffffff, 0x1b8ffff, 0xffff01b8, 0x1b801b8, 0xffffffff, 0x1b8ffff, 0xffff01b8, 0xffff01b8, 0xffffffff, 0x1b8ffff, 0xffff01b8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b801b8, 0x1b8ffff, 0xffffffff, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b801b8, 0x1b8ffff, 0x1b801b8, 0xffff01b8, 0xffffffff, 0xffffffff, 0x1b8ffff, 0x1b801b8, 0x1b801b8, 0x1b8ffff, 0xffff01b8, 0xffff01b8, 0xffffffff, 0xffff01b8, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffff01b8, 0xffff01b8, 0x1b8ffff, 0xffffffff, 0xffffffff, 0x1b801b8, 0xffff01b8, 0xffffffff, 0xffffffff, 0xffff01b8, 0xffffffff, 0xffff01b8, 0x1b801b8, 0xffff01b8, 0x1b801b8, 0x1b8ffff, 0x1b8ffff, 0x1b801b8, 0xffff01b8, 0xffffffff, 0xffff01b8, 0x1b801b8, 0x1b8ffff, 0xffff01b8, 0x1b801b8, 0x1b8ffff, 0xffffffff, 0x1b801b8, 0x1b801b8, 0x1b8ffff, 0xffffffff, 0xffff01b8, 0x1b801b8, 0x1b8ffff, 0x1b8ffff, 0x1b8ffff, 0xffffffff, 0x1b801b8, 0xffffffff, 0xffffffff, 0x1b801b8, 0xffffffff, 0xffff01b8, 0x1b8ffff, 0x1b801b8, 0x1b801b8, 0xffff01b8, 0xffff01b8, 0x1b801b8, 0x1b8ffff, 0xffffffff, 0x1b8ffff, 0xffff01b8, 0x1b8ffff, 0xffffffff, 0x1b8ffff, 0xffffffff, 0xffff01b8, 0x1b8ffff, 0x1b801b8, 0xffff01b8, 0x1b8ffff, 0x1b801b8, 0xffff01b8, 0x1b801b8, 0xffffffff, 0xffff01b8, 0xffffffff, 0x1b8ffff, 0xffff01b8, 0xffff01b8, 0xffffffff, 0x1b801b8, 0x1b801b8, 0x1b801b8, 0xffffffff, 0xffff01b8, 0xffff01b8, 0x1b8ffff, 0x1b801b8, 0x1b8ffff, 0x1b801b8, 0xffff01b8, 0x1b8ffff, 0x1b8ffff, 0xffff01b8, 0x1b801b8, 0x1b801b8, 0x1b801b8, 0xffff01b8, 0x1b801b8, 0xffff01b8, 0xffff01b8, 0x1b8ffff, 0xffff01b8, 0x1b801b8, 0x1b801b8, 0x1b801b8, 0x1b8ffff, 0x1b801b8, 0x1b8ffff, 0xffffffff, 0xffff01b8, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b8ffff, 0x1b8ffff, 0xffffffff, 0x1b8ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x260026, 0xffffffff, 0xffff0026, 0xffffffff, 0x260026, 0xffff0026, 0xffffffff, 0xffffffff, 0xffff01b9, 0x26ffff, 0xffffffff, 0xffffffff, 0xffff01b9, 0x1b9ffff, 0xffff0026, 0xffff01b9, 0xffff03e0, 0xffff0026, 0x1b9ffff, 0xffff01b9, 0x1b9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b9ffff, 0xffff01b9, 0xffffffff, 0x680ffff, 0x1b90026, 0x1b9ffff, 0xffffffff, 0xffff0026, 0x260026, 0xffff01b9, 0xffffffff, 0xffffffff, 0x1b9ffff, 0xffff07b4, 0xffffffff, 0xffff01b9, 0xffff01b9, 0xffff01b9, 0xffffffff, 0x1b9ffff, 0x1b9ffff, 0xffffffff, 0xffff01b9, 0xffff07c1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b90026, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b9, 0xffffffff, 0x1b901b9, 0xffff01b9, 0x1b9ffff, 0xffffffff, 0xffff01b9, 0x1b9ffff, 0xffff01b9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b9, 0xffffffff, 0xffffffff, 0x1b9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b901b9, 0xffffffff, 0x1b9ffff, 0x1b9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b9ffff, 0xffff01b9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b9ffff, 0xffffffff, 0xffffffff, 0x1b9ffff, 0xffff01b9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff01b9, 0x1b901b9, 0x1b901b9, 0x1b9ffff, 0xffffffff, 0xffffffff, 0xffff01b9, 0xffffffff, 0x1b9ffff, 0xffff01b9, 0x1b901b9, 0xffffffff, 0x1b9ffff, 0xffff01b9, 0xffff01b9, 0xffffffff, 0x1b9ffff, 0xffff01b9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b901b9, 0x1b9ffff, 0xffffffff, 0x1b9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b901b9, 0x1b9ffff, 0x1b901b9, 0xffff01b9, 0xffffffff, 0xffffffff, 0x1b9ffff, 0x1b901b9, 0x1b901b9, 0x1b9ffff, 0xffff01b9, 0xffff01b9, 0xffffffff, 0xffff01b9, 0x1b9ffff, 0xffff0026, 0xffffffff, 0xffff01b9, 0xffff01b9, 0x1b9ffff, 0xffffffff, 0xffffffff, 0x1b901b9, 0xffff01b9, 0xffffffff, 0xffffffff, 0xffff01b9, 0xffffffff, 0xffff01b9, 0x1b901b9, 0xffff01b9, 0x1b901b9, 0x1b9ffff, 0x1b9ffff, 0x1b901b9, 0xffff01b9, 0xffffffff, 0xffff01b9, 0x1b901b9, 0x1b9ffff, 0xffff01b9, 0x1b901b9, 0x1b9ffff, 0xffffffff, 0x1b901b9, 0x1b901b9, 0x1b90026, 0xffffffff, 0xffff01b9, 0x1b901b9, 0x1b9ffff, 0x1b9ffff, 0x1b9ffff, 0xffffffff, 0x1b901b9, 0xffffffff, 0xffffffff, 0x1b901b9, 0xffffffff, 0xffff01b9, 0x1b9ffff, 0x1b901b9, 0x1b901b9, 0xffff01b9, 0xffff01b9, 0x1b901b9, 0x1b9ffff, 0xffffffff, 0x1b9ffff, 0xffff01b9, 0x1b9ffff, 0xffffffff, 0x1b9ffff, 0xffffffff, 0xffff01b9, 0x1b9ffff, 0x1b901b9, 0xffff01b9, 0x1b9ffff, 0x1b901b9, 0xffff01b9, 0x1b901b9, 0xffffffff, 0xffff01b9, 0xffffffff, 0x1b9ffff, 0xffff01b9, 0xffff01b9, 0xffffffff, 0x1b901b9, 0x1b901b9, 0x1b901b9, 0xffffffff, 0xffff01b9, 0xffff01b9, 0x1b9ffff, 0x1b901b9, 0x1b9ffff, 0x1b901b9, 0xffff01b9, 0x1b9ffff, 0x1b9ffff, 0xffff01b9, 0x1b901b9, 0x1b901b9, 0x1b901b9, 0x2601b9, 0x1b901b9, 0xffff01b9, 0xffff01b9, 0x1b9ffff, 0x2601b9, 0x1b901b9, 0x1b901b9, 0x1b901b9, 0x1b9ffff, 0x1b901b9, 0x1b9ffff, 0xffffffff, 0xffff01b9, 0x1b9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b9ffff, 0x1b9ffff, 0xffffffff, 0x1b9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x270027, 0x228013a, 0x3120027, 0x13a037c, 0x270027, 0x13a0027, 0x11303a4, 0x191025f, 0x13a01a3, 0x27017a, 0x13a0191, 0x21f021f, 0x43401a3, 0x1a3021f, 0x1e90027, 0x13a01a3, 0x54601a3, 0x1a30027, 0x1a3013a, 0x3a401a3, 0x1a30191, 0x434024e, 0x6af021f, 0x25f021f, 0x21f021f, 0x1a3073b, 0x21f01a3, 0x21f0191, 0x1a30784, 0x1a30027, 0x1a3021f, 0x7af07b5, 0x86f0027, 0x270027, 0x21f01a3, 0x2ee0191, 0x21f0872, 0x1a30113, 0x8d001a3, 0x91a0990, 0x8b201a3, 0x9b401a3, 0x9d101a3, 0x8b20191, 0x1a30a18, 0x1a30434, 0x1e9021f, 0xaaa01a3, 0xaab01a3, 0xaac025f, 0x43408b2, 0xffff08b2, 0x191ffff, 0x24effff, 0xffff025f, 0x1a30027, 0xffff017a, 0x17a017a, 0xffffffff, 0x25f021f, 0xffff01a3, 0x191ffff, 0x1a301a3, 0xffff01a3, 0x1a30410, 0xffffffff, 0xffff01a3, 0x1a30439, 0x25f01a3, 0x21f021f, 0xffffffff, 0xffffffff, 0xffff01a3, 0x21f021f, 0xffff021f, 0x21fffff, 0x25f025f, 0xffff01a3, 0x21fffff, 0x21f021f, 0x1a3ffff, 0x25fffff, 0x25fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a301a3, 0x25fffff, 0x1a3ffff, 0x1a3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a3ffff, 0xffff01a3, 0xffffffff, 0xffffffff, 0xffff02ee, 0xffffffff, 0x1a3ffff, 0xffffffff, 0xffffffff, 0x1a302ee, 0x25f01a3, 0xffffffff, 0x21f021f, 0x410ffff, 0x25fffff, 0x21fffff, 0xffffffff, 0x21f021f, 0xffff021f, 0x21fffff, 0xffff025f, 0xffff021f, 0x21f025f, 0x21f02ee, 0xffffffff, 0x17affff, 0xffff0191, 0xffff066b, 0x1a301a3, 0x1a301a3, 0x66bffff, 0x21fffff, 0xffffffff, 0xffff01a3, 0xffffffff, 0x1a3ffff, 0xffff01a3, 0x1a301a3, 0xffffffff, 0x1a30191, 0xffff01a3, 0x19101a3, 0xffff021f, 0x1a3ffff, 0x25f066b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0191, 0x1a3ffff, 0xffff025f, 0x25fffff, 0x1910191, 0x1910191, 0x1910191, 0xffff0191, 0x1910191, 0x1a301a3, 0x1a3ffff, 0x25f025f, 0x75affff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a301a3, 0x1a3ffff, 0x1a301a3, 0xffff01a3, 0xffffffff, 0xffffffff, 0x1a3ffff, 0x1a301a3, 0x1a3066b, 0x1a302ee, 0x25f01a3, 0xffff01a3, 0x25f02ee, 0xffff01a3, 0x1a3ffff, 0xffff088b, 0xffffffff, 0x25f01a3, 0xffff01a3, 0x1a3ffff, 0xffff025f, 0x25fffff, 0x1a301a3, 0xffff075a, 0x21fffff, 0xffffffff, 0x25f075a, 0xffff02ee, 0xffff01a3, 0x1a301a3, 0xffff01a3, 0x1a301a3, 0x1a3ffff, 0x1a3ffff, 0x1a301a3, 0x2ee01a3, 0x410ffff, 0xffff01a3, 0x1a301a3, 0x1a3ffff, 0x19101a3, 0x1a301a3, 0x1a3ffff, 0xffffffff, 0x1a301a3, 0x1a301a3, 0x1a3088b, 0xffff0191, 0xffff01a3, 0x1a301a3, 0x1a3025f, 0x1a3ffff, 0x1a3ffff, 0xffff025f, 0x1a301a3, 0xffff025f, 0x25fffff, 0x1a301a3, 0xffffffff, 0xffff075a, 0x1a3ffff, 0x1a301a3, 0x1a301a3, 0xffff01a3, 0x25f01a3, 0x1a301a3, 0x1a3025f, 0xffff025f, 0x1a3ffff, 0xffff01a3, 0x1a3025f, 0x25f025f, 0x1a3ffff, 0xffffffff, 0xffff01a3, 0x66bffff, 0x1a301a3, 0xffff01a3, 0x1a3ffff, 0x1a301a3, 0x25f01a3, 0x1a301a3, 0x25fffff, 0xffff01a3, 0xffffffff, 0x1a3ffff, 0xffff01a3, 0xffff01a3, 0xffffffff, 0x1a301a3, 0x1a301a3, 0x1a301a3, 0x25fffff, 0x25f01a3, 0xffff01a3, 0x1a3ffff, 0x1a301a3, 0x1a3ffff, 0x1a301a3, 0xffff01a3, 0x1a3ffff, 0x1a3ffff, 0xffff01a3, 0x1a301a3, 0x1a301a3, 0x1a301a3, 0x88b01a3, 0x1a301a3, 0xffff01a3, 0xffff01a3, 0x1a3ffff, 0x88b01a3, 0x1a301a3, 0x1a301a3, 0x1a301a3, 0x1a3ffff, 0x1a301a3, 0x1a3ffff, 0xffffffff, 0xffff01a3, 0x1a3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1a3ffff, 0x1a3ffff, 0xffffffff, 0x1a3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x280028, 0x280028, 0xffff0028, 0xffffffff, 0xeb0028, 0xffff0028, 0xffffffff, 0xffffffff, 0xffffffff, 0x28ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x280028, 0xffffffff, 0x28ffff, 0xffff0028, 0xffff0028, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x28ffff, 0xffffffff, 0xffffffff, 0xffff00eb, 0xffffffff, 0x280028, 0xffff00eb, 0xeb00eb, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff00eb, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x29ffff, 0xffff0029, 0xffffffff, 0xffff00e1, 0xffffffff, 0xffffffff, 0xffffffff, 0x290029, 0xffffffff, 0x29ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x29ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0029, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe1ffff, 0xffffffff, 0xffffffff, 0xffff0029, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x29ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x2a002a, 0xffffffff, 0xffff002a, 0xffffffff, 0x2a002a, 0xffff002a, 0xffffffff, 0xffffffff, 0xffffffff, 0x2affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002a, 0xffffffff, 0xffffffff, 0xffff002a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002a, 0xffffffff, 0xffffffff, 0xffff002a, 0x2a002a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x2b002b, 0xffffffff, 0xffff002b, 0xffffffff, 0x2b002b, 0xffff002b, 0xffffffff, 0xffffffff, 0xffffffff, 0x2bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002b, 0xffffffff, 0xffffffff, 0xffff002b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002b, 0xffffffff, 0xffffffff, 0xffff002b, 0x2b002b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x2c002c, 0xffffffff, 0xffff002c, 0xffffffff, 0x2c002c, 0xffff002c, 0xffffffff, 0xffffffff, 0xffffffff, 0x2cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002c, 0xffffffff, 0xffffffff, 0xffff002c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002c, 0xffffffff, 0xffffffff, 0xffff002c, 0x2c002c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x2d002d, 0xffff013b, 0xffff002d, 0x13bffff, 0x2d002d, 0x13b002d, 0xffffffff, 0xffffffff, 0x13bffff, 0x2dffff, 0x13bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002d, 0x13bffff, 0xffffffff, 0xffff002d, 0xffff013b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002d, 0xffffffff, 0xffffffff, 0xffff002d, 0x2d002d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x13bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x2e002e, 0x2e002e, 0xffff002e, 0xffffffff, 0xffff002e, 0xffff002e, 0xffffffff, 0xffffffff, 0xffffffff, 0x2effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2e002e, 0xffffffff, 0x2effff, 0xffff002e, 0xffff002e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2e002e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x2fffff, 0xffff002f, 0xffffffff, 0x13c002f, 0xffffffff, 0xffffffff, 0x13cffff, 0x2f002f, 0xffffffff, 0x2f013c, 0xffffffff, 0x13c013c, 0x13cffff, 0xffff013c, 0x2fffff, 0x13cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff002f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2fffff, 0xffffffff, 0xffffffff, 0xffff002f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x2fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x300030, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0174, 0xffffffff, 0x125ffff, 0xffffffff, 0x30ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x310031, 0xffffffff, 0xffff0031, 0xffffffff, 0x310031, 0xffff0031, 0xffffffff, 0xffffffff, 0xffffffff, 0x31ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0031, 0xffffffff, 0xffffffff, 0xffff0031, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0031, 0xffffffff, 0xffffffff, 0xffff0031, 0x310031, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0031, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0031, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0031, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x31ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x31ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x320032, 0xffffffff, 0xffff0032, 0xffffffff, 0x320032, 0xffff0032, 0x114ffff, 0xffffffff, 0xffffffff, 0x32ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0032, 0xffffffff, 0xffffffff, 0xffff0032, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0032, 0xffffffff, 0xffffffff, 0xffff0032, 0x320032, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0114, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0032, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x33ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x33ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x340034, 0xffffffff, 0xffff0034, 0xffffffff, 0x340034, 0xffff0034, 0xffffffff, 0xffffffff, 0xffffffff, 0x34ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0034, 0xffffffff, 0xffffffff, 0xffff0034, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0034, 0xffffffff, 0xffffffff, 0xffff0034, 0x340034, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0034, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x35ffff, 0xffff0035, 0xc4ffff, 0x350035, 0xffffffff, 0xffffffff, 0x35ffff, 0x350035, 0xffffffff, 0x350035, 0xffffffff, 0x350035, 0x35ffff, 0xffff0035, 0x35ffff, 0x35ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0035, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x35ffff, 0xffffffff, 0xffffffff, 0xffff0035, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x35ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x360036, 0xffffffff, 0xffff0036, 0xffffffff, 0x360036, 0xffff0036, 0xffffffff, 0xffffffff, 0xffffffff, 0x36ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0036, 0xffffffff, 0xffffffff, 0xffff0036, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0036, 0xffffffff, 0xffffffff, 0xffff0036, 0x360036, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0036, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0036, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0036, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x36ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x36ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x370037, 0x370037, 0x1090037, 0x370037, 0x370037, 0x1090037, 0x370037, 0x370037, 0x109ffff, 0x370037, 0x1090109, 0xffffffff, 0x37ffff, 0xffffffff, 0x370037, 0x109ffff, 0x37ffff, 0xffff0037, 0xffff0037, 0x109ffff, 0xffff0109, 0x370109, 0x109ffff, 0xffffffff, 0xffffffff, 0xffff0109, 0x37ffff, 0xffff0037, 0xffff0109, 0xffff0037, 0xffffffff, 0x370037, 0x370037, 0x370037, 0xffffffff, 0xffff0037, 0xffff0109, 0xffff0037, 0x109ffff, 0x370109, 0x37ffff, 0x109ffff, 0x109ffff, 0x370109, 0xffff0109, 0xffff0037, 0x109ffff, 0x109ffff, 0x109ffff, 0x109ffff, 0x370037, 0x1090037, 0x109ffff, 0xffffffff, 0xffffffff, 0xffff0037, 0x370037, 0x370037, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0109, 0xffffffff, 0xffffffff, 0x1090109, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x109ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x37ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0109, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff088d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0109, 0xffff0109, 0x109ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x109ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff088d, 0xffff0109, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x88dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x88dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x380038, 0x380038, 0xffff0038, 0xffffffff, 0xffff0038, 0xffff0038, 0xffffffff, 0xffffffff, 0xffffffff, 0x38ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x380038, 0xffffffff, 0x38ffff, 0xffff0038, 0xffff0038, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x38ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x380038, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x390039, 0xffffffff, 0xffff0039, 0xffffffff, 0x390039, 0xffff0039, 0xffffffff, 0xffffffff, 0xffffffff, 0x39ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0039, 0xffffffff, 0xffffffff, 0xffff0039, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0039, 0xffffffff, 0xffffffff, 0xffff0039, 0x390039, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0039, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0039, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0039, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x39ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x39ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x3a003a, 0x3a003a, 0xffff003a, 0xffffffff, 0xffff003a, 0xffff003a, 0xffffffff, 0xffffffff, 0xffffffff, 0x3affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3a003a, 0xffffffff, 0x3affff, 0xffff003a, 0xffff003a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3a003a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x3b003b, 0xffffffff, 0xffff003b, 0xffffffff, 0x3b003b, 0xffff003b, 0x3bffff, 0xffffffff, 0xffffffff, 0x3bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003b, 0xffffffff, 0xffffffff, 0xffff003b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003b, 0xffffffff, 0xffffffff, 0xffff003b, 0x3b003b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x3c003c, 0xffffffff, 0xffff003c, 0xffffffff, 0x3c003c, 0xffff003c, 0xffffffff, 0xffffffff, 0xffffffff, 0x3cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003c, 0xffffffff, 0xffffffff, 0xffff003c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003c, 0xffffffff, 0xffffffff, 0xffff003c, 0x3c003c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x3d003d, 0xffffffff, 0xffff003d, 0xffffffff, 0x3d003d, 0xffff003d, 0xffffffff, 0xffffffff, 0xffffffff, 0x3dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003d, 0xffffffff, 0xffffffff, 0xffff003d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003d, 0xffffffff, 0xffffffff, 0xffff003d, 0x3d003d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x3e003e, 0xffffffff, 0xffff003e, 0xffffffff, 0xec003e, 0xffff003e, 0xffffffff, 0xffffffff, 0xffffffff, 0x3effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003e, 0xffffffff, 0xffffffff, 0xffff003e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff00ec, 0xffffffff, 0xffffffff, 0xffff00ec, 0xec00ec, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff00ec, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
            };
        }

        protected static int[][] value2 = null;

        protected static void value2Init()
        {
            value2 = new int[][]
            {
                { 0x3f003f, 0xffffffff, 0xffff003f, 0xffffffff, 0x3f003f, 0xffff003f, 0xffffffff, 0xffffffff, 0xffffffff, 0x3fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003f, 0xffffffff, 0xffffffff, 0xffff003f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003f, 0xffffffff, 0xffffffff, 0xffff003f, 0x3f003f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff003f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x40ffff, 0xffff00b9, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1260126, 0xffffffff, 0x40ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb9ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0126, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0126, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x126ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x410041, 0x410041, 0xffff0041, 0xffffffff, 0xffff0041, 0xffff0041, 0xffffffff, 0xffffffff, 0xffffffff, 0x41ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x410041, 0xffffffff, 0x41ffff, 0xffff0041, 0xffff0041, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x41ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x410041, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x420042, 0xffffffff, 0xffff0042, 0xffffffff, 0x420042, 0xffff0042, 0xffffffff, 0xffffffff, 0xffffffff, 0x42ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0042, 0xffffffff, 0xffffffff, 0xffff0042, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0042, 0xffffffff, 0xffffffff, 0xffff0042, 0x420042, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0042, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0042, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0042, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x42ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x42ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x430043, 0x430043, 0xffff0043, 0xffffffff, 0xef0043, 0xffff0043, 0xffffffff, 0xffffffff, 0xffffffff, 0x43ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x430043, 0xffffffff, 0x43ffff, 0xffff0043, 0xffff0043, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x43ffff, 0xffffffff, 0xffffffff, 0xffff00ef, 0xffffffff, 0x430043, 0xffff00ef, 0xef00ef, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff00ef, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x440044, 0x440044, 0xffff0044, 0xffffffff, 0xffff0044, 0xffff0044, 0xffffffff, 0xffffffff, 0xffffffff, 0x44ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x440044, 0xffffffff, 0x44ffff, 0xffff0044, 0xffff0044, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x44ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x440044, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x45ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x45ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x460046, 0xffffffff, 0xffff00ca, 0xffffffff, 0xffff0101, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x46ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x470047, 0xffffffff, 0xffff0047, 0xffffffff, 0x470047, 0xffff0047, 0xffffffff, 0xffffffff, 0xffffffff, 0x47ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0047, 0xffffffff, 0xffffffff, 0xffff0047, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0047, 0xffffffff, 0xffffffff, 0xffff0047, 0x470047, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0047, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0047, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0047, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x47ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x47ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x480048, 0xffffffff, 0xffff0048, 0xffffffff, 0x480048, 0xffff0048, 0xffffffff, 0xffffffff, 0xffffffff, 0x48ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0048, 0xffffffff, 0xffffffff, 0xffff0048, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0048, 0xffffffff, 0xffffffff, 0xffff0048, 0x480048, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0048, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0048, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0048, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x48ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x48ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x490049, 0x490049, 0xffff0049, 0xffffffff, 0xffff0049, 0xffff0049, 0xffffffff, 0xffffffff, 0xffffffff, 0x49ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x490049, 0xffffffff, 0x49ffff, 0xffff0049, 0xffff0049, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x49ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x490049, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x4a004a, 0x4a004a, 0xffff004a, 0xffffffff, 0xffff004a, 0xffff004a, 0xffffffff, 0xffffffff, 0xffffffff, 0x4affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4a004a, 0xffffffff, 0x4affff, 0xffff004a, 0xffff004a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4a004a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x4b004b, 0xffffffff, 0xffff004b, 0xffffffff, 0xffff004b, 0xffff0175, 0xffffffff, 0xffffffff, 0xffffffff, 0x4bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0175, 0xffffffff, 0xffffffff, 0xffff0175, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x4c004c, 0x4c004c, 0xffff004c, 0xffffffff, 0xffff004c, 0xffff004c, 0xffffffff, 0xffffffff, 0xffffffff, 0x4cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4c004c, 0xffffffff, 0x4cffff, 0xffff004c, 0xffff004c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4c004c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x4d004d, 0xffffffff, 0xffff004d, 0xffffffff, 0x4d004d, 0xffff004d, 0xffffffff, 0xffffffff, 0xffffffff, 0x4dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004d, 0xffffffff, 0xffffffff, 0xffff004d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004d, 0xffffffff, 0xffffffff, 0xffff004d, 0x4d004d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x4e004e, 0xffffffff, 0xffff004e, 0xffffffff, 0x4e004e, 0xffff004e, 0x115ffff, 0xffffffff, 0xffffffff, 0x4e017c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004e, 0xffffffff, 0xffffffff, 0xffff004e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004e, 0xffffffff, 0xffffffff, 0xffff004e, 0x4e004e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0115, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004e, 0xffff017c, 0x17c017c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x17cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x4f004f, 0xffffffff, 0xffff004f, 0xffffffff, 0x4f004f, 0xffff004f, 0xffffffff, 0xffffffff, 0xffffffff, 0x4fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004f, 0xffffffff, 0xffffffff, 0xffff004f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004f, 0xffffffff, 0xffffffff, 0xffff004f, 0x4f004f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff004f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x500050, 0xffff00bb, 0xffff0050, 0xffffffff, 0xffff0050, 0xffff0176, 0xffffffff, 0xffffffff, 0xffffffff, 0x50ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbb0176, 0xffffffff, 0xffffffff, 0xffff0176, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x510051, 0x510051, 0x42f0051, 0x13d0606, 0x13d0051, 0x13d0051, 0x13dffff, 0xffffffff, 0x13dffff, 0x51ffff, 0x13dffff, 0xffffffff, 0xffff013d, 0x13dffff, 0x510051, 0xffffffff, 0x51ffff, 0xffff0051, 0xffff0051, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x51ffff, 0xffffffff, 0xffffffff, 0x29cffff, 0xffffffff, 0x510051, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x29cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x520052, 0xffffffff, 0xffff0052, 0xffffffff, 0x520052, 0xffff0052, 0xffffffff, 0xffffffff, 0xffffffff, 0x52ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0052, 0xffffffff, 0xffffffff, 0xffff0052, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0052, 0xffffffff, 0xffffffff, 0xffff0052, 0x520052, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0052, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0052, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0052, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x52ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x52ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x530053, 0xffffffff, 0xffff0053, 0xffffffff, 0x530053, 0xffff0053, 0xffffffff, 0xffffffff, 0xffffffff, 0x53ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0053, 0xffffffff, 0xffffffff, 0xffff0053, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0053, 0xffffffff, 0xffffffff, 0xffff0053, 0x530053, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0053, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0053, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0053, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x53ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x53ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x540054, 0x540054, 0xffff0054, 0xffffffff, 0xffff0054, 0xffff0054, 0xffffffff, 0xffffffff, 0xffffffff, 0x54ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x540054, 0xffffffff, 0x54ffff, 0xffff0054, 0xffff0054, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x54ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x540054, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x550055, 0x550055, 0xffff0055, 0xffffffff, 0xffff0055, 0xffff0055, 0xffffffff, 0xffffffff, 0xffffffff, 0x55ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x550055, 0xffffffff, 0x55ffff, 0xffff0055, 0xffff0055, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x55ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x550055, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x56ffff, 0xffff0056, 0xffffffff, 0x13e0056, 0xffffffff, 0xffffffff, 0x13effff, 0x560056, 0xffffffff, 0x56013e, 0xffffffff, 0x13e013e, 0x13effff, 0xffff013e, 0x56ffff, 0x13effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0056, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x56ffff, 0xffffffff, 0xffffffff, 0xffff0056, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x56ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x570057, 0xffffffff, 0xffff0057, 0xffffffff, 0x570057, 0xffff0057, 0x116ffff, 0xffffffff, 0xffffffff, 0x57ffff, 0xffffffff, 0xffffffff, 0x435ffff, 0xffffffff, 0xffff0057, 0xffffffff, 0xffffffff, 0xffff0057, 0xffffffff, 0xffffffff, 0xffffffff, 0x60affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0057, 0xffffffff, 0xffffffff, 0xffff0057, 0x570057, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0116, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0435, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x60affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0057, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0057, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0057, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x57ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x57ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x580058, 0xffffffff, 0xffff0058, 0xffffffff, 0x580058, 0xffff0058, 0x117ffff, 0xffffffff, 0xffffffff, 0x580181, 0xffffffff, 0xffffffff, 0x436ffff, 0xffffffff, 0xffff0058, 0xffffffff, 0xffffffff, 0xffff0058, 0xffffffff, 0xffffffff, 0xffffffff, 0x60bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0058, 0xffffffff, 0xffffffff, 0xffff0058, 0x580058, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0117, 0xffffffff, 0xffffffff, 0x8b3ffff, 0xffffffff, 0xffffffff, 0x978ffff, 0xffffffff, 0xffff0436, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x60b08b3, 0xffff0978, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0058, 0xffff0181, 0x1810181, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x181ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0058, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0058, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x58ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x58ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x590059, 0xffffffff, 0xffff0059, 0xffffffff, 0x590059, 0xffff0059, 0xffffffff, 0xffffffff, 0xffffffff, 0x59ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0059, 0xffffffff, 0xffffffff, 0xffff0059, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0059, 0xffffffff, 0xffffffff, 0xffff0059, 0x590059, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0059, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0059, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0059, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x59ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x59ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x5a005a, 0x5a005a, 0xffff005a, 0xffffffff, 0xffff005a, 0xffff005a, 0xffffffff, 0xffffffff, 0xffffffff, 0x5affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x5a005a, 0xffffffff, 0x5affff, 0xffff005a, 0xffff005a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x5affff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x5a005a, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x5b005b, 0xffffffff, 0xffff005b, 0xffffffff, 0x5b005b, 0xffff005b, 0xffffffff, 0xffffffff, 0xffffffff, 0x5bffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff005b, 0xffffffff, 0xffffffff, 0xffff005b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff005b, 0xffffffff, 0xffffffff, 0xffff005b, 0x5b005b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff005b, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x5c005c, 0xffffffff, 0xffff005c, 0xffffffff, 0x5c005c, 0xffff005c, 0xffffffff, 0xffffffff, 0xffffffff, 0x5cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff005c, 0xffffffff, 0xffffffff, 0xffff005c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff005c, 0xffffffff, 0xffffffff, 0xffff005c, 0x5c005c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff005c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff005c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff005c, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x5cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x5cffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x5dffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
            };
        }

        protected static int[][] value3 = null;

        protected static void value3Init()
        {
            value3 = new int[][]
            {
                { 0x5e00ae, 0xb200bd, 0xffff00cb, 0xdb00e6, 0xf30102, 0xffff00ae, 0x1190121, 0x127012b, 0xffffffff, 0x5e0182, 0xffffffff, 0xffffffff, 0x437ffff, 0xffffffff, 0xbd00cb, 0xffffffff, 0x2bcffff, 0xffff0102, 0xffff02c0, 0xffffffff, 0xffffffff, 0x437ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb2ffff, 0xffff0285, 0xffffffff, 0xffff02ac, 0xffffffff, 0x2bc02c0, 0x2c400f3, 0x2cd00f3, 0xffffffff, 0xffff02fa, 0xffffffff, 0xffff0119, 0xffffffff, 0x321ffff, 0x8b4ffff, 0xffffffff, 0xffffffff, 0x8b4ffff, 0xffffffff, 0xffff0437, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x43708b4, 0xffff08b4, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff02cd, 0x12b0182, 0x18203ab, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3abffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff088e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff088e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x88effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x88effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x5f005f, 0xffffffff, 0xffff00cc, 0xffffffff, 0xffff0103, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x5fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x600060, 0x600060, 0xffff0060, 0xffffffff, 0xffff0060, 0xffff0060, 0xffffffff, 0xffffffff, 0xffffffff, 0x60ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x600060, 0xffffffff, 0x60ffff, 0xffff0060, 0xffff0060, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x60ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x600060, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x61ffff, 0xffff00be, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x1280128, 0xffffffff, 0x61ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbeffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0128, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0128, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x128ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x620062, 0xffffffff, 0xffff0062, 0xffffffff, 0x620062, 0xffff0062, 0x62ffff, 0xffffffff, 0xffffffff, 0x620062, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0062, 0xffffffff, 0xffffffff, 0xffff0062, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0062, 0xffffffff, 0xffffffff, 0xffff0062, 0x620062, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0062, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0062, 0xffff0062, 0x620062, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x62ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x630063, 0xffffffff, 0xffff0063, 0xffffffff, 0x630063, 0xffff0063, 0xffffffff, 0xffffffff, 0xffffffff, 0x63ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0063, 0xffffffff, 0xffffffff, 0xffff0063, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0063, 0xffffffff, 0xffffffff, 0xffff0063, 0x630063, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0063, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0063, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0063, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x63ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x63ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 },
                { 0x640064, 0x640064, 0xffff0064, 0xffffffff, 0xf50064, 0xffff0064, 0xffffffff, 0xffffffff, 0xffffffff, 0x64ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x640064, 0xffffffff, 0x64ffff, 0xffff0064, 0xffff0064, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x64ffff, 0xffffffff, 0xffffffff, 0xffff00f5, 0xffffffff, 0x640064, 0xffff00f5, 0xf500f5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff00f5, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000 }
            };
        }

        protected static short lookupValue(int row, int col)
        {
            if (row <= 30)
                return (short)(col % 2 == 0 ? value[row][col/2] >>> 16 : value[row][col/2] & 0xFFFF);
            else if (row >= 31 && row <= 61)
                return (short)(col % 2 == 0 ? value1[row-31][col/2] >>> 16 : value1[row-31][col/2] & 0xFFFF);
            else if (row >= 62 && row <= 92)
                return (short)(col % 2 == 0 ? value2[row-62][col/2] >>> 16 : value2[row-62][col/2] & 0xFFFF);
            else if (row >= 93)
                return (short)(col % 2 == 0 ? value3[row-93][col/2] >>> 16 : value3[row-93][col/2] & 0xFFFF);
            else
                throw new Error("Unexpected location requested in value3 lookup");
        }

        static
        {
            sigmapInit();
            sigmap1Init();
            sigmap2Init();
            sigmap3Init();
            sigmap4Init();
            valueInit();
            value1Init();
            value2Init();
            value3Init();
        }
    }
}
